/*
 * HISTORY.C: handle history and headline items in a structured binary file.
 *
 *	 9-Mar-88	V1.0	Created					PJT
 *	 1-Jun-88	V1.1	renamed read/write to get/put		PJT
 *	 7-jun-88	V1.2	installed in libT with getparam();
 *                              dynamic HISTORY				PJT
 *      14-Jun-88       V1.3    integrated with IAS NEMO		JEB
 *	15-Dec-88	V1.4	allow creation from scratch		PJT
 *	15-jul-89	V1.5	added reset_history(), fixed order	PJT
 *				of Headline's and History's
 *	29-sep-89	V1.6	added item= keyword                     PJT
 *	 8-dec-89	V1.7	moved some stuff to history.h, no error PJT
 *	10-sep-90       V1.8    << moved history_level around >>	PJT
 *	 9-oct-90	V1.8a   documentation upgraded			PJT
 *      17-jul-91       V1.9    extra option skip=                      PJT
 *	 9-apr-92	V2.0    add_history -> app_history		PJT
 *				because of conflicting libreadline.a
 *	20-feb-94	V2.1    ansi
 *
 */

#include <stdinc.h>
#include <filestruct.h>
#include <history.h>
#include <string.h>

/* 
 * The datastructure to store history is a simple array of pointers to
 * history strings (in safe memory we may hope).
 * The array is declared static, but you may also think of
 * a dynamically allocated linked list in the future.
 *				-- sure, and just how will that work?  JEB
 */

#define MAXHIST 256			/* max size of history array        */

local int nhist = 0;			/* count history data stored so far */
local string histbuf[MAXHIST+1];	/* history string array             */
local string headline = NULL;		/* last headline read in            */

extern int history;            /* 1=history is auto-mode  0=no history done */
                               /* this item should eventually disappear     */

/*
 * GET_HISTORY: read history and headline info into local buffer;
 * returns current number of history items
 */

int get_history(instr)
stream instr;			/* input stream to read from */
{
    for(;;) {                   		/* loop reading input data  */
	if (get_tag_ok(instr, HeadlineTag)) {
	    headline = get_string(instr, HeadlineTag);
	    dprintf(5, "get_history: headline = %s\n", headline);
	} else if (get_tag_ok(instr, HistoryTag)) {
	    if (nhist > MAXHIST) {
		warning("Warning: no more history saved\n");
		return(MAXHIST);
	    }
	    histbuf[nhist] = get_string(instr, HistoryTag);
	    dprintf(5, "get_history: histbuf[%d] = %s\n",
		    nhist, histbuf[nhist]);
	    nhist++;
	} else
            break;                          /* done with reading loop */
    }
    return (nhist);
}

/*
 * PUT_HISTORY:  write current history and headline data to output.
 *               always returns 0 ...
 */

int put_history(outstr)
stream  outstr;
{				
    int i;

    if (!history) {
    	dprintf(5, "put_history: history data suppressed\n");    
        return(0);
    }
    if (headline != NULL) {
	dprintf(5, "put_history: headline = %s\n", headline);
	put_string(outstr, HeadlineTag, headline);
    }
    dprintf(5, "put_history: writing %d history items\n", nhist);
    for (i = 0; i < nhist; i++) {
	dprintf(5, "             histbuf[%d] = %s\n", i, histbuf[i]);
    	put_string(outstr, HistoryTag, histbuf[i]);
    }
/*  nhist = 0;			// reset counter -- why? JEB */
    return(0);
}

/* 
 * APPEND_HISTORY: add to history
 *              returns current length of history list
 *
 */

int app_history(s)
string s;
{
    if (nhist > MAXHIST) {
	warning("app_history: too much history");
	return (nhist);
    } else if (s == NULL || streq(s, "")) {
	dprintf(1, "app_history: null history string\n");
	return (nhist);
    }
    histbuf[nhist] = s;
    dprintf(9,"app_history: histbuf[%d] = %s\n", nhist, s);
    nhist++;
    return(nhist);
}

/*
 * RESET_HISTORY: reset the history list
 */

void reset_history()
{
    nhist = 0;
}

/*
 * SET_HEADLINE: set new value for headline string.
 */

void set_headline(s)
string s;
{
    headline = s;
}

/*
 * ASK_HEADLINE: enquire about value of headline.
 */

string ask_headline()
{
    return (headline);
}

/*
 * ASK_HISTORY: enquire about history data.
 */

string *ask_history()
{
    if (nhist > MAXHIST)
	error("ask_history: too much history");
    histbuf[nhist] = NULL;			/* NULL-term the hist array */
    return (histbuf);
}

#ifdef TOOLBOX
			/* PROGRAM: hisf */
#include <getparam.h>

char *defv[] = {		/* DEFAULT INPUT PARAMETERS */
    "in=\n		Input file name",
    "out=\n		If given, output history",
    "item=all\n		Show which items",
    "history=\n		If given, add this text to history",
    "skip=1\n           Number to skip before display",
    "VERSION=1.9\n	17-jul-91 PJT",
    NULL,
};

string iname,oname;			/* input/output file  name */
stream instr,outstr;

nemo_main()
{
    int i, skip;
    int item;

    iname = getparam("in");
    oname = getparam("out");
    if (streq(getparam("item"),"all"))
        item = -1;
    else
        item = getiparam("item");       /* get a particular history item */
    skip = getiparam("skip");
    if (skip<0) error("Skip=%d cannot be negative");

    if (! streq(iname,"")) {
        instr = stropen(iname, "r");                /* open input file */
        get_history(instr);                         /* get old */
        if (! streq(getparam("history"), ""))
	    app_history(getparam("history"));       /* add new */
        for (i = skip; i < nhist; i++) 
            if (item<0 || i==item) {
                dprintf(1,"%d: ",i);
                printf("%s\n", histbuf[i]);     /* list old history */
            }
        strclose(instr);
    } else if (! streq(getparam("history"), "")) {
        nhist--;				/* delete first one */
        app_history(getparam("history"));           /* add new from scratch */
    }

    if (! streq(oname, "")) {                       /* see if output needed */
        outstr = stropen(oname, "w");               /*  open file */
        put_history(outstr);                        /*  and dump history */
        strclose(outstr);                           
    }
}

#endif
