/*
 * (M)INT program stub for HatCreek
 *	14-jun		added options= after the fact
 *      19-oct-93       added start= and stop= for Jim's new start/stop
 *                      this is going to replace times= eventually
 *	3-nov-93	needvalue(source)
 *     11-nov           compacter output to appease Arie
 *      1-dec 		even more compact output to appease Stephen
 *	6-dec	   V3.0 allow formal new start/stop time method
 *                      idle time is added to slewing (...)
 *     26-jan-94        fixing up start= and stop=
 *     11-feb-94        added nreps= to MINT (but we're not supposed to use it)
 *     28-feb-94        added output of nchan and itime
 *	4-aug-94 	adding grid output for mosaicing - local parsing ...
 *	
 *  todo:  nchan<0 doesn't do doppler tracking; this should be the
 *	   default for calibrators.
 */

#include "checker.h"
#include <filefn.h>

string defv[] = {
    "source=\n              source name from catalog, or planet or COMET",
    "catalog=./j2000.cat\n  catalog where to search for source", 
    "ants=\n",
    "itime=50\n         integraton time in seconds",
#ifdef MINT
    "grid=1x1\n         Grid selector (file or DRA(a1,a2,..),DDEC(d1,d2..)",
    "scale=1\n          Scale factor for grid offsets (final: arcmin)",
    "nreps=1\n          Number of times to integrate at each grid point",
#endif
    "nchan=0\n          0=wide only, 1=spectral+wide, -1=no doppler track",
#ifdef MINT
    "vis=mintdata\n     output miriad dataset",
#else
    "vis=intdata\n      output miriad dataset",
#endif
    "device=\n          pgplot device for SHOW plots (see SETSHOW)",
    "trakopt=\n         CH (CH+ for mint) should be default",
#ifndef MINT
    "options=\n         (p,l)",
#endif
    "times=-1,-1\n      start and stop time (decimal hours) **deprecated**",
    "start=\n           Start time [now]",
    "stop=\n            Stop time [1 hour] [+100 in future]",
    "VERSION=3.4a\n     8-sep-94 PJT",    
    NULL,
};

#ifdef MINT
string usage="HatCreek MINT program stub";
#else
string usage="HatCreek INT program stub";
#endif


char msg1[32], msg2[32], msg3[32], msg4[32];

/* external functions */
char *show_lst();
char *strdup();
real get_lst(), slew(), get_stop();
bool check_time();

/* local forward declarations */
int gridparse(string);
int argcount(string, string);

nemo_main()
{
    bool new_ss;	/* use this for the new start/stop time method */
    real times[2], itime;
    real idle, slew_time, start, stop, new_lst, on_source, lst = get_lst();
    float az0, el0, az1, el1;
    int n, nreps, nchan, ngrid;
    string src, cat, lst0, lst1, name = getargv0();
    char s[32];
    bool Qgrid;


    strtoupper(name);

    needvalue("source");    
    src = getparam("source");
    cat = getparam("catalog");
    nchan = getiparam("nchan");
    itime = getdparam("itime");
#ifdef MINT
    ngrid = gridparse(getparam("grid"));
#else
    Qgrid = FALSE;
    ngrid = 0;
#endif

    new_ss = hasvalue("stop");
    if (!new_ss) {
       n = nemoinpd(getparam("times"),times,2);
       if (n != 2) error("times= needs two values");
    } else {
       if (hasvalue("start")) {
         start = getTime(getparam("start"), (int) 60.0*lst) / 60.0;
         if (check_time(start))
           error("Cannot start before the project start time");
         idle = start - lst;
         if (idle > 0.25)
            warning("sleeping a bit (%d minutes)",(int)60*idle);  
         if (idle < 0.0) {
            warning("you started too late!");
            idle = 0.0;
         }
         lst = start;
	 set_lst(lst);	/* sync the clocks */
       } 
       if (check_time(lst)) 
         error("Current time (%g) outside of project time range",lst);
    }

    check_source(src,cat);
    check_filename("vis",getparam("vis"));

    azel(src,cat,&az0,&el0);
    dprintf(2,"%s: source=%s started at lst=%g az=%g el=%g\n",
		name,src,lst,az0,el0);

    lst0 = strdup(show_lst(lst));

    if (get_azel(&az1,&el1))
        slew_time  = slew(az1,el1,az0,el0) + idle;
    else
        slew_time  = idle;

    if (new_ss) {                           /* new Start/Stop */
      stop = getTime(getparam("stop"), (int) 60.0*lst) / 60.0;
      if (check_time(stop)) {
        stop = getTime("stop", (int) 60.0*lst) / 60.0;
        warning("stoptime truncated by project stoptime");
      }
      on_source = stop-lst;
      if (on_source < 0) on_source += 24.0;
      dprintf(2,"SLEW/OBS %s %f %f %f\n",src,lst,slew_time, on_source);
      lst = stop;
    } else {                                /* old Times= */
      if (times[0] < 0) {
        if (times[1] < 0) {              /* incremental */
            on_source = ABS(times[1]);
            dprintf(2,"SLEW/OBS %s %f %f %f\n",src,lst,slew_time, on_source);
            lst += ABS(times[1]);
        } else {                          /* absolute : yuck */
            sprintf(s,"%g",times[1]);
	    check_real_range("times(2)",s,0.0,24.0,"hours");
            on_source = times[1] - lst;
            dprintf(2,"SLEW/OBS %s %f %f %f\n", src, lst, slew_time, on_source);
            lst = times[1];
        }
      } else {
        error("times(1) > 0 not implemented");
      }
    }

    check_real_range("itime",getparam("itime"),0.32,580.0,"seconds");
    check_int_range("nchan",getparam("nchan"),-1,1,"narrow/wide/doppler mode");
#ifdef MINT
    check_real_range("scale",getparam("scale"),0.0,999999999.0,"");
    nreps = getiparam("nreps");
    if (nreps>1) warning("Mel sez not to use nreps > 1");
#endif

    set_lst(lst);
    dprintf(2,"%s: stopped at lst=%g\n",name,lst);

    azel(src,cat,&az1,&el1);
    dprintf(2,"%s: stopped at az,el=%f %f\n",name,az1,el1);
    lst1 = strdup(show_lst(lst));
    sprintf(msg1,"%s-%s",lst0,lst1);
    sprintf(msg2,"%5.1f-%5.1f",az0,az1);
    sprintf(msg3,"%5.1f-%5.1f",el0,el1);
    sprintf(msg4,"%s  %s",strdup(show_lst(slew_time)), 
			  strdup(show_lst(on_source)));


    if (el0 < 0 || el1 < 0) warning("Source %s was set",src);
    set_azel(az1,el1);

    /* final (and only!) output */

    dprintf(1,"%s\n",OBS_HEADER);   /* this is a printf() in setfreq.c !!! */
    printf("%s: %-10s %2d %2d %5.1f %s %s %s %s\n",name,src,
		nchan, ngrid, itime,msg1,msg2,msg3,msg4);

}

#define GRIDLEN 300

int gridparse(string grid)
{
    char gstring[GRIDLEN], *cp;
    int inc, nra, ndec, nlen=0;
    stream instr;

    if (strchr(grid,'(') || strchr(grid,')')) 
        strcpy(gstring,grid);
    else {
        sprintf(gstring,".:%s",getenv("HATGRID"));
        instr = pathopen(gstring,grid,"r");
        if (instr==NULL) {
            warning("No grid file grid=%s in searchpath %s",grid,gstring);
            return 0;
        }
        for (cp=gstring; nlen<GRIDLEN; ) {
            if (fgets(cp,GRIDLEN-nlen,instr)==NULL) break;
            if (*cp == '#' || *cp == '!' || *cp == ';') {
                warning("grid=%s contains comments",grid);
                continue;
            }
            inc = strlen(cp) - 1;       /* don't copy the newline */
            if (cp[inc] == '\n')        /* patch newline */
                cp[inc] = 0;            
            else                        /* or else not read to EOF ? */
                warning("(%d) Your grid=%s may be too big",GRIDLEN,grid);
            nlen += inc;
            cp += inc;
        } /* for */
    }
    dprintf(1,"GRID: \"%s\"\n",gstring);
    for (cp=gstring; *cp; cp++)
        if (isupper(*cp)) *cp = tolower(*cp);
    nra = argcount("dra",gstring);
    ndec = argcount("ddec",gstring);
    if (nra!=ndec)
        warning("grid=%s contains %d DRA and %d DDEC positions",grid,nra,ndec);
    return MAX(nra,ndec);
}

int argcount(string word, string grid)
{
    char *cp=grid, match=word[0];
    int n, wordlen = strlen(word);

    for (;;) {          /* a poor man's string finder */
        if (*cp==0) return 0;
        cp = strchr(cp,match);
        if (cp==NULL) {
            warning("No \"%s\" found in %s",word,grid);
            return 0;
        }
        if (strncmp(cp,word,wordlen)==0)  /* found a match */
            break;
        cp++;
    }

    /* cp now points to the location in 'grid' where 'word' starts */
    cp += wordlen;
    n = 1;          /* count the number of arguments = #commas + 1 */
    while (*cp) {
        if (*cp == ')') break;
        if (*cp == ',') n++;
        cp++;
    }
    return n;
}

