//======================================================================
//	cora.h --	BOS Correlator control package.
//			jah -- Radio Astronomy Lab -- UC Berkeley
//			4/24/1992
// index of routines
// drivers (cora.cc and cora2.cc)
//	cora_adc --		Reads miscellaneous ADC's
//	cora_alloc --		Allocates the correlator device.
//	cora_atten --		Set IF attenuator in switchyard
//	cora_busy --		Indicates whether correlator busy.
//	cora_clip --		Set clipping voltage in sampler
//	cora_clock_ctrl --	Set correlator clock delays, duty cycle
//	cora_error --		Report any error
//	cora_filter --		Set up bandpass filters
//	cora_free --		Frees correlator device
//	cora_get_bank --	Get bank # of processor RAM (A_BANK,B_BANK)
//	cora_get_clip --	Get clipping voltage level from sampler
//	cora_get_clock_ctrl --	Get clock control word settings
//	cora_get_error_msg --	Gets text of current error message
//	cora_get_mode --	Get configuration information
//	cora_get_switch --	Get switchyard status info.
//	cora_get_wal_demod --	Read current demodulating Walsh fcts.
//	cora_if_power --	Reads IF power level in front of sampler
//	cora_if_select --	Select IF or test inputs from sampler
//	cora_int --		Starts integration
//	cora_load_data_mem --	Loads processor data memory
//	cora_load_sort_mem --	Loads correlator sort memory
//	cora_load_test_pat --	Loads correlator test pattern
//	cora_lo_lock --		Tests to see whether synthesizers locked
//	cora_lo_set --		Set synthesizer (3rd LO) frequency
//	cora_mode --		Sets up correlator in given mode
//	cora_readout --		Read correlator processor memory
//	cora_read_sort_mem --	Reads out the correlator sort memory
//	cora_read_test_pat --	Reads MST clock card test RAM
//	cora_reset --		Resets the correlator
//	cora_set_bank --	Set bank # of processor RAM (A_BANK,B_BANK)
//	cora_set_onebit_mode --	Sets correlator up as single-bit sampling
//	cora_set_twobit_mode --	Sets correlator up for modified 2 bit samps.
//      cora_sort_ptrs --    	Compute sort memory pointrs for config.
//	cora_switch --		Set IF switches, amplifiers in sw. yard
//	cora_test_signal --	Turn correlator test signal ON or OFF
//	cora_tweak --		Adjust sampler clipping levels best S/N
//	cora_wait --		Waits until correlator finshes integ.
//	cora_wait_int --	Wait till done; start new integration
//	cora_wal_demod --	Set correlator demodulating Walsh fcts.
//	cora_wal_switchy --	Set switchyard Walsh functions
//	cora_zero_data --	Zero processor data memory
//======================================================================

#ifndef cora_h
#define cora_h

// I/O Page Address Assignments
#define CORA_MST_CLOCK	0xE800	// MST clock control area
#define CORA_TEST_LEN		0xE806	// Length of test pattern cycle
#define CORA_TEST_ADDR		0xE808	// Test stream address
#define CORA_TEST_SIGNAL	0xE80A	// Test stream register
#define CORA_CLOCK_CTRL		0xE80C	// Clock control reg/test control
#define CORA_SWITCHY	0xE93C	// IF Switchyard device
#define CORA_CLOCK	0xEA00	// Centisecond clock
#define CORA_MBUS_RAM	0xEA04 // Non-volatile random-access memory
#define CORA_MBUS_RAM_FIRST	0xEA04	// 1st avail word
#define CORA_MBUS_RAM_LAST	0xEAFE	// last avail word
#define CORA_IDENTIFIER		0xEA18	// User name for interlock test

#define CORA_PROCESSOR	0xEB80	// Correlator processor control area
#define CORA_PRO_SEL		0xEB80	// Processor select
#define CORA_PRO_ADDR		0xEB82	// Processor address
#define CORA_PRO_DATA		0xEB84	// Processor data register

#define CORA_WALFS	0xEE00	// Correlator demodulating Walsh fct's
#define CORA_INT_CTRL	0xEE1C	// Integration control word
#define CORA_INT_TIMER	0xEE1E	// Integration time counter

#define CORA_SAMS	0xF000	// Correlator sampler cards & related stuff
				// Range 0xF000 -- 0xF800.
#define CORA_ADC_START	0xF7F0	// Where to send command to start ADC
#define CORA_ADC	0xF7F2	// A/D readout register
#define CORA_MUX	0xF7F4	// A/D Multiplexer and test signal input
#define CORA_BWS	0xF7F6	// Bandwidth filter control word
#define CORA_DAC3	0xF7F8	// Sampler clipping voltage control, No. 3
#define CORA_DAC2	0xF7FA	//   "        "       "        "         2
#define CORA_DAC1	0xF7FC	//   "        "       "        "         1
#define CORA_DAC0	0xF7FE	//   "        "       "        "         0

// Offset values used by cora_sam_addr to compute appropriate sampler
// control addresses (simpler to use than the above)
#define CORA_SAM_DACS		0x0000	// 4 DACs associated w/ sampler card
#define CORA_SAM_BWS		0x0008	// 2 BWs associated with card
#define CORA_SAM_MUX		0x000A	// Multiplexed word
#define CORA_SAM_ADC		0x000C	// A/D readout (multiplexed)
#define CORA_SAM_ADC_START	0x000E	// Start A/D integrate (write only)

// Correlation type
#define CROSSCORR   0
#define AUTOCORR    1

// Test pattern bank
#define XBANK 0
#define YBANK 1

// Processor memory area selection codes
#define	DATA_MEMORY	0
#define SORT_MEMORY	1
#define CONTROL_PORTS	2
#define DATA_PORTS	3
#define THIS_BANK	0x0000
#define A_BANK		0x0100
#define B_BANK		0x0200

// Limits
#define NANTS	2	// Temporary, I hope!!!
//#define NCORS	(NANTS*(NANTS-1)/2)
#define NCORS	32

// Boolean "type"
#ifndef BOOLEAN
#define BOOLEAN unsigned int
#define TRUE 1
#define FALSE 0
#endif

#define ON  1
#define OFF 0

// Correlator component
#ifndef RE
#define RE 0
#define IM 1
#endif

// Physical units
typedef float SECONDS;
typedef char USERID[8];
typedef float MHZ;
typedef float DBM;

// Miscellaneous
#define ushort unsigned short

// Prototypes
int	cora_adc	// Read miscellaneous ADC's on sampler card
(	int ant,	// in	Antenna number, 0..11
	int card,	// in	Sampler card for that ant.: 0..1
	int muxno );	// in	ADC selector from multiplexor
BOOLEAN cora_alloc	// Attempt to allocate correlator device to user
(   USERID user );	// in	User identifier.
BOOLEAN cora_atten	// Set IF attenuator in switchyard
(	int ant,	// in	Antenna number, 0..11
	int level );	// in	Attenuator level to set, 1 dB steps
BOOLEAN cora_busy	// Indicate whether correlator now integrating
(   void );
void cora_clear_error();// Clear any error condition; set error msg to "OK"
BOOLEAN cora_clip	// Set clipping level for given sampler
(	int ant,	// in	Antenna number, 0..11
	int sam,	// in	Sampler number (0=>A, ... 7=>H)
	float v );	// in	Clipping voltage 0.0 .. 1.0
void cora_clock_ctrl	// Condition the MST clock signal
(	int test_delay,	// in	Test signal delay, 0..7 nsec.
	int dist_delay, // in	Distributor clock delay, 0..7 nsec.
	int samp_delay,	// in	Sampler clock delay, 0..7 nsec.
	int duty_cycle);// in	Clock duty cycle, 0..7 
void cora_error		// Report any correlator error
(	char * msg );	// in	Text of error message
BOOLEAN cora_filter	// Set filter bandwidth for given sampler
(	int ant,	// in	Antenna number, 0..11
	int sam,	// in	Sampler number (0=>A, ... 7=>H)
			//	But note: May set only A,B,C,D (0,1,2,3)
	MHZ bw );	// in	Bandwidth: 200, 50, 25, 12.5, 6.25
BOOLEAN cora_free	// Free correlator device (assign to null user)
(	void );
int cora_get_bank();	// Get bank # of processor RAM (A_BANK,B_BANK)
float cora_get_clip	// Get clipping voltage level from sampler 
			//	range 0.0 .. 1.0
(	int ant,	// in	Antenna number, 0..11
	int sam);	// in	Sampler number (0=>A, ..., 7=>H)
void cora_get_clock_ctrl  // Fetch contents of MST clock control word
(	int & test_delay, // out Test signal delay, 0..7 nsec.
	int & dist_delay, // out Distributor clock delay, 0..7 nsec.
	int & samp_delay, // out Sampler clock delay, 0..7 nsec.
	int & duty_cycle, // out Clock duty cycle, 0..7 
	int & test_bit);  // out Status of test signal bit
char * cora_get_error_msg	// return pointer to error message
(	void );
MHZ cora_get_filter	// Look up filter bandwidth for given sampler
(	int ant,	// in	Antenna number, 0..11
	int sam);	// in	Sampler number (0=>A, ... 7=>H)
			//	But note: May get only A,B,C,D (0,1,2,3)
BOOLEAN cora_get_if_select	// Get IF state
(	int ant,	// in	Antenna number, 0..11
	int card,	// in	Sampler card
	int & ifsel,	// out	IF 1=>ON or 0=>OFF
	int & testxy,	// out	0=>X or 1=>Y test input
	int & testsel);	// out	Test signal 1=>ON 0=>OFF
int cora_get_mode	// Retrieve correlator mode, mpx. Return 0=>not init
(	int bl,		// in	Baseline number of correlator, 0..69
	int & mode,	// out	Spectrometer mode, 1..4
	int & ctype,	// out	Correlation type: CROSSCOR or AUTOCOR
	int & mpx,	// out	Time multiplexing, 0..2
	MHZ shrate[2] );// out	Correlator shift rate
int cora_get_switch	// Get switchyard status info.
(	int ant,	// in	Antenna number, 0..11
	int & atten,	// out	Attenuator setting, 0..15
	int & opcal,	// out	Output selector, 0=>normal passthru, 1=>calib.
	int & onoff,	// out	Input selector,  0=>calib. (summer), 1=>normal
	int & mode,	// out	Configured for:  0=>normal interf., 1=>VLBI
	int & auxwal );	// out	State of aux. Walsh fct. generator 0=>OFF 1=ON
BOOLEAN cora_get_wal_demod // Read current demodulating Walsh fcts.
(	int bl,		// in	Baseline, 0..69
	int & w0,	// out	Walsh function sequencies
	int & w1,
	int & w2 );
float	cora_if_power	// Read IF power level in front of sampler: 0 .. 1.0
(	int ant,	// in	Antenna number, 0..11
	int sam); 	// in	Sampler number (0=>A,...7=>H)
BOOLEAN cora_if_select	// Select IF or test inputs to sampler
(	int ant,	// in	Antenna number, 0..11
	int card,	// in	Sampler card
	int ifsel,	// in	Turn IF 1=>ON or 0=>OFF
	int testxy,	// in	Choose 0=>X or 1=>Y test input
	int testsel );	// in	Select test signal (disable IF) 1=>ON 0=>OFF
BOOLEAN cora_int	// Start correlator, wait until underway
(	SECONDS tint );	// in	Integration time requested
BOOLEAN cora_load_data_mem  // Load processor data RAM
(	int bl,		// in	Baseline number of correl. to be written
	int compon,	// in	Component, RE or IM
	long int * data,// in	Data to be loaded into memory
	int first,	// in	Index of first channel
	int last );	// in	Index of last channel
BOOLEAN cora_load_sort_mem  // Load associative memory bank
(	int bl, 	// in	Baseline number of correl. to be set up.
	short int * ps,	// in	Pointers to be loaded into sort memory
	int first,	// in	Index of first channel
	int last );	// in	Index of last channel
BOOLEAN cora_load_test_pat // Load test signal for correlator
(	int length,	// in	Length (#short words) of pattern
	ushort * pat1,	// in	Xbank Pattern.  2 bits/sampler
	ushort * pat2);	// in	Ybank Pattern.  "  "     "
BOOLEAN cora_lo_lock	// Test whether 3rd LO successfully locked(TRUE)
(	int lo );	// in	LO number, 1..4
BOOLEAN cora_lo_set	// Set 3rd LO to given frequency
(	int lo,		// in	LO number, 1..4
	MHZ freq,	// 3rd LO frequency, 6.25..990 MHz
	DBM ampl );	// in	Amplitude in dBm units
BOOLEAN cora_mode	// Set spectrometer mode, shift rate, multiplex.
(	int bl, 	// in	Baseline number of correl. to be set up
	int mode,	// in	Spectrometer mode, 1..4
	int ctype,	// in	Correlation type: CROSSCOR or AUTOCOR
	int mpx,	// in	Time multiplexing, 0..2
	MHZ shrate[2] );// in	Shift rates for cards, 12.5, 25, or 50
BOOLEAN cora_readout	// Read out correlator data RAM
(	int bl,		// in	Baseline number, 0..69
	int compon,	// in	Component, RE or IM
	long int * dat,	// out	Data array
	int first,	// in	Index of first channel
	int last );	// in	Index of last channel
BOOLEAN cora_read_sort_mem // Read out correlator associative memory
(	int bl,		// in	Baseline number, 0..69
	short int * dat,// out	Data array
	int first,	// in	Index of first channel
	int last );	// in	Index of last channel
BOOLEAN cora_read_test_pat  // return FALSE on error
(	int & length,	// out	Length (#short words) of pattern
	ushort * pat1,	// out	Xbank Pattern.  2 bits/sampler
	ushort * pat2);	// out	Ybank Pattern.  "  "     "
BOOLEAN cora_reset	// Reset the correlator, halt integration.
(	void );
int cora_set_bank	// Set bank # of processor RAM (A_BANK,B_BANK)
(	int bank );	// in	Bank: A_BANK or B_BANK
BOOLEAN cora_set_onebit_mode //	Sets correlator up as single-bit sampling
(	int bl );	// in	Baseline number, 0..69
BOOLEAN cora_set_twobit_mode //	Sets correlator up for modified 2 bit samps.
(	int bl );	// in	Baseline number, 0..69
BOOLEAN cora_sort_ptrs	// Calculate associative memory pointers
(	int mode,	// in	Spectrometer mode, 1..4
	int ctype,	// in	Correlation type: CROSSCOR or AUTOCOR
	int mpx,	// in	Time multiplexing, 0..2
	short int ps[2048]);
BOOLEAN cora_switch	// Set up correlator swyd. settings for antenna
(	int ant,	// in	Antenna number, 0..11
	int opcal,	// in	Op/Cal selector, 0=>op 1=>cal ?
	int onoff,	// in	On/Off selector, 0=>on 1=>off ?
	int mode,	// in	Mode selector,   0=>normal 1=>VLBI ?
	int auxwal );	// in	Aux Walsh gen,   0=>off 1=>on ?
void cora_test_signal	// Set test signal state
(	int state );	// in	ON or OFF
BOOLEAN cora_tweak	// Adjust sampler clipping voltage for best S/N
(	int ant,	// in	Antenna number, 0..11
	int sam );	// in	Sampler for that antenna, 0..11
BOOLEAN cora_wait();	// Wait until correlator finishes current int.
BOOLEAN cora_wait_int	// Wait until correlator done; start new int.
(	SECONDS tint );	// in	Integration time, seconds, n X 0.320 s
BOOLEAN cora_wal_demod	// Set correlator Walsh demodulating functions
(	int bl,		// in	Baseline number, 0..69
	int w0,		// in	Demodulation Walsh function sequencies
	int w1, int w2);
BOOLEAN cora_wal_switchy// Set switchyard Walsh functions
(	int ant,	// in	Antenna number, 0..11
	int w1,		// in	90 degree Walsh function
	int w2 );	// in	180 degree Walsh function
BOOLEAN cora_zero_data	// Zero correlator data RAM memory
(	int bl,		// in	Baseline number, 0..69
	int first,	// in	Index of first channel
	int last );	// in	Index of last channel

// Semi-private
void cx_chars_ints( char * cb, int nw, short int * ib );
void cx_ints_chars( int nw, short int * ib, char * cb );
void wait( SECONDS t );
SECONDS rawtime( void );
sortindex( unsigned int dataddr, unsigned int mode,	unsigned int ctype,
	unsigned int mux);
int cora_area_select( int bl, int area );
int cora_data_address_init( int bl, int area, int addr );
unsigned int cora_sam_addr( int ant, int card, int offset );
void cora_card_command( int bl, int card, int command );
void cora_chip_command( int bl, int card, int chip, int command );
int cora_get_card_command( int bl, int card );
int cora_get_chip_command( int bl, int card, int chip );

#endif

