//=======================================================================
//	coracalc
//=======================================================================
//  Index of routines
//
//	abssum		-- Find sum of absolute values of array of complex
//	antenna1	-- Find leading antenna for baseline -- X input
//	antenna2	-- Find second antenna for baseline  -- Y input
//	ant_online	-- Tell whether antenna no. a is on-line
//	apply_gains	-- Apply calibrated gains to computed cross spectrum
//	arranger	-- Re-arrange data, separating sidebands
//	baseline	-- Find baseline belonging to antenna pair
//	clipcorr	-- Correct lag data for sampler voltage clipping
//	clipping_corrector-- Correct lag data--all windows
//	Destroy2dArray	-- Unallocate storage for any 2d array
//	divide_passband	-- Divide spectrum by passband cal.  Renormalize.
//	divider		-- Divide one set of complex data by another
//	fixbadchan	-- Fix bad correlator channels in lag data
//	fixbadfreq	-- Fix bad freq. channels in spectrum--zero them!
//	fixlagoutliers	-- Fix bad correlator channels that are too loud
//	floatlags	-- Convert raw lag data into complex type
//	get_ant_bitmap	-- Get on-line antennas bitmap
//	get_ant_logid	-- Get antenna logical identifier, 0..11
//	get_ant_physid	-- Get antenna physical identifer, 1,3,4,...,13
//	get_chan_freq	-- Get frequency value for channel within spectrum
//	get_corf_no	-- Get LO3 mixer number assoc. with piece of spectrum
//	get_freq_chan	-- Get channel number for IF frequency; -1=>outside seg
//	get_lo3_power	-- Get LO3 power settings, given a frequency f
//	get_online_ants -- Get list of on-line antennas
//	get_online_nants-- Get number of on-line antennas
//	get_online_bls	-- Get list of on-line baselines
//	get_online_nbls	-- Get number of on-line baselines
//	get_cora_card	-- Get correlator card no, 0..1 for piece of spectrum
//	get_sam_freq	-- Get actual sampler mid-frequency within IF band
//	get_sam_no	-- Get number of sampler assoc. with piece of spectrum
//	get_walsh_ant	-- Get Walsh functions for antenna
//	get_walsh_cor	-- Get dewalshing sequencies for correlator
//	getmaxants	-- Get max number of antennas we are cabled for
//	getmaxbls	-- Get max number of baselines+extra slots on proc. crd
//	halfshift	-- Shift spectrum 1/2 channel; operate on lag data
//	hanning		-- Hanning smoothing of spectra
//	lin_interp	-- Linear interpolation of table values
//	Make2dArray	-- Allocate storage & initialize any 2d array
//	move		-- Move complex data from one part of array to another
//	normalize	-- Normalize lag data by dividing out sat. count
//	normalize2	-- Normalize lag data gathered in 2-bit mode
//	opacity		-- Compute opacity of atmosphere based on a model
//	passband_adjust	-- Adjust passband spectrum for transmission losses
//	processor	-- Spectrum processor
//	reverse		-- Reverse the order of spectral channels
//	scaler		-- Scale array of complex data
//	setmaxants	-- Set max number of antennas we are cabled for
//	set_walsh_ant	-- Set walsh function pair for given antenna
//	spectrum_maker	-- Compute spectrum from lag data
//	tcal		-- Compute equivalent calibration temperature
//	unbias		-- Subtract bias level from lag data
//	vanvleck	-- Correct lag data for one-bit sampling
//=======================================================================

#ifndef CORACALC_H
#define CORACALC_H

#include "complex.h"
#include "fft.h"

const int MaxPossibleAnts = 12;
enum SideBand{ Lower=0, Upper=1 };
#ifndef CROSSCORR
#define CROSSCORR 0
#define AUTOCORR  1
#endif

int setmaxants(		// Set maximum number of antennas we are cabled for
	int maxants );	// in	Maximum number of antennas for which we
			//	are presently cabled.  VERY IMPORTANT to
			//	get this right!
int getmaxants(	);	// Get maximum number of antennas we are cabled for
int getmaxbls( );	// Get max number of baselines+extra slots on proc. crd
int antenna1(		// Find leading antenna for baseline -- X input
	int bl );	// in	Baseline number, 0 .. 65
int antenna2(		// Find second antenna for baseline  -- Y input
	int bl );	// in	Baseline number, 0 .. 65
int ant_online(		// Tell whether antenna number a is on-line
	int a,		// in	Antenna number, 0..11
	int ants );	// in	Antenna flags, 1=>on line
int get_ant_physid(	// Look up physical antenna identifier
	int a );	// in	Antenna number, 0..11
int get_ant_logid(	// Look up logical antenna identifier
	int pa );	// in	Physical number, 1, 3, 4, ..., 13
int baseline(		// Find baseline belonging to antenna pair
	int a1,		// in	Antenna number, leading antenna, 0 .. 11
	int a2 );	// in	Antenna number, second antenna, 0 .. 11
int get_ant_bitmap();	// Get on-line antennas bitmap, assuming all online
int * get_online_bls(	// Get list of on-line baselines
	int ants );	// in	Antenna flags, 1=>on line
int get_online_nbls(	// Get number of on-line baselines
	int ants );	// in	Antenna flags, 1=>on line
int * get_online_ants(	// Get list of on-line antennas
	int ants );	// in	Antenna flags, 1=>on line
int get_online_nants(	// Get number of on-line antennas
	int ants );	// in	Antenna flags, 1=>on line
int get_sam_no(		// Get number of sampler assoc. with piece of spectrum
	int piece,	// in	Segment of spectrum, 0..15 (depends on mode)
	int mode);	// in	Spectrometer mode, 1..4
float get_sam_freq(	// Get actual sampler mid-frequency within IF band
	int sam,	// in	Sampler number, 0..7
	float corf[4],	// in	Correlator 3rd LO frequencies, MHz
	float bw[4],	// in	Filter bandwidth, MHz
	int ctype);	// in	Correlation type: CROSSCORR or AUTOCORR
float get_chan_freq(	// Get frequency value for channel within spectrum
	int chan,	// in	Channel number, 0..nchan-1
	float corf[4],	// in	Correlator 3rd LO frequencies, MHz
	float shrate[2],// in	Correlator processor shift rates, MHz
	int mode,	// in	Spectrometer mode, 1..4
	int mpx,	// in	Multiplexing code, 0 (1x)..2 (4x)
	int ctype );	// in	Correlation type: CROSSCORR or AUTOCORR
int get_freq_chan(	// Get channel number for IF frequency; -1=>outside seg
	float freq,	// in	IF Frequency, MHz
	float corf[4],	// in	Correlator 3rd LO frequencies, MHz
	float shrate[2],// in	Correlator processor shift rates, MHz
	int seg,	// in	Number of spectral segment, 0..15
	int mode,	// in	Spectrometer mode, 1..4
	int mpx,	// in	Multiplexing code, 0 (1x)..2 (4x)
	int ctype );	// in	Correlation type: CROSSCORR or AUTOCORR
int get_corf_no(	// Get LO3 mixer number assoc. with piece of spectrum
	int piece,	// in	Segment of spectrum, 0..15 (depends on mode)
	int mode);	// in	Spectrometer mode, 1..4
float get_lo3_power(	// Get LO3 power setting, given a frequency f
	float f );	// in	Frequency to look up, MHz
int get_cora_card(	// Get correlator card no, 0..1 for piece of spectrum
	int piece,	// in	Segment of spectrum, 0..15 (depends on mode)
	int mode);	// in	Spectrometer mode, 1..4
int get_walsh_ant(	// Get Walsh functions for antenna
	int ant,	// in	Antenna number, 0..11
	int & w0,	// out	w0 (90 degree phase modulation) sequency
	int & w1 );	// out	w1 (180  "      "       "           "
int set_walsh_ant(	// Set walsh function pair for given antenna
	int ant,	// in	Antenna number, 0..11
	int w0,		// in	w0 (90 degree phase modulation) sequency
	int w1 );	// in	w1 (180  "      "        "    )     "
int get_walsh_cor(	// Get dewalshing sequencies for correlator
	int w0k,	// in	Antenna k Walsh function, 90 deg. phase
	int w1k,	// in	Antenna k Walsh function, 180 deg. phase
	int w0l,	// in	Antenna l Walsh function, 90 deg. phase
	int w1l,	// in	Antenna l Walsh function, 180 deg. phase
	int & w0,	// out	Correlator w0 Walsh seq.
	int & w1,	// out	Correlator w1 Walsh seq.
	int & w2 );	// out	Correlator w2 Walsh seq.
float abssum(		// Find sum of absolute values of array of complex
	int nchan,	// in	Number of data samples
	complex * data);// in	Data array to be summed
void clipcorr(		// Correct lag data for sampler voltage clipping
	int nlags,	// in	Number of lag domain samples
	complex * data,	// in out Data samples, normalized on input,
			//	clipping corrected on output
	float v1,
	float v2);	// in	Clipping levels as fraction of RMS voltage
void vanvleck(		// Correct lag data for one-bit sampling
	int nlags,	// in	Number of lag domain samples
	complex * data);// inout Data samples, normalized on input,
			//	clipping corrected on output
int clipping_corrector(	// Correct lag data--all windows
	float lev1[],	// in	Array containing antenna1 clipping levels
	float lev2[],	// in	  "       "         "   2     "       "
	int mode,	// in	Spectrometer mode, 1..4
	int nchan,	// in	Number of channels
	complex * indata,// in	Input data array
	complex * outdata);// out Output data array
int floatlags(		// Convert raw lag data into complex type
	int nchan,	// in	Number of complex lag channels
	long int * re,	// in	Input data, real part
	long int * im,	// in	Input data, imag part [may set = NULL]
	complex * lags);// out	Converted lag data
int unbias(		// Subtract bias level from lag data
	float tint,	// in	Integration time, seconds
	int mpx,	// in	Degree of multiplexing, 0..2
	int nchan,	// in	Number of complex lag channels
	complex * data);// inout Data to be biased, in place
int normalize(		// Normalize lag data by dividing out sat. count
	float tint,	// in	Integration time, seconds
	float shrate[2],// in	Correlator processor shift rates, MHz
	int mode,	// in	Spectrometer mode, 1..4
	int mpx,	// in	Degree of multiplexing, 0..2
	int nchan,	// in	Number of complex lag channels
	complex * data);// inout Data to be normalized, in place
int normalize2(		// Normalize lag data gathered in 2-bit mode
	float tint,	// in	Integration time, seconds
	float ** a0lags,// in	Autocorrelation 0 lag data for all ants,sams
	float shrate[2],// in	Correlator processor shift rates, MHz
	int mode,	// in	Spectrometer mode, 1..4
	int mpx,	// in	Degree of multiplexing, 0..2
	int a1, int a2,	// in	Antenna numbers for this baseline, 0..11
	int nchan,	// in	Number of complex lag channels
	complex * data);// inout Data to be normalized, in place
int fixbadchan		// Fix bad correlator channels in lag data
(	int nbad,	// in	Number of bad channels
	short * bad,	// in	Bad channel list
	int nchan,	// in	Number of channels of lag data
	complex * data);// inout Data to be fixed
int fixlagoutliers(	// Fix outlier channels; return count of "fixed" chans.
	int mode,	// in	Spectrometer mode, 1..4
	int nchan,	// in	Number of channels of lag data
	complex * data);// inout Data to be fixed, must be normalized
int halfshift(		// Shift spectrum 1/2 channel; operate on lag data
	int mode,	// in	Spectrometer mode, 1..4
	int nchan,	// in	Number of complex lag channels
	complex * indata,// in	Input data array
	complex * outdata);// out Output data array
int spectrum_maker(	// Compute spectrum from lag data
	int mode,	// in	Spectrometer mode, 1..4
	int nchan,	// in	Number of channels
	complex * lags,	// in	Lag data
	complex * spec );// out	Spectrum
int fixbadfreq(		// Fix bad freq. channels in spectrum--zero them!
			// Assumes channels have been "arranged"
	int nbad,	// in	Number of bad frequencies
	float * badf,	// in	Bad frequencies, MHz
	float corf[4],	// in	Correlator LO freqs., MHz
	float shrate[2],// in	Correlator shiftrates, MHz
	int mode,	// in	Spectrometer mode, 1..4
	int mpx,	// in	Multiplexing code, 0 (1x)..2 (4x)
	int ctype,	// in	Correlation type: CROSSCORR or AUTOCORR
	complex * data);// inout Data to be corrected, in place
int arranger(		// Re-arrange data, separating sidebands
	int mode,	// in	Spectrometer mode, 1..4
	int nchan,	// in	Number of channels
	complex * indata,// in	Input data array
	complex * outdata);// out Output data array
int divider(		// Divide one set of complex data by another
	int nchan,	// in	Number of channels in each data set
	complex * num,	// inout Numerator and quotient
	complex * den);	// in	Denominator
//int divide_passband(	// Divide spectrum by passband cal.  Renormalize.
//	int mode,	// in	Spectrometer mode, 1..4
//	int nchan,	// in	Number of channels
//	complex * pb,	// in	Passband
//	complex * sp );	// inout Spectrum
int scaler(		// Scale array of complex data
	int nchan,	// in	Number of channels in data set
	float factor,	// in	Scale factor to multiply by
	complex * data);// inout Data to be scaled
int apply_cal(		// Apply calibrated gains to cross spectrum
	float ** TCalL,	// in	Gain array, lower sideband, Kelvins/norm.correl.
	float ** TCalU,	// in	  "    "    upper      "        "
	int ant1,	// in	Antenna 1, logical ID, 0..11
	int ant2,	// in       "   2,    "     "    "
	int mode,	// in	Spectrometer mode, 1..4
	int nchan,	// in	Spectrum channel count
	complex * indata,// in	Input spectrum, uncalibrated
	complex * outdata);// out Output spectrum, calibrated
int hanning(		// Hanning smoothing of spectra
	int mode,	// in	Spectrometer mode, 1..4
	int nchan,	// in	Number of channels total
	complex * indata,// in	Input data array
	complex * outdata);// out Output data array
int hanning(		// Hanning smoothing of spectra
	int mode,	// in	Spectrometer mode, 1..4
	int nchan,	// in	Number of channels total
	float * indata,// in	Input data array
	float * outdata);// out Output data array
int passband_adjust(	// Adjust passband spectrum for transmission losses
	float corf[4],	// in	Correlator 3rd LO frequencies, MHz
	float shrate[2],// in	Correlator shift rates, MHz
	int mode,	// in	Spectrometer mode, 1..4
	int mpx,	// in	Degree of multiplexing, 0..2
	int nchan,	// in	Number of channels passband spectrometer data
	complex * indata,// in	Input data array: passband
	complex * outdata);// out Output data array: adjusted passband
float opacity(		// Compute opacity of atmosphere based on a model
	float freq );	// in	Frequency, GHz
float tcal(		// Compute equivalent calibration temperature,
			// using approximation from Ulich & Haas
	float Tm,	// in	Mean atmospheric temperature, K
	float Tsbr,	// in	Appar. T from spillover, blockage, rad. losses, K
	float Tamb,	// in	Ambient flap temperature, K
	float tau[2],	// in	Atmospheric opacity, lower, upper sideband
	SideBand s,	// in	Which sideband: Upper or Lower
	float airmass,	// in	Path dist. thru atmosphere, zenith = 1
	float eta );	// in	Overall antenna efficiency
float** Make2dArray(	// Generate 2 dimensional array of float
	int dim1,	// in	First table dimension (slowly varying index)
	int dim2,	// in	Second table dimension (rapidly  "      "  )
	float initval);	// in	Value to load into the array initially
void Destroy2dArray(	// Free storage occupied by 2d array
	float ** array);// in	Pointer to array
float lin_interp(	// Linear interpolation of table values
			// Table assumed organized [nval][2]
	float arg,	// in	Argument for table lookup & interpolation
	int nval,	// in	Number of table entries, as pairs {ord, value}
	float table[][2]);// in	Table array
void reverse( int nchan, complex * data );
void move( int nchan, complex * dfrom, complex * dto );

#endif
