//=======================================================================
//	corafun -- Miscellaneous correlator-related procedures
//=======================================================================
//
//	INDEX TO FUNCTIONS
//
//	cora_adc_power	-- Determine RMS power levels at samplers from ADC's
//	cora_ant_avg_power--Takes average of all sampler power levels for an ant.
//	cora_ant_max_power--Finds max power of all samplers for an ant.
//	cora_auto_cor0(	-- Get zero-lag autocorrelations
//	cora_auto_volts	-- Determine RMS voltage at samplers from autocorrels.
//	cora_calibrate	-- Measures receiver power on sky & ambient,
//				calculates gains & system temps.
//	cora_fix_cor0	-- Correct the correlations for sampler problems
//	cora_flip( );	-- Do a bank flip of the correlator RAM buffers.
//	cora_get_swyard_attenuators -- Retrieve saved attenuator settings
//	cora_get_twobit_levs--Find out (via auto_volts) where clip levs are set
//	cora_get_verbosity -- Find out what the message level is
//	cora_lo_avg_power--Takes average of all adc power levels for an LO
//	cora_lo_tune	-- Tune LO; use table power level
//	cora_passcal	-- Autocorrelate via switchyard; get passband data
//	cora_pop_config	-- Pop up correlator configuration from push-down list
//	cora_push_config-- Save current correlator config. onto push-down list
//	cora_put_swyard_attenuators -- Store attenuator settings for future use
//	cora_set_verbosity -- Set diagnostic message level: 0=>quiet, 9=>verbose
//	cora_setup	-- Configures correlator, adjusts IF power levs, etc.
//	cora_swyard_atten_adj -- Adjusts switchyard attenuators
//	read_and_verify	-- Read out correlator data buffer, compare.

#ifndef CORAFUN_H
#define CORAFUN_H

struct CConfig			// Structure for maintaining correl. config.
{	int mode;		// Spectrometer mode, 1..4
	int ctype;		// Correlation type: AUTOCORR, CROSSCORR
	int mpx;		// Correlator multiplex code, 0 (1x)..2 (4x)
	float shrate[2];	// Correlator shift rates, MHz
};

int cora_auto_cor0(	// Get zero-lag autocorrelations
	int ants,	// in	Antennas on-line bitmap
	float ** cor0s);// out	Normalized correlations
int cora_fix_cor0(	// Correct the correlations for sampler problems
	int ants,	// in	Antennas on-line bitmap
	float bws[4],	// in	Bandwidths, ABCD samplers
	float ** c0s,	// in	Normalized correlations
	float ** cc0s);	// out	Corrected correlations
int cora_auto_volts(	// Determine RMS voltage at samplers from autocorrels.
	int ants,	// in	Antennas on-line bitmap
	float ** volts);// out	RMS voltages in units of clipping voltage
int cora_adc_power(	// Determine power levels from sampler ADC's
	int ants,	// in	Antennas on-line bitmap
	float ** powr );// out	Power levels read from sampler A/D Converters
int cora_get_twobit_levs(//Find out (via auto_volts) where clip levs are set
	int ants,	// in	Antennas on-line bitmap
	float ** a0lags,// in	Zero lag readout for autocorrelators
	float ** clevs);// out	Clipping level/RMS volts [ant][sam]
void cora_set_verbosity(// Establishes how verbose we want these
			// routines to be.
	int lev );	// in	Verbosity level 0=>silent 10=>talkative
int cora_get_verbosity();// Find out how garrulous somebody told us to be
complex * cora_passcal(	// Observes noise thru switchyard, makes p.b. spec.
	int ants,	// in	Antennas on-line (bit set for each)
	float shrate[2],// in	Shift rate, 12.5, 25, 50 MHz
	int mode,	// in	Correlator mode, 1..4
	int mpx,	// in	Degree of multiplexing, 0..2
	int nchan);	// in	Number of correlator channels/spectrum
			// Returns passband spectra: pass[nchan*bl+chan]
			// (pseudo 2-dimensional array for Fortran consumption)
void cora_pop_config( );// Restore correlator configuration from push-down list
void cora_push_config( );// Save current correl. config. on push-down list
int cora_setup(		// Configures correlator and sets up
	int ants,	// in	Antennas on-line bitmap
	int mode,	// in	Spectrometer mode, 1..4
	int ctype,	// in	Type of correlation, CROSSCOR, AUTOCOR
	int bits,	// in	Number of bits correlated, 1..2
	int mpx,	// in	Degree of time multiplexing, 0..2
	float shrate[2],// in	Shift rate, 12.5, 25, 50 MHz
	float corbw[4],	// in	Filter band widths, 6.25 .. 100.0 MHz
	float corf[4],	// in	Synthesizer frequencies, 70..990 MHz
	float corfdbm[4]);// in	Synthesizer amplitudes, -127..10 dBm
int cora_get_swyard_attenuators( // Retrieve saved attenuator settings
	char * fname );	// in	Name of file containing last save
int cora_put_swyard_attenuators( // Store attenuator settings for future use
	char * fname );	// in	Name of save file
//int cora_get_corf_power_correction( // Retrieve saved power level corrections
//	char * fname );	// in	Name of file containing last save
//int cora_put_corf_power_correction( // Store power level corrections
//	char * fname );	// in	Name of save file
int cora_swyard_atten_adj(// Adjusts switchyard attenuators
			// Returns 1=>happy 0=>out of range
	int ant,	// in	Logical Antenna number, 0..11
	int & atten );	// in&	If >0, apply this to attenuator, first guess
			// out	Actual final value set
int cora_lo_tune(	// Tune LO; use table power level
	int lo,		// in	LO number, 1..4
	float freq );	// in	Frequency, MHz
//int cora_lo_power_adj(	// Adjusts 3rd LO power level to balance sampler input
//			// Returns 1=>happy 0=>out of range
//	int ants,	// in	Antennas on-line (bit set for each)
//	int lo,		// in	3rd LO number, 1..4
//	float freq,	// in	Desired frequency, MHz
//	float & dlev );	// in&	If >-127, apply corr. to recommended table setting
float cora_lo_avg_power(// Takes average of all adc power levels for an LO
	int ants,	// in	Antennas on-line (bit set for each)
	int lo );	// in	3rd LO number, 1..4
float cora_ant_avg_power(// Takes average of all sampler power levels for an ant.
	int ant );	// in	Antenna number, 0..11
float cora_ant_max_power(// Finds max power of all samplers for an ant.
	int ant );	// in	Antenna number, 0..11
int cora_calibrate(	// Measures receiver power on sky & ambient,
			// calculates gains & system temps.
	int ants,	// in	Antennas on-line bitmap
	MHZ corf[4],	// in	Correlator frequencies (3rd LO)
	MHZ corbw[4],	// in	Correlator filter bandwidths, ABCD
	float ** GainL, float ** GainU,
			// out	Gains, both 1st LO sidebands [ant][sam]
	float ** TCalL, float ** TCalU
			// out	Calibrated system temperatures [ant][sam]
);
int cora_flip( );	// Do a bank flip of the correlator RAM buffers.
int read_and_verify(	// Read and verify contents of correlator buffer
	int bl, 	// in	Baseline slot: 0..66
	int whichbuf, 	// in	RE or IM to indicate which buffer
	long * data, 	// inout Address of data to receive raw counts
	int first, 	// in	First channel to read out
	int last );	// in	Last channel to read out


#endif
