//=======================================================================
//				CoraTest
//=======================================================================
//  Object that performs correlator bad channel checking
//=======================================================================

#ifndef CORATEST_H
#define CORATEST_H

/**CoraTest Object -- Correlator bad channel testing facility		*/
/*:correlator								*/
/*+									*/
class CoraTest
{
private:
	int * w0, * w1, * w2;	// Arrays to hold Walsh functions
	int mode;		// Spectrometer mode: 1..4
	int ctype;		// Correl. type: CROSSCORR or AUTOCORR
	int mpx;		// Degree of multiplexing: 0,1,2
	float shrate[2];	// Correlator shift rates
	float tint;		// Integrating time, seconds
	int max_ants;		// Maximum number of antennas avail.
	int ant_bitmap;		// Bit map of physical antenna ID's
	int n_bls;		// Number of baselines on-line
	int * basels;		// List of baselines on-line
	int n_chans;		// Number of channels of lag data avail.
	int n_segs;		// Number of correlator segments in lagdata
	int pat_size;		// Size of compressed test pattern (words)
	int pat_length;		// Size of test pattern (channels)
	int test_run;		// Whether a test was run since configuring
	int ramp_computed;	// Whether ramp compar. lag data computed
	int randm_computed;	//     "   random "      "    "    "
	int maxchan;		// Location of peak
	long * comp;		// Comparison lag data
public:
	long int * Data;	// Actual data recorded

	// Methods:
	CoraTest();		// Constructor
	~CoraTest();		// Destructor
	int PatSize();		// Retrieve size of test pattern (words)
	int PatLength();	// Retrieve size of test pattern (chans)
	int NChans();		// Retrieve number of channels of lag data
	int NSegs();		// Retrieve number of correlator segments
	int MaxChan();		// Retrieve channel where peak is located
	long int MaxVal();	// Retrieve peak value from lag data
	int Mpx();		// Retrieve multiplexing state (0..2)
	int Ctype();		// Retrieve correl type (CROSSCORR,AUTOCORR)
	int Mode();		// Retrieve mode (1..4)
	void ExpandPattern	// Expand correlator RAM test pattern
				// (May only load compressed form of it.)
	(	int length,	// in	Test pattern length, channels
		ushort * cpat,	// in	Compressed pattern
		short * epat);	// out	Expanded pattern
	void CompressPattern	// Compress correlator RAM test pattern
	(	int length,	// in	Test pattern length, channels
		short * epat,	// in	Expanded pattern
		ushort * cpat);	// out	Compressed pattern
	void CycSerProd		// Take cyclic serial product of patterns
	(	int length,	// in	Test pattern length, channels
		short * epat1,	// in	Expanded pattern, multiplicand
		short * epat2,	// in	Expanded pattern, multiplier
		long * prod);	// out	Product array: epat*epat
	void TestPattern	// Load test pattern
	(	int patsize,	// in	Test pattern length, words of RAM
		ushort * xpat,	// in	Compressed pattern for X inputs
		ushort * ypat);	// in	Compressed pattern for Y inputs
	void RunCorrelator	// Run correlator, switch banks
	(	float t );	// in	Integrating time, seconds
	void DumpData		// Dump data buffer for given baseline
	(	int bl );	// in	Correlator baseline, 0..65
	void SquareWave		// Run square wave test signal
	(	int length,	// in	Test pattern length, channels
		int n_ones );	// in	How long the 1's pattern is, chans.
	void Random		// Run random test signal
	(	int length );	// in	Test pattern length, channels
	int CompareTest		// Compare channels to expected values
	(	int crit,	// in	Report discrepancies > crit
		int length,	// in	Length of comparison lag data
		long * comp,	// in	Lag data to be compared
		long * errors);	// out	Errors array, entry for each chan.
	int RampSlopeTest	// Test for bad channels by seeking
				// discontinuities in slope in lag data
				// Returns bad channel count
	(	int crit,	// in	Report discrepancies > crit
		long * errors );// out	Errors array, entry for each chan.
	int FitTest		// Compare channels to expected values
	(	int crit,	// in	Report discrepancies > crit
		long * errors );// out	Errors array, as above.
	int SymTest		// Symmetry test
	(	int crit,	// in	Report discrepancies > crit
		long * errors );// out	Errors array: count - avg(right,left)


// Note: CoraTest will not disturb the correlator's configuration, but
// will cheerfully wipe out the data buffers.
};
/*-									*/

#endif // CORATEST_H

