//========================================================================
//				correl
//========================================================================
//  Correlator class
//========================================================================

#ifndef CORREL_H
#define CORREL_H

#include "../inc/complex.h"


#define MAXBIRDIES  20
#define MAXANTS	    12
#define MAXBLS	    66

/**Correlator class							*/
/*:correlator								*/
/*+									*/

// Constants
enum Process { Norm=1, FixBadChan=2, FixClip=4, HalfShift=16,
	FFT=256, FixBirdies=512, FixDelays=1024, AplPass=2048,
	AplGain=4096, Hanning=8192, Arrange=16384 };
enum LagState{ LagsNotAvail, LagsInHeader, LagsInProcessor };

// Structure for representing integer lists
struct IList
{	int nitems;
	int * item;
};

// Structure for representing lists of float
struct FList
{	int nitems;
	float * item;
};

// Structure for representing calibration data (gains)
struct Gains
{	int ants;	// Antennas on-line bits
	int numants;	// Number of antennas on-line
	float ** GainL;	// Lower sideband gains[ant][sampler]
	float ** GainU; // Upper    "       "    "      "
	float ** TCalL;	// Lower sideband System temperatures[ant][sampler]
	float ** TCalU; // Upper    "       "    " 
};

// Structure for representing bad correlator channel flags
struct Flags
{	int numbls;	/* Number of correlators on-line		*/
	int maxbad;	/* Max. Number of bad channels			*/
	int * blno;	/* Pointer to array of active BL identifiers	*/
			/* Array blno[bl], bl=0..numbls-1,		*/
			/* consists of baseline numbers, b=0..69	*/
	short * flags;	/* Pointer to array of bad channel flags	*/
			/* Array flags[bl][bad], bl=0..numbls-1,	*/
			/* bad=0..maxbad-1. -1=>no entry, 1=>bad chan #	*/
};

// Structure for representing spectra
struct Spectra
{
	int ants;	/* Antennas on-line, bit set for each.		*/
	int mode;	/* Spectrometer mode, 1..4			*/
	int ctype;	/* Type of correlation, 0=>Cross, 1=>Auto	*/
	int mpx;	/* 0=>none, 1=>2X, 2=>4X multiplexing		*/
	int han;	/* Whether to hanning-smooth the spectra	*/
			/* 0 => no,   1 => yes				*/
	int process;	/* Processing bits, 1 bit set for each:		*/
			/* 0=>norm, 1=>fixbadchan, 2=>fixclip,		*/
			/* 4=>halfshift,				*/
			/* 8=>FFT, 9=>fixbirdies, 10=>fixdelays,	*/
			/* 11=>aplpass, 12=>aplgain, 13=>hanning	*/
	MHZ corf[4];	/* Correlator LO frequencies, MHz		*/
	MHZ corbw[4];	/* Correlator band widths, MHz			*/
	MHZ shrate[2];	/* Correlator processor shift rates, MHz	*/
	int numants;	/* Number of antennas on-line			*/
	int numbls;	/* Number of active correlators			*/
	int numchans;	/* Number of complex channels/spectrum		*/
	SECONDS tint;	/* Integration time, seconds			*/
	int * blno;	/* Pointer to array of active BL identifiers	*/
	complex * data;	/* Pointer to array data[bl][chan],		*/
			/* chan=0..numchans-1, Complex data type.	*/

public:
	void copy( const Spectra & s )
	{	ants = s.ants;
		mode = s.mode;
		ctype = s.ctype;
		mpx = s.mpx;
		han = s.han;
		process = s.process;
		corf[0] = s.corf[0];
		corf[1] = s.corf[1];
		corf[2] = s.corf[2];
		corf[3] = s.corf[3];
		corbw[0] = s.corbw[0];
		corbw[1] = s.corbw[1];
		corbw[2] = s.corbw[2];
		corbw[3] = s.corbw[3];
		shrate[0] = s.shrate[0];
		shrate[1] = s.shrate[1];
		shrate[2] = s.shrate[2];
		shrate[3] = s.shrate[3];
		numants = s.numants;
		numbls = s.numbls;
		numchans = s.numchans;
		tint = s.tint;
		blno = s.blno;
		data = s.data;	/* Simply copy the array pointer */
	}
};

/* Fringes structure */

struct Fringes
{
	int numbls;	/* Number of active correlators			*/
	int * blno;	/* Pointer to array of active BL identifiers	*/
	complex * data;	/* Array data[bl][sb], bl = 0..numbls-1,	*/
			/* sb = 0..1 (sideband). Complex type.		*/
};

struct CoraHdr
{
// External controls and parameters
	int	ants;		// Antennas on line: bitmap
	int	maxants;	// Maximum number of antennas on-line
	int	mode;		// Correlator mode 1..4
	int	ctype;		// 0=>cross 1=>auto correlation type
	int	bits;		// Number of bits in correlation, 1..2
	int	mpx;		// 0=>none 1=>2X 2=>4X f. multiplexing
	MHZ	corf[4];	// 3rd LO frequencies
	MHZ	corbw[4];	// Correlator bandwitdhs: 6.25, 12.5, 25
	MHZ	shrate[2];	// Correlator shift rate: 12.5, 25, 50
	int *	blno;		// Pointer to array of active BL ID's
	int	han;		// Whether to hanning-smooth the spectra
				// 0 => no,  1 => yes
	SECONDS	tint;		// Integration time, seconds.
	int	process;	// Processing bits, 1 bit set for each.
	int	setup_done;	// Whether correl. was set up, 1=>yes, 0=>no
	int	passb_exists;	// Whether a passband exists, 1=>yes 0=>no
	int	coratest_done;	// Whether a correlator test was done, 1=>yes
	int	calibrate_done;	// Whether calibration was done, 1=>yes
	int	recno;		// Record number for spectra
	FList *	birdie;		// Known IF birdies
	float ** TCalL;		// Gains, lower sideband
	float ** TCalU;		// Gains, upper sideband
	float ** levs;		// Clipping levels, units of RMS voltage
	float ** a0lags;	// Autocorrelator zero lags for all ants,sams
	Flags	flgs;		// Bad channel list
	int **	walants;	// Walsh sequencies for antennas

// Internal parameters
	int	valid;		// Whether this record has been validated
	int	numants;	// Number of active antennas
	int	numbls;		// Number of active correlators
	int	numchans;	// Number of channels/spectrum
	int	maxbad;		// Maximum number of bad correl. chans.
	LagState lstate;	// Status of lag data
	Spectra * savesp;	// Where Cal saves lag data so it won't 
				// get destroyed by calibration procedure.
};

class Correlator
{
public:

Correlator		// Construct and configure
(	int ants,	// in	Antennas on-line bitmap
	int maxants,	// in	Maximum number of antennas on-line
	int mode,	// in	Spectrometer mode: 1..4
	int ctype,	// in	Correlation type: CROSSCORR or AUTOCORR
	int bits,	// in	Number of bits in correlations, 1..2
	int mpx,	// in	Multiplexing 0=>1X, 1=>2X, 2=>4X
	MHZ shrate[2],	// in	Correlator shift rates
	MHZ corbw[4],	// in	Correlator band widths
	MHZ corf[4],	// in	3rd LO frequencies
	int ** walants);// in	Walsh table for antennas

Correlator();		// Construct without configuring

~Correlator();		// Destroy object of correlator class

int ValidateParams( );	// Examine parameter validity
int ValidateTables( );	// Examine table & option validity
void QueryStatus();	// Dump correlator status info

int Calibrate		// Run calibration procedure for all samplers
(	Gains & g);	// out	Gains for each antenna, sampler, sideband

int Test();		// Test correlator channels

int Run			// Run correlator for one cycle
(	float tint );	// in	Integration time, seconds

int Idle( );		// Idle while correlator finishes integration
int GetLags		// Read correlator
(	Spectra & s );	// in out Computed spectra + header info

int MakeSpectra		// Compute spectra from lag data
(	int process,	// in	Bits to indicating what processing to do
	FList & birdie,	// in	Birdie table (within IF passband)
	Gains & g,	// in	Gains for each antenna, sampler, sideband
	Spectra & pb,	// in	Passband spectrum
	Spectra & s );	// in out Computed spectra + header info

int PassBand		// Perform passband cal data collection, make
			// passband spectrum
(	Spectra & pb );	// out	Passband spectrum 

/*-									*/
};

// Misc. procedures -- fellow travelers
int fetchbadchans(	// Fetch master bad channel list
	char * filen,	// in	File name where list is held (ASCII file)
	Flags & badlist);// out	List of known bad channels, read from file

// Global error message for correlator.
extern char cora_error_msg[128];

/**fake_lags -- Fake the lag data (debugging tool)			*/
/*:correlator								*/
/*+									*/
void fake_lags		// Fake the lags by reading from files		*/
(	int bl,		// in	Baseline, 0..69
	int numchans,	// in	Number of complex channels
	long int * re,	// out	Real part of complex crosscorrelation
	long int * im);	// out	Imaginary part
/*-									*/

inline int inrange	// Check floating point quantity for in-bounds
(	float quan,	// in	Quantity to be tested
	float lowlim,	// in	Lower bound, inclusive
	float hilim )	// in	Higher bound, inclusive
{
	return (quan >= lowlim) && (quan <= hilim);
}

inline int inrange	// Check integer quantity for in-bounds
(	int quan,	// in	Quantity to be tested
	int lowlim,	// in	Lower bound, inclusive
	int hilim )	// in	Higher bound, inclusive
{
	return (quan >= lowlim) && (quan <= hilim);
}

#endif // CORREL_H
