C$Procedure      BODEUL ( Return Euler angles for a body )
 
      SUBROUTINE BODEUL ( BODY, ET, RA, DEC, W, LAMBDA )
 
C$ Abstract
C
C      Return the Euler angles needed to compute the transformation
C      from inertial to body-fixed coordinates for any body in the
C      kernel pool.
C
C$ Required_Reading
C
C      KERNEL
C      NAIF_IDS
C      TIME
C
C$ Keywords
C
C      CONSTANTS
C      ROTATION
C      TRANSFORMATION
C
C$ Declarations
 
      INTEGER               BODY
      DOUBLE PRECISION      ET
      DOUBLE PRECISION      RA
      DOUBLE PRECISION      DEC
      DOUBLE PRECISION      W
      DOUBLE PRECISION      LAMBDA
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      BODY       I   ID code of body.
C      ET         I   Epoch of transformation.
C      RA         O   Right ascension of the (IAU) north pole.
C      DEC        O   Declination of the (IAU) north pole of the body.
C      W          O   Angle between the x-axis and the prime meridian.
C      LAMBDA     O   Angle between the prime meridian and longitude of
C                     longest axis.
C
C$ Detailed_Input
C
C      BODY        is the integer ID code of the body for which the
C                  transformation is requested. Bodies are numbered
C                  according to the standard NAIF numbering scheme.
C
C      ET          is the epoch at which the transformation is
C                  requested. (This is typically the epoch of
C                  observation minus the one-way light time from
C                  the observer to the body at the epoch of
C                  observation.)
C
C$ Detailed_Output
C
C      RA,
C      DEC         are the right ascension and declination of the
C                  (IAU) north pole of the body at the epoch of
C                  transformation. RA and DEC are given in radians.
C
C      W           is the angle between the x-axis (inertial) and the
C                  prime meridian of the body. W is given in radians.
C
C      LAMBDA      is the angle between the prime meridian and the
C                  longest axis of the tri-axial ellipsoid which
C                  models the body. LAMBDA is given in radians.
C
C$ Parameters
C
C      None.
C
C$ Exceptions
C
C      1) If the number of phase terms is insufficient, the error
C         SPICE(KERNELVARNOTFOUND) is signalled.
C
C$ Files
C
C      None.
C
C$ Particulars
C
C      The Euler angles RA, DEC, and W are defined as follows:
C
C            RA  = RA0  + RA1*T  + RA2*T*T   + a  sin theta
C                                               i          i
C
C            DEC = DEC0 + DEC1*T + DEC2*T*T  + d  cos theta
C                                               i          i
C
C            W   = W0   + W1*d   + W2*d*d    + w  sin theta
C                                               i          i
C
C      where:
C
C            d = days past J2000.
C
C            T = Julian centuries past J2000.
C
C            a , d , and w  arrays apply to satellites only.
C             i   i       i
C
C            theta  = THETA0 * THETA1*T are specific to each planet.
C                 i
C
C        These angles -- typically nodal rates -- vary in number and
C        definition from one planetary system to the next.
C
C        The offset LAMBDA is a constant for a given body. LAMBDA is
C        needed to distinguish between the latitude and longitude
C        system and the geometric system (where Prime Meridian always
C        intersects the longest axis).
C
C$ Examples
C
C      In the following code fragment, BODEUL is used to get the unit
C      vector (POLE) parallel to the north pole of a target body (BODY)
C      at a specific epoch (ET).
C
C         CALL BODEUL ( BODY, ET, RA, DEC, W, LAMBDA )
C         CALL RADREC ( 1.D0, RA,  DEC, POLE )
C
C      Note that the items necessary to compute the Euler angles
C      must have been loaded into the kernel pool (by one or more
C      previous calls to LDPOOL).
C
C$ Restrictions
C
C      None.
C
C$ Literature_References
C
C      1)  Refer to the NAIF_IDS required reading file for a complete
C          list of the NAIF integer ID codes for bodies.
C
C$ Author_and_Institution
C
C      N.J. Bachman    (JPL)
C      H.A. Neilan     (JPL)
C      W.L. Taber      (JPL)
C      I.M. Underwood  (JPL)
C
C$ Version
C
C-     SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 2.0.0, 04-SEP-1991 (NJB)
C
C         Updated to handle P_constants referenced to different epochs
C         and inertial reference frames.
C
C-     SPICELIB Version 1.1.0, 02-NOV-1990  (NJB)
C
C         Allowed number of nutation precession angles increased to
C         100.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990  (WLT) (IMU)
C
C-&
 
C$ Index_Entries
C
C     euler angles for orientation of a body
C     fetch euler angles for a body
C
C-&
 
 
 
C$ Revisions
C
C-     SPICELIB Version 2.0.0, 04-SEP-1991 (NJB)
C
C         Updated to handle P_constants referenced to different epochs
C         and inertial reference frames.
C
C         Updated to handle P_constants referenced to different epochs
C         and inertial reference frames.
C
C         BODEUL now checks the kernel pool for presence of the
C         variables
C
C            BODY#_CONSTANTS_REF_FRAME
C
C         and
C
C            BODY#_CONSTANTS_JED_EPOCH
C
C         where # is the NAIF integer code of the barycenter of a
C         planetary system or of a body other than a planet or
C         satellite.  If either or both of these variables are
C         present, the P_constants for BODY are presumed to be
C         referenced to the specified inertial frame or epoch.
C         If the epoch of the constants is not J2000, the input
C         time ET is converted to seconds past the reference epoch.
C         If the frame of the constants is not J2000, the Euler angles
C         defining the rotation from the P_constants' frame to
C         body-fixed coordinates are transformed so that they define
C         the rotation from J2000 coordinates to body-fixed
C         coordinates.
C
C
C-     SPICELIB Version 1.1.0, 02-NOV-1990  (NJB)
C
C         Allowed number of nutation precession angles increased to
C         100.
C
C-    Beta Version 2.0.0, 23-JUN-1989 (HAN)
C
C        Mod angles by two pi. Check to see that right ascension and
C        prime meridian angles are within the range 0 to two pi.
C
C        LAMBDA used to be returned in degrees. It has been corrected
C        to return LAMBDA in radians.
C
C-    Beta Version 1.1.0, 16-FEB-1989 (IMU) (NJB)
C
C        Examples section completed.  Declarations of unused variables
C        HALFPI and N removed.
C
C-&
 
 
 
 
C
C     SPICELIB functions
C
      LOGICAL               BODFND
      LOGICAL               RETURN
 
      DOUBLE PRECISION      HALFPI
      DOUBLE PRECISION      J2000
      DOUBLE PRECISION      RPD
      DOUBLE PRECISION      SPD
      DOUBLE PRECISION      TWOPI
      DOUBLE PRECISION      VDOTG
 
C
C     Local parameters
C
      INTEGER               MAXANG
      PARAMETER           ( MAXANG = 100 )
 
 
C
C     Local variables
C
      CHARACTER*(32)        ITEM
 
      DOUBLE PRECISION      CONEPC
      DOUBLE PRECISION      CONREF
      DOUBLE PRECISION      EPOCH
 
      DOUBLE PRECISION      RCOEF  ( 0:2 )
      DOUBLE PRECISION      DCOEF  ( 0:2 )
      DOUBLE PRECISION      WCOEF  ( 0:2 )
 
      LOGICAL               ISSAT
      INTEGER               BARY
 
      INTEGER               NA
      INTEGER               ND
      INTEGER               NW
      INTEGER               NL
      INTEGER               J2CODE
      INTEGER               REF
      INTEGER               REFID
 
      DOUBLE PRECISION      J2BFX  ( 3, 3 )
      DOUBLE PRECISION      J2REF  ( 3, 3 )
      DOUBLE PRECISION      RF2BFX ( 3, 3 )
 
      DOUBLE PRECISION      AC     ( MAXANG )
      DOUBLE PRECISION      DC     ( MAXANG )
      DOUBLE PRECISION      WC     ( MAXANG )
 
      INTEGER               NTHETA
      DOUBLE PRECISION      TCOEF  ( 2, MAXANG )
      DOUBLE PRECISION      THETA
      DOUBLE PRECISION      SINTH  (    MAXANG )
      DOUBLE PRECISION      COSTH  (    MAXANG )
 
      DOUBLE PRECISION      T
      DOUBLE PRECISION      D
 
      INTEGER               I
      INTEGER               DIM
 
      LOGICAL               FOUND
      LOGICAL               FIRST
 
C
C     Saved variables
C
      SAVE                  FIRST
      SAVE                  J2CODE
 
C
C     Initial values
C
      DATA  FIRST / .TRUE. /
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'BODEUL' )
      END IF
 
C
C     Get the code for the J2000 frame, if we don't have it yet.
C
      IF ( FIRST ) THEN
 
         CALL IRFNUM ( 'J2000', J2CODE )
         FIRST = .FALSE.
 
      END IF
 
C
C     Find the body code used to label the reference frame and epoch
C     specifiers for the orientation constants for BODY.
C
C     For planetary systems, the reference frame and epoch for the
C     orientation constants is associated with the system barycenter,
C     not with individual bodies in the system.  For any other bodies,
C     (the Sun or asteroids, for example) the body's own code is used
C     as the label.
C
      IF (  ( BODY .GT. 100 ) .AND. ( BODY .LT. 1000 )  ) THEN
         REFID = BODY / 100
      ELSE
         REFID = BODY
      END IF
 
C
C     Look up the epoch of the constants.  The epoch is specified
C     as a Julian ephemeris date.  The epoch defaults to J2000.
C
      ITEM  =  'BODY#_CONSTANTS_JED_EPOCH'
 
      CALL REPMI  ( ITEM, '#',   REFID,   ITEM   )
      CALL RTPOOL ( ITEM,  DIM,  CONEPC,  FOUND  )
 
 
      IF ( FOUND ) THEN
C
C        The reference epoch is returned as a JED.  Convert to
C        ephemeris seconds past J2000.  Then convert the input
C        ET to seconds past the reference epoch.
C
         CONEPC  =  SPD() * ( CONEPC - J2000() )
         EPOCH   =  ET    -   CONEPC
 
      ELSE
         EPOCH   =  ET
      END IF
 
C
C     Look up the reference frame of the constants.  The reference
C     frame is specified by a code recognized by CHGIRF.  The default
C     frame is J2000, symbolized by the code J2CODE.
C
      CALL IRFNUM ( 'J2000', J2CODE )
 
      ITEM  =  'BODY#_CONSTANTS_REF_FRAME'
 
      CALL REPMI  ( ITEM,  '#',  REFID,   ITEM   )
      CALL RTPOOL ( ITEM,  DIM,  CONREF,  FOUND  )
 
      IF ( FOUND ) THEN
         REF  =  NINT ( CONREF )
      ELSE
         REF  =  J2CODE
      END IF
 
C
C     Whatever the body, it has quadratic time polynomials for
C     the RA and Dec of the pole, and for the rotation of the
C     Prime Meridian.
C
      ITEM = 'POLE_RA'
      CALL CLEARD (              3, RCOEF )
      CALL BODVAR ( BODY, ITEM, NA, RCOEF )
 
      ITEM = 'POLE_DEC'
      CALL CLEARD (              3, DCOEF )
      CALL BODVAR ( BODY, ITEM, ND, DCOEF )
 
      ITEM = 'PM'
      CALL CLEARD (              3, WCOEF )
      CALL BODVAR ( BODY, ITEM, NW, WCOEF )
 
C
C     The offset of the prime meridian is optional.
C
      ITEM = 'LONG_AXIS'
      IF ( BODFND ( BODY, ITEM ) ) THEN
         CALL BODVAR ( BODY, ITEM, NL, LAMBDA )
      ELSE
         LAMBDA = 0
      END IF
 
 
C
C     If the body is a satellite, there may be additional nutation
C     and libration (THETA) terms.
C
      NTHETA = 0
      NA     = 0
      ND     = 0
      NW     = 0
 
      ISSAT  = (        BODY              .GT. 100
     .           .AND.  BODY              .LT. 1000
     .           .AND.  MOD ( BODY, 100 ) .NE. 99   )
 
      IF ( ISSAT ) THEN
 
         BARY = BODY / 100
 
         ITEM = 'NUT_PREC_ANGLES'
         IF ( BODFND ( BARY, ITEM ) ) THEN
            CALL BODVAR ( BARY, ITEM, NTHETA, TCOEF )
            NTHETA = NTHETA / 2
         END IF
 
         ITEM = 'NUT_PREC_RA'
         IF ( BODFND ( BODY, ITEM ) )
     .      CALL BODVAR ( BODY, ITEM, NA, AC )
 
         ITEM = 'NUT_PREC_DEC'
         IF ( BODFND ( BODY, ITEM ) )
     .      CALL BODVAR ( BODY, ITEM, ND, DC )
 
         ITEM = 'NUT_PREC_PM'
         IF ( BODFND ( BODY, ITEM ) )
     .      CALL BODVAR ( BODY, ITEM, NW, WC )
 
      END IF
 
      IF ( MAX ( NA, ND, NW ) .GT. NTHETA ) THEN
         CALL SETMSG ( 'BODEUL: Insufficient number of nutation/'  //
     .                 'precession angles for body * at time #.'     )
         CALL ERRINT ( '*', BODY )
         CALL ERRDP  ( '#', ET   )
         CALL SIGERR ( 'SPICE(KERNELVARNOTFOUND)' )
         CALL CHKOUT ( 'BODEUL' )
         RETURN
      END IF
 
C
C     Evaluate the time polynomials at EPOCH.
C
      D   = EPOCH / SPD()
      T   = D     / 36525.D0
 
      RA  = RCOEF(0)  +  T * ( RCOEF(1) + T * RCOEF(2) )
      DEC = DCOEF(0)  +  T * ( DCOEF(1) + T * DCOEF(2) )
      W   = WCOEF(0)  +  D * ( WCOEF(1) + D * WCOEF(2) )
 
C
C     Add nutation and libration as appropriate.
C
      DO 50001
     .   I = 1, NTHETA
 
         THETA    = ( TCOEF(1,I)  +  T * TCOEF(2,I) ) * RPD()
 
         SINTH(I) = SIN ( THETA )
         COSTH(I) = COS ( THETA )
 
50001 CONTINUE
 
      RA  = RA  + VDOTG ( AC, SINTH, NA )
      DEC = DEC + VDOTG ( DC, COSTH, ND )
      W   = W   + VDOTG ( WC, SINTH, NW )
 
C
C     Convert from degrees to radians and mod by two pi.
C
      RA     = RA     * RPD()
      DEC    = DEC    * RPD()
      W      = W      * RPD()
      LAMBDA = LAMBDA * RPD()
 
      RA     = MOD ( RA,     TWOPI() )
      DEC    = MOD ( DEC,    TWOPI() )
      W      = MOD ( W,      TWOPI() )
      LAMBDA = MOD ( LAMBDA, TWOPI() )
 
C
C     Convert the angles to the J2000 frame if they are not already
C     referenced to J2000.
C
      IF ( REF .NE. J2CODE ) THEN
 
C        Find the transformation from the J2000 frame to the frame
C        designated by REF.  Form the transformation from `REF'
C        coordinates to body-fixed coordinates, using our Euler angles.
C        Compose the transformations to obtain the J2000-to-body-fixed
C        transformation.  Decompose this transformation into Euler
C        angles.
C
         CALL IRFROT ( J2CODE, REF, J2REF )
 
         CALL EUL2M  ( W,
     .                 HALFPI()  -  DEC,
     .                 HALFPI()  +  RA,
     .                 3, 1, 3,
     .                 RF2BFX               )
 
         CALL MXM    ( RF2BFX, J2REF, J2BFX )
 
         CALL M2EUL  ( J2BFX,  3,  1,  3,  W,  DEC,  RA )
 
         RA   =  RA        -  HALFPI()
         DEC  =  HALFPI()  -  DEC
 
      END IF
 
C
C     The Euler angles now give the transformation from J2000 to
C     body-fixed coordinates at epoch ET seconds past J2000,
C     regardless of the epoch and frame of the orientation constants
C     for the specified body.
C
 
C
C     Make sure that the prime meridian and right ascension are in
C     the correct range.
C
      IF ( W .LT. 0 ) THEN
         W = W + TWOPI()
      END IF
 
      IF ( RA .LT. 0 ) THEN
         RA = RA + TWOPI()
      END IF
 
 
      CALL CHKOUT ( 'BODEUL' )
      RETURN
      END
