C$Procedure      BODMAT ( Return transformation matrix for a body )
 
      SUBROUTINE BODMAT ( BODY, ET, TIPM )
 
C$ Abstract
C
C      Return the J2000 to body Equator and Prime Meridian coordinate
C      transformation matrix for a specified body.
C
C$ Required_Reading
C
C      KERNEL
C      NAIF_IDS
C      TIME
C
C$ Keywords
C
C      CONSTANTS
C
C$ Declarations
 
      INTEGER          BODY
      DOUBLE PRECISION ET
      DOUBLE PRECISION TIPM   ( 3,3 )
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      BODY       I   ID code of body.
C      ET         I   Epoch of transformation.
C      TIPM       O   Transformation from Inertial to PM for BODY at ET.
C
C$ Detailed_Input
C
C      BODY        is the integer ID code of the body for which the
C                  transformation is requested. Bodies are numbered
C                  according to the standard NAIF numbering scheme.
C
C      ET          is the epoch at which the transformation is
C                  requested. (This is typically the epoch of
C                  observation minus the one-way light time from
C                  the observer to the body at the epoch of
C                  observation.)
C
C$ Detailed_Output
C
C      TIPM        is the transformation matrix from Inertial to body
C                  Equator and Prime Meridian.  The X axis of the PM
C                  system is directed to the intersection of the
C                  equator and prime meridian. The Z axis points north.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C      None.
C
C$ Exceptions
C
C      1)  If the number of phase angle coefficients found in the
C          kernel pool is insufficient, the error
C          SPICE(KERNELVARNOTFOUND) is signalled.
C
C      2)  If any other required kernel pool constants are not
C          found, the error will be diagnosed by routines called by
C          this routine.
C
C$ Particulars
C
C      This routine is related to the more general routine TIPBOD
C      which returns a matrix that transforms vectors from a
C      specified inertial reference frame to body equator and
C      prime meridian coordinates.  TIPBOD accepts an input argument
C      REF that allows the caller to specify an inertial reference
C      frame.
C
C      The transformation represented by BODMAT's output argument TIPM
C      is defined as follows:
C
C            TIPM = [W] [HALFPI-DEC] [RA+HALFPI]
C                     3            1           3
C
C      The Euler angles RA, DEC, and W are defined as follows:
C
C            RA  = RA0  + RA1*T  + RA2*T*T   + a  sin theta
C                                               i          i
C
C            DEC = DEC0 + DEC1*T + DEC2*T*T  + d  cos theta
C                                               i          i
C
C            W   = W0   + W1*d   + W2*d*d    + w  sin theta
C                                               i          i
C
C      where:
C
C            d = days past J2000.
C
C            T = Julian centuries past J2000.
C
C            a , d , and w  arrays apply to satellites only.
C             i   i       i
C
C            theta  = THETA0 * THETA1*T are specific to each planet.
C                 i
C
C        These angles -- typically nodal rates -- vary in number and
C        definition from one planetary system to the next.
C
C$ Examples
C
C      In the following code fragment, BODMAT is used to rotate
C      the position vector (POS) from a target body (BODY) to a
C      spacecraft from inertial coordinates to body-fixed coordinates
C      at a specific epoch (ET), in order to compute the planetocentric
C      longitude (PCLONG) of the spacecraft.
C
C         CALL BODMAT ( BODY, ET, TIPM )
C         CALL MXV    ( TIPM, POS, POS )
C         CALL RECLAT ( POS, RADIUS, PCLONG, LAT )
C
C      To compute the equivalent planetographic longitude (PGLONG),
C      it is necessary to know the direction of rotation of the target
C      body, as shown below.
C
C         CALL BODVAR ( BODY, 'PM', DIM, VALUES )
C
C         IF ( VALUES(2) .GT. 0.D0 ) THEN
C            PGLONG = PCLONG
C         ELSE
C            PGLONG = TWOPI() - PCLONG
C         END IF
C
C      Note that the items necessary to compute the transformation
C      TIPM must have been loaded into the kernel pool (by one or more
C      previous calls to LDPOOL).
C
C$ Restrictions
C
C      None.
C
C$ Literature_References
C
C      1)  Refer to the NAIF_IDS required reading file for a complete
C          list of the NAIF integer ID codes for bodies.
C
C$ Author_and_Institution
C
C      N.J. Bachman    (JPL)
C      W.L. Taber      (JPL)
C      I.M. Underwood  (JPL)
C
C$ Version
C
C-     SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 2.0.0, 04-SEP-1991 (NJB)
C
C         Updated to handle P_constants referenced to different epochs
C         and inertial reference frames.
C
C         The header was updated to specify that the inertial reference
C         frame used by BODMAT is restricted to be J2000.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (WLT) (IMU)
C
C-&
 
C$ Index_Entries
C
C     fetch transformation matrix for a body
C     transformation from j2000 position to bodyfixed
C     transformation from j2000 to bodyfixed coordinates
C
C-&
 
 
 
 
C$ Revisions
C
C-     SPICELIB Version 2.0.0, 04-SEP-1991 (NJB)
C
C         Updated to handle P_constants referenced to different epochs
C         and inertial reference frames.
C
C         The header was updated to specify that the inertial reference
C         frame used by BODMAT is restricted to be J2000.
C
C         BODMAT now checks the kernel pool for presence of the
C         variables
C
C            BODY#_CONSTANTS_REF_FRAME
C
C         and
C
C            BODY#_CONSTANTS_JED_EPOCH
C
C         where # is the NAIF integer code of the barycenter of a
C         planetary system or of a body other than a planet or
C         satellite.  If either or both of these variables are
C         present, the P_constants for BODY are presumed to be
C         referenced to the specified inertial frame or epoch.
C         If the epoch of the constants is not J2000, the input
C         time ET is converted to seconds past the reference epoch.
C         If the frame of the constants is not J2000, the rotation from
C         the P_constants' frame to body-fixed coordinates is
C         transformed to the rotation from J2000 coordinates to
C         body-fixed coordinates.
C
C         For efficiency reasons, this routine now duplicates much
C         of the code of BODEUL so that it doesn't have to call BODEUL.
C         In some cases, BODEUL must covert Euler angles to a matrix,
C         rotate the matrix, and convert the result back to Euler
C         angles.  If this routine called BODEUL, then in such cases
C         this routine would convert the transformed angles back to
C         a matrix.  That would be a bit much....
C
C
C-    Beta Version 1.1.0, 16-FEB-1989 (IMU) (NJB)
C
C        Examples section completed.  Declaration of unused variable
C        FOUND removed.
C
C-&
 
 
 
C
C     SPICELIB functions
C
      LOGICAL               BODFND
      LOGICAL               RETURN
 
      DOUBLE PRECISION      HALFPI
      DOUBLE PRECISION      J2000
      DOUBLE PRECISION      RPD
      DOUBLE PRECISION      SPD
      DOUBLE PRECISION      TWOPI
      DOUBLE PRECISION      VDOTG
 
C
C     Local parameters
C
      INTEGER               MAXANG
      PARAMETER           ( MAXANG = 100 )
 
 
C
C     Local variables
C
      CHARACTER*(32)        ITEM
 
      DOUBLE PRECISION      CONEPC
      DOUBLE PRECISION      CONREF
      DOUBLE PRECISION      EPOCH
 
      DOUBLE PRECISION      RCOEF  ( 0:2 )
      DOUBLE PRECISION      DCOEF  ( 0:2 )
      DOUBLE PRECISION      WCOEF  ( 0:2 )
 
      LOGICAL               ISSAT
      INTEGER               BARY
 
      INTEGER               NA
      INTEGER               ND
      INTEGER               NW
      INTEGER               NL
      INTEGER               J2CODE
      INTEGER               REF
      INTEGER               REFID
 
      DOUBLE PRECISION      J2REF  ( 3, 3 )
 
      DOUBLE PRECISION      AC     ( MAXANG )
      DOUBLE PRECISION      DC     ( MAXANG )
      DOUBLE PRECISION      WC     ( MAXANG )
 
      INTEGER               NTHETA
      DOUBLE PRECISION      TCOEF  ( 2, MAXANG )
      DOUBLE PRECISION      THETA
      DOUBLE PRECISION      SINTH  (    MAXANG )
      DOUBLE PRECISION      COSTH  (    MAXANG )
 
      DOUBLE PRECISION      T
      DOUBLE PRECISION      D
 
      DOUBLE PRECISION      RA
      DOUBLE PRECISION      DEC
      DOUBLE PRECISION      W
      DOUBLE PRECISION      LAMBDA
 
      INTEGER               I
      INTEGER               DIM
 
      LOGICAL               FOUND
      LOGICAL               FIRST
 
C
C     Saved variables
C
      SAVE                  FIRST
      SAVE                  J2CODE
 
C
C     Initial values
C
      DATA  FIRST / .TRUE. /
 
 
C
C     Standard SPICE Error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'BODMAT' )
      END IF
 
C
C     Get the code for the J2000 frame, if we don't have it yet.
C
      IF ( FIRST ) THEN
 
         CALL IRFNUM ( 'J2000', J2CODE )
         FIRST = .FALSE.
 
      END IF
 
C
C     Find the body code used to label the reference frame and epoch
C     specifiers for the orientation constants for BODY.
C
C     For planetary systems, the reference frame and epoch for the
C     orientation constants is associated with the system barycenter,
C     not with individual bodies in the system.  For any other bodies,
C     (the Sun or asteroids, for example) the body's own code is used
C     as the label.
C
      IF (  ( BODY .GT. 100 ) .AND. ( BODY .LT. 1000 )  ) THEN
         REFID = BODY / 100
      ELSE
         REFID = BODY
      END IF
 
C
C     Look up the epoch of the constants.  The epoch is specified
C     as a Julian ephemeris date.  The epoch defaults to J2000.
C
      ITEM  =  'BODY#_CONSTANTS_JED_EPOCH'
 
      CALL REPMI  ( ITEM, '#',   REFID,   ITEM   )
      CALL RTPOOL ( ITEM,  DIM,  CONEPC,  FOUND  )
 
 
      IF ( FOUND ) THEN
C
C        The reference epoch is returned as a JED.  Convert to
C        ephemeris seconds past J2000.  Then convert the input
C        ET to seconds past the reference epoch.
C
         CONEPC  =  SPD() * ( CONEPC - J2000() )
         EPOCH   =  ET    -   CONEPC
 
      ELSE
         EPOCH   =  ET
      END IF
 
C
C     Look up the reference frame of the constants.  The reference
C     frame is specified by a code recognized by CHGIRF.  The default
C     frame is J2000, symbolized by the code J2CODE.
C
      ITEM  =  'BODY#_CONSTANTS_REF_FRAME'
 
      CALL REPMI  ( ITEM,  '#',  REFID,   ITEM   )
      CALL RTPOOL ( ITEM,  DIM,  CONREF,  FOUND  )
 
      IF ( FOUND ) THEN
         REF  =  NINT ( CONREF )
      ELSE
         REF  =  J2CODE
      END IF
 
C
C     Whatever the body, it has quadratic time polynomials for
C     the RA and Dec of the pole, and for the rotation of the
C     Prime Meridian.
C
      ITEM = 'POLE_RA'
      CALL CLEARD (              3, RCOEF )
      CALL BODVAR ( BODY, ITEM, NA, RCOEF )
 
      ITEM = 'POLE_DEC'
      CALL CLEARD (              3, DCOEF )
      CALL BODVAR ( BODY, ITEM, ND, DCOEF )
 
      ITEM = 'PM'
      CALL CLEARD (              3, WCOEF )
      CALL BODVAR ( BODY, ITEM, NW, WCOEF )
 
C
C     The offset of the prime meridian is optional.
C
      ITEM = 'LONG_AXIS'
      IF ( BODFND ( BODY, ITEM ) ) THEN
         CALL BODVAR ( BODY, ITEM, NL, LAMBDA )
      ELSE
         LAMBDA = 0
      END IF
 
 
C
C     If the body is a satellite, there may be additional nutation
C     and libration (THETA) terms.
C
      NTHETA = 0
      NA     = 0
      ND     = 0
      NW     = 0
 
      ISSAT  = (        BODY              .GT. 100
     .           .AND.  BODY              .LT. 1000
     .           .AND.  MOD ( BODY, 100 ) .NE. 99   )
 
      IF ( ISSAT ) THEN
 
         BARY = BODY / 100
 
         ITEM = 'NUT_PREC_ANGLES'
         IF ( BODFND ( BARY, ITEM ) ) THEN
            CALL BODVAR ( BARY, ITEM, NTHETA, TCOEF )
            NTHETA = NTHETA / 2
         END IF
 
         ITEM = 'NUT_PREC_RA'
         IF ( BODFND ( BODY, ITEM ) )
     .      CALL BODVAR ( BODY, ITEM, NA, AC )
 
         ITEM = 'NUT_PREC_DEC'
         IF ( BODFND ( BODY, ITEM ) )
     .      CALL BODVAR ( BODY, ITEM, ND, DC )
 
         ITEM = 'NUT_PREC_PM'
         IF ( BODFND ( BODY, ITEM ) )
     .      CALL BODVAR ( BODY, ITEM, NW, WC )
 
      END IF
 
      IF ( MAX ( NA, ND, NW ) .GT. NTHETA ) THEN
         CALL SETMSG ( 'Insufficient number of nutation/'             //
     .                 'precession angles for body * at time #.'     )
         CALL ERRINT ( '*', BODY )
         CALL ERRDP  ( '#', ET   )
         CALL SIGERR ( 'SPICE(KERNELVARNOTFOUND)' )
         CALL CHKOUT ( 'BODMAT' )
         RETURN
      END IF
 
C
C     Evaluate the time polynomials at EPOCH.
C
      D   = EPOCH / SPD()
      T   = D     / 36525.D0
 
      RA  = RCOEF(0)  +  T * ( RCOEF(1) + T * RCOEF(2) )
      DEC = DCOEF(0)  +  T * ( DCOEF(1) + T * DCOEF(2) )
      W   = WCOEF(0)  +  D * ( WCOEF(1) + D * WCOEF(2) )
 
C
C     Add nutation and libration as appropriate.
C
      DO 50001
     .   I = 1, NTHETA
 
         THETA    = ( TCOEF(1,I)  +  T * TCOEF(2,I) ) * RPD()
 
         SINTH(I) = SIN ( THETA )
         COSTH(I) = COS ( THETA )
 
50001 CONTINUE
 
      RA  = RA  + VDOTG ( AC, SINTH, NA )
      DEC = DEC + VDOTG ( DC, COSTH, ND )
      W   = W   + VDOTG ( WC, SINTH, NW )
 
C
C     Convert from degrees to radians and mod by two pi.
C
      RA     = RA     * RPD()
      DEC    = DEC    * RPD()
      W      = W      * RPD()
      LAMBDA = LAMBDA * RPD()
 
      RA     = MOD ( RA,     TWOPI() )
      DEC    = MOD ( DEC,    TWOPI() )
      W      = MOD ( W,      TWOPI() )
      LAMBDA = MOD ( LAMBDA, TWOPI() )
 
C
C     Produce the rotation matrix defined by the Euler angles.
C
      CALL EUL2M  ( W,
     .              HALFPI()  -  DEC,
     .              HALFPI()  +  RA,
     .              3, 1, 3,
     .              TIPM              )
 
C
C     Convert TIPM to the J2000-to-bodyfixed rotation, if is is not
C     already referenced to J2000.
C
      IF ( REF .NE. J2CODE ) THEN
 
C        Find the transformation from the J2000 frame to the frame
C        designated by REF.  Form the transformation from `REF'
C        coordinates to body-fixed coordinates.  Compose the
C        transformations to obtain the J2000-to-body-fixed
C        transformation.
C
         CALL IRFROT ( J2CODE,  REF,    J2REF )
         CALL MXM    ( TIPM,    J2REF,  TIPM  )
 
      END IF
 
C
C     TIPM now gives the transformation from J2000 to
C     body-fixed coordinates at epoch ET seconds past J2000,
C     regardless of the epoch and frame of the orientation constants
C     for the specified body.
C
 
      CALL CHKOUT ( 'BODMAT' )
      RETURN
      END
