 
C$Procedure      CKBSR ( C-kernel, buffer segments for readers )
 
      SUBROUTINE CKBSR ( FNAME,
     .                   HANDLE,
     .                   INST,
     .                   SCLKDP,
     .                   TOL,
     .                   NEEDAV,
     .                   DESCR,
     .                   SEGID,
     .                   FOUND   )
 
C$ Abstract
C
C     Buffer segments for readers.
C
C$ Required_Reading
C
C     CK
C     DAF
C
C$ Keywords
C
C     POINTING
C
C$ Declarations
 
      CHARACTER*(*)         FNAME
      INTEGER               HANDLE
      INTEGER               INST
      DOUBLE PRECISION      SCLKDP
      DOUBLE PRECISION      TOL
      LOGICAL               NEEDAV
      DOUBLE PRECISION      DESCR    ( * )
      CHARACTER*(*)         SEGID
      LOGICAL               FOUND
 
      INTEGER               FTSIZE
      PARAMETER           ( FTSIZE =   20 )
 
      INTEGER               ITSIZE
      PARAMETER           ( ITSIZE =   20 )
 
      INTEGER               STSIZE
      PARAMETER           ( STSIZE =   1000 )
 
C$ Brief_I/O
C
C     Variable  I/O  Entry points
C     --------  ---  --------------------------------------------------
C     FNAME      I   CKLPF
C     HANDLE    I,O  CKLPF, CKUPF, CKSNS
C     INST       I   CKBSS
C     SCLKDP     I   CKBSS
C     TOL        I   CKBSS
C     NEEDAV     I   CKBSS
C     DESCR      O   CKSNS
C     SEGID      O   CKSNS
C     FOUND      O   CKSNS
C
C$ Detailed_Input
C
C     FNAME      is the name of a binary C-kernel file to be loaded.
C
C     HANDLE     on input is the handle of a binary C-kernel file to be
C                unloaded.
C
C
C     The purpose of entry points CKBSS and CKSNS is to search for
C     segments in CK files matching certain criteria. The four
C     quantities below establish these search criteria.
C
C
C     INST       is the NAIF ID of an instrument.
C
C     SCLKDP     is an encoded spacecraft clock time.
C
C     TOL        is a time tolerance, measured in the same units as
C                encoded spacecraft clock.
C
C     NEEDAV     indicates whether or not angular velocity data is
C                required.
C
C                If true, only segments containing pointing and angular
C                velocity data will be checked. If false, segments
C                containing just pointing data will also be considered.
C
C
C     A segment matches the CKBSS/CKSNS search criteria when the
C     following statements are true.
C
C        1) INST matches the instrument number for the segment.
C
C        2) The time interval [SCLKDP - TOL, SCLKDP + TOL] intersects
C           the time interval of the segment.
C
C        3) If angular velocity data is required, as indicated by
C           NEEDAV, the segment contains angular velocity data.
C
C
C$ Detailed_Output
C
C     HANDLE     on output is the handle of the C-kernel file
C                containing a located segment.
C
C     DESCR      is the packed descriptor of a located segment.
C
C     SEGID      is the identifier of a located segment.
C
C     FOUND      indicates whether a requested segment was found or not.
C
C$ Parameters
C
C     FTSIZE     is the maximum number of pointing files that can
C                be loaded by CKLPF at any given time for use by the
C                readers.
C
C     ITSIZE     is the maximum number of instruments whose segments
C                are buffered by CKSNS.
C
C     STSIZE     is the maximum number of segments that can be buffered
C                at any given time by CKSNS.
C
C$ Files
C
C     C-kernel pointing files are indicated by filename before loading
C     (see CKLPF) and handle after loading (all other places).
C
C$ Exceptions
C
C     1) If CKBSR is called directly, the error 'SPICE(CKBOGUSENTRY)'
C        is signalled.
C
C     2) See entry points CKLPF, CKUPF, CKBSS, and CKSNS for exceptions
C        specific to them.
C
C$ Particulars
C
C     CKBSR serves as an umbrella, allowing data to be shared by its
C     entry points:
C
C        CKLPF       Load pointing file.
C        CKUPF       Unload pointing file.
C        CKBSS       Begin search for segment.
C        CKSNS       Select next segment.
C
C     Before a file can be read by the C-kernel readers, it must be
C     loaded by CKLPF, which among other things opens the file.
C
C     Several files may be loaded for use simultaneously, and a file
C     only has to be loaded once to become a potential search target
C     for any number of subsequent reads.
C
C     Once a C-kernel file is loaded and opened, it is assigned a file
C     handle, which is used to keep track of the file internally, and
C     which is used by the calling program to refer to the file in all
C     subsequent calls to CK routines.
C
C     A file may be removed from the list of files for potential
C     searching by unloading it with CKUPF.
C
C     CKBSS and CKSNS are used together to search through loaded files
C     for segments.
C
C     CKBSS sets up the search. You tell it the instrument and time
C     that you are interested in, and whether you require segments
C     containing angular velocity data.
C
C     CKSNS finds segments matching the search criteria set up by
C     CKBSS. Last-loaded files get searched first, and individual files
C     are searched backwards.
C
C     When an applicable segment is found, CKSNS returns that segment's
C     descriptor and identifier, along with the handle of the file
C     containing the segment.
C
C     Subsequent calls to CKSNS continue the search, picking up where
C     the previous call to this routine left off.
C
C     CKSNS uses information on loaded files to manage a buffer
C     of saved segment descriptors and identifiers. The buffer is used
C     to speed up access time by minimizing file reads.
C
C
C$ Examples
C
C     Suppose that pointing data for the Voyager 2 narrow angle camera
C     for a certain interval of time is contained in three separate
C     files:  ORIGINAL.CK contains an original complete set of pointing
C     data and UPDATE_1.CK and UPDATE_2.CK contain two separate pointing
C     updates for certain pictures in the same time period.
C
C     In the following example, pointing from the C-kernel is extracted
C     in two different ways for the purpose of comparing the two
C     updates:
C
C     First, the original pointing file and one of the update files are
C     both loaded and pointing is retrieved for all of the pictures.
C     The update file is searched through first, and if no data for the
C     desired picture is located, then the original file provides the
C     requested pointing.
C
C     Then, the first update file is unloaded, the second update file
C     is loaded, and the same search is performed, as above.
C
C     Throughout the two searches, a ficticious non-SPICELIB routine
C     named WRTABL writes an entry into a table that contains
C     the pointing of the camera and the file from which the pointing
C     came, if such pointing was found.  WRERR, another ficticious,
C     non-SPICELIB routine writes an error message if no such pointing
C     was found.
C
C     It is assumed that an array (FDS) exists that contains character
C     representations of the spacecraft clock time for each picture,
C     and that there are NPICS pictures.
C
C           INTEGER               NPICS
C           PARAMETER           ( NPICS = 100 )
C
C           INTEGER               HANDLE
C           INTEGER               HNORIG
C           INTEGER               HUPDT
C           INTEGER               UPDATE
C           INTEGER               INST
C           INTEGER               SC
C           INTEGER               I
C
C           DOUBLE PRECISION      DESCR    (    5 )
C           DOUBLE PRECISION      SCLKDP
C           DOUBLE PRECISION      TOL
C           DOUBLE PRECISION      CLKOUT
C           DOUBLE PRECISION      CMAT     ( 3, 3 )
C           DOUBLE PRECISION      AV       (    3 )
C
C           CHARACTER*(12)        FDS      ( NPICS )
C           CHARACTER*(25)        FNAME
C           CHARACTER*(40)        SEGID
C           CHARACTER*(12)        OUTFDS
C           CHARACTER*(12)        TOLSTR
C           CHARACTER*(25)        UDFILE   (    2 )
C
C           LOGICAL               PFOUND
C           LOGICAL               SFOUND
C           LOGICAL               NEEDAV
C
C
C           UDFILE ( 1 ) = 'UPDATE_1.CK'
C           UDFILE ( 2 ) = 'UPDATE_2.CK'
C
C     C
C     C     The NAIF integer ID codes for the Voyager 2 spacecraft
C     C     and the narrow angle camera on Voyager 2 are -32 and
C     C     -32001, respectively.
C     C
C           SC           = -32
C           INST         = -32001
C     C
C     C     Load the Voyager SCLK file.
C     C
C           CALL LDPOOL ( 'VG2_SCLK.TSC' )
C
C     C
C     C     Allow a time tolerance of 400 line counts.  Convert
C     C     the tolerance to 'ticks', the units of encoded spacecraft
C     C     clock time.
C     C
C           TOLSTR  = '0:00:400'
C           CALL SCTIKS ( SC, TOLSTR, TOL )
C
C     C
C     C     Don't care about angular velocity data.
C     C
C           NEEDAV = .FALSE.
C
C     C
C     C     Load the original CK file first.
C     C
C           CALL CKLPF ( 'ORIGINAL.CK', HNORIG )
C
C
C           DO UPDATE = 1, 2
C
C     C
C     C        Load the update file.  Last-loaded files get searched
C     C        first, so the update file will be searched before
C     C        the original file.
C     C
C              CALL CKLPF ( UDFILE ( UPDATE ), HUPDT )
C
C              DO I = 1, NPICS
C
C     C
C     C           Encode the character string representation of
C     C           spacecraft clock time in FDS.
C     C
C                 CALL SCENCD ( SC, FDS( I ), SCLKDP )
C
C     C
C     C           Begin a search for this instrument and time, and
C     C           get the first applicable segment.
C     C
C                 CALL CKBSS ( INST,   SCLKDP, TOL,   NEEDAV  )
C                 CALL CKSNS ( HANDLE, DESCR,  SEGID, SFOUND  )
C
C     C
C     C           Keep trying candidate segments until a segment can
C     C           produce a pointing instance within the specified
C     C           time tolerance of SCLKDP, the encoded spacecraft
C     C           clock time.
C     C
C                 PFOUND = .FALSE.
C
C                 DO WHILE (  SFOUND .AND. ( .NOT. PFOUND )  )
C
C                    CALL CKPFS ( HANDLE, DESCR, SCLKDP, TOL,  NEEDAV,
C          .                      CMAT,   AV,    CLKOUT, PFOUND       )
C
C                    IF ( PFOUND ) THEN
C
C     C                 Get the name of the file from whence the
C     C                 pointing instance came, decode the spacecraft
C     C                 clock time associated with the instance, and
C     C                 write the results to the table.
C     C
C                       CALL DAFHFN ( HANDLE, FNAME          )
C                       CALL SCDECD ( SC,     CLKOUT, OUTFDS )
C
C                       CALL WRTABL ( FDS( I ), OUTFDS, CMAT, FNAME )
C
C                    ELSE
C     C
C     C                 Look for another candidate segment.
C     C
C                       CALL CKSNS ( HANDLE, DESCR, SEGID, SFOUND )
C
C                    END IF
C
C                 END DO
C
C                 IF ( .NOT. PFOUND ) THEN
C
C                    CALL WRERR ( FDS( I ) )
C
C                 END IF
C
C              END DO
C
C     C
C     C        Unload the update file.  The original file stays loaded.
C     C
C              CALL CKUPF  ( HUPDT )
C
C           END DO
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     J.M. Lynch     (JPL)
C     J.E. McLean    (JPL)
C     M.J. Spencer   (JPL)
C     R.E. Thurman   (JPL)
C     I.M. Underwood (JPL)
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 25-NOV-1992 (JML)
C
C     1) When loading a file, CKLPF now checks if the file table is
C        full only after determining that the file is not currently
C        loaded. Previously, if the file table was full and an attempt
C        was made to reload a file, an error was signalled. A new
C        exception was added as a result of this change.
C
C     2) A bug in the way that CKLPF and CKUPF clean up the instrument
C        tables after a file is unloaded was fixed.
C
C     3) Variable declarations were added to the example program
C        so that it can now be compiled.
C
C     4) The length of the elements in the array of segment
C        indentifiers ( STIDNT ) was changed from 56 to 40.
C
C-    SPICELIB Version 1.1.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.1.0, 01-NOV-1990 (JML)
C
C        An intial value was assigned to the variable STATUS so
C        that an error will be signalled if CKSNS is called
C        without CKBSS ever having been called to initiate the
C        search.
C
C-    SPICELIB Version 1.0.0, 07-SEP-1990 (RET) (IMU)
C
C-&
 
C$ Index_Entries
C
C     buffer ck segments for readers
C
C-&
 
 
 
 
C$ Revisions
C
C-    SPICELIB Version 2.0.0, 25-NOV-1992 (JML)
C
C     1) When loading a file, CKLPF now checks if the file table is
C        full only after determining that the file is not currently
C        loaded. Previously, if the file table was full and an attempt
C        was made to reload a file, an error was signalled. A new
C        exception was added as a result of this change.
C
C     2) A bug in the way that CKLPF and CKUPF clean up the instrument
C        tables after a file is unloaded was fixed.
C
C     3) Variable declarations were added to the example program
C        so that it can now be compiled.
C
C     4) The length of the elements in the array of segment
C        indentifiers ( STIDNT ) was changed from 56 to 40.
C
C-    SPICELIB Version 1.1.0, 01-NOV-1990 (JML)
C
C        An intial value was assigned to the variable STATUS so
C        that an error will be signalled if CKSNS is called
C        without CKBSS ever having been called to initiate the
C        search.
C
C
C-    Beta Version 1.1.0, 28-AUG-1990 (MJS) (JEM)
C
C        The following changes were made as a result of the
C        NAIF CK Code and Documentation Review:
C
C           1) The variable SCLK  was changed to SCLKDP.
C           2) The variable IDENT was changed to SEGID.
C           3) The parameterized values for FTSIZE and ITSIZE were
C               increased from 5 to 20.
C           4) The paramterized value for STSIZE was increased from 100
C              to 1000.
C           5) The local variables INTDES and DPDES were changed to
C              ICD and DCD.
C           6) The extended SAVE statement was broken in to single
C              SAVE statements.
C           7) Header and internal documentation was corrected and
C              updated.
C
C-    Beta Version 1.0.0, 14-MAR-1990 (RET) (IMU)
C
C-&
 
C
C     SPICELIB functions
C
      LOGICAL               FAILED
      LOGICAL               RETURN
 
      INTEGER               INTMAX
      INTEGER               ISRCHI
 
C
C     Local parameters
C
C        NDC        is the number of double precision components in an
C                   unpacked C-kernel descriptor.
C
C        NIC        is the number of integer components in an unpacked
C                   C-kernel descriptor.
C
C        NC         is the number of components in a packed C-kernel
C                   descriptor.  All DAF summaries have this formulaic
C                   relationship between the number of its integer and
C                   double precision components and the number of packed
C                   components.
C
 
      INTEGER               NDC
      PARAMETER           ( NDC = 2 )
 
      INTEGER               NIC
      PARAMETER           ( NIC = 6 )
 
      INTEGER               NC
      PARAMETER           ( NC = NDC + ( NIC + 1 )/2 )
 
 
C
C     Local variables
C
 
C
C     The file table contains the handle and file number of each file
C     that has been loaded for use with the CK readers. File
C     numbers begin at one, and are incremented until they reach the
C     maximum integer representable by the compiler, at which point
C     the routine fails.
C
C     A file number is similar to a file handle, but it is assigned
C     and used exclusively by this module. The purpose of file numbers
C     is to keep track of the order in which files are loaded and the
C     order in which they are searched.
C
C     All names begin with FT.
C
C        HAN      Handle
C        NUM      File number
C
C     NFT is the number of files that have been loaded. NEXT is
C     incremented whenever a new file is loaded to give the file
C     number for that file. FINDEX is the index of whatever file is
C     of current interest.
C
C     New files are added at the end of the table. As files are
C     removed, succeeding files are moved forward to take up the
C     slack. This keeps the table ordered by file number.
C
      INTEGER               NFT
      INTEGER               FTHAN    ( FTSIZE )
      INTEGER               FTNUM    ( FTSIZE )
      INTEGER               NEXT
      INTEGER               FINDEX
 
C
C     The instrument table contains the beginning of the list of the
C     stored segments for each spacecraft/instrument pair, and the
C     expense at which that list was constructed. (The expense of an
C     instrument list is the number of segment descriptors examined
C     during the construction of the list.) It also contains the
C     highest and lowest file numbers searched during the construction
C     of the list.
C
C     All names begin with IT.
C
C        INS      Spacecraft/instrument number
C        EXP      Expense
C        HFS      Highest file (number) searched
C        LFS      Lowest  file (number) searched
C        BEG      Beginning of segment list
C
C     NIT is the number of instruments for which segments are currently
C     being stored in the table. IINDEX is the index of whatever
C     instrument is of current interest at any given time.
C
C     New instruments are added at the end of the table. As instruments
C     are removed, the last instrument is moved forward to take up the
C     slack. This keeps the entries in the table contiguous.
C
      INTEGER               NIT
      INTEGER               ITINS    ( ITSIZE )
      INTEGER               ITEXP    ( ITSIZE )
      INTEGER               ITHFS    ( ITSIZE )
      INTEGER               ITLFS    ( ITSIZE )
      INTEGER               ITBEG    ( ITSIZE )
      INTEGER               IINDEX
 
C
C     The segment table contains the handle, descriptor, and identifier
C     for each segment that has been found so far. The segment contains
C     data for the instrument on whose list it appears. The list is
C     one-way:  each node points to the next node. The final node
C     points to nothing (zero). Nodes not used by any list are on the
C     free list.
C
C     All names begin with ST.
C
C        PTR      Pointer to next element
C        HAN      Handle
C        DCD      Double Precision component of descriptor
C        ICD      Integer component of descriptor
C        IDNT     Identifier
C
C     FREE points to the beginning of the free list.
C
C     New segments are added to the front or end of an instrument list
C     as appropriate, according to the rules spelled out under
C     entry point CKSNS.
C
C     We don't have to keep track of how many segments are in the table
C     because the table is a linked list, with the end marked by a zero
C     pointer.
C
      INTEGER               STPTR    (      STSIZE )
      INTEGER               STHAN    (      STSIZE )
      DOUBLE PRECISION      STDCD    ( NDC, STSIZE )
      INTEGER               STICD    ( NIC, STSIZE )
      CHARACTER*40          STIDNT   (      STSIZE )
      INTEGER               FREE
 
C
C     Other stuff
C
      INTEGER               SLEN
      PARAMETER           ( SLEN = 40 )
 
      INTEGER               ICD   ( NIC )
      INTEGER               SCINST
      INTEGER               TOP
      INTEGER               MINEXP
      INTEGER               CHEAP
      INTEGER               NXTSEG
      INTEGER               PREV
      INTEGER               TAIL
      INTEGER               NEW
      INTEGER               BEG
      INTEGER               P
      INTEGER               I
      INTEGER               J
 
      DOUBLE PRECISION      DCD    ( NDC )
      DOUBLE PRECISION      ALPHA
      DOUBLE PRECISION      OMEGA
 
      CHARACTER*(SLEN)      STATUS
      CHARACTER*(SLEN)      DOING
      CHARACTER*(SLEN)      URGENT
      CHARACTER*(SLEN)      STACK    ( 2 )
 
      LOGICAL               AVNEED
      LOGICAL               FND
 
C
C     Saved variables
C
      SAVE                  NFT
      SAVE                  FTHAN
      SAVE                  FTNUM
      SAVE                  NEXT
      SAVE                  NIT
      SAVE                  ITINS
      SAVE                  ITEXP
      SAVE                  ITHFS
      SAVE                  ITLFS
      SAVE                  ITBEG
      SAVE                  IINDEX
      SAVE                  STPTR
      SAVE                  STHAN
      SAVE                  STDCD
      SAVE                  STICD
      SAVE                  STIDNT
      SAVE                  ALPHA
      SAVE                  OMEGA
      SAVE                  AVNEED
      SAVE                  FREE
      SAVE                  BEG
      SAVE                  FND
      SAVE                  P
      SAVE                  FINDEX
      SAVE                  ICD
      SAVE                  DCD
      SAVE                  SCINST
      SAVE                  TOP
      SAVE                  MINEXP
      SAVE                  CHEAP
      SAVE                  NXTSEG
      SAVE                  STATUS
      SAVE                  DOING
      SAVE                  URGENT
      SAVE                  STACK
 
C
C     Initial values
C
      DATA                  NFT       / 0 /
      DATA                  NIT       / 0 /
      DATA                  NEXT      / 0 /
 
      DATA                  STATUS    / 'BOGUS ENTRY' /
 
 
C
C     Nobody has any business calling CKBSR directly.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN  ( 'CKBSR' )
         CALL SIGERR ( 'SPICE(CKBOGUSENTRY)' )
         CALL CHKOUT ( 'CKBSR' )
      END IF
 
      RETURN
 
 
C$Procedure CKLPF ( C-kernel, load pointing file )
 
      ENTRY CKLPF ( FNAME, HANDLE )
 
C$ Abstract
C
C     Load a CK pointing file for use by the CK readers.  Return that
C     file's handle, to be used by other CK routines to refer to the
C     file.
C
C$ Required_Reading
C
C     CK
C     DAF
C
C$ Keywords
C
C     POINTING
C
C$ Declarations
C
C     CHARACTER*(*)         FNAME
C     INTEGER               HANDLE
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     FNAME      I   Name of the CK file to be loaded.
C     HANDLE     O   Loaded file's handle.
C
C$ Detailed_Input
C
C     FNAME      is the name of a C-kernel file to be loaded.
C
C$ Detailed_Output
C
C     HANDLE     is an integer handle assigned to the file upon loading.
C                Almost every other CK routine will subsequently use
C                this number to refer to the file.
C
C$ Parameters
C
C     FTSIZE     is the maximum number of pointing files that can
C                be loaded by CKLPF at any given time for use by the
C                readers.
C
C$ Exceptions
C
C     1) If an attempt is made to load more files than is specified
C        by the parameter FTSIZE, the error 'SPICE(CKTOOMANYFILES)'
C        is signalled.
C
C     2) If an attempt is made to open more DAF files than is specified
C        by the parameter FTSIZE in DAFAH, an error is signalled by a
C        routine that this routine calls.
C
C     3) If the file specified by FNAME can not be opened, an error
C        is signalled by a routine that this routine calls.
C
C     4) If the file specified by FNAME has already been loaded,
C        it will become the "last-loaded" file.  (The readers
C        search the last-loaded file first.)
C
C$ Files
C
C     The C-kernel file specified by FNAME is loaded.  The file is
C     assigned an integer handle by CKLPF.  Other CK routines will refer
C     to this file by its handle.
C
C$ Particulars
C
C     See Particulars above, in CKBSR.
C
C     If there is room for a new file, CKLPF opens the file for
C     reading.  This routine must be called prior to a call to CKGP or
C     CKGPAV.
C
C     CK readers search files loaded with CKLPF in the reverse order
C     in which they were loaded.  That is, last-loaded files are
C     searched first.
C
C$ Examples
C
C     See the Example above, in CKBSR.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     J.M. Lynch     (JPL)
C     J.E. McLean    (JPL)
C     M.J. Spencer   (JPL)
C     R.E. Thurman   (JPL)
C     I.M. Underwood (JPL)
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 25-NOV-1992 (JML)
C
C     1) When loading a file, CKLPF now checks if the file table is
C        full only after determining that the file is not currently
C        loaded. Previously, if the file table was full and an attempt
C        was made to reload a file, an error was signalled.  A new
C        exception was added as a result of this change.
C
C     2) A bug in the way that CKLPF and CKUPF clean up the instrument
C        tables after a file is unloaded was fixed.
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 07-SEP-1990 (RET) (IMU)
C
C-&
 
C$ Index_Entries
C
C     load ck pointing file
C
C-&
 
 
C$ Revisions
C
C-    SPICELIB Version 2.0.0, 25-NOV-1992 (JML)
C
C     1) When loading a file, CKLPF now checks if the file table is
C        full only after determining that the file is not currently
C        loaded. Previously, if the file table was full and an attempt
C        was made to reload a file, an error was signalled.  A new
C        exception was added as a result of this change.
C
C     2) A bug in the way that CKLPF and CKUPF clean up the instrument
C        tables after a file is unloaded was fixed.
C
C        If as the result of loading a file that was previously loaded,
C        there are no more segments buffered for a particular
C        instrument, the counter variable for the instruments is no
C        longer incremented.
C
C        The following code fragment changed:
C
C           IF ( ITBEG( I ) .EQ. 0 ) THEN
C
C              .
C              .
C              .
C              NIT = NIT - 1
C
C           END IF
C
C           I = I + 1
C
C        This is the fix:
C
C           IF ( ITBEG( I ) .EQ. 0 ) THEN
C
C              .
C              .
C              .
C              NIT = NIT - 1
C
C           ELSE
C
C              I = I + 1
C
C           END IF
C
C-    Beta Version 1.1.0, 28-AUG-1990 (MJS) (JEM)
C
C        Header documentation was updated, and error handling was
C        modified.
C
C-    Beta Version 1.0.0, 14-MAR-1990 (RET) (IMU)
C
C-&
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'CKLPF' )
      END IF
 
C
C     Nothing works unless at least one file has been loaded, so
C     this is as good a place as any to initialize the free list
C     whenever the instrument table is empty.
C
C     Each node except the last should point to the adjacent node;
C     the last points to nothing (zero).
C
      IF ( NIT .EQ. 0 ) THEN
 
         FREE = 1
 
         DO 50001
     .      I = 1, STSIZE - 1
            STPTR( I ) = I + 1
50001    CONTINUE
 
         STPTR( STSIZE ) = 0
 
      END IF
 
C
C     To load a new file, first try to open it for reading.
C
      CALL DAFOPR ( FNAME, HANDLE )
 
      IF ( FAILED() ) THEN
         CALL CHKOUT ( 'CKLPF' )
         RETURN
      END IF
 
C
C     If the file is already in the table, remove it, remove its
C     segments from the segment table (using the same algorithm
C     used by CKUPF, below), and then store it.  Last loaded files are
C     searched first, so we remove this file and store it again so it
C     will be the last loaded.
C
      FINDEX = ISRCHI ( HANDLE, NFT, FTHAN )
 
      IF ( FINDEX .GT. 0 ) THEN
C
C        Handle is already in the table.  Remove it.
C
         NFT = NFT - 1
 
         DO 50002
     .      I = FINDEX, NFT
            FTHAN( I ) = FTHAN( I + 1 )
            FTNUM( I ) = FTNUM( I + 1 )
50002    CONTINUE
 
C
C        Unlink any segments that came from this file.
C
         I = 1
 
50003    IF       ( I .LE. NIT )
     .   THEN
 
            PREV = 0
            P    = ITBEG( I )
 
50004       IF       ( P .NE. 0 )
     .      THEN
 
               IF ( STHAN( P ) .EQ. HANDLE ) THEN
 
                  IF ( P .EQ. ITBEG( I ) ) THEN
 
                     ITBEG( I ) = STPTR( P )
                     STPTR( P ) = FREE
                     FREE       = P
                     P          = ITBEG( I )
 
                  ELSE
 
                     STPTR( PREV ) = STPTR( P )
                     STPTR( P    ) = FREE
                     FREE          = P
                     P             = STPTR( PREV )
 
                  END IF
 
               ELSE
 
                  PREV = P
                  P    = STPTR( P )
 
               END IF
 
               GO TO 50004
            END IF
 
C
C        If the list for this instrument is now empty, shorten the
C        current table by one: put all the entries for the last
C        instrument in the table into the space occupied by the
C        one we've deleted.
C
            IF ( ITBEG( I ) .EQ. 0 ) THEN
 
               ITINS( I ) = ITINS( NIT )
               ITEXP( I ) = ITEXP( NIT )
               ITHFS( I ) = ITHFS( NIT )
               ITLFS( I ) = ITLFS( NIT )
               ITBEG( I ) = ITBEG( NIT )
 
               NIT = NIT - 1
 
            ELSE
 
               I = I + 1
 
            END IF
 
            GO TO 50003
         END IF
 
      ELSE
C
C        This is a new file.  Make sure that there are unused slots
C        in the file table.
C
         IF ( NFT .EQ. FTSIZE ) THEN
 
            CALL DAFCLS ( HANDLE )
 
            CALL SETMSG ( 'Number of files loaded is at a maximum, ' //
     .                    'as specified by the parameter FTSIZE, '   //
     .                    'the value of which is #. You will need '  //
     .                    'to either load fewer files, or change '   //
     .                    'the parameter FTSIZE.'                     )
            CALL ERRINT ( '#', FTSIZE                                 )
            CALL SIGERR ( 'SPICE(CKTOOMANYFILES)'                     )
            CALL CHKOUT ( 'CKLPF'                                     )
            RETURN
 
         END IF
 
      END IF
 
C
C     Now add file to file table.
C
      NFT        = NFT  + 1
      NEXT       = NEXT + 1
      FTHAN(NFT) = HANDLE
      FTNUM(NFT) = NEXT
 
      CALL CHKOUT ( 'CKLPF' )
      RETURN
 
 
 
C$Procedure CKUPF ( C-kernel, Unload pointing file )
 
      ENTRY CKUPF ( HANDLE )
 
C$ Abstract
C
C     Unload a CK pointing file so that it will no longer be searched
C     by the readers.
C
C$ Required_Reading
C
C     CK
C     DAF
C
C$ Keywords
C
C     POINTING
C
C$ Declarations
C
C     INTEGER               HANDLE
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   Handle of CK file to be unloaded
C
C$ Detailed_Input
C
C     HANDLE     Integer handle assigned to the file upon loading.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C     1) If the file specified by HANDLE does not appear in the file
C        table, nothing happens.
C
C$ Files
C
C     The file referred to by HANDLE is unloaded.
C
C$ Particulars
C
C     See Particulars section above, in CKBSR.
C
C     Unloading a file with CKUPF removes that file from consideration
C     by the CK readers.  In doing so, it frees up space for another
C     file to be loaded.
C
C$ Examples
C
C     See the Example above, in CKBSR.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     J.M. Lynch     (JPL)
C     R.E. Thurman   (JPL)
C     I.M. Underwood (JPL)
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 25-NOV-1992 (JML)
C
C     1) A bug in the way that CKLPF and CKUPF clean up the instrument
C        tables after a file is unloaded was fixed.
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 07-SEP-1990 (RET) (IMU)
C
C-&
 
C$ Index_Entries
C
C     unload ck pointing file
C
C-&
 
 
 
C$ Revisions
C
C-    SPICELIB Version 2.0.0, 25-NOV-1992 (JML)
C
C     1) A bug in the way that CKLPF and CKUPF clean up the instrument
C        tables after a file is unloaded was fixed.
C
C        If as the result of unloading a file there are no more
C        segments buffered for a particular instrument, the counter
C        variable for the instruments in the instrument table is no
C        longer incremented.
C
C        The following code fragment changed:
C
C           IF ( ITBEG( I ) .EQ. 0 ) THEN
C
C              .
C              .
C              .
C              NIT = NIT - 1
C
C           END IF
C
C           I = I + 1
C
C        This is the fix:
C
C           IF ( ITBEG( I ) .EQ. 0 ) THEN
C
C              .
C              .
C              .
C              NIT = NIT - 1
C
C           ELSE
C
C              I = I + 1
C
C           END IF
C
C-    Beta Version 1.0.1, 29-AUG-1990 (MJS) (JEM)
C
C        Comments were updated.
C
C-    Beta Version 1.0.0, 07-SEP-1990 (RET) (IMU)
C
C-&
 
 
 
C
C     The file must be removed from the file table, and all of the
C     stored segments from the file must be removed from the segment
C     table (by returning the corresponding nodes to the front of the
C     free list.)
C
C     Don't do anything if the given handle is not in the file table.
C
      FINDEX = ISRCHI ( HANDLE, NFT, FTHAN )
 
      IF ( FINDEX .NE. 0 ) THEN
 
C
C        First get rid of the entry in the file table.
C
         NFT = NFT - 1
 
         DO 50005
     .      I = FINDEX, NFT
            FTHAN( I ) = FTHAN( I + 1 )
            FTNUM( I ) = FTNUM( I + 1 )
50005    CONTINUE
 
C
C        Check each instrument list individually. Note that the first
C        node on each list, having no predecessor, must be handled
C        specially.
C
         I = 1
 
50006    IF       ( I .LE. NIT )
     .   THEN
 
            PREV = 0
            P    = ITBEG( I )
 
50007       IF       ( P .NE. 0 )
     .      THEN
 
                IF ( STHAN( P ) .EQ. HANDLE ) THEN
 
C
C                 Unlink segment table entry.
C
                  IF ( P .EQ. ITBEG( I ) ) THEN
 
                     ITBEG( I ) = STPTR( P )
                     STPTR( P ) = FREE
                     FREE       = P
                     P          = ITBEG( I )
 
                  ELSE
 
                     STPTR( PREV ) = STPTR( P )
                     STPTR( P )    = FREE
                     FREE          = P
                     P             = STPTR( PREV )
 
                  END IF
 
               ELSE
 
                  PREV = P
                  P    = STPTR( P )
 
               END IF
 
               GO TO 50007
            END IF
 
C
C           If the list for this instrument is now empty, shorten the
C           current table by one: put all the entries for the last
C           instrument in the table into the space occupied by the
C           one we've deleted.
C
            IF ( ITBEG( I ) .EQ. 0 ) THEN
 
               ITINS( I ) = ITINS( NIT )
               ITEXP( I ) = ITEXP( NIT )
               ITHFS( I ) = ITHFS( NIT )
               ITLFS( I ) = ITLFS( NIT )
               ITBEG( I ) = ITBEG( NIT )
 
               NIT = NIT - 1
 
            ELSE
 
               I = I + 1
 
            END IF
 
            GO TO 50006
         END IF
 
C
C        Finally, close the file.
C
         CALL DAFCLS ( HANDLE )
 
      END IF
 
      RETURN
 
 
 
C$Procedure CKBSS ( C-kernel, begin search for segment )
 
      ENTRY CKBSS ( INST, SCLKDP, TOL, NEEDAV )
 
C$ Abstract
C
C     Initiate search through loaded files to find segments applicable
C     to the spacecraft instrument and time specified.
C
C$ Required_Reading
C
C     CK
C     DAF
C
C$ Keywords
C
C     POINTING
C
C$ Declarations
C
C     INTEGER               INST
C     DOUBLE PRECISION      SCLKDP
C     DOUBLE PRECISION      TOL
C     LOGICAL               NEEDAV
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     INST       I   Spacecraft and instrument ID.
C     SCLKDP     I   Encoded spacecraft clock time.
C     TOL        I   Time tolerance.
C     NEEDAV     I   Is there a need for angular velocity?
C
C$ Detailed_Input
C
C     CKBSS sets up a search for segments. The four quantities below
C     establish this search criteria.
C
C
C     INST       is the NAIF ID of an instrument.
C
C     SCLKDP     is an encoded spacecraft clock time.
C
C     TOL        is a time tolerance, measured in the same units as
C                encoded spacecraft clock.
C
C     NEEDAV     indicates whether or not angular velocity data is
C                required.
C
C                If true, only segments containing pointing and angular
C                velocity data will be checked. If false, segments
C                containing just pointing data will also be considered.
C
C
C     A segment matches the CKBSS/CKSNS search criteria when the
C     following statements are true.
C
C        1) INST matches the instrument number for the segment.
C
C        2) The time interval [SCLKDP - TOL, SCLKDP + TOL] intersects
C           the time interval of the segment.
C
C        3) If angular velocity data is required, as indicated by
C           NEEDAV, the segment contains angular velocity data.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If no files have been loaded, the error 'SPICE(NOLOADEDFILES)'
C        is signalled.
C
C$ Files
C
C     All files loaded by CKLPF are potential search targets for
C     CKSNS.
C
C$ Particulars
C
C     CKBSS sets up a search for segments by CKSNS. It records the
C     instrument and time to be searched for, and whether to require
C     segments containing angular velocity data. If angular velocity
C     data are required, only segments containing angular velocity
C     data will be returned by CKSNS. If angular velocity data are
C     not required, segments returned by CKSNS may or may not contain
C     angular velocity data.
C
C     CKBSS determines the first task that CKSNS will have to perform
C     if it is called to get an applicable segment.
C
C$ Examples
C
C     See Examples in CKBSR.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     M.J. Spencer   (JPL)
C     J.E. McLean    (JPL)
C     R.E. Thurman   (JPL)
C     I.M. Underwood (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 07-SEP-1990 (RET) (IMU)
C
C-&
 
C$ Index_Entries
C
C     begin search for ck segment
C
C-&
 
 
 
C$ Revisions
C
C-    Beta Version 1.1.0, 28-AUG-1990 (MJS) (JEM)
C
C        The following changes were made as a result of the
C        NAIF CK Code and Documentation Review:
C
C          1) The variable SCLK was changed to SCLKDP.
C          2) Header documentation was updated.
C
C-    Beta Version 1.0.0, 20-APR-1990 (RET) (IMU)
C
C-&
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'CKBSS' )
      END IF
 
C
C     Make copies of the instrument ID code and angular velocity flag.
C
C     And form the endpoints of the acceptable time interval using the
C     input time and time tolerance.
C
      SCINST = INST
      ALPHA  = SCLKDP - TOL
      OMEGA  = SCLKDP + TOL
      AVNEED = NEEDAV
 
C
C     There must be at least one file loaded.
C
      IF ( NFT .EQ. 0 ) THEN
 
         CALL SETMSG ( 'At least one CK file needs must be loaded ' //
     .                 'by CKLPF before beginning a search.'       )
         CALL SIGERR ( 'SPICE(NOLOADEDFILES)'                      )
         CALL CHKOUT ( 'CKBSS'                                     )
         RETURN
 
      END IF
 
C
C     The stack of suspended tasks is empty.
C
      TOP = 0
 
C
C     Is the instrument already in the instrument table?  The answer
C     determines what the first task for CKSNS will be.
C
      IINDEX = ISRCHI ( SCINST, NIT, ITINS )
 
      IF ( IINDEX .EQ. 0 ) THEN
         STATUS = 'NEW INSTRUMENT'
      ELSE
         STATUS = '?'
      END IF
 
      CALL CHKOUT ( 'CKBSS' )
      RETURN
 
 
 
C$Procedure CKSNS ( C-kernel, Select next segment )
 
      ENTRY CKSNS ( HANDLE, DESCR, SEGID, FOUND )
 
C$ Abstract
C
C     Search through loaded files to find a segment matching the
C     requested instrument, time, and need for angular velocity.
C     Buffer segment descriptors, identifiers, and handles in the
C     process to minimize file reads.
C
C$ Required_Reading
C
C     CK
C     DAF
C
C$ Keywords
C
C     POINTING
C
C$ Declarations
C
C     INTEGER               HANDLE
C     DOUBLE PRECISION      DESCR  ( * )
C     CHARACTER*(*)         SEGID
C     LOGICAL               FOUND
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     O   Handle of file containing the applicable segment.
C     DESCR      O   Descriptor of the applicable segment.
C     SEGID      O   Identifier of the applicable segment.
C     FOUND      O   True if a segment was found.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     HANDLE     is an integer handle of the file containing the
C                segment matching the instrument and time
C                specifications made in the last call to CKBSS.
C
C     DESCR,
C     SEGID      are the descriptor and identifier of the segment found
C                which matches the instrument and time specifications
C                made in the last call to CKBSS.
C
C     FOUND      is true if an applicable segment was found.  False
C                otherwise.  If FOUND is false, the values of the
C                other arguments are meaningless.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If CKSNS is called without CKBSS ever having been called,
C        the error 'SPICE(CALLCKBSSFIRST)' is signalled.
C
C     2) If no segment is found that matches the search criteria,
C        FOUND is set to false, but the values of HANDLE, DESCR,
C        and SEGID will be meaningless.
C
C$ Files
C
C     All files loaded by CKLPF are potential search targets for
C     CKSNS. The files are all referred to by their integer handles.
C
C$ Particulars
C
C     CKSNS is used to locate segments based on the search criteria
C     established by the most recent call to CKBSS.  When a segment
C     is found it will have the following characteristics:
C
C        1) Its instrument will match the instrument specified in the
C           call to CKBSS.
C
C        2) Its time interval will intersect the time interval
C
C              [SCLKDP - TOL, SCLKDP + TOL],
C
C           where SCLKDP and TOL were specified in the call to CKBSS.
C
C        3) If there is a need for angular velocity data, as specified
C           by NEEDAV in the call to CKBSS, a returned segment
C           will contain angular velocity data. If there is no need
C           for such data, the returned segment may or may not contain
C           angular velocity data.
C
C     The first call to CKSNS following a call to CKBSS starts a search
C     through loaded files and either returns the first applicable
C     segment, or indicates that no segment was found.
C
C     CKSNS searches through last-loaded files first. Individual
C     files are searched backwards, so that segments that were inserted
C     last into the file get checked first.
C
C     Subsequent calls to CKSNS pick up the search exactly where the
C     previous calls left off. If a segment is not found, future calls
C     will also indicate that no segment could be found, until a new
C     search is begun.
C
C     CKSNS also buffers segment descriptors and identifiers, to
C     attempt to minimize file reads.
C
C$ Examples
C
C     See Examples in CKBSR.
C
C$ Restrictions
C
C     This subroutine assumes that a search has been initiated by
C     a call to CKBSS.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     J.E. McLean    (JPL)
C     M.J. Spencer   (JPL)
C     R.E. Thurman   (JPL)
C     I.M. Underwood (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.1.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.1.0, 01-NOV-1990 (JML)
C
C        A check on the initial value of the variable STATUS
C        was added in order to detect the situation in which
C        CKBSS was never called to initiate a search.
C
C
C-    SPICELIB Version 1.0.0, 07-SEP-1990 (RET) (IMU)
C
C-&
 
C$ Index_Entries
C
C     select next ck segment
C
C-&
 
 
C$ Revisions
C
C-    SPICELIB Version 1.1.0, 01-NOV-1990 (JML)
C
C        A check on the initial value of the variable STATUS
C        was added in order to detect the situation in which
C        CKBSS was never called to initiate a search.
C
C
C-    Beta Version 1.1.0, 28-AUG-1990 (MJS) (JEM)
C
C        The following changes were made as a result of the
C        NAIF CK Code and Documentation Review:
C
C           1) The variable IDENT was changed to SEGID.
C           2) The local variables INTDES and DPDES were changed to
C              ICD and DCD.
C           3) Header and internal documentation was corrected and
C              updated.
C
C-    Beta Version 1.0.0, 20-APR-1990 (RET) (IMU)
C
C-&
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'CKSNS' )
      END IF
 
C
C     CKSNS buffers segment descriptors and identifiers, to
C     attempt to minimize file reads. Buffering segments involves
C     maintaining three tables:  the file table, the instrument table,
C     and the segment table. CKSNS is broken down into various tasks,
C     described in the code below, which perform these manipulations.
C
C     A description of the components of each table is provided in
C     the declarations section of CKBSR.
C
C     Basically, the buffering is performed as follows: Once a request
C     for a segment for a particular instrument is made, if there are
C     no adequate entries in the buffer already, a search is made
C     through loaded files for applicable segments.  Every segment
C     pertaining to that instrument in a searched file is buffered,
C     before a check of the current buffer is made.  If the search
C     doesn't turn up a segment matching the specified search criteria
C     the next file is searched and new segments are added to the list,
C     and so on.
C
C     The information in the segment table (ST) is stored in a
C     singly-linked list. Each node in the list contains several
C     individual pieces of data, which are stored in parallel
C     arrays. One of these arrays, PTR, contains the pointers
C     for the nodes. The pointer for each node contains the array
C     index of the next node on the list. The final node on any
C     list contains index zero, indicating that it is the end
C     of the list.
C
C     A list begins with a head, in this case either ITBEG(IINDEX)
C     or FREE. The head is the array index of the first node in the
C     list. Initially, all nodes are on the free list:
C
C                      Next   Data
C                    +------+--------+
C           FREE --> |    2 |        |
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    3 |        | <--`
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    4 |        | <--`
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    5 |        | <--`
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    6 |        | <--`
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    7 |        | <--`
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    8 |        | <--`
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    0 |        | <--`
C                    |      |        |
C                    +------+--------+
C
C     The various ITBEG( I ) all contain zero, indicating that they
C     contain no nodes.
C
C     Later, some of the nodes are associated with the instrument lists.
C
C                      Next   Data
C                    +------+--------+
C       ITBEG(2) --> |    2 |        |
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    8 |        | <--`
C                    |      |        | -------------,
C                    +------+--------+              |
C                    |    5 |        |              |
C       FREE ------> |      |        | ------,      |
C                    +------+--------+       |      |
C       ITBEG(1) --> |    6 |        |       |      |
C                    |      |        | ---,  |      |
C                    +------+--------+    |  |      |
C                    |    0 |        |    |  |      |
C                    |      |        | <-----`      |
C                    +------+--------+    |         |
C                    |    0 |        | <--`         |
C                    |      |        |              |
C                    +------+--------+              |
C                    |    0 |        |              |
C                    |      |        | <--,         |
C                    +------+--------+    |         |
C                    |    7 |        | ---`         |
C                    |      |        | <------------`
C                    +------+--------+
C
C     Note that every node is always on exactly one list.
C
C     The whole point of using a list is to avoid shuffling the
C     contents of the arrays back and forth every time a segment
C     is added or deleted from the list of stored segments.
C
C     The algorithms for adding and deleting nodes from lists are
C     adequately documented elsewhere, and will not be descibed
C     here. However, note that elements are added only to the front
C     or end of a list, never the middle. Note also that when an
C     instrument is removed from the instrument table, the nodes
C     associated with the instrument are added to the beginning of the
C     free list in a single operation, rather than a node at a time.
C
 
C
C     In the following loop, we will try to simplify things by
C     doing exactly one thing on each pass through the loop.
C     After each pass, the status of the loop (STATUS) will be
C     adjusted to reflect the next thing that needs to be done.
C     The first task is set by CKBSS.
C
C     Occasionally, the current task will have to be interrupted
C     until another task can be carried out. (For example, when
C     collecting new segments, an interrupt might place a segment
C     at the front or end of the current instrument list; when placing
C     the segment on the list, a second interrupt might free
C     room in the segment table in order to allow the addition
C     to proceed.) In this case, the current task will be saved and
C     restored after the more urgent task has been completed.
C
C     The loop can terminate in only one of two ways. First, if
C     an applicable segment is found in the segment table, the
C     handle, descriptor, and identifier for the segment are returned
C     immediately.  Second, if the table does not contain an applicable
C     segment, and if no files remain to be searched, the loop
C     terminates normally, and no data is returned.
C
C     The status is saved on exit, however, so that subsequent calls
C     will pick up the action exactly where previous calls left off.
C
C     Each status is described below.
C
C     'NEW INSTRUMENT'
C
C        This indicates that the specified spacecraft/instrument has
C        no segments stored for it at all. It must be added to the
C        instrument table.  (This is followed immediately by an
C        OLD FILES search, in which every file loaded is considered an
C        old file.)
C
C     'NEW FILES'
C
C        This indicates that at least one new file has been added
C        since the last time the segment list for the specified
C        instrument was searched. Find the oldest of these new files,
C        and begin a NEW SEGMENTS search in forward order for
C        segments to add to the front of the list.
C
C     'NEW SEGMENTS'
C
C        Continue a NEW FILES search, adding segments for the specified
C        instrument to the front of the list.
C
C     'OLD FILES'
C
C        This indicates that although the list has been searched
C        and found to contain no applicable segment, some of the
C        older files remain to be searched. Find the newest of these
C        old files, and begin an OLD SEGMENTS search in backward order.
C
C     'OLD SEGMENTS'
C
C        Continue an OLD FILES search, adding segments for the specified
C        instrument to the end of the list.
C
C     'CHECK LIST'
C
C        This indicates that the list is ready to be searched,
C        either because no new files have been added, or because
C        segments from a new file or an old file have recently
C        been added.
C
C        The list is never checked until all new files have been
C        searched.
C
C        If an applicable segment is found, it is returned.
C
C     'MAKE ROOM' (Interrupt)
C
C        This indicates that one of the instruments must be removed,
C        along with its stored segments, to make room for another
C        instrument or segment.  The instrument (other than the
C        specified instrument) with the smallest expense is selected
C        for this honor.
C
C     'ADD TO FRONT' (Interrupt)
C
C        This indicates that a segment has been found (during the
C        course of a NEW FILES search) and must be added to the front
C        of the list.
C
C     'ADD TO END' (Interrupt)
C
C        This indicates that a segment has been found (during the
C        course of an OLD FILES search) and must be added to the end
C        of the list.
C
C     'FREE AND CHECK'
C
C        This indicates that an attempt to 'MAKE ROOM' failed when
C        trying to 'ADD TO END' because all of the segments in the
C        table were for the instrument being searched on. All of the
C        segments that were in the process of being added to the table
C        for the current old file get freed from the list, and a 'CHECK
C        FREED LIST' is performed. Following that, a 'SEARCH W/O BUFF'
C        is performed on all unsearched files.
C
C     'CHECK FREED LIST'
C
C        This indicates that a portion of the list was just freed,
C        and the freed portion is to be checked for applicable segments.
C
C     'SEARCH W/O BUFF'
C
C        This indicates that the segment table was too small to handle
C        all of the segments for the current instrument, and that the
C        remaining unchecked old files should be searched for applicable
C        segments, without buffering the segments.
C
C     'SUSPEND'
C
C        This indicates that the current task (DOING) should be
C        interrupted until a more urgent task (URGENT) can be
C        carried out. The current task is placed on a stack for
C        safekeeping.
C
C     'RESUME'
C
C        This indicates that the most recently interrupted task
C        should be resumed immediately.
C
C     '?'
C
C        This indicates that the next task is not immediately
C        apparent: if new files exist, they should be searched;
C        otherwise the list should be checked.
C
C     'HOPELESS'
C
C        This indicates that the table does not contain an applicable
C        segment, and no files remain to be searched.
C
C      'BOGUS ENTRY'
C
C        This is the initial value of STATUS and indicates that no
C        call to CKBSS was ever made. If this is the case then an
C        error will be signalled.
C
 
      IF ( STATUS .EQ. 'BOGUS ENTRY' )  THEN
 
         CALL SETMSG ( 'Must begin a search by calling CKBSS first.' )
         CALL SIGERR ( 'SPICE(CALLCKBSSFIRST)' )
         CALL CHKOUT ( 'CKSNS'                 )
         RETURN
 
      END IF
 
50008 IF       ( STATUS .NE. 'HOPELESS' )
     .THEN
 
C
C        If new files have been added, they have to be searched.
C        Otherwise, go right to the list of stored segments.
C
         IF ( STATUS .EQ. '?' ) THEN
 
            IF ( ITHFS( IINDEX ) .LT. FTNUM( NFT ) ) THEN
 
               STATUS = 'NEW FILES'
 
            ELSE
 
               P      =  ITBEG( IINDEX )
               STATUS = 'CHECK LIST'
 
            END IF
 
C
C        New instruments are added to the end of the instrument table.
C        If the table is full, one of the current occupants must be
C        removed to make room for the new one.
C
C        Setting LFS to one more than the highest current
C        file number means the 'OLD FILES' search that follows will
C        begin with the last-loaded file.
C
         ELSE IF ( STATUS .EQ. 'NEW INSTRUMENT' ) THEN
 
            IF ( NIT .EQ. ITSIZE ) THEN
 
               DOING  = 'NEW INSTRUMENT'
               URGENT = 'MAKE ROOM'
               STATUS = 'SUSPEND'
 
            ELSE
 
               NIT          = NIT + 1
               ITINS( NIT ) = SCINST
               ITEXP( NIT ) = 0
               ITHFS( NIT ) = FTNUM( NFT )
               ITLFS( NIT ) = FTNUM( NFT ) + 1
               ITBEG( NIT ) = 0
               IINDEX       = NIT
               STATUS       = 'OLD FILES'
 
            END IF
 
C
C        When new files exist, they should be searched in forward
C        order, beginning with the oldest new file not yet searched.
C        All new files must be searched before the list can be
C        checked, to ensure that the best (newest) segments are
C        being used.
C
C        Begin a forward search, and prepare to look for individual
C        segments from the file.
C
         ELSE IF ( STATUS .EQ. 'NEW FILES' ) THEN
 
            FINDEX = 1
 
50009       IF       ( ITHFS( IINDEX ) .GE. FTNUM( FINDEX ) )
     .      THEN
 
               FINDEX = FINDEX + 1
 
               GO TO 50009
            END IF
 
            ITHFS( IINDEX ) = FTNUM( FINDEX )
 
            CALL DAFBFS ( FTHAN( FINDEX ) )
            STATUS = 'NEW SEGMENTS'
 
C
C        New files are searched in forward order. Segments, when
C        found, are inserted at the front of the list. Invisible
C        segments (initial time > final time) are ignored.
C
C        Each segment examined, whether applicable or not, adds to
C        the expense of the list.
C
         ELSE IF ( STATUS .EQ. 'NEW SEGMENTS' ) THEN
 
            CALL DAFFNA ( FND )
 
            IF ( .NOT. FND ) THEN
               STATUS = '?'
 
            ELSE
 
               CALL DAFGS ( DESCR )
               CALL DAFUS ( DESCR, NDC, NIC, DCD, ICD )
 
               IF ( ICD( 1 ) .EQ. SCINST      .AND.
     .              DCD ( 1 ) .LE. DCD( 2 ) ) THEN
 
                  DOING  = 'NEW SEGMENTS'
                  URGENT = 'ADD TO FRONT'
                  STATUS = 'SUSPEND'
 
               END IF
 
               ITEXP( IINDEX ) = ITEXP( IINDEX ) + 1
 
            END IF
 
C
C        When old files must be searched (because the segments in the
C        list are inadequate), they should be searched in backward
C        order, beginning with the newest old file not yet searched.
C        Not every old file must be searched. The list should be
C        re-checked after each file.
C
C        Begin a backwards search, and prepare to look for individual
C        segments from the file.
C
         ELSE IF ( STATUS .EQ. 'OLD FILES' ) THEN
 
            FINDEX = NFT
 
50010       IF       ( ITLFS( IINDEX ) .LE. FTNUM( FINDEX ) )
     .      THEN
               FINDEX = FINDEX - 1
               GO TO 50010
            END IF
 
            CALL DAFBBS ( FTHAN( FINDEX ) )
            STATUS = 'OLD SEGMENTS'
 
C
C        Old files are searched in backward order. Segments, when
C        found, are inserted at the end of the list. Invisible
C        segments (initial time > final time) are ignored.
C
C        Each segment examined, whether applicable or not, adds to
C        the expense of the list.
C
         ELSE IF ( STATUS .EQ. 'OLD SEGMENTS' ) THEN
 
            CALL DAFFPA ( FND )
 
            IF ( .NOT. FND ) THEN
 
C
C              All of the segments in this file have been exhausted.
C              Change the lowest file searched indicator for this
C              instrument to be the current file, and go check the
C              current list.
C
               ITLFS( IINDEX ) =  FTNUM( FINDEX )
               P               =  ITBEG( IINDEX )
               STATUS          = 'CHECK LIST'
 
            ELSE
 
               CALL DAFGS ( DESCR )
               CALL DAFUS ( DESCR, NDC, NIC, DCD, ICD )
 
               IF ( ICD( 1 ) .EQ. SCINST      .AND.
     .              DCD ( 1 ) .LE. DCD( 2 ) ) THEN
 
                  DOING  = 'OLD SEGMENTS'
                  URGENT = 'ADD TO END'
                  STATUS = 'SUSPEND'
 
               END IF
 
               ITEXP( IINDEX ) = ITEXP( IINDEX ) + 1
 
            END IF
 
C
C        Okay, all the new files (and maybe an old file or two)
C        have been searched. Time to look at the list of segments
C        stored for the instrument, to see if there is one applicable
C        to the specified epoch and need for angular velocity data.
C
C        If so, return it.  If not, try another old file.  If there
C        are no more old files, give up the ghost.
C
         ELSE IF ( STATUS .EQ. 'CHECK LIST' ) THEN
 
C
C           P always points to the current segment in the list. Reject
C           a segment if there is a need for angular velocity data and
C           the segment doesn't have it.
C
50011       IF       ( P .NE. 0 )
     .      THEN
 
               IF (          OMEGA .GE.  STDCD( 1, P )           .AND.
     .                       ALPHA .LE.  STDCD( 2, P )           .AND.
     .              .NOT. ( AVNEED .AND. STICD( 4, P ) .EQ. 0 ) ) THEN
 
                  CALL DAFPS ( NDC,           NIC,
     .                         STDCD( 1, P ), STICD( 1, P ), DESCR )
 
                  SEGID  =  STIDNT( P )
                  HANDLE =  STHAN ( P )
                  FOUND  = .TRUE.
 
C
C                 Go ahead and move the pointer up before returning so
C                 that the search for the next applicable segment will
C                 start at the right place.
C
                  P = STPTR( P )
                  CALL CHKOUT ( 'CKSNS' )
                  RETURN
 
               END IF
 
               P = STPTR( P )
 
               GO TO 50011
            END IF
 
            IF ( ITLFS( IINDEX ) .GT. FTNUM( 1 ) ) THEN
               STATUS = 'OLD FILES'
            ELSE
               STATUS = 'HOPELESS'
            END IF
 
C
C        When adding a new instrument or segment to a full table, one of
C        the current instruments must be dropped. The ideal candidate
C        is the one whose list was constructed at the lowest expense.
C        The candidate should be removed from the instrument table, and
C        its list transferred (intact) to the head of the free list.
C
C        As files are unloaded, some instruments may lose all their
C        segments (ITBEG=0). These should be removed from the
C        instrument table, possibly obviating the need to change the
C        segment table at all.
C
C        There is ``room'' if another instrument can be added, and if
C        the free list contains at least one node.
C
C        It is possible that a single instrument requires more than the
C        entire segment table for its own segments. Two things might
C        happen in such a case:
C
C           1) If an old file was being searched, free the segments
C              for that file (and this instrument) from the list. Check
C              those freed segments and then search the rest of the
C              old unchecked files without buffering any segments.
C
C           2) If the list was being added to at the beginning, then
C              that means there was a NEW FILES search going on, and
C              so a brand new list is constructed for the instrument,
C              much as in a NEW INSTRUMENT task.
C
         ELSE IF ( STATUS .EQ. 'MAKE ROOM' ) THEN
 
            J = 0
            DO 50012
     .         I = 1, NIT
 
               IF ( ITBEG( I ) .GT. 0 ) THEN
 
                  J        = J + 1
                  ITINS( J ) = ITINS( I )
                  ITEXP( J ) = ITEXP( I )
                  ITHFS( J ) = ITHFS( I )
                  ITLFS( J ) = ITLFS( I )
                  ITBEG( J ) = ITBEG( I )
 
               END IF
 
50012       CONTINUE
 
            NIT = J
 
            IF ( NIT .EQ. ITSIZE  .OR.  FREE .EQ. 0 ) THEN
               MINEXP = INTMAX()
               CHEAP  = 0
 
               DO 50013
     .            I = 1, NIT
 
                  IF ( I          .NE. IINDEX  .AND.
     .                 ITEXP( I ) .LT. MINEXP ) THEN
 
                    CHEAP  = I
                    MINEXP = ITEXP( I )
 
                  END IF
 
50013          CONTINUE
 
               IF ( CHEAP .EQ. 0 ) THEN
 
C
C                 If there are no deleteable segments, the Thing To
C                 Do depends on the task that was suspended before
C                 entering MAKE ROOM.
C
                  IF ( STACK( TOP ) .EQ. 'ADD TO END' ) THEN
                     STATUS = 'FREE AND CHECK'
 
                  ELSE IF ( STACK( TOP ) .EQ. 'ADD TO FRONT' ) THEN
C
C                    If there is no room left in the table in the middle
C                    of an attempt to add to the front of the list, just
C                    start from scratch by effectively initiating a 'NEW
C                    INSTRUMENT' task.
C
C                    Turn the current list into a free list (FREE is
C                    currently zero)
C
                     FREE            = ITBEG( IINDEX )
                     ITBEG( IINDEX ) = 0
C
C                    Re-initialize the table for this instrument, and
C                    initiate an 'OLD FILES' search, just as in 'NEW
C                    INSTRUMENT'.
C
                     ITEXP( IINDEX ) = 0
                     ITHFS( IINDEX ) = FTNUM( NFT )
                     ITLFS( IINDEX ) = FTNUM( NFT ) + 1
                     STATUS          = 'OLD FILES'
 
                  END IF
 
C
C                 Any suspended task can't be resumed.
C
                  TOP = 0
 
               ELSE
 
C
C                 Turn this cheapest list into a free list by pointing
C                 its tail to the free list, and pointing the FREE
C                 pointer to its beginning.
C
                  TAIL   = ITBEG( CHEAP )
                  NXTSEG = STPTR( TAIL  )
 
50014             IF       ( NXTSEG .NE. 0 )
     .            THEN
                     TAIL   = NXTSEG
                     NXTSEG = STPTR( NXTSEG )
                     GO TO 50014
                  END IF
 
                  STPTR( TAIL )  = FREE
                  FREE           = ITBEG( CHEAP )
 
C
C                 Fill the deleted instrument's space in the table with
C                 the final entry in the table.
C
                  ITINS( CHEAP ) = ITINS( NIT )
                  ITEXP( CHEAP ) = ITEXP( NIT )
                  ITHFS( CHEAP ) = ITHFS( NIT )
                  ITLFS( CHEAP ) = ITLFS( NIT )
                  ITBEG( CHEAP ) = ITBEG( NIT )
 
C
C                 If the final entry in the table happened to be the
C                 current instrument of interest, then the current
C                 instrument index has to change
C
                  IF ( IINDEX .EQ. NIT ) THEN
                     IINDEX = CHEAP
                  END IF
 
C
C                 One less instrument now.
C
                  NIT    = NIT - 1
                  STATUS = 'RESUME'
 
               END IF
 
            ELSE
               STATUS = 'RESUME'
 
            END IF
 
C
C        Adding a node to the front of a list is a standard algorithm.
C        The only possible wrinkle is that the free list may be empty.
C
         ELSE IF ( STATUS .EQ. 'ADD TO FRONT' ) THEN
 
            IF ( FREE .EQ. 0 ) THEN
               DOING  = 'ADD TO FRONT'
               URGENT = 'MAKE ROOM'
               STATUS = 'SUSPEND'
 
            ELSE
               STHAN( FREE ) = FTHAN( FINDEX )
 
               CALL DAFUS ( DESCR, NDC, NIC,
     .                      STDCD ( 1, FREE ),
     .                      STICD ( 1, FREE ) )
               CALL DAFGN ( STIDNT(    FREE ) )
 
               NEW             = FREE
               FREE            = STPTR( FREE )
               STPTR( NEW    ) = ITBEG( IINDEX )
               ITBEG( IINDEX ) = NEW
 
               STATUS        = 'RESUME'
 
            END IF
 
C
C        Similarly for adding a node to the end of a list.
C
         ELSE IF ( STATUS .EQ. 'ADD TO END' ) THEN
 
            IF ( FREE .EQ. 0 ) THEN
               DOING  = 'ADD TO END'
               URGENT = 'MAKE ROOM'
               STATUS = 'SUSPEND'
 
            ELSE
C
C              The first free node is our newest node.  Move the free
C              pointer on down the free list.
C
               NEW  = FREE
               FREE = STPTR( FREE )
 
               STHAN( NEW ) = FTHAN( FINDEX )
 
               CALL DAFUS ( DESCR, NDC, NIC,
     .                      STDCD ( 1, NEW ),
     .                      STICD ( 1, NEW ) )
               CALL DAFGN ( STIDNT(    NEW ) )
 
               TAIL = ITBEG( IINDEX )
 
               IF ( TAIL .EQ. 0 ) THEN
C
C                 This is the first node in the list for this instrument
C
                  ITBEG( IINDEX ) = NEW
 
               ELSE
C
C                 Need to move to the tail of the list before adding the
C                 pointer to the new segment.
C
                  NXTSEG = STPTR( TAIL )
 
50015             IF       ( NXTSEG .NE. 0 )
     .            THEN
                     TAIL   = NXTSEG
                     NXTSEG = STPTR( NXTSEG )
                     GO TO 50015
                  END IF
 
                  STPTR( TAIL ) = NEW
 
               END IF
 
C
C              The newest segment has no segment following it.
C
               STPTR( NEW ) = 0
 
               STATUS      = 'RESUME'
 
            END IF
 
C
C        When the segment table is completely full with the current
C        instrument's segments, free and check those segments from
C        the current file. Once that's done, resume searching unchecked
C        files without buffering their segments.
C
         ELSE IF ( STATUS .EQ. 'FREE AND CHECK' ) THEN
 
C
C           Find the portion of the current instrument's segment list
C           which comes from the current file of interest.
C
            PREV = 0
            BEG  = ITBEG( IINDEX )
 
50016       IF       ( STHAN( BEG ) .NE. FTHAN( FINDEX ) .AND.
     .                        BEG   .NE. 0                     )
     .      THEN
 
               PREV = BEG
               BEG  = STPTR( BEG )
 
               GO TO 50016
            END IF
 
C
C           If anything was found, free that portion of the list, and
C           then check what's been freed.
C
C           Otherwise, finish searching old files without buffering
C           anything.
C
            IF ( BEG  .NE. 0 ) THEN
 
               FREE = BEG
 
               IF ( PREV .NE. 0 ) THEN
                  STPTR( PREV ) = 0
               END IF
 
               STATUS = 'CHECK FREED LIST'
 
            ELSE
               STATUS = 'SEARCH W/O BUFF'
 
            END IF
 
C
C        When freeing up a portion of the segment table because
C        the table is full of segments for a single instrument,
C        search through the freed portion for applicable segments.
C
C        Once the freed portion has been completely checked,
C        search the remaining old files without buffering any
C        segments in the segment table.
C
C
         ELSE IF ( STATUS .EQ. 'CHECK FREED LIST' ) THEN
 
C
C           BEG points to the current segment in the freed portion.
C           Reject a segment if there is a need for angular velocity
C           data and the segment doesn't have it.
C
50017       IF       ( BEG  .NE. 0 )
     .      THEN
 
               IF (         OMEGA .GE.  STDCD( 1, BEG )            .AND.
     .                      ALPHA .LE.  STDCD( 2, BEG )            .AND.
     .             .NOT. ( AVNEED .AND. STICD( 4, BEG ) .EQ. 0 ) )  THEN
 
                  CALL DAFPS ( NDC,             NIC,
     .                         STDCD( 1, BEG ), STICD( 1, BEG ), DESCR )
 
                  SEGID  = STIDNT( BEG )
                  HANDLE = STHAN ( BEG )
                  FOUND  = .TRUE.
C
C                 Go ahead and move the pointer up before returning so
C                 that the search for the next applicable segment will
C                 start at the right place.
C
                  BEG  = STPTR( BEG )
                  CALL CHKOUT ( 'CKSNS' )
                  RETURN
 
               END IF
 
            BEG  = STPTR( BEG )
 
               GO TO 50017
            END IF
 
            STATUS = 'SEARCH W/O BUFF'
 
C
C        When the segment table is full with the current instrument's
C        segments and any freed up portions have been checked, continue
C        the search for applicable segments in old files, without
C        buffering any of the segments in the segment table.
C
         ELSE IF ( STATUS .EQ. 'SEARCH W/O BUFF' ) THEN
 
C
C           Recall that a search is already in progress and that a
C           segment is currently under consideration (FND = .TRUE.).
C
50018       IF       ( FINDEX .GT. 0 )
     .      THEN
 
50019          IF       ( FND )
     .         THEN
 
                  CALL DAFGS ( DESCR )
                  CALL DAFUS ( DESCR, NDC, NIC, DCD, ICD )
 
                  IF (         SCINST .EQ.  ICD( 1 )           .AND.
     .                          OMEGA .GE.  DCD ( 1 )           .AND.
     .                          ALPHA .LE.  DCD ( 2 )           .AND.
     .                 .NOT. ( AVNEED .AND. ICD( 4 ) .EQ. 0 ) ) THEN
 
                     CALL DAFGN ( SEGID )
                     HANDLE = FTHAN( FINDEX )
                     FOUND  = .TRUE.
 
C
C                    Go ahead and point to the next segment in the
C                    file in case an attempt is made to continue the
C                    search: you want to pick up exactly where you left
C                    off.
C
                     CALL DAFFPA ( FND )
                     CALL CHKOUT( 'CKSNS' )
                     RETURN
 
                  END IF
 
                  CALL DAFFPA ( FND )
 
                  GO TO 50019
               END IF
 
C
C              Try the next oldest file.
C
               FINDEX = FINDEX -1
 
               IF ( FINDEX .GT. 0 ) THEN
 
                  CALL DAFBBS ( FTHAN( FINDEX ) )
                  CALL DAFFPA ( FND             )
 
               END IF
 
               GO TO 50018
            END IF
 
C
C           There's nothing nowhere if you get to here.
C
            STATUS = 'HOPELESS'
 
C
C        When a task is suspended, the current activity is placed on
C        a stack, to be restored later. Two levels are provided, since
C        some interrupts can be interrupted by others.
C
C        Trying to resume a task that was never suspended is a boo-boo.
C
         ELSE IF ( STATUS .EQ. 'SUSPEND' ) THEN
 
            TOP          = TOP + 1
            STACK( TOP ) = DOING
            STATUS       = URGENT
 
         ELSE IF ( STATUS .EQ. 'RESUME' ) THEN
 
            IF ( TOP .EQ. 0 ) THEN
               STATUS = '?'
            ELSE
               STATUS = STACK( TOP )
               TOP    = TOP - 1
            END IF
 
         END IF
 
         GO TO 50008
      END IF
 
C
C     Can only get here if status is 'HOPELESS', in which case a
C     segment was not found.
C
      FOUND = .FALSE.
 
      CALL CHKOUT ( 'CKSNS' )
      RETURN
 
      END
