 
C$Procedure DAFARR ( DAF, add reserved records )
 
      SUBROUTINE DAFARR ( HANDLE, RESV )
 
C$ Abstract
C
C     Add a specified number of reserved records to a Double Precision
C     Array File (DAF).
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     FILES
C
C$ Declarations
 
      INTEGER               HANDLE
      INTEGER               RESV
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   DAF, opened for writing.
C     RESV       I   Number of records to reserve.
C
C$ Detailed_Input
C
C     HANDLE      is the handle associated with a DAF that has been
C                 opened for write access.  Use DAFOPW, for example,
C                 to open an existing file and get its handle.
C
C     RESV        is the number of reserved records to be added
C                 to the specified file.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If RESV is less than one, the file is not changed.
C
C$ Files
C
C     See argument HANDLE.
C
C$ Particulars
C
C     Normally, the reserved records in an array file are reserved
C     when the file is created.  However, it may occasionally become
C     necessary to add reserved records---when the contents of one
C     file are appended to another, for example. (In this case, any
C     information in the reserved records of either file should
C     be included in the resulting file.)
C
C     The new reserved records are appended to the old ones.
C
C$ Examples
C
C     In the following code fragment, the call to DAFOPN
C     creates a new array file with 12 reserved records (located in
C     records 2-13 of the physical file). A subsequent call to DAFARR,
C     increases the number of reserved records by 7, to a total of 19.
C     The new reserved records are located in records 14-20 of the
C     physical file.
C
C        CALL DAFOPN ( FNAME, ND, NI, IFNAME, 12, HANDLE )
C              .
C              .
C              .
C
C        CALL DAFARR ( HANDLE, 7 )
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     I.M. Underwood (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 17-JUL-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     add daf reserved records
C
C-&
 
 
 
 
C
C     SPICELIB functions
C
      LOGICAL               FAILED
      LOGICAL               RETURN
 
C
C     Local parameters
C
C     IFNLEN      is the length of a DAF internal file name.
C
      INTEGER               IFNLEN
      PARAMETER           ( IFNLEN = 60       )
 
C
C     WPR         is the maximum number of double precision numbers
C                 (words) per record.
C
C     MAXD,       are the maximum number of double precision
C     MAXI,       numbers, integers, and characters, respectively,
C     MAXC        per record, not including space reserved for
C                 control information (3 dp numbers are reserved).
C                 There are two integers per double precision word,
C                 and eight characters per word.
C
      INTEGER               WPR
      PARAMETER           ( WPR    = 128      )
 
      INTEGER               MAXD
      PARAMETER           ( MAXD   = WPR  - 3 )
 
      INTEGER               MAXI
      PARAMETER           ( MAXI   = MAXD * 2 )
 
      INTEGER               MAXC
      PARAMETER           ( MAXC   = MAXD * 8 )
 
C
C     Local variables
C
      CHARACTER*(MAXC)      CREC
      CHARACTER*(IFNLEN)    IFNAME
 
      DOUBLE PRECISION      DC      ( MAXD )
      DOUBLE PRECISION      DREC    ( WPR  )
      DOUBLE PRECISION      SUM     ( MAXD )
 
      INTEGER               BEGBLK
      INTEGER               BWARD
      INTEGER               ENDBLK
      INTEGER               FREE
      INTEGER               FWARD
      INTEGER               I
      INTEGER               INCR
      INTEGER               IC      ( MAXI )
      INTEGER               ND
      INTEGER               NEXT
      INTEGER               NI
      INTEGER               RECNO
      INTEGER               WORD
 
      LOGICAL               FOUND
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
 
      ELSE IF ( RESV .LT. 1 ) THEN
         RETURN
 
      ELSE
         CALL CHKIN ( 'DAFARR' )
      END IF
 
C
C     Get the contents of the file record.
C
      CALL DAFRFR ( HANDLE, ND, NI, IFNAME, FWARD, BWARD, FREE )
 
C
C     Okay, here's the plan. We are just going to move records
C     in the direction of the end of the file, starting
C     with the last record in the file and ending with the first
C     summary record.
C
C     After everything has been moved, the initial and final
C     addresses of all the arrays have to be incremented by the
C     same amount: the number of words per record (128) times
C     the number of new records.
C
      INCR = WPR * RESV
 
 
C
C     Before we do that, however, we should write some bogus records
C     to the end of the file, to make sure we don't run out of space
C     later on. If this doesn't work, we will leave the logical
C     contents of the file uncorrupted (although it may get larger).
C
      CALL DAFARW ( FREE, RECNO, WORD )
 
      DO 50001
     .   I = 1, RESV
         CALL DAFWDR ( HANDLE, RECNO + I, DREC )
50001 CONTINUE
 
      IF ( FAILED() ) THEN
         CALL CHKOUT ( 'DAFARR' )
         RETURN
      END IF
 
C
C     Records will be moved in `blocks', where each block contains
C
C        -- a summary record
C
C        -- a name record
C
C        -- one or more data records
C
C     The first block to be moved (that is, the last block in
C     the file) lies between the final summary record (BWARD) and
C     whatever record contains the first free address in the file.
C
      BEGBLK = BWARD
      CALL DAFARW ( FREE, ENDBLK, WORD )
 
50002 IF       ( ( BEGBLK .GT. 0 )  .AND.  ( .NOT. FAILED() ) )
     .THEN
 
C
C        Move the data records first.
C
         DO 50003
     .      RECNO = ENDBLK,  BEGBLK + 2,  -1
            CALL DAFRDR ( HANDLE, RECNO,        1, WPR, DREC, FOUND )
            CALL DAFWDR ( HANDLE, RECNO + RESV,         DREC        )
50003    CONTINUE
 
C
C        Then the name record.
C
         RECNO = BEGBLK + 1
         CALL DAFRCR ( HANDLE, RECNO,        CREC )
         CALL DAFWCR ( HANDLE, RECNO + RESV, CREC )
 
C
C        Finally, the summary record.
C
C        To find the beginning of the next block, look at the backward
C        pointer from the summary record of the current block.
C
C        Be sure to adjust the forward and backward pointers;
C        otherwise, we won't be able to find the summaries again.
C
         RECNO = BEGBLK
         CALL DAFRDR ( HANDLE, RECNO, 1, WPR, DREC, FOUND )
 
         NEXT = DREC(2)
 
         IF ( DREC(1) .GT. 0 ) THEN
            DREC(1) = DREC(1) + RESV
         END IF
 
         IF ( DREC(2) .GT. 0 ) THEN
            DREC(2) = DREC(2) + RESV
         END IF
 
         CALL DAFWDR ( HANDLE, RECNO + RESV, DREC )
 
C
C        The next block ends just before the current block begins.
C
         ENDBLK = BEGBLK  - 1
         BEGBLK = NEXT
 
         GO TO 50002
      END IF
 
C
C     Rewrite the file record, to reflect the new organization of
C     the file.
C
      FWARD = FWARD + RESV
      BWARD = BWARD + RESV
      FREE  = FREE  + INCR
 
      CALL DAFWFR ( HANDLE, ND, NI, IFNAME, FWARD, BWARD, FREE )
 
C
C     Get the summary for each array, increment the addresses (stored
C     in the final two integer components), and replace the summary.
C
      CALL DAFBFS ( HANDLE )
      CALL DAFFNA ( FOUND  )
 
50004 IF       ( FOUND  .AND.  ( .NOT. FAILED() ) )
     .THEN
 
         CALL DAFGS  ( SUM                 )
         CALL DAFUS  ( SUM, ND, NI, DC, IC )
 
         IC(NI-1) = IC(NI-1) + INCR
         IC(NI  ) = IC(NI  ) + INCR
 
         CALL DAFPS ( ND, NI, DC, IC, SUM )
         CALL DAFWS (                 SUM )
 
         CALL DAFFNA ( FOUND )
 
         GO TO 50004
      END IF
 
      CALL CHKOUT ( 'DAFARR' )
      RETURN
      END
