C$Procedure DAFRFR ( DAF, read file record )
 
      SUBROUTINE DAFRFR ( HANDLE, ND, NI, IFNAME, FWARD, BWARD, FREE )
 
C$ Abstract
C
C     Read the contents of the file record of a DAF.
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     FILES
C
C$ Declarations
 
      INTEGER               HANDLE
      INTEGER               ND
      INTEGER               NI
      CHARACTER*(*)         IFNAME
      INTEGER               FWARD
      INTEGER               BWARD
      INTEGER               FREE
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   Handle of a DAF.
C     ND         O   Number of double precision components in summaries.
C     NI         O   Number of integer components in summaries.
C     IFNAME     O   Internal file name.
C     FWARD      O   Forward list pointer.
C     BWARD      O   Backward list pointer.
C     FREE       O   Free address pointer.
C
C$ Detailed_Input
C
C     HANDLE      is the handle assigned to a DAF.
C
C$ Detailed_Output
C
C     ND,
C     NI          are the numbers of double precision and integer
C                 components, respectively, in each array summary in
C                 the specified file.
C
C     IFNAME      is the internal file name stored in the first
C                 (or file) record of the specified file.
C
C     FWARD       is the forward list pointer. This points to the
C                 first summary record in the file. (Records between
C                 the first record and the first summary record are
C                 reserved when the file is created, and are invisible
C                 to DAF routines.)
C
C     BWARD       is the backward list pointer. This points
C                 to the final summary record in the file.
C
C     FREE        is the free address pointer. This contains the
C                 first free address in the file. (That is, the
C                 initial address of the next array to be added
C                 to the file.)
C
C$ Parameters
C
C      None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If the specified file is not a DAF (does not contain the
C        proper internal identifier), the error SPICE(DAFNOIDWORD)
C        is signalled.
C
C     2) If the specified record cannot (for some reason) be read,
C        the error SPICE(DAFFRNOTFOUND) is signalled.
C
C$ Particulars
C
C     The file record of a DAF is the only record that contains
C     any global information about the file. This record is created
C     when the file is created, and is updated only when new arrays
C     are added.
C
C     Like character records, file records are not buffered.
C
C$ Examples
C
C     In the following example, the value of the forward list
C     pointer is examined in order to determine the number of
C     reserved records in the DAF. These records are then read
C     and the contents printed to the screen.
C
C        CALL DAFRFR ( HANDLE, ND, NI, IFNAME, FWARD, BWARD, FREE )
C        CALL DAFHLU ( HANDLE, UNIT )
C
C        DO I = 2, FWARD - 1
C           READ  (UNIT,REC=I) PRIVATE(1:1000)
C           WRITE (*,*)        PRIVATE(1:1000)
C        END DO
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 167.0, "Double Precision Array Files (DAF)
C     Specification and User's Guide"
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.3, 6-OCT-1992 (HAN)
C
C        Corrected a typo in the Brief_I/O section. ND was listed
C        twice as an input, and NI was not listed.
C
C-    SPICELIB Version 1.0.2, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     read daf file record
C
C-&
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Local variables
C
      INTEGER               UNIT
      INTEGER               IOSTAT
 
      CHARACTER*8           DAFWRD
      CHARACTER*60          IFN
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DAFRFR' )
      END IF
 
C
C     What unit?
C
      CALL DAFHLU ( HANDLE, UNIT )
 
C
C     Read all but the internal file name directly. Read that into a
C     temporary string, to be sure of the length.
C
      READ (UNIT,
     .      REC=1,
     .      IOSTAT=IOSTAT) DAFWRD, ND, NI, IFN, FWARD, BWARD, FREE
 
      IF ( IOSTAT .EQ. 0 ) THEN
         IF (       DAFWRD .EQ. 'NAIF/DAF'
     .        .OR.  DAFWRD .EQ. 'NAIF/NIP'  ) THEN
 
            IFNAME = IFN
 
         ELSE
            CALL SETMSG ( 'Missing ''NAIF/DAF'' ID word.' )
            CALL SIGERR ( 'SPICE(DAFNOIDWORD)' )
         END IF
 
      ELSE
         CALL SETMSG ( 'File record not found.  Value of IOSTAT was #' )
         CALL ERRINT ( '#', IOSTAT                                     )
         CALL SIGERR ( 'SPICE(DAFFRNOTFOUND)'                          )
      END IF
 
      CALL CHKOUT ( 'DAFRFR' )
      RETURN
      END
