 
C$Procedure DAFRRR ( DAF, remove reserved records )
 
      SUBROUTINE DAFRRR ( HANDLE, RESV )
 
C$ Abstract
C
C     Remove a specified number of reserved records from a Double
C     Precision Array File (DAF).
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     FILES
C
C$ Declarations
 
      INTEGER               HANDLE
      INTEGER               RESV
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   DAF, opened for writing.
C     RESV       I   Number of records to remove.
C
C$ Detailed_Input
C
C     HANDLE      is the handle associated with a DAF that has been
C                 opened for write access.  Use DAFOPW, for example,
C                 to open an existing file and get its handle.
C
C     RESV        is the number of reserved records to be removed
C                 from the specified file.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If RESV is less than one, the file is not changed.
C
C     2) If RESV is greater than the number of reserved records in the
C        file, all of them are removed.
C
C$ Files
C
C     See argument HANDLE.
C
C$ Particulars
C
C     Normally, the reserved records in an array file are reserved
C     when the file is created. However, it may occasionally become
C     desirable to remove reserved records---when their contents are
C     significantly reduced, for example.
C
C     The records nearest the end of the file are removed. Note
C     that the physical size of the file is not reduced when reserved
C     records are removed.
C
C$ Examples
C
C     In the following code fragment, the call to DAFOPN creates a
C     new array file with 12 reserved records (located in records
C     2-13 of the physical file).  The subsequent call to DAFRRR,
C     decreases the number of reserved records by 7, leaving only
C     the first five original records.
C
C        CALL DAFOPN ( FNAME, ND, NI, IFNAME, 12, HANDLE )
C              .
C              .
C              .
C
C        CALL DAFRRR ( HANDLE, 7 )
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     I.M. Underwood (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 18-JUL-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     remove daf reserved records
C
C-&
 
 
 
 
C
C     SPICELIB functions
C
      LOGICAL               FAILED
      LOGICAL               RETURN
 
C
C     Local parameters
C
C     IFNLEN      is the length of a DAF internal file name.
C
      INTEGER               IFNLEN
      PARAMETER           ( IFNLEN = 60       )
 
C
C     WPR         is the maximum number of double precision
C                 numbers per record.  WPR stands for words
C                 per record.
C
      INTEGER               WPR
      PARAMETER           ( WPR    = 128      )
 
C
C     MAXD,       are the maximum number of double precision
C     MAXI,       numbers, integers, and characters, respectively,
C     MAXC        not including space reserved for control information.
C
      INTEGER               MAXD
      PARAMETER           ( MAXD   = WPR  - 3 )
 
      INTEGER               MAXI
      PARAMETER           ( MAXI   = MAXD * 2 )
 
      INTEGER               MAXC
      PARAMETER           ( MAXC   = MAXD * 8 )
 
C
C     Local variables
C
      CHARACTER*(MAXC)      CREC
      CHARACTER*(IFNLEN)    IFNAME
 
      DOUBLE PRECISION      DC      ( MAXD )
      DOUBLE PRECISION      DREC    ( WPR  )
      DOUBLE PRECISION      SUM     ( MAXD )
 
      INTEGER               BEGBLK
      INTEGER               BWARD
      INTEGER               DECR
      INTEGER               ENDBLK
      INTEGER               FREE
      INTEGER               FWARD
      INTEGER               IC      ( MAXI )
      INTEGER               ND
      INTEGER               NEXT
      INTEGER               NI
      INTEGER               RECNO
      INTEGER               REMOVE
      INTEGER               WORD
 
      LOGICAL               FOUND
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DAFRRR' )
      END IF
 
C
C     Get the contents of the file record.
C
      CALL DAFRFR ( HANDLE, ND, NI, IFNAME, FWARD, BWARD, FREE )
 
C
C     Don't remove more than the current number of reserved records!
C     If there are none, check out.
C
      REMOVE = MIN ( RESV, FWARD - 2 )
 
      IF ( REMOVE .LT. 1 ) THEN
         CALL CHKOUT ( 'DAFRRR' )
         RETURN
      END IF
 
C
C     Okay, here's the plan. We are just going to move records
C     forwards, starting with the first summary record in the file
C     and ending with the last data record.
C
C     After everything has been moved, the initial and final
C     addresses of all the arrays have to be decremented by the
C     same amount: the number of words per record (128) times
C     the number of records removed.
C
      DECR = WPR * REMOVE
 
C
C     Records will be moved in `blocks', where each block contains
C
C        -- a summary record
C
C        -- a name record
C
C        -- one or more data records
C
C     Most blocks lie between one summary record and the next.
C     The final block lies between the final summary record and
C     whatever data record contains the first free address.
C
C     BEGBLK is initially the first summary record location.
C
      BEGBLK = FWARD
 
50001 IF       ( ( BEGBLK .GT. 0 )  .AND.  ( .NOT. FAILED() ) )
     .THEN
 
C
C        Move the summary record first. The location of the next
C        summary record determines the end of this block, and the
C        beginning of the next.
C
C        Be sure to adjust the forward and backward pointers;
C        otherwise, we won't be able to find the summaries again.
C
         RECNO = BEGBLK
         CALL DAFRDR ( HANDLE, RECNO, 1, WPR, DREC, FOUND )
 
         IF ( DREC(1) .GT. 0 ) THEN
            ENDBLK = DREC(1) - 1
            NEXT   = DREC(1)
         ELSE
            CALL DAFARW ( FREE, ENDBLK, WORD )
            NEXT = 0
         END IF
 
         IF ( DREC(1) .GT. 0 ) THEN
            DREC(1) = DREC(1) - REMOVE
         END IF
 
         IF ( DREC(2) .GT. 0 ) THEN
            DREC(2) = DREC(2) - REMOVE
         END IF
 
         CALL DAFWDR ( HANDLE, RECNO - REMOVE, DREC )
 
C
C        Then the name record.
C
         RECNO = BEGBLK + 1
         CALL DAFRCR ( HANDLE, RECNO,          CREC )
         CALL DAFWCR ( HANDLE, RECNO - REMOVE, CREC )
 
C
C        Finally, the data records.
C
         DO 50002
     .      RECNO = BEGBLK + 2, ENDBLK
            CALL DAFRDR ( HANDLE, RECNO,          1, WPR, DREC, FOUND )
            CALL DAFWDR ( HANDLE, RECNO - REMOVE,         DREC        )
50002    CONTINUE
 
C
C        Start the next block, if one exists.
C
         BEGBLK = NEXT
 
         GO TO 50001
      END IF
 
C
C     Rewrite the file record, to reflect the new organization of
C     the file.
C
      FWARD = FWARD - REMOVE
      BWARD = BWARD - REMOVE
      FREE  = FREE  - DECR
 
      CALL DAFWFR ( HANDLE, ND, NI, IFNAME, FWARD, BWARD, FREE )
 
C
C     Get the summary for each array, decrement the addresses (stored
C     in the final two integer components), and replace the summary.
C
      CALL DAFBFS ( HANDLE )
      CALL DAFFNA ( FOUND  )
 
50003 IF       ( FOUND  .AND.  ( .NOT. FAILED() ) )
     .THEN
 
         CALL DAFGS  ( SUM                 )
         CALL DAFUS  ( SUM, ND, NI, DC, IC )
 
         IC(NI-1) = IC(NI-1) - DECR
         IC(NI  ) = IC(NI  ) - DECR
 
         CALL DAFPS ( ND, NI, DC, IC, SUM )
         CALL DAFWS (                 SUM )
 
         CALL DAFFNA ( FOUND )
 
         GO TO 50003
      END IF
 
      CALL CHKOUT ( 'DAFRRR' )
      RETURN
      END
