 
C$Procedure DAFTB ( DAF, text file to binary file )
 
      SUBROUTINE DAFTB ( TXTLUN, BINFIL )
 
C$ Abstract
C
C     Convert the contents of an ASCII DAF encoded text file into an
C     equivalent binary DAF file.
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     CONVERSION
C     FILES
C
C$ Declarations
 
      INTEGER               TXTLUN
      CHARACTER*(*)         BINFIL
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     TXTLUN     I   Logical unit of a previously opened text file.
C     BINFIL     I   Name of a binary DAF file to be created.
C
C$ Detailed_Input
C
C     TXTLUN      The Fortran logical unit number, to which a
C                 previously opened DAF encoded text file has been
C                 attached by the calling program.
C
C                 The file pointer should be positioned ready to read
C                 the DAF file ID word.
C
C     BINFIL      The name of the binary DAF file to be created.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     See arguments TXTLUN, BINFIL.
C
C$ Exceptions
C
C     1)   If the text file cannot be read, the error
C          SPICE(FILEREADFAILED) will be signalled.
C
C     2)   If the DAF ID word found in the file is not 'NAIF/DAF'
C          or 'NAIF/NIP' the error SPICE(DAFIDWORDNOTKNOWN) will
C          be signalled.
C
C     3)   If an error occurs while attempting to decode data in the
C          DAF encoded text file, the error SPICE(BADDAFTEXTFILE) will
C          be signalled.
C
C     4)   If the DAF file cannot be written, a DAF file access routine
C          will signal an error with an appropriate error message.
C
C     5)   The binary DAF file opened by this routine, BINFIL, is only
C          GUARANTEED to be closed upon successful completion of the
C          text to binary conversion process. In the event of an error,
C          the caller of this routine is required to close the binary
C          DAF file BINFIL.
C
C$ Particulars
C
C     This routine provides a mechanism for converting an ASCII DAF
C     encoded text file created by DAFBT, or an equivalent procedure,
C     into an equivalent binary DAF file which may be used with the
C     SPICE system. It is one of a pair of routines for performing text
C     to binary and binary to text DAF file conversions using an
C     encoded ASCII text file format. The inverse of this routine is
C     performed by the routine DAFBT.
C
C     This routine makes NO use of the DAF reserved record area. It
C     can only deal with the data portion of a DAF file in the encoded
C     ASCII text file.
C
C     Upon successful completion, the binary DAF file specified by
C     BINFIL will have been created. The binary DAF file that was
C     created will be closed when this routine exits. The text file
C     will remain open, as it was on entry, and it will be positioned
C     to read the first line after the encoded DAF file data.
C
C$ Examples
C
C     Let
C
C        TXTLUN   be the Fortran logical unit attached to an encoded
C                 ASCII text version of a DAF file which is to be
C                 converted into its binary DAF equivalent.
C
C        BINFIL   be the name of the binary DAF file, which will be
C                 created by the encoded ASCII text to binary DAF
C                 file conversion.
C
C     Then, the following subroutine call would read the encoded
C     ASCII text DAF file attached to the Fortran logical unit
C     TXTLUN, convert its data into binary format, and write that
C     data to the binary DAF file which has been created:
C
C        CALL DAFTB( TXTLUN, BINFIL )
C
C$ Restrictions
C
C     1) This routine assumes that it is positioned ready to read the
C        file ID word from the text DAF file.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     K.R. Gehringer  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 24-JUN-1993 (KRG)
C
C        Modified the description of the DAF encoded text file format
C        appearing before the program code.
C
C-    SPICELIB Version 1.0.0, 02-NOV-1992 (KRG)
C
C-&
 
C$ Index_Entries
C
C     convert encoded text daf to binary daf
C
C-&
 
C$ Revisions
C
C-    SPICELIB Version 1.0.1, 24-JUN-1993 (KRG)
C
C        Modified the description of the DAF encoded text file format
C        appearing before the program code. Changed the line:
C
C           C        < DAF ND value > < DAF NI value >
C
C        to the lines:
C
C           C        < DAF ND value >
C           C        < DAF NI value >
C
C        This change was necessary because the output format for the
C        low level routines which encode and write the data were
C        modified to fix a problem. See the routines WRENCD and WRENCI
C        for details of the modification.
C
C-    SPICELIB Version 1.0.0, 29-OCT-1992 (KRG)
C
C-&
 
C
C     SPICELIB functions
C
      LOGICAL               FAILED
      LOGICAL               RETURN
C
C     Local Parameters
C
      CHARACTER*(*)         BEGARR
      PARAMETER           ( BEGARR = 'BEGIN_ARRAY' )
 
      CHARACTER*(*)         ENDARR
      PARAMETER           ( ENDARR = 'END_ARRAY' )
 
      CHARACTER*(*)         TOTARR
      PARAMETER           ( TOTARR = 'TOTAL_ARRAYS' )
 
      INTEGER               BUFSIZ
      PARAMETER           ( BUFSIZ = 1024 )
 
      INTEGER               LINLEN
      PARAMETER           ( LINLEN = 255 )
 
      INTEGER               ERRLEN
      PARAMETER           ( ERRLEN = 320 )
 
      INTEGER               IDWLEN
      PARAMETER           ( IDWLEN = 8    )
 
      INTEGER               IFNLEN
      PARAMETER           ( IFNLEN = 60   )
 
      INTEGER               CRECL
      PARAMETER           ( CRECL  = 1000 )
 
      INTEGER               RESREC
      PARAMETER           ( RESREC = 0 )
C
C     Local variables
C
      CHARACTER*(IDWLEN)    IDWORD
      CHARACTER*(IFNLEN)    IFNAME
      CHARACTER*(ERRLEN)    ERRMSG
      CHARACTER*(LINLEN)    LINE
      CHARACTER*(CRECL)     NAME
      CHARACTER*(LINLEN)    REST
      CHARACTER*(LINLEN)    WORD
 
      DOUBLE PRECISION      BUFFER( BUFSIZ )
      DOUBLE PRECISION      DSUMRY( 125    )
      DOUBLE PRECISION      SUMMRY( 125    )
 
      INTEGER               ARRCNT
      INTEGER               BARR
      INTEGER               BCNT
      INTEGER               BINHDL
      INTEGER               DTACNT
      INTEGER               EARR
      INTEGER               ECNT
      INTEGER               ERRPTR
      INTEGER               IOSTAT
      INTEGER               ISUMRY( 250    )
      INTEGER               LFTOVR
      INTEGER               ND
      INTEGER               NI
      INTEGER               NUMARR
      INTEGER               NUMDTA
      INTEGER               NUMLFT
      INTEGER               SNMLEN
 
      LOGICAL               INARR
      LOGICAL               MORE
 
 
C
C     Standard/ SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DAFTB' )
      END IF
C
C     A brief description of the DAF encoded text file format and its
C     intended uses follows. This description is intended to provide a
C     simple ``picture'' of the DAF encoded text file format to aid in
C     the understanding of this routine. This description is NOT
C     intended to be a detailed specification of the file format.
C
C     A DAF encoded text file contains all of the data from a binary
C     DAF file, except for the reserved record area, in an encoded
C     ASCII text format, and also contains some bookkeeping
C     information for maintaining the integrity of the data. The DAF
C     encoded text file format allows the full precision of both
C     integer and floating point numeric data to be maintained in a
C     portable fashion. The DAF encoded text file format is intended
C     to provide a reliable and accurate means for porting data among
C     multiple computer systems and for the archival storage of data.
C
C     A DAF encoded text file is not intended to be used directly to
C     provide data to a program, the equivalent binary DAF file is
C     to be used for this purpose. In no way should any program, other
C     than a DAF binary <-> text conversion program, rely on the DAF
C     encoded text file format.
C
C     To correctly understand the DAF encoded text file description
C     the reader should be familiar with the DAF file architecture.
C     Items enclosed in angle brackets, '<' and '>', are used to
C     represent the data which is to be placed at that position in
C     the file. The bookkeeping information is represented exactly
C     as it would appear in a DAF encoded text file.
C
C     Let
C
C       BOF    denote the beginning of the file
C       EOF    denote the end of the file
C
C    and
C
C       n      denote the total number of arrays in a DAF file
C       NA(i)  denote the number of double precision numbers in array i
C       m(i)   denote the number of blocks of encoded data for array i
C       N(i,j) denote the number of encoded double precision numbers
C              in block j of array i
C
C     and
C
C              m(i)
C             -----
C             \
C              >   N(i,k) = NA(i),   i = 1, ..., n.
C             /
C             -----
C              k=1
C
C     A DAF encoded text file has the following format:
C
C        <BOF>
C        < Information line >
C        < DAF file ID word >
C        < DAF ND value >
C        < DAF NI value >
C        < DAF internal file name >
C        BEGIN_ARRAY 1 NA(1)
C        < Name for array 1 >
C        < ND double precision summary values >
C        < NI-2 integer summary values >
C        N(1,1)
C        < N(1,1) Encoded double precision numbers >
C        N(1,2)
C        < N(1,2) Encoded double precision numbers >
C                          .
C                          .
C                          .
C        N(1,m(1))
C        < N(1,m(1)) Encoded double precision numbers >
C        END_ARRAY 1 NA(1)
C        BEGIN_ARRAY 2 NA(2)
C        < Name for array 2 >
C        < ND double precision summary values >
C        < NI-2 integer summary values >
C        N(2,1)
C        < N(2,1) Encoded double precision numbers >
C        N(2,2)
C        < N(2,2) Encoded double precision numbers >
C                          .
C                          .
C                          .
C        N(2,m(2))
C        < N(2,m(2)) Encoded double precision numbers >
C        END_ARRAY 2 NA(2)
C                          .
C                          .
C                          .
C        BEGIN_ARRAY n NA(n)
C        < Name for array n >
C        < ND double precision summary values >
C        < NI-2 integer summary values >
C        N(n,1)
C        < N(n,1) Encoded double precision numbers >
C        N(n,2)
C        < N(n,2) Encoded double precision numbers >
C                          .
C                          .
C                          .
C        N(n,m(n))
C        < N(n,m(n)) Encoded double precision numbers >
C        END_ARRAY n NA(n)
C        TOTAL_ARRAYS  n
C        <EOF>
C
C     We begin by reading the DAF file ID word from the text file. We
C     should have been positioned ready to read this. If an error
C     occurs, set an appropriate error message and signal the error.
C
      IDWORD = ' '
      READ (TXTLUN,*,IOSTAT=IOSTAT) IDWORD
 
      IF ( IOSTAT .NE. 0 ) THEN
 
         CALL SETMSG ( 'Error reading the file ID word from the'  //
     .                 ' text DAF file: #. IOSTAT = #.'            )
         CALL ERRFNM ( '#', TXTLUN                                 )
         CALL ERRINT ( '#', IOSTAT                                 )
         CALL SIGERR ( 'SPICE(FILEREADFAILED)'                     )
         CALL CHKOUT ( 'DAFTB'                                     )
         RETURN
 
      END IF
C
C     Check the DAF ID word. When checking the ID word, allow either
C     'NAIF/DAF' or 'NAIF/NIP'.
C
      IF ( ( IDWORD .NE. 'NAIF/DAF' ) .AND.
     .     ( IDWORD .NE. 'NAIF/NIP' )       ) THEN
 
         CALL SETMSG ( 'File ID word ''#'' was not recognized in'  //
     .                 ' DAF text file: #'                          )
         CALL ERRCH  ( '#', IDWORD                                  )
         CALL ERRFNM ( '#', TXTLUN                                  )
         CALL SIGERR ( 'SPICE(DAFIDWORDNOTKNOWN)'                   )
         CALL CHKOUT ( 'DAFTB'                                      )
         RETURN
 
      END IF
C
C     Before we can open the binary DAF, we need to get the summary
C     format and the internal file name from the text file. We begin
C     doing this here.
C
C     Read in the ND and NI values for the DAF file.
C
      CALL RDENCI( TXTLUN, 2, ISUMRY )
 
      IF ( FAILED() ) THEN
 
         CALL CHKOUT ( 'DAFTB' )
         RETURN
 
      END IF
 
      ND = ISUMRY(1)
      NI = ISUMRY(2)
C
C     Read the internal filename for the DAF file.
C
      READ (TXTLUN,*,IOSTAT=IOSTAT) IFNAME
 
      IF ( IOSTAT .NE. 0 ) THEN
 
         CALL SETMSG ( 'Error reading the internal filename from' //
     .                 ' the text DAF file: #. IOSTAT = #.'        )
         CALL ERRFNM ( '#', TXTLUN                                 )
         CALL ERRINT ( '#', IOSTAT                                 )
         CALL SIGERR ( 'SPICE(FILEREADFAILED)'                     )
         CALL CHKOUT ( 'DAFTB'                                     )
         RETURN
 
      END IF
C
C     Open a new binary DAF file.
C
      CALL DAFOPN ( BINFIL, ND, NI, IFNAME, RESREC, BINHDL )
 
      IF ( FAILED() ) THEN
 
         CALL CHKOUT ( 'DAFTB' )
         RETURN
 
      END IF
C
C     Calculate the length of the segment names.
C
      SNMLEN = 8 * ( ND + ( NI + 1 ) / 2 )
C
C     Initialize a few things: the array counter, and the data counter,
C     MORE data flag, and the INARR flag.
C
      ARRCNT = 0
      DTACNT = 0
C
C     We currently have more to process.
C
      MORE  = .TRUE.
C
C     We are currently not processing an array.
C
      INARR = .FALSE.
C
C     Begin converting the DAF encoded text file into a binary DAF file
C     here.
C
50001 IF       ( MORE )
     .THEN
 
         READ (TXTLUN,FMT='(A)',IOSTAT=IOSTAT) LINE
 
         IF ( IOSTAT .NE. 0 ) THEN
 
            CALL SETMSG ( 'Error reading from the text DAF'  //
     .                    ' file #. IOSTAT = #.'              )
            CALL ERRFNM ( '#', TXTLUN                         )
            CALL ERRINT ( '#', IOSTAT                         )
            CALL SIGERR ( 'SPICE(FILEREADFAILED)'             )
            CALL CHKOUT ( 'DAFTB'                             )
            RETURN
 
         END IF
C
C        At this point, we should be beginning an array, ending an
C        array, or scanning for the total number of arrays. So look
C        for the appropriate keyword.
C
         CALL NEXTWD( LINE, WORD, REST )
 
         IF ( WORD .EQ. BEGARR ) THEN
C
C           Get the array number.
C
            CALL NEXTWD( REST, WORD, REST )
            CALL NPARSI( WORD, BARR, ERRMSG, ERRPTR )
 
            IF ( ERRMSG .NE. ' ' ) THEN
 
               CALL SETMSG ( 'Begin array error, could not parse'  //
     .                       ' array number. Error: # File: #'      )
               CALL ERRCH  ( '#', ERRMSG                            )
               CALL ERRFNM ( '#', TXTLUN                            )
               CALL SIGERR ( 'SPICE(BADDAFTEXTFILE)'                )
               CALL CHKOUT ( 'DAFTB'                                )
               RETURN
 
            END IF
C
C           Parse the count of double precision numbers in the array.
C
            CALL NEXTWD( REST, WORD, REST )
            CALL NPARSI( WORD, BCNT, ERRMSG, ERRPTR )
 
            IF ( ERRMSG .NE. ' ' ) THEN
 
               CALL SETMSG ( 'Begin array error, could not parse' //
     .                       ' the data count for array: #.'      //
     .                       ' Error: # File: #'                   )
               CALL ERRINT ( '#', BARR                             )
               CALL ERRCH  ( '#', ERRMSG                           )
               CALL ERRFNM ( '#', TXTLUN                           )
               CALL SIGERR ( 'SPICE(BADDAFTEXTFILE)'               )
               CALL CHKOUT ( 'DAFTB'                               )
               RETURN
 
            END IF
C
C           If we got to here, we are inside an array, so set the in
C           array flag, INARR, to .TRUE. and increment the array
C           counter.
C
            INARR  = .TRUE.
            ARRCNT = ARRCNT + 1
 
         ELSE IF ( WORD .EQ. ENDARR ) THEN
C
C           Get the array number.
C
            CALL NEXTWD( REST, WORD, REST )
            CALL NPARSI( WORD, EARR, ERRMSG, ERRPTR )
 
            IF ( ERRMSG .NE. ' ' ) THEN
 
               CALL SETMSG ( 'End array error, could not parse'  //
     .                       ' array number. Error: # File: #'    )
               CALL ERRCH  ( '#', ERRMSG                          )
               CALL ERRFNM ( '#', TXTLUN                          )
               CALL SIGERR ( 'SPICE(BADDAFTEXTFILE)'              )
               CALL CHKOUT ( 'DAFTB'                              )
               RETURN
 
            END IF
C
C           Parse the count of double precision numbers in the array.
C
            CALL NEXTWD( REST, WORD, REST )
            CALL NPARSI( WORD, ECNT, ERRMSG, ERRPTR )
 
            IF ( ERRMSG .NE. ' ' ) THEN
 
               CALL SETMSG ( 'End array error, could not parse' //
     .                       ' the data count for array: #.'    //
     .                       ' Error: # File: #'                 )
               CALL ERRINT ( '#', EARR                           )
               CALL ERRCH  ( '#', ERRMSG                         )
               CALL ERRFNM ( '#', TXTLUN                         )
               CALL SIGERR ( 'SPICE(BADDAFTEXTFILE)'             )
               CALL CHKOUT ( 'DAFTB'                             )
               RETURN
 
            END IF
C
C           Check to see if the beginning and ending array numbers
C           match. If not, signal an appropriate error.
C
            IF ( EARR .NE. BARR ) THEN
 
               CALL SETMSG ( 'Data array number mismatch:'         //
     .                       ' Beginning number: #; Ending'        //
     .                       ' number: #. File: #'                  )
               CALL ERRINT ( '#', BARR                              )
               CALL ERRINT ( '#', EARR                              )
               CALL ERRFNM ( '#', TXTLUN                            )
               CALL SIGERR ( 'SPICE(BADDAFTEXTFILE)'                )
               CALL CHKOUT ( 'DAFTB'                                )
               RETURN
 
            END IF
C
C           Check to see if the beginning and ending array data counts
C           match. If not, signal an appropriate error.
C
            IF ( ECNT .NE. BCNT ) THEN
 
               CALL SETMSG ( 'Data array count mismatch:'      //
     .                       ' Beginning count: #;'            //
     .                       ' Ending count: #. File: #'        )
               CALL ERRINT ( '#', BCNT                          )
               CALL ERRINT ( '#', ECNT                          )
               CALL ERRFNM ( '#', TXTLUN                        )
               CALL SIGERR ( 'SPICE(BADDAFTEXTFILE)'            )
               CALL CHKOUT ( 'DAFTB'                            )
               RETURN
 
            END IF
C
C           If we got to here, we have successfully ended the
C           processing of an array, so set the in array flag, INARR,
C           to  .FALSE..
C
            INARR = .FALSE.
 
         ELSE IF ( WORD .EQ. TOTARR ) THEN
C
C           We have the total arrays keyword to parse, so get
C           the total number of arrays processed.
C
            CALL NEXTWD( REST, WORD, REST )
            CALL NPARSI( WORD, NUMARR, ERRMSG, ERRPTR )
 
            IF ( ERRMSG .NE. ' ' ) THEN
 
               CALL SETMSG ( 'Array count error, could not parse' //
     .                       ' the total number of arrays: #.'    //
     .                       ' File: #'                            )
               CALL ERRCH  ( '#', ERRMSG                           )
               CALL ERRFNM ( '#', TXTLUN                           )
               CALL SIGERR ( 'SPICE(BADDAFTEXTFILE)'               )
               CALL CHKOUT ( 'DAFTB'                               )
               RETURN
 
            END IF
 
            IF ( ARRCNT .NE. NUMARR ) THEN
 
                  CALL SETMSG ( 'The number of data arrays'          //
     .                          ' processed (#) was not equal to'    //
     .                          ' the number of data arrays placed'  //
     .                          ' in the DAF text file (#). File: #'  )
                  CALL ERRINT ( '#', ARRCNT                           )
                  CALL ERRINT ( '#', NUMARR                           )
                  CALL ERRFNM ( '#', TXTLUN                           )
                  CALL SIGERR ( 'SPICE(BADDAFTEXTFILE)'               )
                  CALL CHKOUT ( 'DAFTB'                               )
                  RETURN
 
            END IF
C
C           If we got to here, we have successfully processed the
C           entire DAF data portion of the text file, so there is
C           no more data.
C
            MORE = .FALSE.
 
         ELSE
 
            CALL SETMSG ( 'Unknown keyword ''#'' encountered'      //
     .                    ' while processing the DAF text file #.'  )
            CALL ERRCH  ( '#', WORD                                 )
            CALL ERRFNM ( '#', TXTLUN                               )
            CALL SIGERR ( 'SPICE(BADDAFTEXTFILE)'                   )
            CALL CHKOUT ( 'DAFTB'                                   )
            RETURN
 
         END IF
C
C        If we have begun an array, then process it. Otherwise, we
C        have either ended an array or ended the file.
C
         IF ( INARR ) THEN
 
            DTACNT = 0
 
            READ (TXTLUN,*,IOSTAT=IOSTAT) NAME( :SNMLEN)
 
            IF ( IOSTAT .NE. 0 ) THEN
 
               CALL SETMSG ( 'Error reading the array name from the' //
     .                       ' text DAF file #. IOSTAT = #.'          )
               CALL ERRFNM ( '#', TXTLUN                              )
               CALL ERRINT ( '#', IOSTAT                              )
               CALL SIGERR ( 'SPICE(FILEREADFAILED)'                  )
               CALL CHKOUT ( 'DAFTB'                                  )
               RETURN
 
            END IF
C
C           Read in the double precision part of the summary.
C
            CALL RDENCD ( TXTLUN, ND, DSUMRY )
 
            IF ( FAILED() ) THEN
 
               CALL CHKOUT ( 'DAFTB' )
               RETURN
 
            END IF
C
C           Read in the integer part of the summary. The beginning and
C           ending addresses, ISUMRY(NI-1) and ISUMRY(NI), for the
C           array are not known currently. They will be filled in when
C           the array is actually written to the DAF file.
C
            CALL RDENCI( TXTLUN, NI-2, ISUMRY )
 
            IF ( FAILED() ) THEN
 
               CALL CHKOUT ( 'DAFTB' )
               RETURN
 
            END IF
C
C           Pack the summary information into the DAF array summary.
C
            CALL DAFPS  ( ND, NI, DSUMRY, ISUMRY, SUMMRY )
C
C           Begin a new array in the binary DAF file.
C
            CALL DAFBNA ( BINHDL, SUMMRY, NAME( :SNMLEN) )
 
            IF ( FAILED() ) THEN
 
               CALL CHKOUT ( 'DAFTB' )
               RETURN
 
            END IF
C
C           Read and decode the data in the current DAF array.
C
C           First set the count of numbers yet to be decoded and placed
C           in the binary DAF file.
C
            NUMLFT = BCNT
 
50002       IF       ( NUMLFT .GT. 0 )
     .      THEN
C
C              First, read in the count of encoded numbers in the
C              current data block.
C
               READ (TXTLUN,*,IOSTAT=IOSTAT) NUMDTA
 
               IF ( IOSTAT .NE. 0 ) THEN
 
                  CALL SETMSG ( 'Error reading array data from the' //
     .                          ' text DAF file #. IOSTAT = #.'      )
                  CALL ERRFNM ( '#', TXTLUN                          )
                  CALL ERRINT ( '#', IOSTAT                          )
                  CALL SIGERR ( 'SPICE(FILEREADFAILED)'              )
                  CALL CHKOUT ( 'DAFTB'                              )
                  RETURN
 
               END IF
C
C              Now read and decode the data in the current data block,
C              placing the data in the current array in the binary DAF
C              file.
C
               LFTOVR = NUMDTA
 
50003          IF       ( LFTOVR .GT. 0 )
     .         THEN
 
                  IF ( LFTOVR .GE. BUFSIZ ) THEN
 
                     NUMDTA = BUFSIZ
 
                  ELSE
 
                     NUMDTA = LFTOVR
 
                  END IF
C
C                 Read and decode a buffer of encoded double precision
C                 data from the text file.
C
                  CALL RDENCD ( TXTLUN, NUMDTA, BUFFER )
 
                  IF ( FAILED() ) THEN
 
                     CALL CHKOUT ( 'DAFTB' )
                     RETURN
 
                  END IF
C
C                 Write the double precision data to the current array
C                 in the binary DAF file.
C
                  CALL DAFADA ( BUFFER, NUMDTA )
 
                  IF ( FAILED() ) THEN
 
                     CALL CHKOUT ( 'DAFTB' )
                     RETURN
 
                  END IF
C
C                 Decrement the counters for the amount of data
C                 remaining to be moved from the current data block,
C                 LFTOVR, and the current array, NUMLFT.
C
                  LFTOVR = LFTOVR - NUMDTA
                  NUMLFT = NUMLFT - NUMDTA
C
C                 Increment the counter for the amount of data that
C                 has been successfully moved into the current array
C                 in the binary DAF file.
C
                  DTACNT = DTACNT + NUMDTA
 
                  GO TO 50003
               END IF
C
C              At this point, we have either finished reading in the
C              entire array, or we have just completed reading the
C              current encoded block of data for the current array
C              from the text file.
C
               GO TO 50002
            END IF
C
C           If we got to here, we have successfully written an array
C           to the binary file, so we need to end it.
C
            CALL DAFENA
 
            IF ( FAILED() ) THEN
 
               CALL CHKOUT ( 'DAFTB' )
               RETURN
 
            END IF
 
         END IF
 
         GO TO 50001
      END IF
C
C     Close only the binary file.
C
      CALL DAFCLS ( BINHDL )
 
      CALL CHKOUT ( 'DAFTB' )
      RETURN
      END
