C$Procedure      ET2UTC ( Ephemeris Time to UTC )
 
      SUBROUTINE ET2UTC ( ET, FORMAT, PREC, UTCSTR )
 
C$ Abstract
C
C      Convert an input time from ephemeris seconds past J2000
C      to Calendar, Day-of-Year, or Julian Date format, UTC.
C
C$ Required_Reading
C
C      TIME
C
C$ Keywords
C
C      TIME
C
C$ Declarations
 
      DOUBLE PRECISION ET
      CHARACTER*1      FORMAT
      INTEGER          PREC
      CHARACTER*(*)    UTCSTR
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      ET         I   Input epoch, ephemeris seconds past J2000.
C      FORMAT     I   Format of output epoch.
C      PREC       I   Digits of precision in fractional seconds or days.
C      UTCSTR     O   Output time string, UTC.
C
C$ Detailed_Input
C
C      ET          is the input epoch, ephemeris seconds past J2000.
C
C      FORMAT      is the format of the output time string. It may be
C                  any of the following:
C
C                        'C'      Calendar format, UTC.
C                        'D'      Day-of-Year format, UTC.
C                        'J'      Julian Date format, UTC.
C
C      PREC        is the number of digits of precision to which
C                  fractional seconds (for Calendar and Day-of-Year
C                  formats) or days (for Julian Date format) are to
C                  be computed. If PREC is zero, no decimal point is
C                  appended to the output string. PREC may not be
C                  less than zero.
C
C$ Detailed_Output
C
C      UTCSTR      is the output time string equivalent to the input
C                  epoch, in the specified format, as shown below.
C
C                        'C'      '1986 APR 12 16:31:09.814'
C                        'D'      '1986-102 // 16:31:12.814'
C                        'J'      'JD 2446533.18834276'
C
C                  If an error occurs, UTCSTR is not changed.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C      If the format for the output string is not recognized, the
C      error SPICE(INVALIDTIMEFORMAT) is signalled.
C
C$ Files
C
C      None.
C
C$ Particulars
C
C      The value of Delta ET corresponding to the input epoch is
C      determined, and subtracted from the input epoch to give UTC
C      seconds past J2000, and the result converted to the specified
C      format.
C
C$ Examples
C
C      Let the value of ET be -527644192.5403653 ephemeris seconds
C      past J2000. Assuming that the nominal values in the kernel pool
C      have not been altered, the following calls
C
C            CALL ET2UTC ( ET, 'C', 0, UTCSTR )
C            CALL ET2UTC ( ET, 'C', 3, UTCSTR )
C            CALL ET2UTC ( ET, 'D', 5, UTCSTR )
C            CALL ET2UTC ( ET, 'J', 7, UTCSTR )
C
C      produce the following output strings
C
C            '1983 APR 13 12:09:14'
C            '1983 APR 13 12:09:14.274'
C            '1983-103 // 12:09:14.27400'
C            'JD 2445438.0064152'
C
C      respectively.
C
C$ Restrictions
C
C      The conversion between ET and UTC depends on the values in the
C      kernel pool. The kernel pool should be loaded prior to calling
C      this subroutine.
C
C$ Literature_References
C
C      Jesperson and Fitz-Randolph, From Sundials to Atomic Clocks,
C      Dover Publications, New York, 1977.
C
C$ Author_and_Institution
C
C      W.M. Owen       (JPL)
C      I.M. Underwood  (JPL)
C
C$ Version
C
C-      SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C          Comment section for permuted index source lines was added
C          following the header.
C
C-      SPICELIB Version 2.0.0, 21-MAR-1991 (NJB) (JML)
C
C          Two bugs involving rounding errors were corrected.  One of
C          the bugs caused conversion errors of magnitude as large as
C          1 second.  See $Revisions for details.
C
C
C-      SPICELIB Version 1.0.0, 31-JAN-1990 (WMO) (IMU)
C
C-&
 
C$ Index_Entries
C
C     ephemeris time to utc
C
C-&
 
 
 
 
C$ Revisions
C
C-      SPICELIB Version 2.0.0, 21-MAR-1991 (NJB) (JML)
C
C
C       1) In the previous version of this routine, the algorithm
C          that was used permitted inconsistent conversion of the
C          integer and fractional parts of the UTC value corresponding
C          to the input ET value.
C
C          In the case where rounding the double precision UTC time
C          corresponding to the input ET value to PREC decimal places
C          resulted in a carry (to the integer portion of the UTC
C          value), the integer portion of the UTC value was treated
C          correctly, but the fraction was not always rounded correctly.
C          The specific case where the problem occurred was when the
C          input ET value mapped to a UTC time having a fractional
C          part that rounded up to 1.0 when rounded PREC decimal places,
C          but that did not round up to 1.0 when rounded to the nearest
C          PREC+1 decimal places.  The set of such fractions can be
C          represented as
C
C             { 1 - EPSILON :      EPSILON  <  5 * ( 10 ** -(PREC+1) )
C                                           -
C                             and
C
C                                  EPSILON  >  5 * ( 10 ** -(PREC+2) )
C
C                                                                     }
C
C          For example, if the input ET mapped to the UTC time
C
C             2 JAN 1991 00:34:12.99994,
C
C          then a call to this routine with PREC set to 3 would result
C          in the output
C
C             2 JAN 1991 00:34:13.999
C
C          instead of the correct value
C
C             2 JAN 1991 00:34:13.000
C
C          On the other hand, if the input ET mapped to the UTC time
C
C             2 JAN 1991 00:34:12.99996,
C
C          then a call to this routine with PREC set to 3 would result
C          in the correct output.
C
C
C          This error was apparently difficult to generate:  it has
C          never been reported by any SPICELIB users, and was eventually
C          discovered by NAIF staff.
C
C
C
C       2) The second bug is somewhat less severe, as far as the
C          magnitude of the error is concerned.  However, it's easier
C          to generate this error.  Namely, in some cases, the
C          fractional part of the input ET value is rounded to PREC
C          SIGNIFICANT DIGITS, rather than to PREC decimal places.
C          The effect of this is that the fraction is occasionally
C          truncated rather than rounded.  For example, the ET value
C          equivalent to the UTC string
C
C             1991 JAN 2 00:34:12.0009
C
C          would be converted to
C
C             1991 JAN 2 00:34:12.000
C
C          instead of the correct value
C
C             1991 JAN 2 00:34:12.001
C
C          when the input argument PREC was set equal to 3.
C
C
C
C          The modifications made to solve these problems are as
C          follows:
C
C             1)  The input ET value, after conversion to `UTC seconds
C                 past 2000', is broken up into the sum of a whole
C                 number of seconds and a non-negative, fractional
C                 number of seconds.  The fact that the fractional
C                 part is non-negative simplifies the conversion of the
C                 fraction.
C
C             2)  The fraction is rounded to PREC decimal places---
C                 that is, to the nearest integer multiple of
C                 10**(-PREC).  If the rounding results in a carry,
C                 the whole number portion of the time value is
C                 incremented by 1 second.  After this step, the
C                 whole number of seconds correctly accounts for
C                 any necessary rounding of the fraction.
C
C             3)  The whole number portion of the time value is passed
C                 through the inverse Muller-Wimberly algorithm to
C                 obtain years, months, days, hours, minutes, and
C                 whole seconds.  A small fraction is added to the
C                 whole number to prevent round-off error from occurring
C                 when divisions are performed.
C
C             4)  The fraction is converted to a string using the
C                 SPICELIB routine DPSTRF.  To ensure that DPSTRF
C                 produces an output string containing PREC decimal
C                 places, an integer is added to the fraction value
C                 before supplying it to DPSTRF.  This integer
C                 `anchors' the first significant digit of the input
C                 value in the units place.
C
C-&
 
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
      DOUBLE PRECISION      J2000
      DOUBLE PRECISION      SPD
      LOGICAL               FAILED
 
C
C     Local variables
C
 
      CHARACTER*1      FMT
      CHARACTER*80     STR
      CHARACTER*80     FRACT
 
      DOUBLE PRECISION UTCSEC
      DOUBLE PRECISION DELTA
      DOUBLE PRECISION DP2000
      DOUBLE PRECISION FRCSEC
      DOUBLE PRECISION SCALE
      DOUBLE PRECISION SECOND
      DOUBLE PRECISION WHLSEC
 
      INTEGER          DAY
      INTEGER          MONTH
      INTEGER          YEAR
      INTEGER          HOUR
      INTEGER          MINUTE
      INTEGER          DOY
 
      INTEGER          I
 
      CHARACTER*3      MTHNAM   ( 12 )
      INTEGER          DAYS     ( 12 )
 
C
C     Save everything between calls
C
      SAVE
 
C
C     Initial values
C
      DATA         MTHNAM       / 'JAN', 'FEB', 'MAR', 'APR',
     .                            'MAY', 'JUN', 'JUL', 'AUG',
     .                            'SEP', 'OCT', 'NOV', 'DEC'  /
 
      DATA         DAYS         /    0,    31,    59,    90,
     .                             120,   151,   181,   212,
     .                             243,   273,   304,   334   /
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'ET2UTC' )
      END IF
 
C
C     Convert FORMAT to uppercase for ease of comparison. Make sure it's
C     one of the recognized formats.
C
      CALL NUCASE ( FORMAT, FMT )
 
      IF ( INDEX ( 'CDJ', FMT ) .EQ. 0 ) THEN
 
         CALL SETMSG ( 'ET2UTC: Format specification for output time '//
     .                 'string not recognized.  Valid specifications '//
     .                 'are:  ''C'', ''J'', or ''D''.  The supplied  '//
     .                 'value was  '// FORMAT
     .               )
 
 
         CALL SIGERR ( 'SPICE(INVALIDTIMEFORMAT)' )
         CALL CHKOUT ( 'ET2UTC' )
         RETURN
 
      END IF
 
C
C     Convert to UTC seconds past J2000.
C
      CALL DELTET ( ET, 'ET', DELTA )
 
      IF ( .NOT. FAILED () ) THEN
         UTCSEC = ET - DELTA
      ELSE
         CALL CHKOUT ( 'ET2UTC' )
         RETURN
      END IF
 
 
 
C
C     If the output is Julian Date, we're ready to go. Remember that
C     the day part of Julian Date already has seven digits built in.
C
      IF ( FMT .EQ. 'J' ) THEN
 
 
         CALL DPSTRF ( J2000()+(UTCSEC/SPD()), PREC+7, 'F', STR )
         CALL PREFIX ( 'JD', 0,                             STR )
 
 
C
C     It's one of the calendar formats.
C
      ELSE
 
C
C        We're going to break up UTCSEC into an integer and a
C        fractional part.  The integer will be the greatest
C        integer less than or equal to UTCSEC, and the fraction
C        will be the difference between UTCSEC and the integer
C        part.  The fraction will always be in the interval
C
C           [0, 1)
C
C        After making this decomposition, we'll adjust the integer
C        and fraction to take rounding into account.  The result
C        of the adjustment is that the fraction will be an integer
C        number of time units of length 10**(-PREC) seconds, where
C        the integer is in the range [0, (10**PREC)-1].  If the
C        fraction rounds up to 1, the fraction will be set to zero,
C        and the whole number portion of UTCSEC will be incremented.
C
C        Since the integers involved may be too large to represent
C        using the INTEGER data type, we'll represent them with
C        double precision numbers.  We'll use the intrinsic ANINT
C        function to keep round-off from creeping into these d.p.
C        numbers representing integers.
C
 
C
C        Find the greatest integer less than or equal to UTCSEC.
C        Recall that INT truncates toward the origin.  If UTCSEC
C        is negative and is not already an integer, the result we
C        want is one less than AINT(UTCSEC).
C
C
         WHLSEC = AINT ( UTCSEC )
 
         IF (       ( UTCSEC .LT. 0.D0   )
     .        .AND. ( UTCSEC .NE. WHLSEC )  )  THEN
 
            WHLSEC = WHLSEC - 1.D0
 
         END IF
 
C
C        The fractional part of UTCSEC must be rounded to the
C        nearest multiple of 10**(-PREC).  Fractions that are
C        equidistant from two multiples of 10**(-PREC) are
C        rounded up.
C
C        To accomplish the rounding, we scale the fraction by
C        10**PREC.
C
C
         SCALE  = ANINT (  10.D0 ** PREC  )
 
         FRCSEC = ANINT (  SCALE * ( UTCSEC - WHLSEC )  )
 
C
C        If a carry occurred, the fraction becomes zero, and
C        we must increment WHLSEC.
C
         IF (  FRCSEC  .EQ.  SCALE  ) THEN
 
            WHLSEC = WHLSEC + 1.0D0
            FRCSEC = 0.D0
 
         END IF
 
C
C        Now we can un-scale FRCSEC.  It will be, within a tiny
C        round-off error introduced by scaling and unscaling, a
C        multiple of 10**(-PREC) less than 1.  Therefore,
C        FRCSEC won't cause a carry when it is processed by DPSTRF
C        later on.
C
         FRCSEC = FRCSEC / SCALE
 
C
C        Now, the whole number of seconds will `go a progress
C        through the guts of' the inverse Muller-Wimberly formula.
C        This whole number already takes into account any rounding
C        the fraction may have undergone.  Years, months, days,
C        hours, minutes, and (an integer number of) seconds will
C        be extracted from WHLSEC.
C
C        FRCSEC will be converted to a string containing PREC digits.
C        This will be done later on when the output string is
C        assembled.
C
 
 
C
C        Convert WHLSEC to days past 2000.  `Days past 2000' are
C        evidently measured from midnight, not noon.
C
         DP2000 = WHLSEC / SPD()    +   0.5D0
 
C
C        Add a smidgen to DP2000 just to protect against round-off
C        problems.
C
         DP2000 = DP2000   +   0.1D0 / SPD()
 
C
C        Get the year, month, and day. This somewhat convoluted code
C        is the inverse of the Muller-Wimberly formula.
C        Make sure that the integer DAY
C        corresponds to the preceding midnight even for times before
C        the J2000 origin.  Also note that 1461 = 4 years (4*365 + 1)
C        and 146097 = 4 Gregorian centuries (400*365 + 97).
C        The constant 730426 is the number of days from March 1, 1 BC
C        (in the proleptic Gregorian calendar; everything is Gregorian)
C        to J2000; it moves the time origin to then, which
C        makes the problem more tractable.  In particular, it
C        enables the elegant (5*DAY-3)/153 expression, which gives
C        months of 31,30,31,30,31, 31,30,31,30,31, 31,30,... days.
C        (This series is in fact the lengths of the first 11 months if
C        one starts with March.)  Finally, MONTH is changed to start
C        at January; months 11 and 12 correspond to Jan or Feb of the
C        following year.
C
         IF ( DP2000 .GE. 0.D0  .OR.  DP2000 .EQ. DINT ( DP2000) ) THEN
            DAY = DP2000
         ELSE
            DAY = DP2000 - 1.D0
         END IF
 
         SECOND   = SPD()*(DP2000-DBLE(DAY))
         I        = DAY + 730426
         YEAR     = (4*I-1)/146097
         DAY      = ((4*I-1)-(146097*YEAR)) / 4
         I        = (4*DAY+3) / 1461
         DAY      = (((4*DAY+3)-(1461*I))+4) / 4
         MONTH    = (5*DAY-3) / 153
         DAY      = (((5*DAY-3)-(153*MONTH))+5) / 5
         YEAR     = 100*YEAR + I
 
         IF ( MONTH .LT. 10 ) THEN
            MONTH = MONTH + 3
 
         ELSE
            MONTH = MONTH - 9
            YEAR  = YEAR + 1
         END IF
 
C
C        Obtain hours, minutes, and seconds. (This is somewhat
C        more straightforward.)
C
         HOUR   = SECOND / 3600.D0
         SECOND = SECOND - 3600.D0*DBLE(HOUR)
         MINUTE = SECOND / 60.D0
         SECOND = SECOND - 60.D0*DBLE(MINUTE)
 
C
C        Time to make a decision about the first 11 characters of
 
 
C        the output string. If the output format is Calendar, convert
C        YEAR, MONTH, and DAY to character format.
C
         IF ( FMT .EQ. 'C' ) THEN
 
            STR = ' '
            WRITE (STR(1:4),FMT='(I4)') YEAR
 
            STR(6:8) = MTHNAM(MONTH)
 
            WRITE (STR(10:11),FMT='(I2.2)') DAY
 
C
C        Otherwise, convert MONTH and DAY to Day-of-Year.
C
         ELSE
 
C
C           In leap years, add an extra day after February.
C
            IF ( MONTH .GT. 2 ) THEN
               IF (      ( MOD (YEAR, 400) .EQ. 0 )
     .              .OR. ( MOD (YEAR,   4) .EQ. 0    .AND.
     .                     MOD (YEAR, 100) .NE. 0          ) ) THEN
                 DAY = DAY + 1
               END IF
            END IF
 
            DOY = DAYS(MONTH) + DAY
 
            STR = '    -    //'
 
            WRITE (STR(1:4),FMT='(I4)')   YEAR
            WRITE (STR(6:8),FMT='(I3.3)') DOY
 
         END IF
 
C
C        Convert HOUR, MINUTE, integral part of seconds to character
C        format. Leading zeros required.
C
         WRITE (STR(13:14),FMT='(I2.2)') HOUR
         STR(15:15) = ':'
 
         WRITE (STR(16:17),FMT='(I2.2)') MINUTE
         STR(18:18) = ':'
 
         I = SECOND
         WRITE (STR(19:20),FMT='(I2.2)') I
 
C
C        Convert the fractional part of the second to the character
C        format.
C
         IF ( PREC .GT. 0 ) THEN
 
C
C           DPSTRF gives PREC significant digits in the output,
C           not necessarily PREC digits to the right of the
C           decimal point.  We will add a one-digit integer to
C           FRCSEC to `anchor' the first significant digit of
C           FRCSEC in a known place.  That way, we can get DPSTRF
C           to give us a known number of digits after the decimal
C           point.
C
C           The integer part of FRCSEC will not affect the output
C           string.
C
            FRCSEC = FRCSEC + 1.D0
 
            CALL DPSTRF ( FRCSEC, PREC+1, 'F', FRACT )
 
            I = INDEX ( FRACT, '.' )
            STR(21: ) = FRACT(I:I+PREC)
 
         END IF
 
      END IF
 
C
C     Assign to UTCSTR. If the output string is too short, it will
C     be truncated on the right.
C
      UTCSTR = STR
 
 
      CALL CHKOUT ( 'ET2UTC' )
      RETURN
      END
