 
C$Procedure      HYPTOF ( Hyperbolic time of flight )
 
      SUBROUTINE HYPTOF ( MA, ECC, F )
 
C$ Abstract
C
C     Solve the time of flight equation MA = e sinh(F) - F for the
C     hyperbolic eccentric anomaly F, given the mean anomaly, MA,
C     and the eccentricity, e.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     CONIC
C
C$ Declarations
 
      DOUBLE PRECISION      MA
      DOUBLE PRECISION      ECC
      DOUBLE PRECISION      F
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     MA         I   Mean anomaly at epoch.
C     ECC        I   Eccentricity.
C     F          O   Hyperbolic eccentric anomaly.
C
C$ Detailed_Input
C
C     MA          is the hyperbolic mean anomaly of an orbiting body at
C                 some epoch t,
C
C                                         3 1/2
C                       MA = (t-T)(mu/(-a) )
C
C                 where T is the time of periapsis passage, a is
C                 the semi-major axis of the orbit, and mu is the
C                 gravitational parameter of the primary body.
C
C     ECC         is the eccentricity of the orbit.
C
C$ Detailed_Output
C
C     F           is the corresponding eccentric anomaly. This is the
C                 solution to the time of flight equation
C
C                       MA = e sinh(F) - F
C
C$ Parameters
C
C      None.
C
C$ Exceptions
C
C     1) If the eccentricity (ECC) is less than one, the error
C        'SPICE(WRONGCONIC)' is signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     Iterate to solve
C
C           f(F,MA,e) = e sinh(F) - F - MA = 0
C
C$ Examples
C
C     ELLTOF, HYPTOF, and PARTOF are used by CONICS.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     [1] Roger Bate, Fundamentals of Astrodynamics, Dover, 1971.
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C     W.L. Taber      (JPL)
C
C$ Version
C
C-    SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 2.0.0, 19-APR-1990 (WLT)
C
C        A bad initial guess at bracketing the solution to the
C        hyperbolic time of flight equation was corrected so that
C        floating point overflows are now avoided.  In addition, the
C        Newton's method used before has been replaced by simply
C        bisection.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     hyperbolic time of flight
C
C-&
 
 
 
 
C$ Revisions
C
C-    SPICELIB Version 2.0.0, 19-APR-1990 (WLT)
C
C        A bad initial guess at bracketing the solution to the
C        hyperbolic time of flight equation was corrected so that
C        floating point overflows are now avoided.  In addition, the
C        Newton's method used before has been replaced by simply
C        bisection.
C
C-    Beta Version 1.1.1, 27-JAN-1989 (IMU)
C
C        Examples section completed.
C
C-&
 
 
C$ Revisions
C
C-    Beta Version 1.1.0, 8-JAN-1989 (IMU)
C
C        The routine now verifies that the eccentricity is in the
C        proper range---(1,+infinity)---before proceeding.
C
C-&
 
 
C
C     SPICELIB functions
C
      DOUBLE PRECISION      DPMAX
      DOUBLE PRECISION      DCBRT
 
      LOGICAL               RETURN
 
 
C
C     Local parameters
C
      DOUBLE PRECISION      TOL
      PARAMETER           ( TOL = 1.D-15 )
 
C
C     Local variables
C
 
      DOUBLE PRECISION      MAXLOG
 
      DOUBLE PRECISION      M
 
      DOUBLE PRECISION      LOWER
      DOUBLE PRECISION      MIDDLE
      DOUBLE PRECISION      MIDVAL
      DOUBLE PRECISION      UPPER
 
      DOUBLE PRECISION      DIFF
 
      LOGICAL               FIRST
      SAVE                  FIRST
 
      DATA                  FIRST   / .TRUE. /
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'HYPTOF' )
      END IF
 
      IF ( FIRST ) THEN
         MAXLOG = LOG ( DPMAX() )
      END IF
 
      IF ( ECC .LT. 1.D0 ) THEN
         CALL SIGERR ( 'SPICE(WRONGCONIC)' )
         CALL CHKOUT ( 'HYPTOF'            )
         RETURN
      END IF
 
C
C     For reasons of numerical stability, we have to intercept cases
C     where the mean anomaly is zero or negative (since log x is not
C     defined for nonpositive x). If the mean anomaly is zero, the
C     eccentric anomaly is also zero (by inspection). If the mean
C     anomaly is negative, we can pretend it's positive (by symmetry).
C
      IF ( MA .EQ. 0.D0 ) THEN
         F = 0.D0
         CALL CHKOUT ( 'HYPTOF' )
         RETURN
      ELSE
         M = ABS ( MA )
      END IF
 
 
C
C     The initial bounds for the eccentric anomaly F are determined
C     as follows:
C
C     For the value of F we seek,
C
C        M = e sinh F - F
C
C     Thus
C
C        M < e sinh F <    (e/2) Exp F
C
C     Hence
C
C        2 M
C        ---   < Exp F
C         e
C
C     and
C
C     Therefore
C
C
C
C     Returning to our initial equation:
C
C        M = e sinh F - F
C
C                         3        5
C                        F        F
C          =  e ( F  +  ---   +  --- + ...   )  -  F
C                        3!       5!
C
C               3
C          >  eF / 6
C
C     Thus
C
C
C             3 __________
C        F <  \/  6M / e
C
C
C     Thus our solution must satisfy the inequalities
C
C
C                                  3 __________
C         LOG (2M/e)  <   F   <   \/  6M/e
C
C
C     In addition we know that the solution must lie somewhere
C     in the region between 0 and the maximum value of F for which
C     (e sinh F - F) can be computed.  This turns out to be
C     approximately LOG( DPMAX() / e ) = LOG(DPMAX()) - LOG(e) .
C
C
      LOWER = MAX ( 0.0D0,                       LOG (2.0D0*M / ECC ) )
      UPPER = MIN ( DCBRT(6.0D0*M/ECC), MAXLOG - LOG (          ECC ) )
 
 
C
C     Now we simply perform bisection to locate the root.
C
      MIDDLE = MAX( LOWER, MIN( UPPER, 0.5D0*UPPER + 0.5D0*LOWER ) )
      MIDVAL = ECC * SINH(MIDDLE) - MIDDLE - M
      DIFF   = UPPER - LOWER
 
 
50001 IF       ( DIFF   .GT. 0.0D0 )
     .THEN
 
C
C        Move one of the endpoints to the middle.
C
         IF ( MIDVAL .GT. 0 ) THEN
 
            UPPER = MIDDLE
 
         ELSE IF ( MIDVAL .LT. 0 ) THEN
 
            LOWER = MIDDLE
 
         ELSE
 
            LOWER = MIDDLE
            UPPER = MIDDLE
 
         END IF
 
C
C        Compute the next middle point.
C
         MIDDLE = MAX ( LOWER, MIN ( UPPER, 0.5D0*UPPER + 0.5D0*LOWER ))
 
C
C        If we are on an endpoint, we are ready to call it quits.
C
         IF (      ( MIDDLE .EQ. LOWER )
     .        .OR. ( MIDDLE .EQ. UPPER ) ) THEN
 
            DIFF   = 0.0D0
 
         ELSE
 
            DIFF   = UPPER - LOWER
            MIDVAL = ECC * SINH ( MIDDLE ) - MIDDLE - M
 
         END IF
 
 
         GO TO 50001
      END IF
 
C
C     Retore the proper sign, if necessary.
C
      IF ( MA .LT. 0 ) THEN
         F = - MIDDLE
      ELSE
         F =   MIDDLE
      END IF
 
 
      CALL CHKOUT ( 'HYPTOF' )
      RETURN
      END
