C$Procedure      INRYPL ( Intersection of ray and plane )
 
      SUBROUTINE INRYPL ( VERTEX, DIR, PLANE, NXPTS, XPT )
 
C$ Abstract
C
C     Find the intersection of a ray and a plane.
C
C$ Required_Reading
C
C     PLANES
C
C$ Keywords
C
C     GEOMETRY
C
C$ Declarations
 
      INTEGER               UBPL
      PARAMETER           ( UBPL    =   4 )
 
      DOUBLE PRECISION      VERTEX (    3 )
      DOUBLE PRECISION      DIR    (    3 )
      DOUBLE PRECISION      PLANE  ( UBPL )
      INTEGER               NXPTS
      DOUBLE PRECISION      XPT    (    3 )
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     VERTEX,
C     DIR        I   Vertex and direction vector of ray.
C     PLANE      I   A SPICELIB plane.
C     NXPTS      O   Number of intersection points of ray and plane.
C     XPT        O   Intersection point, if NXPTS = 1.
C
C$ Detailed_Input
C
C     VERTEX,
C     DIR            are a point and direction vector that define a
C                    ray in three-dimensional space.
C
C     PLANE          is a SPICELIB plane.
C
C$ Detailed_Output
C
C     NXPTS          is the number of points of intersection of the
C                    input ray and plane.  Values and meanings of
C                    NXPTS are:
C
C                       0     No intersection.
C
C                       1     One point of intersection.  Note that
C                             this case may occur when the ray's
C                             vertex is in the plane.
C
C                      -1     An infinite number of points of
C                             intersection; the ray lies in the plane.
C
C
C     XPT            is the point of intersection of the input ray
C                    and plane, when there is exactly one point of
C                    intersection.  Otherwise, XPT is the zero vector.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1)  If the ray's direction vector is the zero vector, the error
C         SPICE(ZEROVECTOR) is signalled.  NXPTS and XPT are not
C         modified.
C
C
C     2)  If the ray's vertex is further than DPMAX() / 3 from the
C         origin, the error SPICE(VECTORTOOBIG) is signalled.  NXPTS
C         and XPT are not modified.
C
C
C     3)  If the input plane is s further than DPMAX() / 3 from the
C         origin, the error SPICE(VECTORTOOBIG) is signalled.  NXPTS
C         and XPT are not modified.
C
C
C     4)  The input plane should be created by one of the SPICELIB
C         routines
C
C            NVC2PL
C            PNV2PL
C            PSV2PL
C
C         Invalid input planes will cause unpredictable results.
C
C
C     5)  In the interest of good numerical behavior, in the case
C         where the ray's vertex is not in the plane, this routine
C         considers that an intersection of the ray and plane occurs
C         only if the distance between the ray's vertex and the
C         intersection point is less than DPMAX() / 3.
C
C         If VERTEX is not in the plane and this condition is not
C         met, then NXPTS is set to 0 and XPT is set to the zero
C         vector.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The intersection of a ray and plane in three-dimensional space
C     can be a the empty set, a single point, or the ray itself.
C
C$ Examples
C
C     1)  Find the camera projection of the center of an extended
C         body.  For simplicity, we assume:
C
C            -- The camera has no distortion;  the image of a point
C               is determined by the intersection of the focal plane
C               and the line determined by the point and the camera's
C               focal point.
C
C            -- The camera's pointing matrix (C-matrix) is available
C               in a C-kernel.
C
C
C            C
C            C     Load Leapseconds and SCLK kernels to support time
C            C     conversion.
C            C
C                  CALL LDPOOL ( 'LEAP.KER' )
C                  CALL LDPOOL ( 'SCLK.KER' )
C
C            C
C            C     Load an SPK file containing ephemeris data for
C            C     observer (a spacecraft, whose NAIF integer code
C            C     is SC) and target at the UTC epoch of observation.
C            C
C                  CALL SPKLEF ( 'SPK.BSP', HANDLE )
C
C            C
C            C     Load a C-kernel containing camera pointing for
C            C     the UTC epoch of observation.
C            C
C                  CALL CKLPF ( 'CK.BC',    HANDLE )
C
C            C
C            C     Find the ephemeris time (barycentric dynamical time)
C            C     and encoded spacecraft clock times corresponding to
C            C     the UTC epoch of observation.
C            C
C                  CALL UTC2ET ( UTC, ET          )
C                  CALL SCE2T  ( SC,  ET,  SCLKDP )
C
C            C
C            C     Encode the pointing lookup tolerance.
C            C
C                  CALL SCTIKS ( SC, TOLCH,  TOLDP  )
C
C            C
C            C     Find the observer-target vector at the observation
C            C     epoch.  In this example, we'll use a light-time
C            C     corrected state vector.
C            C
C                  CALL SPKEZ ( TARGET,  ET,  'J2000',  'LT',  SC,
C                 .             STATE,   LT                        )
C
C            C
C            C     Look up camera pointing.
C            C
C                  CALL CKGP  ( CAMERA, SCLKDP, TOLDP, 'J2000', CMAT,
C                 .             CLKOUT, FOUND                        )
C
C            C
C            C     Negate the spacecraft-to-target body vector and
C            C     convert it to camera coordinates.
C            C
C                  CALL VMINUS ( STATE, DIR       )
C                  CALL MXV    ( CMAT,  DIR,  DIR )
C
C            C
C            C     If FL is the camera's focal length, the effective
C            C     focal point is
C            C
C            C        FL * ( 0, 0, 1 )
C            C
C                  CALL VSCL ( FL, ZVEC, FOCUS )
C
C            C
C            C     The camera's focal plane contains the origin in
C            C     camera coordinates, and the z-vector is orthogonal
C            C     to the plane.  Make a SPICELIB plane representing
C            C     the focal plane.
C            C
C                  CALL NVC2PL ( ZVEC, 0.D0, FPLANE )
C
C            C
C            C     The image of the target body's center in the focal
C            C     plane is defined by the intersection with the focal
C            C     plane of the ray whose vertex is the focal point and
C            C     whose direction is DIR.
C            C
C                  CALL INRYPL ( FOCUS, DIR, FPLANE, NXPTS, IMAGE )
C
C                  IF ( NXPTS .EQ. 1 ) THEN
C            C
C            C        The body center does project to the focal plane.
C            C        Check whether the image is actually in the
C            C        camera's field of view...
C            C
C                               .
C                               .
C                               .
C                  ELSE
C
C            C
C            C        The body center does not map to the focal plane.
C            C        Handle this case...
C            C
C                               .
C                               .
C                               .
C                  END IF
C
C
C
C     2)  Find the ring plane intercept of a spacecraft-mounted
C         instrument's boresight vector.  In this example, we assume
C
C            -- The ring is truly planar.
C
C            -- The ring plane is equatorial.
C
C            -- Light travels in a straight line.
C
C            -- The light time correction for the ring plane intercept
C               can be obtained by performing three light-time
C               correction iterations.  If this assumption does not
C               lead to a sufficiently accurate result, additional
C               iterations can be performed.
C
C            -- The boresight vector is given in J2000 coordinates.
C
C            -- The observation epoch is ET ephemeris seconds past
C               J2000.
C
C            -- The boresight vector, spacecraft and planetary
C               ephemerides, and ring plane orientation are all known
C               with sufficient accuracy for the application.
C
C         (A similar technique could be used to obtain low-accuracy
C         predictions of radio occultations.  In that case, the
C         instrument boresight ray's direction vector would be replaced
C         by the vector from the observer to the light-time corrected
C         radio source position.)
C
C         We omit display of the portion of the code that loads SPICE
C         kernels.
C
C            C
C            C     Start out with light time set equal to zero.
C            C
C                  TAU = 0.D0
C
C            C
C            C     Find the ring plane intercept and calculate the
C            C     light time (zero-order) from it to the spacecraft.
C            C     Perform three iterations.
C            C
C                  I     = 1
C                  NXPTS = 1
C
C                  DO WHILE (  ( I .LE. 3 )  .AND.  ( NXPTS .EQ. 1 )  )
C            C
C            C        Find position of the spacecraft relative to the
C            C        target body in J2000 coordinates at the light-time
C            C        retarded epoch
C            C
C                     CALL SPKEZ  ( TARG,  ET-TAU, 'J2000', 'NONE', SC,
C                 .                 STATE, LT                          )
C
C                     CALL VMINUS ( STATE, SCPOS )
C
C            C
C            C        Obtain the J2000 to body equator and prime
C            C        meridian transformation TIPM for the epoch
C            C        ET - TAU.
C            C
C                     CALL BODMAT ( TARG,  ET - TAU,  TIPM )
C
C            C
C            C        Convert both the SC position vector and instrument
C            C        boresight vector BORVEC into body-fixed coordinate
C            C
C                     CALL MXV ( TIPM,  SCPOS,  SCPOS  )
C                     CALL MXV ( TIPM,  BORVEC, BORVEC )
C
C            C
C            C        Make a SPICELIB plane representing the ring plane.
C            C        In body-fixed coordinates, this plane has normal
C            C        vector ( 0, 0, 1 ) (we'll call this vector ZVEC)
C            C        and plane constant 0.
C            C
C                     CALL NVC2PL ( ZVEC, 0.D0, PLANE )
C
C            C
C            C        Find the intersection of the ring plane and the
C            C        ray having vertex SCPOS and direction vector
C            C        BORVEC.
C            C
C                     CALL INRYPL ( SCPOS, BORVEC, PLANE, NXPTS, XPT )
C
C            C
C            C        If the number of intersection points is 1,
C            C        find the next light time estimate.
C            C
C                     IF ( NXPTS .EQ. 1 ) THEN
C            C
C            C           Find the light time (zero-order) from the
C            C           intercept point to the spacecraft.
C            C
C                        TAU  =  VDIST ( SCPOS, XPT )  /  CLIGHT()
C                        I    =  I + 1
C
C                     END IF
C
C                  END DO
C
C            C
C            C     At this point, if NXPTS is still 1, we iterated
C            C     three times, and XPT should be a good estimate
C            C     of the light-time corrected position of the ring
C            C     plane intercept point.  If NXPTS is not 1, the
C            C     boresight ray does not intersect the ring plane.
C            C
C
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman   (JPL)
C     W.L. Taber     (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 01-APR-1991 (NJB) (WLT)
C
C-&
 
C$ Index_Entries
C
C     intersection of ray and plane
C
C-&
 
 
 
C
C     SPICELIB functions
C
      DOUBLE PRECISION      DPMAX
      DOUBLE PRECISION      VDOT
      DOUBLE PRECISION      VNORM
 
      LOGICAL               RETURN
      LOGICAL               SMSGND
      LOGICAL               VZERO
 
C
C     Local parameters
C
      DOUBLE PRECISION      MARGIN
      PARAMETER           ( MARGIN = 3.D0 )
 
C
C     Local variables
C
      DOUBLE PRECISION      CONST
      DOUBLE PRECISION      PRJDIF
      DOUBLE PRECISION      PRJDIR
      DOUBLE PRECISION      PRJVN
      DOUBLE PRECISION      MSCALE
      DOUBLE PRECISION      NORMAL ( 3 )
      DOUBLE PRECISION      SCALE
      DOUBLE PRECISION      SCLCON
      DOUBLE PRECISION      SCLVTX ( 3 )
      DOUBLE PRECISION      TOOBIG
      DOUBLE PRECISION      UDIR   ( 3 )
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'INRYPL' )
      END IF
 
C
C     We'll give the name TOOBIG to the bound DPMAX() / MARGIN.
C     If we let VTXPRJ be the orthogonal projection of VERTEX onto
C     PLANE, and let DIFF be the vector VTXPRJ - VERTEX, then
C     we know that
C
C        ||  DIFF  ||    <     2 * TOOBIG
C
C     Check the distance of the ray's vertex from the origin.
C
      TOOBIG = DPMAX() / MARGIN
 
      IF (  VNORM (VERTEX)  .GE.  TOOBIG  )  THEN
 
         CALL SETMSG ( 'Ray''s vertex is too far from the origin.' )
         CALL SIGERR ( 'SPICE(VECTORTOOBIG)'                       )
         CALL CHKOUT ( 'INRYPL'                                    )
         RETURN
 
      END IF
 
C
C     Check the distance of the plane from the origin.  (The returned
C     plane constant IS this distance.)
C
      CALL PL2NVC ( PLANE, NORMAL, CONST )
 
      IF (  CONST  .GE.  TOOBIG  )  THEN
 
         CALL SETMSG ( 'Plane is too far from the origin.' )
         CALL SIGERR ( 'SPICE(VECTORTOOBIG)'               )
         CALL CHKOUT ( 'INRYPL'                            )
         RETURN
 
      END IF
 
C
C     Check the ray's direction vector.
C
      CALL VHAT ( DIR, UDIR )
 
      IF ( VZERO (UDIR) ) THEN
 
         CALL SETMSG ( 'Ray''s direction vector is the zero vector.' )
         CALL SIGERR ( 'SPICE(ZEROVECTOR)'                           )
         CALL CHKOUT ( 'INRYPL'                                      )
         RETURN
 
      ENDIF
 
C
C     That takes care of the error cases.  Now scale the input vertex
C     and plane to improve numerical behavior.
C
      MSCALE = MAX ( CONST, VNORM(VERTEX) )
 
      IF ( MSCALE .NE. 0.D0 ) THEN
 
         CALL VSCL ( 1.D0 / MSCALE, VERTEX, SCLVTX )
         SCLCON  =  CONST / MSCALE
 
      ELSE
 
         CALL VEQU ( VERTEX, SCLVTX )
         SCLCON   =  CONST
 
      END IF
 
      IF ( MSCALE .GT. 1.D0 ) THEN
         TOOBIG = TOOBIG / MSCALE
      END IF
 
 
C     Find the projection (coefficient) of the ray's vertex along the
C     plane's normal direction.
C
      PRJVN = VDOT ( SCLVTX, NORMAL )
 
C
C     If this projection is the plane constant, the ray's vertex lies in
C     the plane.  We have one intersection or an infinite number of
C     intersections.  It all depends on whether the ray actually lies
C     in the plane.
C
C     The absolute value of PRJDIF is the distance of the ray's vertex
C     from the plane.
C
      PRJDIF  =  SCLCON - PRJVN
 
      IF ( PRJDIF .EQ. 0.D0 ) THEN
C
C        XPT is the original, unscaled vertex.
C
         CALL VEQU ( VERTEX, XPT )
 
         IF (  VDOT ( NORMAL, UDIR )  .EQ.  0.D0  )  THEN
C
C           The ray's in the plane.
C
            NXPTS  =  -1
         ELSE
            NXPTS  =   1
         END IF
 
         CALL CHKOUT ( 'INRYPL' )
         RETURN
 
      END IF
 
C
C     Ok, the ray's vertex is not in the plane.  The ray may still be
C     parallel to or may point away from the plane.  If the ray does
C     point towards the plane, mathematicians would say that the
C     ray does intersect the plane, but the computer may disagree.
C
C     For this routine to find an intersection, both of the following
C     conditions must be met:
C
C        -- The ray must point toward the plane; this happens when
C           PRJDIF has the same sign as < UDIR, NORMAL >.
C
C        -- The vector difference XPT - SCLVTX must not overflow.
C
C      Qualitatively, the case of interest looks something like the
C      picture below:
C
C
C                      * SCLVTX
C                      |\
C                      | \   <-- UDIR
C                      |  \
C    length of this    |   \|
C      segment is      |   -*
C                      |
C     | PRJDIF |   --> | ___________________________
C                      |/                          /
C                      |       *                  /   <-- PLANE
C                     /|        XPT              /
C                    / ^                        /
C                   /  | NORMAL                /
C                  /   | .                    /
C                 /    |/|                   /
C                / .---| /                  /
C               /  |   |/                  /
C              /   `---*                  /
C             /          Projection of SCLVTX onto the plane
C            /                          /
C           /                          /
C          ----------------------------
C
C
C
 
C
C     Find the projection of the direction vector along the the plane's
C     normal vector.
C
      PRJDIR  =  VDOT ( UDIR, NORMAL )
 
C
C     We're done if the ray doesn't point toward the plane.  PRJDIF
C     has already been found to be non-zero at this point; PRJDIR is
C     zero if the ray and plane are parallel.  The SPICELIB routine
C     SMSGND will return a value of .FALSE. if PRJDIR is zero.
C
      IF (  .NOT. SMSGND (PRJDIR, PRJDIF)  )  THEN
C
C        The ray is parallel to or points away from the plane.
C
         NXPTS = 0
         CALL CLEARD ( 3, XPT )
 
         CALL CHKOUT ( 'INRYPL' )
         RETURN
 
      END IF
 
C
C     The difference XPT - SCLVTX is the hypotenuse of a right triangle
C     formed by SCLVTX, XPT, and the orthogonal projection of SCLVTX
C     onto the plane.  We'll obtain the hypotenuse by scaling UDIR.
C     We must make sure that this hypotenuse does not overflow.  The
C     scale factor has magnitude
C
C         | PRJDIF |
C       --------------
C         | PRJDIR |
C
C     and UDIR is a unit vector, so as long as
C
C         | PRJDIF |   <   | PRJDIR |  *  TOOBIG
C
C     the hypotenuse is no longer than TOOBIG.  The product can be
C     computed safely since PRJDIR has magnitude 1 or less.
C
 
      IF (  ABS(PRJDIF)  .GE.  ABS(PRJDIR) * TOOBIG  ) THEN
C
C        If the hypotenuse is too long, we say that no intersection
C        exists.
C
         NXPTS = 0
         CALL CLEARD ( 3, XPT )
 
         CALL CHKOUT ( 'INRYPL' )
         RETURN
 
      END IF
 
C
C     We conclude that it's safe to compute XPT.  Scale UDIR and add
C     the result to SCLVTX.  The addition is safe because both addends
C     have magnitude no larger than TOOBIG.  The vector thus obtained
C     is the intersection point.
C
      NXPTS    =   1
      SCALE    =   ABS (PRJDIF)  /   ABS (PRJDIR)
 
      CALL VLCOM ( 1.0D0, SCLVTX, SCALE, UDIR, XPT )
 
C
C     Re-scale XPT.  This is safe, since TOOBIG has already been
C     scaled to allow for any growth of XPT at this step.
C
      CALL VSCL ( MSCALE, XPT, XPT )
 
      CALL CHKOUT ( 'INRYPL' )
      RETURN
      END
