C$Procedure      M2Q ( Matrix to quaternion )
 
      SUBROUTINE M2Q ( R, Q )
 
C$ Abstract
C
C     Find a unit quaternion corresponding to a specified rotation
C     matrix.
C
C$ Required_Reading
C
C     ROTATION
C
C$ Keywords
C
C     MATH
C     MATRIX
C     ROTATION
C
C$ Declarations
 
      DOUBLE PRECISION      R ( 3,  3 )
      DOUBLE PRECISION      Q ( 0 : 3 )
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     R          I   A rotation matrix.
C     Q          O   A unit quaternion representing R.
C
C$ Detailed_Input
C
C     R              is a rotation matrix.
C
C$ Detailed_Output
C
C     Q              is a unit quaternion representing R.  Q is a
C                    4-dimensional vector.  If R rotates vectors by an
C                    angle of r radians about a unit vector A, where
C                    r is in [0, pi], then if h = r/2,
C
C                       Q = ( cos(h), sin(h)A ,  sin(h)A ,  sin(h)A ).
C                                            1          2          3
C
C                    The restriction that r must be in the range [0, pi]
C                    determines the output quaternion Q uniquely
C                    except when r = pi; in this special case, both of
C                    the quaternions
C
C                       Q = ( 0,  A ,  A ,  A  )
C                                  1    2    3
C                    and
C
C                       Q = ( 0, -A , -A , -A  )
C                                  1    2    3
C
C                   are possible outputs, if A is a choice of rotation
C                   axis for R.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1)   If R is not a rotation matrix, the error SPICE(NOTAROTATION)
C          is signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     A unit quaternion is a 4-dimensional vector for which the sum of
C     the squares of the components is 1.  Unit quaternions can be used
C     to represent rotations in the following way:  Given a rotation
C     angle r in [0, pi] and a unit vector A that acts as a rotation
C     axis, we define the quaternion Q by
C
C        Q = ( cos(r/2), sin(r/2)A , sin(r/2)A , sin(r/2)A ).
C                                 1           2           3
C
C     As mentioned in $Detailed_Output, our restriction on the range of
C     r determines Q uniquely, except when r = pi.
C
C     The SPICELIB routine Q2M is an one-sided inverse of this routine:
C     given any rotation matrix R, the calls
C
C        CALL M2Q ( R, Q )
C        CALL Q2M ( Q, R )
C
C     leave R unchanged, except for round-off error.  However, the
C     calls
C
C        CALL Q2M ( Q, R )
C        CALL M2Q ( R, Q )
C
C     might preserve Q or convert Q to -Q.
C
C$ Examples
C
C     1)  A case amenable to checking by hand calculation:
C
C            To convert the rotation matrix
C
C                     +-              -+
C                     |  0     1    0  |
C                     |                |
C               R  =  | -1     0    0  |
C                     |                |
C                     |  0     0    1  |
C                     +-              -+
C
C            also represented as
C
C               [ pi/2 ]
C                       3
C
C            to a quaternion, we can use the code fragment
C
C               CALL ROTATE (  HALFPI(),  3,  R  )
C               CALL M2Q    (  R,             Q  )
C
C            M2Q will return Q as
C
C               ( sqrt(2)/2, 0, 0, -sqrt(2)/2 ).
C
C            Why?  Well, R is a coordinate transformation that
C            rotates vectors by -pi/2 radians about the axis vector
C
C               A  = ( 0, 0, 1 ),
C
C            so our definition of Q,
C
C               Q =
C
C               ( cos(r/2), sin(r/2)A , sin(r/2)A , sin(r/2)A  ),
C                                    1           2           3
C
C            implies that in this case,
C
C               Q =  ( cos(pi/4),  0,  0,  sin(pi/4)  )
C
C                 =  ( sqrt(2)/2,  0,  0, -sqrt(2)/2  ).
C
C
C     2)  Finding a quaternion that represents a rotation specified by
C         a set of Euler angles:
C
C            Suppose our original rotation R is the product
C
C               [ TAU ]  [ pi/2 - DELTA ]  [ ALPHA ] .
C                      3                 2          3
C
C            The code fragment
C
C               CALL EUL2M  ( TAU,   HALFPI() - DELTA,   ALPHA,
C              .              3,     2,                  3,      R )
C
C               CALL M2Q    ( R, Q )
C
C            yields a quaternion Q that represents R.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF document 179.0, "Rotations and their Habits", by
C     W. L. Taber.
C
C$ Author_and_Institution
C
C     N.J. Bachman   (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 30-AUG-1990 (NJB)
C
C-&
 
C$ Index_Entries
C
C     matrix to quaternion
C
C-&
 
 
 
C
C     SPICELIB functions
C
      LOGICAL               ISROT
      LOGICAL               RETURN
 
C
C     Local parameters
C
 
C
C     NTOL and DETOL are used to determine whether R is a rotation
C     matrix.
C
C     NTOL is the tolerance for the norms of the columns of R.
C
C     DTOL is the tolerance for the determinant of a matrix whose
C     columns are the unitized columns of R.
C
C
      DOUBLE PRECISION      NTOL
      PARAMETER           ( NTOL = 0.1D0 )
 
      DOUBLE PRECISION      DTOL
      PARAMETER           ( DTOL = 0.1D0 )
 
 
C
C     Local variables
C
      DOUBLE PRECISION      ANGLE
      DOUBLE PRECISION      AXIS   ( 3 )
      DOUBLE PRECISION      S
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'M2Q' )
      END IF
 
C
C     If R is not a rotation matrix, we can't proceed.
C
      IF (  .NOT. ISROT ( R, NTOL, DTOL )  ) THEN
 
         CALL SETMSG ( 'Input matrix was not a rotation.' )
         CALL SIGERR ( 'SPICE(NOTAROTATION)'              )
         CALL CHKOUT ( 'M2Q'                              )
         RETURN
 
      END IF
 
C
C     Find the rotation angle and axis of R, and form the corresponding
C     quaternion Q.
C
      CALL RAXISA ( R, AXIS, ANGLE )
 
      Q(0)  =  COS ( ANGLE / 2.D0 )
      S     =  SIN ( ANGLE / 2.D0 )
 
      CALL VSCL   ( S, AXIS, Q(1)  )
 
      CALL CHKOUT ( 'M2Q' )
      RETURN
      END
