C$Procedure      NPARSI ( Integer parsing of a character string)
 
      SUBROUTINE NPARSI (  STRING,  N, ERROR, PNTER)
 
C$ Abstract
C
C     Parse a character string that represents a number and return
C     the FORTRAN-truncated integer value.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      ALPHANUMERIC
C      CONVERSION
C      PARSING
C
C$ Declarations
 
      CHARACTER*(*)    STRING
      INTEGER          N
      CHARACTER*(*)    ERROR
      INTEGER          PNTER
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  ---------------------------------------------------
C     STRING     I   Character string representing a numeric value.
C     N          O   Translated integer value of STRING.
C     ERROR      O   Message indicating what errors have occurred.
C     PNTER      O   Position in character string where an error
C                    occurred.
C
C$ Detailed_Input
C
C     STRING     A character string that represents a numeric value.
C                Commas and spaces may be used in this string for
C                ease of reading and writing the number.  They
C                are treated as insignificant but non-error-producing
C                characters.
C
C                For exponential representation and of the characters
C                'E','D','e','d' may be used.
C
C                The following are legitimate numeric expressions
C
C                 +12.2 e-1
C                 -3. 1415 9276
C                 1e12
C                 E10
C
C                The program also recognizes the following  mnemonics
C                'PI', 'pi', 'Pi', 'pI'
C                '+PI', '+pi', '+Pi', '+pI'
C                '-PI', '-pi', '-Pi', '-pI'
C                and returns the value ( + OR - ) 3 as appropriate.
C
C$ Detailed_Output
C
C     N          Integer parsed value of input string  ( with
C                the implied limits on precision).  If an error is
C                encountered, N is not changed from whatever the
C                input value was.  If the input string has a fractional
C                part, the fractional part will be truncated.  Thus
C                3.18 is interpreted as 3.  -4.98 is interpreted as -4.
C
C     ERROR      This is a message indicating that the string could
C                not be parsed due to ambiguous use of symbols or
C                due to a string representing a number too large for
C                VAX double precision or integer variables. If no
C                error occurred, ERROR is blank.
C
C                In particular, blank strings, or strings that do not
C                contain either a digit or exponent character will
C                be regarded as errors.
C
C     PNTER      This indicates which character was being used when
C                the error occurred. If no error occurred, PNTER is 0.
C
C$ Parameters
C
C      None.
C
C$ Particulars
C
C     Basically, all this routine does is pass the input string to
C     NPARSD which does the parsing in double precision.  If nothing
C     goes wrong in the double precision parsing of the number, the
C     returned value is checked to determine whether or not it will fit
C     into a VAX integer.  If it doesn't, an error message is returned.
C
C$ Examples
C
C     Let   LINE = 'DELTA_T_A       =   32'
C
C     The following code fragment parses the line and obtains the
C     integer value.
C
C
C        CALL NEXTWD ( LINE,  FIRST,  REST )
C        CALL NEXTWD ( REST, SECOND,  REST )
C        CALL NEXTWD ( REST,  THIRD,  REST )
C
C        CALL NPARSI (  THIRD,  VALUE, ERROR, POINTR )
C
C$ Restrictions
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C     1) If the string is non-numeric, PNTER indicates the location in
C        the string where the error occurred, and ERROR contains a
C        descriptive error message.
C
C     2) If the string is blank, ERROR is returned with a message
C        indicating the problem.
C
C$ Files
C
C     None.
C
C$ Author_and_Institution
C
C     H.A. Neilan     (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 15-OCT-1992 (WLT)
C
C        The abstract of this routine was modified to reflect what
C        the routine actually does---truncate the value to an
C        integer.
C
C        In addition, a blank string is no longer considered to be
C        valid input.
C
C        Finally the instances of DFLOAT in the previous version were
C        replaced by the standard intrinsic function DBLE and the
C        function DINT was replaced by IDINT in one place to make types
C        match up on both sides of an assignment.
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (WLT)
C
C-&
 
C$ Index_Entries
C
C     parse a character_string to an integer
C
C-&
 
C
 
 
C$ Revisions
C
C-    SPICELIB Version 2.0.0, 15-OCT-1992 (WLT)
C
C        The abstract of this routine was modified to reflect what
C        the routine actually does---truncate the value to an
C        integer.
C
C        In addition, a blank string is no longer considered to be
C        valid input.
C
C        Finally the instances of DFLOAT in the previous version were
C        replaced by the standard intrinsic function DBLE and the
C        function DINT was replaced by IDINT in one place to make types
C        match up on both sides of an assignment.
C
C-     Beta Version 1.2.0, 23-FEB-1989 (WLT)
C
C         Due to a programming error, the routine was not leaving N
C         unchanged if the input string was blank.  This bug was
C         fixed and the exceptional case noted in exceptions.
C
C-     Beta Version 1.1.0, 28-OCT-1988 (HAN)
C
C         Peter Wolff (JPL) informed the NAIF staff that he found
C         an "IMPLICIT NONE" statement in the ANSI Standard Fortran
C         77 version of this routine. Because the statement is a
C         VAX extension not used by NAIF, the statement was removed.
C
C-&
 
C
C     SPICELIB functions
C
 
      INTEGER               INTMAX
      INTEGER               INTMIN
 
C
C     Local Variables
C
      DOUBLE PRECISION      XMXINT
      DOUBLE PRECISION      XMNINT
      DOUBLE PRECISION      X
C
C
C     No errors yet.
C
      ERROR = ' '
 
      XMXINT =  DBLE ( INTMAX () )
      XMNINT =  DBLE ( INTMIN () )
      X      =  DBLE ( N         )
 
      CALL NPARSD (  STRING,  X, ERROR, PNTER)
 
      IF ( ERROR .EQ. ' ' ) THEN
 
         IF ((DINT(X) .LT. XMNINT) .OR. ( DINT(X) .GT. XMXINT  )) THEN
 
            ERROR = 'NPARSI: Value entered is beyond the bounds of '//
     >              'representable integers.'
 
         ELSE
 
            N     = IDINT(X)
 
         END IF
 
      END IF
 
      RETURN
      END
