C$Procedure      POOL ( Maintain a pool of kernel variables )
 
      SUBROUTINE POOL ( KERNEL, UNIT, NAME,   NAMES, NNAMES,
     .                  AGENT,  N,    VALUES, FOUND, UPDATE )
 
C$ Abstract
C
C     Maintain a pool of variables read from SPICE ASCII kernel files.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     CONSTANTS
C     FILES
C
C$ Declarations
 
      CHARACTER*(*)         KERNEL
      INTEGER               UNIT
      CHARACTER*(*)         NAME
      CHARACTER*(*)         NAMES     ( * )
      INTEGER               NNAMES
      CHARACTER*(*)         AGENT
      INTEGER               N
      DOUBLE PRECISION      VALUES    ( * )
      LOGICAL               FOUND
      LOGICAL               UPDATE
 
      INTEGER               MAXVAR
      PARAMETER           ( MAXVAR =  500 )
 
      INTEGER               MAXVAL
      PARAMETER           ( MAXVAL = 5000 )
 
      INTEGER               MXNOTE
      PARAMETER           ( MXNOTE = 1000 )
 
      INTEGER               MAXLEN
      PARAMETER           ( MAXLEN =   32 )
 
      INTEGER               MAXAGT
      PARAMETER           ( MAXAGT =  500 )
 
C$ Brief_I/O
C
C     Variable  I/O  Entry
C     --------  ---  --------------------------------------------------
C     KERNEL     I   LDPOOL
C     UNIT       I   WRPOOL
C     NAME       I   RTPOOL, EXPOOL
C     NAMES      I   SWPOOL
C     AGENT      I   CVPOOL, SWPOOL
C     NNAMES     I   SWPOOL
C     N          O   RTPOOL
C     VALUES     O   RTPOOL
C     FOUND      O   RTPOOL, EXPOOL
C     UPDATE     O   CVPOOL
C     MAXVAR     P   (All)
C     MAXLEN     P   (All)
C     MXNOTE     P   (All)
C     MAXVAL     P   (All)
C     MAXAGT     P   (All)
C     BEGDAT     P   WRPOOL
C     BEGTXT     P   WRPOOL
C
C$ Detailed_Input
C
C     See the ENTRY points for a discussion of their arguments.
C
C$ Detailed_Output
C
C     See the ENTRY points for a discussion of their arguments.
C
C$ Parameters
C
C     MAXVAR      is the maximum number of variables that the
C                 kernel pool may contain at any one time.
C
C     MAXLEN      is the maximum length of the variable names
C                 that can be stored in the kernel pool.
C
C     MAXVAL      is the maximum number of distinct values that
C                 may belong to the variables in the kernel pool.
C                 Each variable must have at least one value, and
C                 may have any number, so long as the total number
C                 does not exceed MAXVAL. MAXVAL must be at least
C                 as large as MAXVAR.
C
C     MXNOTE      is the maximum number of distinct variable-agents
C                 pairs that can be maintained by the kernel pool.
C                 (A variable is "paired" with an agent, if that agent
C                 is to be notified whenever the variable is updated.)
C
C     MAXAGT      is the maximum number of agents that can be kept
C                 on the distribution list for notification of updates
C                 to kernel variables.
C
C$ Files
C
C     See the ENTRY points for a discussion of their arguments.
C
C$ Exceptions
C
C     1) If POOL is called directly, the error SPICE(BOGUSENTRY) is
C        signalled.
C
C$ Particulars
C
C     POOL should never be called directly, but should instead be
C     accessed only through its entry points.
C
C     The purpose of this routine is to maintain a pool of variables
C     read from ASCII kernel files. The following entry points may be
C     used to access the pool.
C
C           CLPOOL         Clears the pool.
C
C           LDPOOL         Loads the variables from a kernel file into
C                          the pool.
C
C           RTPOOL         Returns the value of a variable from
C                          the pool.
C
C           EXPOOL         Confirms the existence of a variable in
C                          the pool.
C
C           WRPOOL         Writes the contents of the pool to an
C                          ASCII kernel file.
C
C           SWPOOL         Sets up a "watcher" on a variable so that
C                          various "agents" can be notified when a
C                          variable has been updated.
C
C           CVPOOL         Indicates whether or not an agent's
C                          variable has been updated since the last
C                          time an agent checked with the pool.
C
C     Nominally, the kernel pool contains up to 500 separate variables
C     and up to 5000 separate values. The names of the individual
C     variables may contain up to 32 characters. All of these figures
C     may be increased or decreased as necessary.
C
C$ Examples
C
C     The following code fragment demonstrates how the data from
C     several kernel files can be loaded into a kernel pool. After the
C     pool is loaded, the values in the pool are written to a kernel
C     file.
C
C     C
C     C     Store in an array the names of the kernel files whose
C     C     values will be loaded into the kernel pool.
C     C
C           KERNEL (1) = 'AXES.KER'
C           KERNEL (2) = 'GM.KER'
C           KERNEL (3) = 'LEAP_SECONDS.KER'
C
C     C
C     C     Clear the kernel pool. (This is optional.)
C     C
C           CALL CLPOOL
C
C     C
C     C     Load the variables from the three kernel files into the
C     C     the kernel pool.
C     C
C           DO I = 1, 3
C             CALL LDPOOL ( KERNEL (I) )
C           END DO
C
C     C
C     C     We can examine the values associated with any variable
C     C     in the kernel pool using RTPOOL.
C     C
C           CALL RTPOOL ( VARIABLE, NUMVAL, VALUES, FOUND )
C
C     C
C     C     Get a free logical unit and open the file 'NEWKERNEL.KER'.
C     C
C           CALL GETLUN ( UNIT )
C
C           OPEN ( FILE            = 'NEWKERNEL.KER',
C          .       UNIT            = UNIT,
C          .       STATUS          = 'NEW',
C          .       IOSTAT          = IOSTAT,
C          .       CARRIAGECONTROL = 'LIST'  )
C
C     C
C     C     Write the values in the kernel pool to the file.
C     C
C           CALL WRPOOL ( UNIT )
C
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     H.A. Neilan     (JPL)
C     W.L. Taber      (JPL)
C     R.E. Thurman    (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 6.0.0, 31-MAR-1992 (WLT)
C
C        The entry points SWPOOL and CVPOOL were added.
C
C-    SPICELIB Version 5.0.0, 22-AUG-1990 (NJB)
C
C        Increased value of parameter MAXVAL to 5000 to accommodate
C        storage of SCLK coefficients in the kernel pool.
C
C-    SPICELIB Version 4.0.0, 12-JUN-1990 (IMU)
C
C        All entry points except POOL and CLPOOL now initialize the
C        pool if it has not been done yet.
C
C-    SPICELIB Version 3.0.0, 23-OCT-1989 (HAN)
C
C        Added declaration of FAILED. FAILED is checked in the
C        DO-loops in LDPOOL and WRPOOL to prevent infinite looping.
C
C-    SPICELIB Version 2.0.0, 18-OCT-1989 (RET)
C
C       A FAILED test was inserted into the control of the DO-loop which
C       reads in each kernel variable in LDPOOL.
C
C-    SPICELIB Version 1.2.0, 9-MAR-1989 (HAN)
C
C        Parameters BEGDAT and BEGTXT have been moved into the
C        Declarations section.
C
C-    SPICELIB Version 1.1.0, 16-FEB-1989 (IMU) (NJB)
C
C        Parameters MAXVAR, MAXVAL, MAXLEN moved into Declarations.
C        (Actually, MAXLEN was implicitly 32 characters, and has only
C        now been made an explicit---and changeable---limit.)
C
C        Declaration of unused function FAILED removed.
C
C-    SPICELIB Version 1.0.0, 8-JAN-1989 (IMU)
C
C-&
 
C$ Index_Entries
C
C     MAINTAIN a pool of kernel variables
C
C-&
 
C$ Revisions
C
C-    SPICELIB Version 6.0.0, 31-MAR-1992 (WLT)
C
C        The entry points SWPOOL (set watch on a pool variable)
C        and CVPOOL (check variable for update) so that routines
C        that buffer data stored in the kernel pool can fetch
C        that data only when it is updated.
C
C        Also the control of initializations was modified to be
C        consistent with other SPICELIB practices.
C
C        Finally, the revision history was upgraded so that the
C        version number increases over time.  This wasn't true
C        before. In addition some early revision data that referred to
C        pre-SPICELIB modifications were removed. This editing of
C        the version numbers makes it unlikely that anyone can track
C        down which previous version of this routine they have by
C        looking at the version number.  The best way to determine
C        the routine you had previously is to compare the dates
C        stored in the Version line of the routine.
C
C-    SPICELIB Version 5.0.0, 22-AUG-1990 (NJB)
C
C        Increased value of parameter MAXVAL to 5000 to accommodate
C        storage of SCLK coefficients in the kernel pool.
C
C        Also, changed version number in previous `Revisions' entry
C        from SPICELIB Version 2.0.0 to SPICELIB Version 2.0.0.  The
C        last version entry in the `Version' section had been
C        Version 1.0.0, dated later than the entry for `version 2'
C        in the revisions section!
C
C-    SPICELIB Version 4.0.0, 12-JUN-1990 (IMU)
C
C        All entry points except POOL and CLPOOL now initialize the
C        pool if it has not been done yet.
C
C-    SPICELIB Version 3.0.0, 23-OCT-1989 (HAN)
C
C        Added declaration of FAILED. FAILED is checked in the
C        DO-loops in LDPOOL and WRPOOL to prevent infinite looping.
C
C-    SPICELIB Version 2.0.0, 18-OCT-1989 (RET)
C
C       A FAILED test was inserted into the control of the DO-loop which
C       reads in each kernel variable.
C
C       Previously, if the error action 'RETURN' had been set by a
C       calling program, and the call to RDKNEW by LDPOOL failed,
C       then execution would continue through LDPOOL, with SPICELIB
C       routines returning upon entry. This meant that the routine
C       RDKVAR never got a chance to set the EOF flag, which was the
C       only control of the DO-loop. An infinite loop resulted in such
C       cases.  The FAILED test resolves that situation.
C
C-    SPICELIB Version 1.2.0, 9-MAR-1989 (HAN)
C
C        Parameters BEGDAT and BEGTXT have been moved into the
C        Declarations section.
C
C-    SPICELIB Version 1.1.0, 16-FEB-1989 (IMU) (NJB)
C
C        Parameters MAXVAR, MAXVAL, MAXLEN moved into Declarations.
C        (Actually, MAXLEN was implicitly 32 characters, and has only
C        now been made an explicit---and changeable---limit.)
C
C        Declaration of unused function FAILED removed.
C
C-    SPICELIB Version 1.0.0, 8-JAN-1989 (IMU)
C
C-&
 
 
 
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
      LOGICAL               FAILED
      LOGICAL               ELEMC
      INTEGER               BSRCHC
      INTEGER               CARDC
      INTEGER               SYDIMD
 
C
C     Local variables
C
 
 
C
C     Because some environments (such as the SUN) are too stupid to
C     treat the backslash character correctly we have to go through
C     some girations to put it into a variable in a "portable" way.
C     This is the reason for the following block of declarations.
C     Admittedly this is bizarre, but it works.
C
      INTEGER               MRKLEN
      PARAMETER           ( MRKLEN = 10 )
 
      INTEGER               SHRTLN
      PARAMETER           ( SHRTLN =  9 )
 
      INTEGER               BSLASH
      PARAMETER           ( BSLASH = 92 )
 
      CHARACTER*(MRKLEN)    BEGDAT
      CHARACTER*(MRKLEN)    BEGTXT
 
      CHARACTER*(SHRTLN)    DAT
      PARAMETER           ( DAT = 'begindata' )
 
      CHARACTER*(SHRTLN)    TXT
      PARAMETER           ( TXT = 'begintext' )
 
 
 
C
C     The following is the symbol table used for holding kernel pool
C     variables.
C
      INTEGER               LBCELL
      PARAMETER           ( LBCELL = -5 )
 
      CHARACTER*(MAXLEN)    TABSYM   ( LBCELL:MAXVAR )
      INTEGER               TABPTR   ( LBCELL:MAXVAR )
      DOUBLE PRECISION      TABVAL   ( LBCELL:MAXVAL )
 
 
 
C
C     The WAT... variables make up the symbol table that contains
C     variables (WATSYM) and their associated agents (WATVAL).
C
      CHARACTER*(MAXLEN)    WATSYM   ( LBCELL:MAXVAR )
      INTEGER               WATPTR   ( LBCELL:MAXVAR )
      CHARACTER*(MAXLEN)    WATVAL   ( LBCELL:MXNOTE )
 
 
C
C     Agents contains the list of agents that need to be notified
C     about updates to their variables.  NOTIFY and ACTIVE are both
C     temporary sets.
C
      CHARACTER*(MAXLEN)    ACTIVE   ( LBCELL:MAXAGT )
      CHARACTER*(MAXLEN)    AGENTS   ( LBCELL:MAXAGT )
      CHARACTER*(MAXLEN)    NOTIFY   ( LBCELL:MAXAGT )
 
C
C     First is our initialization flag.
C
      LOGICAL               FIRST
 
C
C     The remaining local variables...
C
      CHARACTER*(MAXLEN)    VARNAM
      LOGICAL               EOF
      LOGICAL               GOT
      LOGICAL               GOTIT
      LOGICAL               MORE
      LOGICAL               NOAGNT
      INTEGER               IOSTAT
      INTEGER               I
      INTEGER               NFETCH
 
C
C     Save EVERYTHING.
C
      SAVE
 
 
 
C
C     Initial values
C
      DATA                  FIRST / .TRUE. /
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'POOL' )
      END IF
 
C
C     This routine should never be called. If this routine is called,
C     an error is signalled.
C
      CALL SETMSG ( 'POOL: You have called an entry which performs '  //
     .              'performs no run-time function. This may '        //
     .              'indicate a bug. Please check the documentation ' //
     .              'for the subroutine POOL.' )
 
      CALL SIGERR ( 'SPICE(BOGUSENTRY)' )
 
      CALL CHKOUT ( 'POOL' )
      RETURN
 
 
 
 
 
C$Procedure CLPOOL ( Clear the pool of kernel variables )
 
      ENTRY CLPOOL
 
C$ Abstract
C
C     Remove all variables from the kernel pool.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     CONSTANTS
C     FILES
C
C$ Declarations
C
C     None.
C
C$ Brief_I/O
C
C     None.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) All known agents (those established through SWPOOL) will
C        be "notified" that their watched variables have been updated
C        whenever CLPOOL is called.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     CLPOOL clears the pool of kernel variables maintained by
C     the subroutine POOL. All the variables in the pool are deleted.
C     However, all watcher information is retained.
C
C     Each watched variable will be regarded as having been updated.
C     Any agent associated with that variable will have a notice
C     posted for it indicating that it's watched variable has been
C     updated.
C
C$ Examples
C
C
C     The following code fragment demonstrates how the data from
C     several kernel files can be loaded into a kernel pool. After the
C     pool is loaded, the values in the pool are written to a kernel
C     file.
C
C
C     C
C     C     Store in an array the names of the kernel files whose
C     C     values will be loaded into the kernel pool.
C     C
C           KERNEL (1) = 'AXES.KER'
C           KERNEL (2) = 'GM.KER'
C           KERNEL (3) = 'LEAP_SECONDS.KER'
C
C     C
C     C     Clear the kernel pool. (This is optional.)
C     C
C           CALL CLPOOL
C
C     C
C     C     Load the variables from the three kernel files into the
C     C     the kernel pool.
C     C
C           DO I = 1, 3
C             CALL LDPOOL ( KERNEL (I) )
C           END DO
C
C     C
C     C     We can examine the values associated with any variable
C     C     in the kernel pool using RTPOOL.
C     C
C           CALL RTPOOL ( VARIABLE, NUMVAL, VALUES, FOUND )
C
C     C
C     C     Get a free logical unit and open the file 'NEWKERNEL.KER'.
C     C
C           CALL GETLUN ( UNIT )
C
C           OPEN ( FILE            = 'NEWKERNEL.KER',
C          .       UNIT            = UNIT,
C          .       STATUS          = 'NEW',
C          .       IOSTAT          = IOSTAT,
C          .       CARRIAGECONTROL = 'LIST'  )
C
C
C     C
C     C     Write the values in the kernel pool to the file.
C     C
C           CALL WRPOOL ( UNIT )
C
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C     W.L. Taber      (JPL)
C
C$ Version
C
C-    SPICELIB Version 6.0.0, 31-MAR-1992 (WLT)
C
C        The entry points SWPOOL and CVPOOL were added.
C
C-    SPICELIB Version 4.0.0, 12-JUN-1990 (IMU)
C
C        All entry points except POOL and CLPOOL now initialize the
C        pool if it has not been done yet.
C
C-    SPICELIB Version 1.0.0, 8-JAN-1989 (IMU)
C
C-&
 
C$ Index_Entries
C
C     CLEAR the pool of kernel variables
C
C-&
 
C$ Revisions
C
C-    SPICELIB Version 6.0.0, 31-MAR-1992 (WLT)
C
C        The entry points SWPOOL (set watch on a pool variable)
C        and CVPOOL (check variable for update) so that routines
C        that buffer data stored in the kernel pool can fetch
C        that data only when it is updated.
C
C
C        Also the control of initializations was modified to be
C        consistent with other SPICELIB practices.
C
C        Finally, the revision history was upgraded so that the
C        version number increases over time.  This wasn't true
C        before. In addition some early revision data that referred to
C        pre-SPICELIB modifications were removed. This editing of
C        the version numbers makes it unlikely that anyone can track
C        down which previous version of this routine they have by
C        looking at the version number.  The best way to determine
C        the routine you had previously is to compare the dates
C        stored in the Version line of the routine.
C
C-    SPICELIB Version 4.0.0, 12-JUN-1990 (IMU)
C
C        All entry points except POOL and CLPOOL now initialize the
C        pool if it has not been done yet.
C
C-    SPICELIB Version 1.0.0, 8-JAN-1989 (IMU)
C
C-&
 
 
 
 
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'CLPOOL' )
      END IF
 
 
C
C     Just initialize the component table.
C
      CALL SSIZEC ( MAXVAR, TABSYM )
      CALL SSIZEI ( MAXVAR, TABPTR )
      CALL SSIZED ( MAXVAL, TABVAL )
 
      IF ( FIRST ) THEN
 
         BEGDAT = CHAR(BSLASH) // DAT
         BEGTXT = CHAR(BSLASH) // TXT
 
         CALL SSIZEC ( MAXVAR, WATSYM )
         CALL SSIZEI ( MAXVAR, WATPTR )
         CALL SSIZEC ( MXNOTE, WATVAL )
 
         CALL SSIZEC ( MAXAGT, AGENTS )
         CALL SSIZEC ( MAXAGT, ACTIVE )
         CALL SSIZEC ( MAXAGT, NOTIFY )
 
         IF ( .NOT. FAILED() ) THEN
            FIRST = .FALSE.
         END IF
 
         CALL CHKOUT ( 'CLPOOL' )
         RETURN
 
      END IF
 
C
C     Copy all of the current AGENTS to notify onto the ACTIVE list.
C
      CALL COPYC ( AGENTS, ACTIVE )
 
      DO 50001
     .   I = 1, CARDC ( WATSYM )
 
C
C        Get the name of the i'th variable to watch and look up its
C        associated agents.
C
         CALL SYFETC( I,      WATSYM, WATPTR, WATVAL, VARNAM, GOTIT    )
         CALL SYGETC( VARNAM, WATSYM, WATPTR, WATVAL, NFETCH, NOTIFY(1),
     .                                                        GOTIT    )
 
         CALL VALIDC( MAXAGT,                         NFETCH, NOTIFY   )
         CALL UNIONC( ACTIVE, NOTIFY,                         AGENTS   )
         CALL COPYC ( AGENTS,                                 ACTIVE   )
 
50001 CONTINUE
 
 
      CALL CHKOUT ( 'CLPOOL' )
      RETURN
 
 
 
C$Procedure LDPOOL ( Load variables from a kernel file into the pool )
 
      ENTRY LDPOOL ( KERNEL )
 
C$ Abstract
C
C     Load the variables contained in a NAIF ASCII kernel file into the
C     kernel pool.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     CONSTANTS
C     FILES
C
C$ Declarations
C
C     CHARACTER*(*)         KERNEL
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     KERNEL     I   Name of the kernel file.
C
C$ Detailed_Input
C
C     KERNEL     is the name of the kernel file whose variables will be
C                loaded into the pool.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     The NAIF ASCII kernel file KERNEL is opened by RDKNEW.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C     The following code fragment demonstrates how the data from
C     several kernel files can be loaded into a kernel pool. After the
C     pool is loaded, the values in the pool are written to a kernel
C     file.
C
C     C
C     C     Store in an array the names of the kernel files whose
C     C     values will be loaded into the kernel pool.
C     C
C           KERNEL (1) = 'AXES.KER'
C           KERNEL (2) = 'GM.KER'
C           KERNEL (3) = 'LEAP_SECONDS.KER'
C
C     C
C     C     Clear the kernel pool. (This is optional.)
C     C
C           CALL CLPOOL
C
C     C
C     C     Load the variables from the three kernel files into the
C     C     the kernel pool.
C     C
C           DO I = 1, 3
C             CALL LDPOOL ( KERNEL (I) )
C           END DO
C
C     C
C     C     We can examine the values associated with any variable
C     C     in the kernel pool using RTPOOL.
C     C
C           CALL RTPOOL ( VARIABLE, NUMVAL, VALUES, FOUND )
C
C     C
C     C     Get a free logical unit and open the file 'NEWKERNEL.KER'.
C     C
C           CALL GETLUN ( UNIT )
C
C           OPEN ( FILE            = 'NEWKERNEL.KER',
C          .       UNIT            = UNIT,
C          .       STATUS          = 'NEW',
C          .       IOSTAT          = IOSTAT,
C          .       CARRIAGECONTROL = 'LIST'  )
C
C
C     C
C     C     Write the values in the kernel pool to the file.
C     C
C           CALL WRPOOL ( UNIT )
C
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     R.E. Thurman    (JPL)
C     I.M. Underwood  (JPL)
C     W.L. Taber      (JPL)
C
C$ Version
C
C-    SPICELIB Version 6.0.0, 31-MAR-1992 (WLT)
C
C        The entry points SWPOOL and CVPOOL were added.
C
C-    SPICELIB Version 5.0.0, 22-AUG-1990 (NJB)
C
C        Increased value of parameter MAXVAL to 5000 to accommodate
C        storage of SCLK coefficients in the kernel pool.
C
C-    SPICELIB Version 4.0.0, 12-JUN-1990 (IMU)
C
C        All entry points except POOL and CLPOOL now initialize the
C        pool if it has not been done yet.
C
C-    SPICELIB Version 3.0.0, 23-OCT-1989 (HAN)
C
C        Added declaration of FAILED. FAILED is checked in the
C        DO-loops in LDPOOL and WRPOOL to prevent infinite looping.
C
C-    SPICELIB Version 2.0.0, 18-OCT-1989 (RET)
C
C       A FAILED test was inserted into the control of the DO-loop which
C       reads in each kernel variable in LDPOOL.
C
C-    SPICELIB Version 1.2.0, 9-MAR-1989 (HAN)
C
C        Parameters BEGDAT and BEGTXT have been moved into the
C        Declarations section.
C
C-    SPICELIB Version 1.1.0, 16-FEB-1989 (IMU) (NJB)
C
C        Parameters MAXVAR, MAXVAL, MAXLEN moved into Declarations.
C        (Actually, MAXLEN was implicitly 32 characters, and has only
C        now been made an explicit---and changeable---limit.)
C
C        Declaration of unused function FAILED removed.
C
C-    SPICELIB Version 1.0.0, 8-JAN-1989 (IMU)
C
C-&
 
C$ Index_Entries
C
C     LOAD variables from a text kernel file into the pool
C
C-&
 
C$ Revisions
C
C-    SPICELIB Version 6.0.0, 31-MAR-1992 (WLT)
C
C        The entry points SWPOOL (set watch on a pool variable)
C        and CVPOOL (check variable for update) so that routines
C        that buffer data stored in the kernel pool can fetch
C        that data only when it is updated.
C
C        In addition, the revision history was upgraded so that the
C        version number increases over time.  This wasn't true
C        before. In addition some early revision data that referred to
C        pre-SPICELIB modifications were removed. This editing of
C        the version numbers makes it unlikely that anyone can track
C        down which previous version of this routine they have by
C        looking at the version number.  The best way to determine
C        the routine you had previously is to compare the dates
C        stored in the Version line of the routine.
C
C-    SPICELIB Version 5.0.0, 22-AUG-1990 (NJB)
C
C        Increased value of parameter MAXVAL to 5000 to accommodate
C        storage of SCLK coefficients in the kernel pool.
C
C        Also, changed version number in previous `Revisions' entry
C        from SPICELIB Version 2.0.0 to SPICELIB Version 2.0.0.  The
C        last version entry in the `Version' section had been
C        Version 1.0.0, dated later than the entry for `version 2'
C        in the revisions section!
C
C-    SPICELIB Version 4.0.0, 12-JUN-1990 (IMU)
C
C        All entry points except POOL and CLPOOL now initialize the
C        pool if it has not been done yet.
C
C-    SPICELIB Version 3.0.0, 23-OCT-1989 (HAN)
C
C        Added declaration of FAILED. FAILED is checked in the
C        DO-loops in LDPOOL and WRPOOL to prevent infinite looping.
C
C-    SPICELIB Version 2.0.0, 18-OCT-1989 (RET)
C
C       A FAILED test was inserted into the control of the DO-loop which
C       reads in each kernel variable.
C
C       Previously, if the error action 'RETURN' had been set by a
C       calling program, and the call to RDKNEW by LDPOOL failed,
C       then execution would continue through LDPOOL, with SPICELIB
C       routines returning upon entry. This meant that the routine
C       RDKVAR never got a chance to set the EOF flag, which was the
C       only control of the DO-loop. An infinite loop resulted in such
C       cases.  The FAILED test resolves that situation.
C
C-    SPICELIB Version 1.2.0, 9-MAR-1989 (HAN)
C
C        Parameters BEGDAT and BEGTXT have been moved into the
C        Declarations section.
C
C-    SPICELIB Version 1.1.0, 16-FEB-1989 (IMU) (NJB)
C
C        Parameters MAXVAR, MAXVAL, MAXLEN moved into Declarations.
C        (Actually, MAXLEN was implicitly 32 characters, and has only
C        now been made an explicit---and changeable---limit.)
C
C        Declaration of unused function FAILED removed.
C
C-    SPICELIB Version 1.0.0, 8-JAN-1989 (IMU)
C
C-&
 
 
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'LDPOOL' )
      END IF
 
C
C     Initialize the pool if necessary.
C
      IF ( FIRST ) THEN
 
         BEGDAT = CHAR(BSLASH) // DAT
         BEGTXT = CHAR(BSLASH) // TXT
 
         CALL SSIZEC ( MAXVAR, TABSYM )
         CALL SSIZEI ( MAXVAR, TABPTR )
         CALL SSIZED ( MAXVAL, TABVAL )
 
         CALL SSIZEC ( MAXVAR, WATSYM )
         CALL SSIZEI ( MAXVAR, WATPTR )
         CALL SSIZEC ( MXNOTE, WATVAL )
 
         CALL SSIZEC ( MAXAGT, AGENTS )
         CALL SSIZEC ( MAXAGT, ACTIVE )
         CALL SSIZEC ( MAXAGT, NOTIFY )
 
         IF ( FAILED() ) THEN
            CALL CHKOUT ( 'LDPOOL' )
            RETURN
         ELSE
            FIRST = .FALSE.
         END IF
      END IF
 
C
C     Open the kernel file and read the first variable.
C
      CALL RDKNEW ( KERNEL )
      CALL RDKVAR ( TABSYM, TABPTR, TABVAL, VARNAM, EOF )
 
C
C     Read the variables in the file, one at a time.
C
50002 IF       ( .NOT. EOF .AND. .NOT. FAILED() )
     .THEN
 
         IF ( VARNAM .NE. ' ' ) THEN
 
C
C           See if this variable is being watched ...
C
            CALL SYGETC ( VARNAM, WATSYM,
     .                            WATPTR,
     .                            WATVAL, NFETCH, NOTIFY(1), GOT )
 
C
C           ... if it is, add its associated agents to the list of
C           AGENTS to be notified of a watched variable update.
C
            IF ( GOT ) THEN
               CALL COPYC  ( AGENTS, ACTIVE         )
               CALL VALIDC ( MAXAGT, NFETCH, NOTIFY )
               CALL UNIONC ( NOTIFY, ACTIVE, AGENTS )
            END IF
 
         END IF
 
         CALL RDKVAR ( TABSYM, TABPTR, TABVAL, VARNAM, EOF )
 
         GO TO 50002
      END IF
 
      CALL CHKOUT ( 'LDPOOL' )
      RETURN
 
 
 
 
C$Procedure RTPOOL ( Return the value of a pooled kernel variable )
 
      ENTRY RTPOOL ( NAME, N, VALUES, FOUND )
 
C$ Abstract
C
C     Return the value of a kernel variable from the kernel pool.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     CONSTANTS
C     FILES
C
C$ Declarations
C
C     CHARACTER*(*)         NAME
C     INTEGER               N
C     DOUBLE PRECISION      VALUES   ( * )
C     LOGICAL               FOUND
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NAME       I   Name of the variable whose value is to be returned.
C     N          O   Number of values associated with NAME.
C     VALUES     O   Values associated with NAME.
C     FOUND      O   True if variable is in pool.
C
C$ Detailed_Input
C
C     NAME       is the name of the variable whose values are to be
C                returned. If the variable is not in the pool, FOUND
C                will be FALSE.
C
C$ Detailed_Output
C
C     N          is the number of values associated with NAME.
C                If NAME is not in the pool, no value is given to
C                N.
C
C     VALUES     is the array of values associated with NAME.
C                If NAME is not in the pool, no values are given to
C                the elements of VALUES.
C
C     FOUND      is TRUE if the variable is in the pool, FALSE if it
C                is not.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C
C     The following code fragment demonstrates how the data from
C     several kernel files can be loaded into a kernel pool. After the
C     pool is loaded, the values in the pool are written to a kernel
C     file.
C
C
C     C
C     C     Store in an array the names of the kernel files whose
C     C     values will be loaded into the kernel pool.
C     C
C           KERNEL (1) = 'AXES.KER'
C           KERNEL (2) = 'GM.KER'
C           KERNEL (3) = 'LEAP_SECONDS.KER'
C
C     C
C     C     Clear the kernel pool. (This is optional.)
C     C
C           CALL CLPOOL
C
C     C
C     C     Load the variables from the three kernel files into the
C     C     the kernel pool.
C     C
C           DO I = 1, 3
C             CALL LDPOOL ( KERNEL (I) )
C           END DO
C
C     C
C     C     We can examine the values associated with any variable
C     C     in the kernel pool using RTPOOL.
C     C
C           CALL RTPOOL ( VARIABLE, NUMVAL, VALUES, FOUND )
C
C     C
C     C     Get a free logical unit and open the file 'NEWKERNEL.KER'.
C     C
C           CALL GETLUN ( UNIT )
C
C           OPEN ( FILE            = 'NEWKERNEL.KER',
C          .       UNIT            = UNIT,
C          .       STATUS          = 'NEW',
C          .       IOSTAT          = IOSTAT,
C          .       CARRIAGECONTROL = 'LIST'  )
C
C
C     C
C     C     Write the values in the kernel pool to the file.
C     C
C           CALL WRPOOL ( UNIT )
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 4.0.0, 12-JUN-1990 (IMU)
C
C        All entry points except POOL and CLPOOL now initialize the
C        pool if it has not been done yet.
C
C-    SPICELIB Version 1.0.0, 8-JAN-1989 (IMU)
C
C-&
 
C$ Index_Entries
C
C     RETURN the value of a pooled kernel variable
C
C-&
 
C$ Revisions
C
C-    SPICELIB Version 4.0.0, 12-JUN-1990 (IMU)
C
C        All entry points except POOL and CLPOOL now initialize the
C        pool if it has not been done yet.
C
C-    SPICELIB Version 1.0.0, 8-JAN-1989 (IMU)
C
C-&
 
 
 
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'RTPOOL' )
      END IF
 
 
C
C     Initialize the pool if necessary.
C
      IF ( FIRST ) THEN
 
         BEGDAT = CHAR(BSLASH) // DAT
         BEGTXT = CHAR(BSLASH) // TXT
 
         CALL SSIZEC ( MAXVAR, TABSYM )
         CALL SSIZEI ( MAXVAR, TABPTR )
         CALL SSIZED ( MAXVAL, TABVAL )
 
         CALL SSIZEC ( MAXVAR, WATSYM )
         CALL SSIZEI ( MAXVAR, WATPTR )
         CALL SSIZEC ( MXNOTE, WATVAL )
 
         CALL SSIZEC ( MAXAGT, AGENTS )
         CALL SSIZEC ( MAXAGT, ACTIVE )
         CALL SSIZEC ( MAXAGT, NOTIFY )
 
         IF ( FAILED() ) THEN
            CALL CHKOUT ( 'RTPOOL' )
            RETURN
         ELSE
            FIRST = .FALSE.
         END IF
      END IF
 
C
C     Either it's there or it's not.
C
      CALL SYGETD ( NAME, TABSYM, TABPTR, TABVAL, N, VALUES, FOUND )
 
 
 
      CALL CHKOUT ( 'RTPOOL' )
      RETURN
 
 
 
 
C$Procedure EXPOOL ( Confirm the existence of a pooled kernel variable )
 
      ENTRY EXPOOL ( NAME, FOUND )
 
C$ Abstract
C
C     Confirm the existence of a kernel variable in the kernel pool.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     CONSTANTS
C     FILES
C
C$ Declarations
C
C     CHARACTER*(*)         NAME
C     LOGICAL               FOUND
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NAME       I   Name of the variable whose value is to be returned.
C     FOUND      O   True when the variable is in the pool.
C
C$ Detailed_Input
C
C     NAME       is the name of the variable whose values are to be
C                returned.
C
C$ Detailed_Output
C
C     FOUND      is true whenever the specified variable is included
C                in the pool.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C     See BODFND.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 4.0.0, 12-JUN-1990 (IMU)
C
C        All entry points except POOL and CLPOOL now initialize the
C        pool if it has not been done yet.
C
C-    SPICELIB Version 1.0.0, 8-JAN-1989 (IMU)
C
C-&
 
C$ Index_Entries
C
C     CONFIRM the existence of a pooled kernel variable
C
C-&
 
C$ Revisions
C
C-    SPICELIB Version 4.0.0, 12-JUN-1990 (IMU)
C
C        All entry points except POOL and CLPOOL now initialize the
C        pool if it has not been done yet.
C
C-    SPICELIB Version 1.0.0, 8-JAN-1989 (IMU)
C
C-&
 
 
 
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'EXPOOL' )
      END IF
 
 
C
C     Initialize the pool if necessary.
C
      IF ( FIRST ) THEN
 
         BEGDAT = CHAR(BSLASH) // DAT
         BEGTXT = CHAR(BSLASH) // TXT
 
         CALL SSIZEC ( MAXVAR, TABSYM )
         CALL SSIZEI ( MAXVAR, TABPTR )
         CALL SSIZED ( MAXVAL, TABVAL )
 
         CALL SSIZEC ( MAXVAR, WATSYM )
         CALL SSIZEI ( MAXVAR, WATPTR )
         CALL SSIZEC ( MXNOTE, WATVAL )
 
         CALL SSIZEC ( MAXAGT, AGENTS )
         CALL SSIZEC ( MAXAGT, ACTIVE )
         CALL SSIZEC ( MAXAGT, NOTIFY )
 
         IF ( FAILED() ) THEN
            CALL CHKOUT ( 'EXPOOL' )
            RETURN
         ELSE
            FIRST = .FALSE.
         END IF
      END IF
 
C
C     Either it's there or it's not. If it is, it has at least
C     one value.
C
      FOUND = ( SYDIMD ( NAME, TABSYM, TABPTR, TABVAL ) .GT. 0 )
 
      CALL CHKOUT ( 'EXPOOL' )
      RETURN
 
 
 
C$Procedure WRPOOL ( Write the values in pool to a specified unit )
 
      ENTRY WRPOOL ( UNIT )
 
C$ Abstract
C
C     Write the values in the pool to the specified unit.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     CONSTANTS
C     FILES
C
C$ Declarations
C
C     INTEGER        UNIT
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     UNIT       I   Logical unit to which the values in the pool will
C                    be written.
C
C$ Detailed_Input
C
C     UNIT       is the logical unit to which the values in the pool
C                will be written.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     If the values are to be written to an output kernel file, the
C     file should be opened with a logical unit determined by the
C     calling program.
C
C$ Exceptions
C
C     None.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C
C     The following code fragment demonstrates how the data from
C     several kernel files can be loaded into a kernel pool. After the
C     pool is loaded, the values in the pool are written to a kernel
C     file.
C
C
C     C
C     C     Store in an array the names of the kernel files whose
C     C     values will be loaded into the kernel pool.
C     C
C           KERNEL (1) = 'AXES.KER'
C           KERNEL (2) = 'GM.KER'
C           KERNEL (3) = 'LEAP_SECONDS.KER'
C
C     C
C     C     Clear the kernel pool. (This is optional.)
C     C
C           CALL CLPOOL
C
C     C
C     C     Load the variables from the three kernel files into the
C     C     the kernel pool.
C     C
C           DO I = 1, 3
C             CALL LDPOOL ( KERNEL (I) )
C           END DO
C
C     C
C     C     We can examine the values associated with any variable
C     C     in the kernel pool using RTPOOL.
C     C
C           CALL RTPOOL ( VARIABLE, NUMVAL, VALUES, FOUND )
C
C     C
C     C     Get a free logical unit and open the file 'NEWKERNEL.KER'.
C     C
C           CALL GETLUN ( UNIT )
C
C           OPEN ( FILE            = 'NEWKERNEL.KER',
C          .       UNIT            = UNIT,
C          .       STATUS          = 'NEW',
C          .       IOSTAT          = IOSTAT,
C          .       CARRIAGECONTROL = 'LIST'  )
C
C
C     C
C     C     Write the values in the kernel pool to the file.
C     C
C           CALL WRPOOL ( UNIT )
C
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     H.A. Neilan     (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 5.0.0, 22-AUG-1990 (NJB)
C
C        Increased value of parameter MAXVAL to 5000 to accommodate
C        storage of SCLK coefficients in the kernel pool.
C
C-    SPICELIB Version 4.0.0, 12-JUN-1990 (IMU)
C
C        All entry points except POOL and CLPOOL now initialize the
C        pool if it has not been done yet.
C
C-    SPICELIB Version 3.0.0, 23-OCT-1989 (HAN)
C
C        Added declaration of FAILED. FAILED is checked in the
C        DO-loops in LDPOOL and WRPOOL to prevent infinite looping.
C
C-    SPICELIB Version 1.2.0, 9-MAR-1989 (HAN)
C
C        Parameters BEGDAT and BEGTXT have been moved into the
C        Declarations section.
C
C-    SPICELIB Version 1.1.0, 16-FEB-1989 (IMU) (NJB)
C
C        Parameters MAXVAR, MAXVAL, MAXLEN moved into Declarations.
C        (Actually, MAXLEN was implicitly 32 characters, and has only
C        now been made an explicit---and changeable---limit.)
C
C        Declaration of unused function FAILED removed.
C
C-    SPICELIB Version 1.0.0, 8-JAN-1989 (IMU)
C
C-&
 
C$ Index_Entries
C
C     WRITE the values in pool to a specified unit
C
C-&
 
C$ Revisions
C
C-    SPICELIB Version 5.0.0, 22-AUG-1990 (NJB)
C
C        Increased value of parameter MAXVAL to 5000 to accommodate
C        storage of SCLK coefficients in the kernel pool.
C
C        Also, changed version number in previous `Revisions' entry
C        from SPICELIB Version 2.0.0 to SPICELIB Version 2.0.0.  The
C        last version entry in the `Version' section had been
C        Version 1.0.0, dated later than the entry for `version 2'
C        in the revisions section!
C
C-    SPICELIB Version 4.0.0, 12-JUN-1990 (IMU)
C
C        All entry points except POOL and CLPOOL now initialize the
C        pool if it has not been done yet.
C
C-    SPICELIB Version 3.0.0, 23-OCT-1989 (HAN)
C
C        Added declaration of FAILED. FAILED is checked in the
C        DO-loops in LDPOOL and WRPOOL to prevent infinite looping.
C
C-    SPICELIB Version 2.0.0, 18-OCT-1989 (RET)
C
C       A FAILED test was inserted into the control of the DO-loop which
C       reads in each kernel variable.
C
C       Previously, if the error action 'RETURN' had been set by a
C       calling program, and the call to RDKNEW by LDPOOL failed,
C       then execution would continue through LDPOOL, with SPICELIB
C       routines returning upon entry. This meant that the routine
C       RDKVAR never got a chance to set the EOF flag, which was the
C       only control of the DO-loop. An infinite loop resulted in such
C       cases.  The FAILED test resolves that situation.
C
C-    SPICELIB Version 1.2.0, 9-MAR-1989 (HAN)
C
C        Parameters BEGDAT and BEGTXT have been moved into the
C        Declarations section.
C
C-    SPICELIB Version 1.1.0, 16-FEB-1989 (IMU) (NJB)
C
C        Parameters MAXVAR, MAXVAL, MAXLEN moved into Declarations.
C        (Actually, MAXLEN was implicitly 32 characters, and has only
C        now been made an explicit---and changeable---limit.)
C
C        Declaration of unused function FAILED removed.
C
C-    SPICELIB Version 1.0.0, 8-JAN-1989 (IMU)
C
C-&
 
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'WRPOOL' )
      END IF
 
 
C
C     Initialize the pool if necessary.
C
      IF ( FIRST ) THEN
 
         BEGDAT = CHAR(BSLASH) // DAT
         BEGTXT = CHAR(BSLASH) // TXT
 
         CALL SSIZEC ( MAXVAR, TABSYM )
         CALL SSIZEI ( MAXVAR, TABPTR )
         CALL SSIZED ( MAXVAL, TABVAL )
 
         CALL SSIZEC ( MAXVAR, WATSYM )
         CALL SSIZEI ( MAXVAR, WATPTR )
         CALL SSIZEC ( MXNOTE, WATVAL )
 
         CALL SSIZEC ( MAXAGT, AGENTS )
         CALL SSIZEC ( MAXAGT, ACTIVE )
         CALL SSIZEC ( MAXAGT, NOTIFY )
 
         IF ( FAILED() ) THEN
            CALL CHKOUT ( 'WRPOOL' )
            RETURN
         ELSE
            FIRST = .FALSE.
         END IF
      END IF
 
C
C     Indicate the beginning of a data section.
C
      WRITE (UNIT,*,IOSTAT=IOSTAT) BEGDAT
 
C
C     Obtain the first variable name.
C
      I = 1
      CALL SYFETD ( I, TABSYM, TABPTR, TABVAL, VARNAM, MORE )
 
 
C
C     While there are variables in the pool, write the
C     variables and their values to the specified unit.
C
50003 IF       ( MORE .AND. ( .NOT. FAILED() ) )
     .THEN
 
         WRITE (UNIT,*,IOSTAT=IOSTAT)
         CALL WRKVAR ( UNIT,   VARNAM, '=',
     .                 TABSYM, TABPTR, TABVAL )
 
         I = I + 1
         CALL SYFETD ( I, TABSYM, TABPTR, TABVAL, VARNAM, MORE )
 
         GO TO 50003
      END IF
 
 
C
C     Indicate the beginning of a text section. Data sections and
C     text sections must alternate, even if the text section is blank.
C
      WRITE (UNIT,*,IOSTAT=IOSTAT) BEGTXT
 
      CALL CHKOUT ( 'WRPOOL' )
      RETURN
 
 
C$Procedure SWPOOL ( Set watch on a pool variable )
 
      ENTRY SWPOOL ( AGENT, NNAMES, NAMES )
 
C$ Abstract
C
C     Add a name to the list of agents to notify whenever a member of
C     a list of kernel variables is updated.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     CONSTANTS
C     FILES
C
C$ Declarations
C
C     CHARACTER*(*)         AGENT
C     INTEGER               NNAMES
C     CHARACTER*(*)         NAMES  ( * )
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     AGENT      I   The name of an agent to be notified after updates.
C     NNAMES     I   The number of variables to associate with AGENT.
C     NAMES      I   Variable names whose update causes the notice.
C
C$ Detailed_Input
C
C     AGENT       is the name of a routine or entry point (agency) that
C                 will want to know when a some variables in the kernel
C                 pool have been updated.
C
C     NNAMES      is the number of kernel pool variable names that will
C                 be associated with AGENT.
C
C     NAMES       is an array of names of variables in the kernel pool.
C                 Whenever any of these is updated, a notice will be
C                 posted for AGENT so that one can quickly check
C                 whether needed data has been modified.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If sufficient room is not available to hold the name or
C        new agent, a routine in the calling tree for this routine
C        will signal an error.
C
C$ Particulars
C
C     The kernel pool is a convenient place to store a wide
C     variety of data needed by routines in SPICELIB and routines
C     that interface with SPICELIB routines.  However, when
C     a single name has a large quantity of data associated with
C     it, it becomes inefficient to constantly query the kernel
C     pool for values that are not updated on a frequent basis.
C
C     This entry point allows a routine to instruct the kernel pool
C     to post a message whenever a particular value gets updated.
C     In this way, a routine can quickly determine whether or not
C     data it requires has been updated since the last time the
C     data was accessed.  This makes it reasonable to buffer
C     the data in local storage and update it only when
C     a variable in the kernel pool that affects this data has
C     been updated.
C
C     Note that SWPOOL has a side effect.  Whenever a call to
C     SWPOOL is made, the agent specified in the calling sequence
C     is added to the list of agents that should be notified that
C     an update of its variables has occurred.  In other words
C     the code
C
C         CALL SWPOOL ( AGENT, NNAMES, NAMES  )
C         CALL CVPOOL ( AGENT,         UPDATE )
C
C     will always return UPDATE as .TRUE.
C
C     This feature allows for a slightly cleaner use of SWPOOL and
C     CVPOOL as shown in the example below.  Because SWPOOL
C     automatically loads AGENT into the list of agents to notify of
C     a kernel pool update, you do not have to include the code for
C     fetching the initial values of the kernel variables in the
C     initialization portion of a subroutine.  Instead, the code for
C     the first fetch from the pool is the same as the code for
C     fetching when the pool is updated.
C
C$ Examples
C
C     Suppose that you have an application subroutine, MYTASK, that
C     needs to access a large data set in the kernel pool.  If this
C     data could be kept in local storage and kernel pool queries
C     performed only when the data in the kernel pool has been
C     updated, the routine can perform much more efficiently.
C
C     The code fragment below illustrates how you might make use of this
C     feature.
C
C     C
C     C     On the first call to this routine establish those variables
C     C     that we will want to read from the kernel pool only when
C     C     new values have been established.
C     C
C           IF ( FIRST ) THEN
C
C              FIRST = .FALSE.
C              CALL SWPOOL ( 'MYTASK', NNAMES, NAMES )
C
C           END IF
C
C      C
C      C    If any of the variables has been updated fetch
C      C    them from the kernel pool. (Note that this also
C      C    handles getting variables the for the first time.)
C      C
C           CALL CVPOOL ( 'MYTASK', UPDATE )
C
C           IF ( UPDATE ) THEN
C
C              CALL RTPOOL ( 'MYTASK_VARIABLE_1', N1, VALS1, FOUND(1) )
C              CALL RTPOOL ( 'MYTASK_VARIABLE_2', N2, VALS2, FOUND(2) )
C                      .
C                      .
C                      .
C              CALL RTPOOL ( 'MYTASK_VARIABLE_N', NN, VALSN, FOUND(N) )
C
C           END IF
C
C           IF ( FAILED() ) THEN
C                 .
C                 .
C              do something about the failure
C                 .
C                 .
C
C           END IF
C
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber      (JPL)
C
C$ Version
C
C-    SPICELIB Version 6.0.0, 31-MAR-1992 (WLT)
C
C        The entry points SWPOOL and CVPOOL were added.
C
C-&
 
C$ Index_Entries
C
C     Watch for an update to a kernel pool variable
C     Notify a routine of an update to a kernel pool variable
C-&
 
C$ Revisions
C
C-    SPICELIB Version 6.0.0, 31-MAR-1992 (WLT)
C
C        The entry points SWPOOL (set watch on a pool variable)
C        and CVPOOL (check variable for update) so that routines
C        that buffer data stored in the kernel pool can fetch
C        that data only when it is updated.
C
C        In addition, the revision history was upgraded so that the
C        version number increases over time.  This wasn't true
C        before. In addition some early revision data that referred to
C        pre-SPICELIB modifications were removed. This editing of
C        the version numbers makes it unlikely that anyone can track
C        down which previous version of this routine they have by
C        looking at the version number.  The best way to determine
C        the routine you had previously is to compare the dates
C        stored in the Version line of the routine.
C
C
C-&
 
 
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SWPOOL' )
      END IF
 
 
C
C     Initialize the pool if necessary.
C
      IF ( FIRST ) THEN
 
         BEGDAT = CHAR(BSLASH) // DAT
         BEGTXT = CHAR(BSLASH) // TXT
 
         CALL SSIZEC ( MAXVAR, TABSYM )
         CALL SSIZEI ( MAXVAR, TABPTR )
         CALL SSIZED ( MAXVAL, TABVAL )
 
         CALL SSIZEC ( MAXVAR, WATSYM )
         CALL SSIZEI ( MAXVAR, WATPTR )
         CALL SSIZEC ( MXNOTE, WATVAL )
 
         CALL SSIZEC ( MAXAGT, AGENTS )
         CALL SSIZEC ( MAXAGT, ACTIVE )
         CALL SSIZEC ( MAXAGT, NOTIFY )
 
         IF ( FAILED() ) THEN
            CALL CHKOUT ( 'SWPOOL' )
            RETURN
         ELSE
            FIRST = .FALSE.
         END IF
      END IF
 
C
C     For each variable specified by the array NAMES, put AGENT
C     into its list of guys to be notified when a variable change
C     occurs.
C
      DO 50004
     .   I = 1, NNAMES
 
C
C        Get the agents associated with NAMES(I).
C
         CALL SYGETC ( NAMES(I), WATSYM,
     .                           WATPTR,
     .                           WATVAL, NFETCH, ACTIVE(1), GOT )
 
         IF ( GOT ) THEN
            NOAGNT = BSRCHC ( AGENT, NFETCH, ACTIVE(1) ) .EQ. 0
         ELSE
            NOAGNT = .TRUE.
         END IF
 
C
C        If we didn't find this agent in the list of agents for this
C        name, put him in and then sort the list of agents for
C        this name (NAMES(I)).
C
         IF ( NOAGNT .AND. CARDC(ACTIVE) .LT. MAXAGT ) THEN
 
            CALL SYPSHC ( NAMES(I), AGENT, WATSYM, WATPTR, WATVAL )
            CALL SYORDC ( NAMES(I),        WATSYM, WATPTR, WATVAL )
 
         ELSE IF ( NOAGNT ) THEN
 
            CALL SETMSG ( 'The list of agents to notify when # is '   //
     .                    'updated '                                  //
     .                    'is too big.  The maximum number of agents '//
     .                    'that any kernel pool variable can '        //
     .                    'activate is ?.'                            )
            CALL ERRCH  ( '#', NAMES(I)          )
            CALL ERRINT ( '?', MAXAGT            )
            CALL SIGERR ( 'SPICE(TOOMANYAGENTS)' )
            CALL CHKOUT ( 'SWPOOL'               )
            RETURN
 
         END IF
 
50004 CONTINUE
 
C
C     We ALWAYS put this agent into the list of agents to be notified.
C
      CALL INSRTC ( AGENT, AGENTS )
 
C
C     That is all.
C
      CALL CHKOUT ( 'SWPOOL' )
      RETURN
 
 
 
C$Procedure CVPOOL ( Check variable in the pool for update)
 
      ENTRY CVPOOL ( AGENT, UPDATE )
 
C$ Abstract
C
C     Determine whether or not any of the variables that are to
C     be watched and have AGENT on their distribution list have been
C     updated.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     SYMBOLS
C     UTILITY
C
C$ Declarations
C
C     CHARACTER*(*)         AGENT
C     LOGICAL               UPDATE
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     AGENT      I   Name of the agent to check for notices.
C     UPDATE     O   .TRUE. if variables for AGENT have been updated.
C
C$ Detailed_Input
C
C     AGENT     is the name of a subroutine, entry point, or significant
C               portion of code that needs to access variables in the
C               kernel pool.  Generally this agent will buffer these
C               variables internally and fetch them from the kernel
C               pool only when they are updated.
C
C$ Detailed_Output
C
C     UPDATE    is a logical flag that will be set to true if the
C               variables in the kernel pool that are required by
C               AGENT have been updated since the last call to CVPOOL.
C
C$ Parameters
C
C     See the umbrella subroutine POOL.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This entry point allows the calling program to determine
C     whether or not variables associated with with AGENT have
C     been updated.  Making use of this entry point in conjunction
C     with the entry point SWPOOL (set watch on pool variables)
C     modules can buffer kernel pool variables they need and
C     fetch values from the kernel pool only when variables have
C     been updated.
C
C     Note that the call to CVPOOL has a side effect.
C     Two consecutive calls to CVPOOL with the same
C     AGENT will always result in the UPDATE being .FALSE.
C     on the second call.  In other words, if you imbed
C     the following two lines of code in a piece of code
C
C                  CALL CVPOOL ( AGENT, UPDATE )
C                  CALL CVPOOL ( AGENT, UPDATE )
C
C     and then test UPDATE, it will be FALSE.  The idea is
C     that once a call to CVPOOL has been made, the
C     kernel pool has performed its duty and notified the
C     calling routine that one of the AGENT's variables
C     has been updated.  Consequently, on the second call
C     to CVPOOL above, the kernel pool will not have any
C     updates to report about any of AGENT's variables.
C
C     If, on the other hand, you have code such as
C
C                  CALL CVPOOL ( AGENT, UPDATE )
C                  CALL LDPOOL ( 'MYFILE.DAT'  )
C                  CALL CVPOOL ( AGENT, UPDATE )
C
C     the value of UPDATE will be true if one of the variables
C     associated with AGENT was updated by the call to
C     LDPOOL (and that variable has been specified as one
C     to watch by call a call to SWPOOL).
C
C     It should also be noted that any call to CVPOOL that
C     occurs immediately after a call to SWPOOL will result in
C     UPDATE being returned as .TRUE.  In other words, code
C     such as shown below, will always result in the value
C     of UPDATE as being returned .TRUE.
C
C                  CALL SWPOOL ( AGENT, NNAMES, NAMES  )
C                  CALL CVPOOL ( AGENT,         UPDATE )
C
C     See the header for SWPOOL for a full discussion of this
C     feature.
C
C$ Examples
C
C     Suppose that you have an application subroutine, MYTASK, that
C     needs to access a large data set in the kernel pool.  If this
C     data could be kept in local storage and kernel pool queries
C     performed only when the data in the kernel pool has been
C     updated, the routine can perform much more efficiently.
C
C     The code fragment below illustrates how you might make use of this
C     feature.
C
C     C
C     C     On the first call to this routine establish those variables
C     C     that we will want to read from the kernel pool only when
C     C     new values have been established.
C     C
C           IF ( FIRST ) THEN
C
C              FIRST = .FALSE.
C
C              CALL SWPOOL ( 'MYTASK', NNAMES, NAMES )
C
C           END IF
C
C      C
C      C    If any of the variables has been updated fetch
C      C    them from the kernel pool.
C      C
C           CALL CVPOOL ( 'MYTASK', UPDATE )
C
C           IF ( UPDATE ) THEN
C
C              CALL RTPOOL ( 'MYTASK_VARIABLE_1', N1, VALS1, FOUND(1) )
C              CALL RTPOOL ( 'MYTASK_VARIABLE_2', N2, VALS2, FOUND(2) )
C                      .
C                      .
C                      .
C              CALL RTPOOL ( 'MYTASK_VARIABLE_N', NN, VALSN, FOUND(N) )
C
C           END IF
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber     (JPL)
C
C$ Version
C
C-    SPICELIB Version 6.0.0, 31-MAR-1992 (WLT)
C
C        The entry points SWPOOL and CVPOOL were added.
C
C-&
 
C$ Index_Entries
C
C     Check the kernel pool for updated variables
C
C-&
 
C$ Revisions
C
C-    SPICELIB Version 6.0.0, 31-MAR-1992 (WLT)
C
C        The entry points SWPOOL (set watch on a pool variable)
C        and CVPOOL (check variable for update) so that routines
C        that buffer data stored in the kernel pool can fetch
C        that data only when it is updated.
C
C        In addition, the revision history was upgraded so that the
C        version number increases over time.  This wasn't true
C        before. In addition some early revision data that referred to
C        pre-SPICELIB modifications were removed. This editing of
C        the version numbers makes it unlikely that anyone can track
C        down which previous version of this routine they have by
C        looking at the version number.  The best way to determine
C        the routine you had previously is to compare the dates
C        stored in the Version line of the routine.
C
C-&
 
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'CVPOOL' )
      END IF
C
C     Initialize the pool if necessary.
C
      IF ( FIRST ) THEN
 
         BEGDAT = CHAR(BSLASH) // DAT
         BEGTXT = CHAR(BSLASH) // TXT
 
         CALL SSIZEC ( MAXVAR, TABSYM )
         CALL SSIZEI ( MAXVAR, TABPTR )
         CALL SSIZED ( MAXVAL, TABVAL )
 
         CALL SSIZEC ( MAXVAR, WATSYM )
         CALL SSIZEI ( MAXVAR, WATPTR )
         CALL SSIZEC ( MXNOTE, WATVAL )
 
         CALL SSIZEC ( MAXAGT, AGENTS )
         CALL SSIZEC ( MAXAGT, ACTIVE )
         CALL SSIZEC ( MAXAGT, NOTIFY )
 
         IF ( FAILED() ) THEN
            CALL CHKOUT ( 'CVPOOL' )
            RETURN
         ELSE
            FIRST = .FALSE.
         END IF
      END IF
 
C
C     Check to see if our agent is on the list of agents to be
C     notified.  If it is, we take this agent off the list---he's
C     now considered to have been notified.
C
      UPDATE = ELEMC  ( AGENT, AGENTS )
 
      IF ( UPDATE ) THEN
         CALL REMOVC ( AGENT, AGENTS )
      END IF
 
      CALL CHKOUT ( 'CVPOOL' )
      RETURN
      END
 
 
