C$Procedure      RDKER ( Read a kernel file )
 
      SUBROUTINE RDKER ( KERNEL, LINE, EOF )
 
C$ Abstract
C
C     Open and read the contents of a SPICE ASCII kernel file.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     FILES
C
C$ Declarations
 
      CHARACTER*(*)         KERNEL
      CHARACTER*(*)         LINE
      LOGICAL               EOF
 
C$ Brief_I/O
C
C     VARIABLE  I/O  ENTRY
C     --------  ---  --------------------------------------------------
C     KERNEL     I   RDKNEW
C     LINE       O   RDKDAT
C     EOF        O   RDKDAT
C
C$ Detailed_Input
C
C     All input is through entry RDKNEW.
C
C$ Detailed_Output
C
C     All output is through entry RDKDAT.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     The SPICE ASCII kernel file KERNEL is opened by RDKNEW and read
C     by RDKDAT.
C
C$ Exceptions
C
C     1) If RDKER is called directly, the error SPICE(BOGUSENTRY) is
C        signalled.
C
C$ Particulars
C
C     RDKER should never be called directly, but should instead be
C     accessed only through its entry points, RDKNEW and RDKDAT.
C
C$ Examples
C
C     In the following example, RDKNEW and RDKDAT are used to read
C     the contents of a kernel file.
C
C     Let the file KERNEL contain the following lines.
C
C        =============================================================
C
C        DELTA_T_A is defined to be 32.184 seconds, and should not
C        be changed except under the most unusual circumstances.
C
C        \begindata
C
C        DELTA_T_A       =   32.184
C
C        \begintext
C
C        The next three items determine the relativistic correction
C        in the difference ET - TAI. To turn the correction off,
C        just set K to zero.
C
C        \begindata
C
C        K               =    1.657D-3
C        ORBIT_ECC       =    1.671D-2
C        MEAN_ANOM       = (  6.239996D0,  1.99096871D-7 )
C
C        =============================================================
C
C     Then the code fragment
C
C        CALL RDKNEW ( KERNEL )
C        CALL RDKDAT ( LINE, EOF )
C
C        DO WHILE ( (.NOT. EOF) .AND. ( .NOT. FAILED () ) )
C           WRITE (6,*) LINE
C           CALL RDKDAT ( LINE, EOF )
C        END DO
C
C     prints the following lines.
C
C        =============================================================
C        DELTA_T_A       =   32.184
C        K               =    1.657D-3
C        ORBIT_ECC       =    1.671D-2
C        MEAN_ANOM       = (  6.239996D0,  1.99096871D-7 )
C        =============================================================
C
C$ Restrictions
C
C     The input file must be opened and initialized by RDKNEW prior
C     to the first call to RDKDAT.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     H.A. Neilan     (JPL)
C     M.J. Spencer    (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.2.0, 01-JUN-1992 (MJS)
C
C         RDKER now initializes the variables BEGDAT and BEGTXT
C         in a portable way. On the first valid entry to this routine,
C         the backslash character in the form CHAR(92) is concatenated
C         individually to 'begindata' and 'begintext'.
C
C-     SPICELIB Version 1.1.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.1.0, 7-DEC-1990 (HAN)
C
C         The declarations for BEGDAT and BEGTXT were changed from
C         CHARACTER*10 to CHARACTER*(*).
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     read a kernel file
C
C-&
 
 
 
 
C$ Revisions
C
C-     SPICELIB Version 1.2.0, 01-JUN-1992 (MJS)
C
C         RDKER now initializes the variables BEGDAT and BEGTXT
C         in a portable way. On the first valid entry to this routine,
C         the backslash character in the form CHAR(92) is concatenated
C         individually to 'begindata' and 'begintext'. As a result of
C         this change, this module is no longer considered environment
C         specific. All references in the header to the previous method
C         of initialization were removed.
C
C         FILE is now initialized to ' '. Before this modification, if
C         a call to RDKDAT was performed prior to RDKNEW, RDTEXT
C         would have printed out garbage (on some machines) in its
C         error message when notifiying the user that it couldn't read
C         from FILE.
C
C-     SPICELIB Version 1.1.0, 7-DEC-1990 (HAN)
C
C         The declarations for BEGDAT and BEGTXT were changed from
C         CHARACTER*10 to CHARACTER*(*). The fixed length of 10 was
C         not long enough.
C
C-     Beta Version 1.1.0, 9-MAR-1989 (HAN)
C
C         Moved the declaration of the parameters BEGDAT and
C         BEGTXT from the code to the Declarations section.
C         Filled out the Brief I/O and Parameters sections.
C
C-&
 
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
      LOGICAL               FAILED
 
 
C
C     Local parameters
C
 
C
C     Because some environments (such as the SUN) treat the backslash
C     character as a special character, some girations are needed to
C     put it into a variable in a "portable" way. This is the reason
C     for the following block of declarations. Admittedly this is
C     bizarre, but it works.
C
      INTEGER               MRKLEN
      PARAMETER           ( MRKLEN = 10 )
 
      INTEGER               SHRTLN
      PARAMETER           ( SHRTLN =  9 )
 
      INTEGER               BSLASH
      PARAMETER           ( BSLASH = 92 )
 
      CHARACTER*(MRKLEN)    BEGDAT
      CHARACTER*(MRKLEN)    BEGTXT
 
      CHARACTER*(SHRTLN)    DAT
      PARAMETER           ( DAT = 'begindata' )
 
      CHARACTER*(SHRTLN)    TXT
      PARAMETER           ( TXT = 'begintext' )
 
      CHARACTER*1           BLANK
      PARAMETER           ( BLANK = ' ' )
 
C
C     Local variables
C
      CHARACTER*255         FILE
      CHARACTER*80          FIRST
      CHARACTER*6           STATUS
 
      LOGICAL               END
      LOGICAL               FRSTIM
 
C
C     Save EVERYTHING.
C
      SAVE
 
C
C     Initial values
C
      DATA                  FRSTIM / .TRUE. /
      DATA                  FILE   / ' '    /
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'RDKER' )
      END IF
 
C
C     Calling RDKER directly is a serious breach of protocol.
C     If RDKER is called, an error is signalled.
C
 
      CALL SETMSG ( 'RDKER: You have called an entry which '   //
     .              'performs no run-time function. This '     //
     .              'may indicate a bug. Please check the '    //
     .              'documentation for the subroutine RDKER.'     )
 
 
      CALL SIGERR ( 'SPICE(BOGUSENTRY)' )
 
 
      CALL CHKOUT ( 'RDKER' )
      RETURN
 
 
 
C$Procedure RDKNEW ( Open and initialize a new kernel file )
 
      ENTRY RDKNEW ( KERNEL )
 
C$ Abstract
C
C     Open and initialize a SPICE ASCII kernel file.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     FILES
C
C$ Declarations
C
C     CHARACTER*(*)         KERNEL
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     KERNEL     I   Kernel file.
C
C$ Detailed_Input
C
C     KERNEL      is the name of the SPICE ASCII kernel file to be
C                 opened and initialized.
C
C$ Detailed_Output
C
C      None.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     The SPICE ASCII kernel file KERNEL is opened by RDKNEW and read
C     by RDKDAT.
C
C$ Exceptions
C
C     None.
C
C$ Particulars
C
C     RDKNEW should be called prior to the first call to RDKDAT.
C     RDKNEW opens the kernel file and RDKDAT reads the lines of
C     data in the file.
C
C$ Examples
C
C     In the following example, RDKNEW and RDKDAT are used to read
C     the contents of a kernel file.
C
C     Let the file KERNEL contain the following lines.
C
C        =============================================================
C
C        DELTA_T_A is defined to be 32.184 seconds, and should not
C        be changed except under the most unusual circumstances.
C
C        \begindata
C
C        DELTA_T_A       =   32.184
C
C        \begintext
C
C        The next three items determine the relativistic correction
C        in the difference ET - TAI. To turn the correction off,
C        just set K to zero.
C
C        \begindata
C
C        K               =    1.657D-3
C        ORBIT_ECC       =    1.671D-2
C        MEAN_ANOM       = (  6.239996D0,  1.99096871D-7 )
C
C        =============================================================
C
C     Then the code fragment
C
C        CALL RDKNEW ( KERNEL )
C        CALL RDKDAT ( LINE, EOF )
C
C        DO WHILE ( (.NOT. EOF) .AND. ( .NOT. FAILED () ) )
C           WRITE (6,*) LINE
C           CALL RDKDAT ( LINE, EOF )
C        END DO
C
C     prints the following lines.
C
C        =============================================================
C        DELTA_T_A       =   32.184
C        K               =    1.657D-3
C        ORBIT_ECC       =    1.671D-2
C        MEAN_ANOM       = (  6.239996D0,  1.99096871D-7 )
C        =============================================================
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.2.0, 01-JUN-1992 (MJS)
C
C         RDKER now initializes the variables BEGDAT and BEGTXT
C         in a portable way. On the first valid entry to this routine,
C         the backslash character in the form CHAR(92) is concatenated
C         individually to 'begindata' and 'begintext'.
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     open and initialize a new kernel file
C
C-&
 
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'RDKNEW' )
      END IF
 
C
C     Initialize the data delimiters if it hasn't been done already.
C
      IF ( FRSTIM ) THEN
 
         BEGDAT = CHAR(BSLASH) // DAT
         BEGTXT = CHAR(BSLASH) // TXT
 
         FRSTIM = .FALSE.
 
      ENDIF
 
C
C     Close the previous file, if it hasn't been closed already.
C     Close the new file, too, in case they are the same. No sense
C     burning up logical units.
C
      CALL CLTEXT ( FILE   )
      CALL CLTEXT ( KERNEL )
 
C
C     Read the first line of the file. It can't possibly be a data
C     line, since data must be preceded by a \begindata marker, so
C     we needn't take any pains to save it.
C
      CALL RDTEXT ( KERNEL, FIRST, END )
      CALL LJUST  ( FIRST,  FIRST      )
 
C
C     The first line is enough to set the status for subsequent
C     calls to RDKDAT.
C
      IF ( END ) THEN
         STATUS = 'EOF'
 
      ELSE IF ( FIRST .EQ. BEGDAT ) THEN
         STATUS = 'INDATA'
 
      ELSE
         STATUS = 'INTEXT'
      END IF
 
C
C     Save the name of the file for future reference.
C
      FILE = KERNEL
 
 
      CALL CHKOUT ( 'RDKNEW' )
      RETURN
 
 
 
C$Procedure RDKDAT ( Read the next data line from a kernel file )
 
      ENTRY RDKDAT ( LINE, EOF )
 
C$ Abstract
C
C     Read the next line of data from a SPICE ASCII kernel file.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     FILES
C
C$ Declarations
C
C     CHARACTER*(*)         LINE
C     LOGICAL               EOF
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     LINE       O   Next line of kernel data.
C     EOF        O   End of file indicator.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     LINE        is the next line of data from the kernel file
C                 most recently opened by NEWKER. Data lines are
C                 non-blank lines which lie between \begindata
C                 and \begintext markers. Lines are returned left
C                 justified.
C
C     EOF         is true when the end of the kernel file has been
C                 reached, and is false otherwise. The kernel file
C                 is closed automatically when the end of the file
C                 is reached.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     The SPICE ASCII kernel file KERNEL is opened by RDKNEW and read
C     by RDKDAT.
C
C$ Exceptions
C
C     None.
C
C$ Particulars
C
C     RDKDAT is used internally by RDKVAR to retrieve successive lines
C     of data from the current kernel file. It exists primarily to
C     relieve RDKVAR of the responsibility of dealing with comment
C     blocks and blank lines.
C
C$ Examples
C
C     In the following example, RDKNEW and RDKDAT are used to read
C     the contents of a kernel file.
C
C     Let the file KERNEL contain the following lines.
C
C        =============================================================
C
C        DELTA_T_A is defined to be 32.184 seconds, and should not
C        be changed except under the most unusual circumstances.
C
C        \begindata
C
C        DELTA_T_A       =   32.184
C
C        \begintext
C
C        The next three items determine the relativistic correction
C        in the difference ET - TAI. To turn the correction off,
C        just set K to zero.
C
C        \begindata
C
C        K               =    1.657D-3
C        ORBIT_ECC       =    1.671D-2
C        MEAN_ANOM       = (  6.239996D0,  1.99096871D-7 )
C
C        =============================================================
C
C     Then the code fragment
C
C        CALL RDKNEW ( KERNEL )
C        CALL RDKDAT ( LINE, EOF )
C
C        DO WHILE ( (.NOT. EOF) .AND. ( .NOT. FAILED () ) )
C           WRITE (6,*) LINE
C           CALL RDKDAT ( LINE, EOF )
C        END DO
C
C     prints the following lines.
C
C        =============================================================
C        DELTA_T_A       =   32.184
C        K               =    1.657D-3
C        ORBIT_ECC       =    1.671D-2
C        MEAN_ANOM       = (  6.239996D0,  1.99096871D-7 )
C        =============================================================
C
C$ Restrictions
C
C     The input file must be opened and initialized by NEWKER prior
C     to the first call to RDKDAT.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     H.A. Neilan     (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     read the next data line from a kernel file
C
C-&
 
C$ Revisions
C
C-     Beta Version 2.0.0, 23-OCT-1989 (HAN)
C
C        A FAILED test was added to the DO-loop which reads
C        lines in the kernel file.
C
C        If the error action was set to 'RETURN' an infinite loop
C        could have resulted if RDTEXT failed and the loop conditions
C        were satisfied.
C
C-&
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'RDKDAT' )
      END IF
 
C
C     If the previous call detected the end of the file,
C     this one should do the same.
C
      IF ( STATUS .EQ. 'EOF' ) THEN
         EOF = .TRUE.
         CALL CHKOUT ( 'RDKDAT' )
         RETURN
      END IF
 
C
C     Well, at least we can try to read a line. Adjust the status as
C     needed, return if appropriate, read another line if necessary.
C     Basically, we're looking for a non-blank line in a data segment.
C
      LINE = BLANK
 
50001 IF       ( ( .NOT. FAILED() )            .AND.
     .           ( STATUS .EQ. 'INTEXT'  .OR.  LINE .EQ. BLANK ) )
     .THEN
 
         CALL RDTEXT ( FILE, LINE, EOF )
         CALL LJUST  ( LINE, LINE      )
 
         IF ( EOF ) THEN
            STATUS = 'EOF'
            CALL CHKOUT ( 'RDKDAT' )
            RETURN
 
         ELSE IF ( LINE .EQ. BEGTXT ) THEN
            STATUS = 'INTEXT'
 
         ELSE IF ( LINE .EQ. BEGDAT ) THEN
            STATUS = 'INDATA'
            LINE   = BLANK
         END IF
 
         GO TO 50001
      END IF
 
 
      CALL CHKOUT ( 'RDKDAT' )
      RETURN
      END
