C$Procedure      SCE2S ( ET to SCLK string )
 
      SUBROUTINE SCE2S ( SC, ET, SCLKCH )
 
C$ Abstract
C
C     Convert ephemeris seconds past J2000 (ET) to a character string
C     representation of an SCLK value.
C
C$ Required_Reading
C
C     SCLK
C     TIME
C
C$ Keywords
C
C     CONVERSION
C     TIME
C
C$ Declarations
 
      INTEGER               SC
      DOUBLE PRECISION      ET
      CHARACTER*(*)         SCLKCH
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     SC         I   NAIF spacecraft ID code.
C     ET         I   Ephemeris time, seconds past J2000.
C     SCLKCH     O   An SCLK string.
C
C$ Detailed_Input
C
C     SC             is a NAIF ID code for a spacecraft, whose clock
C                    reading at the epoch specified by ET is desired.
C                    The set of supported spacecraft clocks is listed
C                    in the SCLK Required Reading.
C
C     ET             is an epoch, specified as ephemeris seconds past
C                    J2000.
C
C$ Detailed_Output
C
C     SCLKCH         is a character string representation of the
C                    spacecraft clock value that corresponds to ET, for
C                    the spacecraft specified by the input argument SC.
C                    SCLKCH is an absolute spacecraft clock value, so
C                    a partition number is included in the string.
C                    The precise format of SCLKCH is specified in the
C                    SCLK Required Reading.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1)  This routine assumes that that an SCLK kernel appropriate
C         to the spacecraft clock identified by the input argument SC
C         has been loaded.  If an SCLK kernel has not been loaded,
C         does not contain all of the required data, or contains
C         invalid data, error diagnoses will be performed by routines
C         called by this routine.  The output argument SCLKCH will not
C         be modified.
C
C     2)  When using SCLK kernels that map SCLK to a time system other
C         than ET (also called barycentric dynamical time---`TDB'), it
C         is necessary to have a leapseconds kernel loaded at the time
C         this routine is called.  If a leapseconds kernel is required
C         for conversion between SCLK and ET but is not loaded, the
C         error will be diagnosed by routines called by this routine.
C         The output argument SCLKCH will not be modified.
C
C         The time system that an SCLK kernel maps SCLK to is indicated
C         by the variable SCLK_TIME_SYSTEM_nn in the kernel, where nn
C         is the negative of the NAIF integer code for the spacecraft.
C         The time system used in a kernel is TDB if and only if the
C         variable is assigned the value 1.
C
C
C     3)  If the input ET value is not representable in the spacecraft
C         clock string format for the spacecraft clock identified by
C         SC, the error will be diagnosed by routines called by this
C         routine.  The output argument SCLKCH will not be modified.
C
C     4)  If the output argument SCLKCH is too short to contain the
C         output spacecraft clock string produced by this routine,
C         the error will be diagnosed by routines called by this
C         routine.  The output argument SCLKCH may contain a portion
C         of the truncated string.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This routine is provided as a convenience; it is simply shorthand
C     for the code fragment
C
C        CALL SCE2T  ( SC,  ET,      SCLKDP )
C        CALL SCDECD ( SC,  SCLKDP,  SCLKCH )
C
C     The set of SCLK conversion routines is
C
C        SCENCD (Encode SCLK)
C        SCDECD (Decode SCLK)
C        SCS2E  (SCLK ticks to ET)
C        SCE2T  (ET to SCLK ticks)
C        SCS2E  (SCLK string to ET)
C        SCE2S  (ET to SCLK string)
C
C$ Examples
C
C     1)  Find the Galileo SCLK value corresponding to the ET
C
C            -322452420.5593641.
C
C
C            C
C            C     Start out by loading the SCLK kernel.  In your own
C            C     program, you must use the name of a real SCLK kernel.
C            C     The name shown here is fictitious.
C            C
C                  CALL LDPOOL ( 'SCLK.KER' )
C
C            C
C            C     Load a leapseconds kernel in case it is needed for
C            C     SCLK-to-ET conversion.  Depending on the SCLK kernel
C            C     used, it may not be necessary to load this file; it's
C            C     just a simple, reliable way of making sure that the
C            C     leapseconds kernel constants are available if we need
C            C     them.  Again, a fictitious name is used.
C            C
C                  CALL LDPOOL ( 'LEAPSECONDS.KER' )
C
C            C
C            C     The spacecraft ID code for Galileo is -77.
C            C
C                  SC = -77
C                  ET = -322452420.5593641
C
C                  CALL SCE2S ( SC, ET, SCLKCH )
C
C
C         The returned value of SCLKCH will be
C
C            1/00010001:44:2:0.
C
C
C     2)  Convert the UTC time
C
C            August 25 1989 4:00:00
C
C         to a Voyager 2 SCLK value.
C
C         To enable you to perform UTC to ET conversion, your
C         initialization code must load the leapseconds and SCLK
C         kernels:
C
C               C
C               C     Load leapseconds and SCLK kernels:
C               C
C                     CALL LDPOOL ( 'LEAPSECONDS.KER' )
C                     CALL LDPOOL ( 'VGR2SCLK.KER'    )
C
C
C            To find Voyager 2 SCLK string corresponding to the
C            specified UTC time, you can use the code fragment
C
C                     CALL UTC2ET ( 'Aug 25 1989 4:00:00',  ET     )
C                     CALL SCE2S  ( -32,        ET,         SCLKCH )
C
C
C$ Restrictions
C
C     1)  An SCLK kernel appropriate to the spacecraft clock identified
C         by SC must be loaded at the time this routine is called.
C
C     2)  If the SCLK kernel used with this routine does not map SCLK
C         directly to barycentric dynamical time, a leapseconds kernel
C         must be loaded at the time this routine is called.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman   (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.2.0, 10-APR-1992 (NJB) (WLT)
C
C        Truncation of the output string is now treated as an error.
C        Header was updated to reflect possibility of needing to load
C        a leapseconds kernel before calling this routine.  Comment
C        section for permuted index source lines was added following the
C        header.
C
C-    SPICELIB Version 1.0.1, 12-OCT-1990 (NJB)
C
C        Missing example added to the $ Examples section.  Restrictions
C        section no longer states that you must load the leapseconds
C        kernel prior to calling this routine.
C
C-    SPICELIB Version 1.0.0, 03-SEP-1990 (NJB)
C
C-&
 
C$ Index_Entries
C
C     ephemeris time to spacecraft_clock string
C
C-&
 
C$ Revisions
C
C-    SPICELIB Version 1.2.0, 10-APR-1992 (NJB) (WLT)
C
C        Truncation of the output string is now treated as an error.
C        The code changes made to implement the error checking were
C        in SCDECD and other lower-level routines.
C
C        The header was updated to reflect possibility of needing to
C        load a leapseconds kernel before calling this routine.
C
C        The comment section for permuted index source lines was added
C        following the header.
C
C
C-    SPICELIB Version 1.0.1, 12-OCT-1990 (NJB)
C
C        Missing example added to the $ Examples section.  Restrictions
C        section no longer states that you must load the leapseconds
C        kernel prior to calling this routine.
C
C        The second example no longer uses a call to CLPOOL.
C
C-&
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Local variables
C
      DOUBLE PRECISION      SCLKDP
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SCE2S' )
      END IF
 
C
C     Convert ET to encoded SCLK, and then to an SCLK string.
C
      CALL SCE2T  ( SC, ET,     SCLKDP )
      CALL SCDECD ( SC, SCLKDP, SCLKCH )
 
      CALL CHKOUT ( 'SCE2S' )
      RETURN
      END
