 
C$Procedure SPKEZ ( S/P Kernel, easy reader )
 
      SUBROUTINE SPKEZ ( TARG, ET, REF, ABCORR, OBS, STARG, LT )
 
C$ Abstract
C
C     Return the apparent state (position and velocity) of a target
C     body relative to an observing body, optionally corrected for
C     planetary (light time) and stellar aberrations.
C
C$ Required_Reading
C
C     SPK
C
C$ Keywords
C
C     EPHEMERIS
C
C$ Declarations
 
      INTEGER               TARG
      DOUBLE PRECISION      ET
      CHARACTER*(*)         REF
      CHARACTER*(*)         ABCORR
      INTEGER               OBS
      DOUBLE PRECISION      STARG    ( 6 )
      DOUBLE PRECISION      LT
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     TARG       I   Target body.
C     ET         I   Target epoch.
C     REF        I   Target inertial reference frame.
C     ABCORR     I   Aberration correction.
C     OBS        I   Observing body.
C     STARG      O   State of target.
C     LT         O   Light time.
C
C$ Detailed_Input
C
C     TARG        is the standard NAIF ID code for a target body.
C
C     ET          is the epoch (ephemeris time) at which the state
C                 of the target body is to be computed.
C
C     REF         is the name of the inertial reference frame to
C                 which the vectors returned by the routine should
C                 be rotated. This may be any frame supported by
C                 the SPICELIB subroutine CHGIRF.
C
C     ABCORR      indicates the aberration corrections to be applied
C                 in order to determine the state of the target body.
C                 ABCORR may be any of the following.
C
C                    'NONE'     Apply no correction. Return the true
C                               state of the target body relative to
C                               the observer.
C
C                    'LT'       Correct for planetary (light time)
C                               aberration.
C
C                    'LT+S'     Correct for planetary (light time)
C                               and stellar aberrations.
C
C     OBS         is the standard NAIF ID code for an observing body.
C
C$ Detailed_Output
C
C     STARG       contains the position and velocity of the target
C                 body, relative to the observing body, corrected
C                 for the specified aberrations, at epoch ET.  STARG
C                 has six elements:  the first three contain the
C                 target's position; the last three contain the target's
C                 velocity.  These vectors are rotated into the
C                 specified inertial reference frame. Units are always
C                 km and km/sec.
C
C     LT          is the one-way light time from the observing body
C                 to the geometric position of the target body at the
C                 specified epoch. The value of LT is not affected
C                 by the value of ABCORR.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     See: $Restrictions.
C
C$ Particulars
C
C     The combinations
C
C        CALL SPKSSB ( OBS,  ET, REF, SOBS                    )
C        CALL SPKAPP ( TARG, ET, REF, SOBS, ABCORR, STARG, LT )
C
C     and
C
C        CALL SPKSSB ( OBS, ET, REF, SOBS )
C
C        DO I = 1, N
C           CALL SPKAPP ( TARG(I), ET,   REF, SOBS, ABCORR,
C       .                                     STARG(1,I),   LT )
C        END DO
C
C     occur so frequently in practice that this routine has been
C     created to allow you to do in one call what you would normally
C     do in two. That is,
C
C        CALL SPKEZ ( TARG, ET, REF, ABCORR, OBS, STARG, LT )
C
C     and
C
C        DO I = 1, N
C           CALL SPKEZ ( TARG(I), ET, REF, ABCORR, OBS, STARG(1,I), LT )
C        END DO
C
C     are exactly equivalent to the combinations shown above when
C     ABCORR is not 'NONE'.
C
C     When ABCORR is 'NONE', that is, when no aberration corrections
C     are requested, SPKEZ just calls SPKGEO to compute the geometric
C     state.  It is not necessary, in this case, to compute the
C     state of the target and observer relative to the SSB.
C
C     For most applications, SPKEZ is the only reader you will ever
C     need to use.
C
C$ Examples
C
C     In the following code fragment, SPKEZ is used to display the
C     astrographic position of any body as seen from any other body
C     at any epoch. The user enters commands interactively. Possible
C     commands are:
C
C        Keyword   Argument       Default
C        -------   ------------   -------------
C        TARG      body_number     499
C        OBS       body_number     399
C        EPOCH     utc_epoch      '1989 DEC 09'
C        STEP      seconds         n/a
C
C     After each command, the astrographic right ascension and
C     declination of the current target relative to the current
C     observer are displayed, along with the epoch. The user
C     enters 'EXIT' to terminate the session.
C
C     (All subroutines and functions are from SPICELIB.)
C
C        READ (*,FMT='(A)') LINE
C
C        DO WHILE ( .TRUE. )
C           CALL NEXTWD ( LINE, KEY, LINE )
C           CALL NUCASE  ( KEY,  KEY       )
C
C           IF ( KEY .EQ. 'TARG' ) THEN
C              CALL NPARSI ( LINE, TARG, ERR, PTR )
C
C           ELSE IF ( KEY .EQ. 'OBS' ) THEN
C              CALL NPARSI ( LINE, OBS, ERR, PTR )
C
C           ELSE IF ( KEY .EQ. 'EPOCH' ) THEN
C              CALL UTC2ET ( LINE, EPOCH )
C
C           ELSE IF ( KEY .EQ. 'STEP' ) THEN
C              CALL NPARSD ( LINE, STEP, ERR, PTR )
C              EPOCH = EPOCH + STEP
C
C           ELSE IF ( KEY .EQ. 'EXIT' ) THEN
C              WRITE (*,*)
C              WRITE (*,*) 'Thank you. Please come again.'
C              STOP
C
C           ELSE IF ( KEY .NE. ' ' ) THEN
C              WRITE (*,*)
C              WRITE (*,*) 'No habla ', KEY
C           END IF
C
C           CALL ET2UTC ( EPOCH, 'C', 3, UTCSTR )
C           CALL SPKEZ  ( TARG, EPOCH, 'J2000', 'LT', OBS, V, LT )
C           CALL RECRAD ( V, RANGE, RA, DEC )
C
C           WRITE (*,*)
C           WRITE (*,*)  OBS,
C       .               ' --> ', TARG,
C       .               ' at ',  UTCSTR,
C       .               ' : ',   RA  * DPR(), DEC * DPR()
C
C           READ (*,FMT='(A)') LINE
C        END DO
C
C$ Restrictions
C
C     1) The ephemeris files to be used by SPKEZ must be loaded
C        by SPKLEF before SPKEZ  is called.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     J.E. McLean     (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 2.0.0, 18-JUL-1991 (JEM) (NJB)
C
C        The old SPKEZ did not compute the geometric state of one body
C        with respect to another unless data existed for each body with
C        respect to the solar system barycenter.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     easy reader for spk file
C     get state relative observer corrected for aberrations
C
C-&
 
 
 
C$ Revisions
C
C-    SPICELIB Version 2.0.0, 18-JUL-1991 (JEM) (NJB)
C
C        The previous version of SPKEZ could not
C        compute the geometric state (no aberration
C        correction) of one body with respect to
C        another if the ephemeris data for each
C        body relative to the Solar System Barycenter
C        (body 0) had not been loaded.  Now, if
C        sufficient data is loaded, SPKEZ can always
C        compute the state.
C
C        For example, suppose the file GLL.BSP contains
C        segments of SPK data for the Galileo spacecraft
C        (body -77) relative to the Jupiter Barycenter
C        (body 5) over a period of time.  If SPKEZ Version
C        1.0.0 was called to compute the geometric state of
C        -77 relative to 5 (or vice versa), a routine that
C        SPKEZ calls, SPKSSB, would signal an error stating
C        that there is insufficient data for computing the
C        state of body 5 (relative to 0).  Version 1.0.0
C        of SPKEZ could not compute the requested state even
C        though sufficent data had been loaded.
C
C        It is necessary to compute the states of each
C        of the target and observing bodies relative to
C        the solar system barycenter when aberration
C        corrections are being applied.  However, when
C        computing geometric states, it is only necessary
C        to trace back to the first common node.  Positive
C        side effects include the maintenance of precision
C        and reduction in number of look ups.
C
C        The changes to the code in SPKEZ involved calling a new
C        routine, SPKGEO, which computes the geometric state if
C        no aberration corrections are requested.
C
C        The other cosmetic changes include the removal of a reference
C        to the SPK User's Guide in C$ Literature_References because
C        the User's Guide is the same as SPK Required Reading.
C
C        Also, the item in C$ Restrictions previously said
C
C           1) The ephemeris files to be used by SPKEZ must be loaded
C              by SPKLEF before SPKSSB is called.
C
C        SPKSSB was replaced with SPKEZ.
C
C        The location of the position and velocity information in the
C        output state vector argument STARG is now spelled out.
C
C        Finally, the C$ Particulars section was updated.  In Version
C        1.0.0 it said that calling SPKEZ was equivalent to calling
C        SPKSSB and SPKAPP.
C
C-&
 
C
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Local variables
C
      DOUBLE PRECISION      SOBS     ( 6 )
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SPKEZ' )
      END IF
 
C
C     If we only want geometric states, then compute just that.
C
C     Otherwise, compute the state of the observer relative to
C     the SSB.  Then feed that state into SPKAPP to compute the
C     apparent state of the target body relative to the observer
C     with the requested aberration corrections.
C
      IF ( ABCORR .EQ. 'NONE' ) THEN
 
         CALL SPKGEO ( TARG, ET, REF, OBS, STARG, LT )
 
      ELSE
 
         CALL SPKSSB ( OBS,  ET, REF, SOBS                    )
         CALL SPKAPP ( TARG, ET, REF, SOBS, ABCORR, STARG, LT )
 
      END IF
 
      CALL CHKOUT ( 'SPKEZ' )
      RETURN
      END
