C$Procedure SPKS09 ( S/P Kernel, subset, type 9 )
 
      SUBROUTINE SPKS09 ( HANDLE, BADDR, EADDR, BEGIN, END )
 
C$ Abstract
C
C     Extract a subset of the data in an SPK segment of type 9
C     into a new segment.
C
C$ Required_Reading
C
C     SPK
C     DAF
C
C$ Keywords
C
C     EPHEMERIS
C
C$ Declarations
 
      INTEGER               HANDLE
      INTEGER               BADDR
      INTEGER               EADDR
      DOUBLE PRECISION      BEGIN
      DOUBLE PRECISION      END
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   Handle of file containing source segment.
C     BADDR      I   Beginning address in file of source segment.
C     EADDR      I   Ending address in file of source segment.
C     BEGIN      I   Beginning (initial epoch) of subset.
C     END        I   End (final epoch) of subset.
C
C$ Detailed_Input
C
C     HANDLE,
C     BADDR,
C     EADDR       are the file handle assigned to an SPK file, and the
C                 beginning and ending addresses of a segment within
C                 that file.  Together they determine a complete set of
C                 ephemeris data, from which a subset is to be
C                 extracted.
C
C     BEGIN,
C     END         are the initial and final epochs (ephemeris time)
C                 of the subset.
C
C                 The first epoch for which there will be ephemeris
C                 data in the new segment will be the greatest time
C                 in the source segment that is less than or equal
C                 to BEGIN.
C
C                 The last epoch for which there will be ephemeris
C                 data in the new segment will be the smallest time
C                 in the source segment that is greater than or equal
C                 to END.
C
C$ Detailed_Output
C
C     See $Files section.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1)  This routine relies on the caller to ensure that the
C         interval [BEGIN, END] is contained in the coverage
C         interval of the segment.
C
C     2)  If BEGIN > END, no data is written to the target file.
C
C$ Files
C
C     Data is extracted from the file connected to the input
C     handle, and written to the current DAF open for writing.
C
C     The segment descriptor and summary must already have been written
C     prior to calling this routine.  The segment must be ended
C     external to this routine.
C
C$ Particulars
C
C     This routine is intended solely for use as a utility by the
C     routine SPKSUB.
C
C     It transfers a subset of a type 09 SPK data segment to
C     a properly initialized segment of a second SPK file.
C
C     The exact structure of a segment of data type 09 is described
C     in the section on type 09 in the SPK Required Reading.
C
C$ Examples
C
C     This routine is intended only for use as a utility by SPKSUB.
C     To use this routine successfully, you must:
C
C        Open the SPK file from which to extract data.
C        Locate the segment from which data should be extracted.
C
C        Open the SPK file to which this data should be written.
C        Begin a new segment (array).
C        Write the summary information for the array.
C
C        Call this routine to extract the appropriate data from the
C        SPK open for read.
C
C        End the array to which this routine writes data.
C
C     Much of this procedure is carried out by the routine SPKSUB.  The
C     examples of that routine illustrate more fully the process
C     described above.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     J.M. Lynch      (JPL)
C     W.L. Taber      (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 08-AUG-1993 (NJB) (JML) (WLT) (IMU)
C
C-&
C
C$ Index_Entries
C
C     subset type_9 spk segment
C
C-&
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Local variables
C
      DOUBLE PRECISION      DATA     ( 6 )
 
      INTEGER               DEGREE
      INTEGER               I
      INTEGER               NDIR
      INTEGER               NREC
      INTEGER               OFFE
      INTEGER               OFFSET
      INTEGER               REC      ( 2 )
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SPKS09' )
      END IF
 
C
C     Get the number of records in the segment.  Get the polynomial
C     degree as well.
C
      CALL DAFRDA ( HANDLE, EADDR-1, EADDR, DATA )
 
      DEGREE = NINT ( DATA(1) )
      NREC   = NINT ( DATA(2) )
 
C
C     From the number of records, we can compute
C
C        NDIR      The number of directory epochs.
C
C        OFFE      The offset of the first epoch.
C
      NDIR = ( NREC - 1 ) / 100
      OFFE = EADDR - NDIR - NREC - 2
 
C
C     Examine the epochs in forward order, looking for the first
C     epoch greater than or equal to END (or the final epoch,
C     whichever comes first). This epoch corresponds to the last
C     state to be transferred.
C
      REC(2) = 1
      CALL DAFRDA ( HANDLE, OFFE+REC(2), OFFE+REC(2), DATA )
 
50001 IF        ( REC(2) .LT. NREC  .AND.  DATA(1) .LT. END )
     .THEN
         REC(2) = REC(2) + 1
         CALL DAFRDA ( HANDLE, OFFE+REC(2), OFFE+REC(2), DATA )
         GO TO 50001
      END IF
 
C
C     Now examine them in reverse order, looking for the first
C     epoch less than or equal to BEGIN (or the initial epoch,
C     whichever comes first). This epoch corresponds to the first
C     state to be transferred.
C
      REC(1) = NREC
      CALL DAFRDA ( HANDLE, OFFE+REC(1), OFFE+REC(1), DATA )
 
50002 IF       ( REC(1) .GT. 1  .AND.  DATA(1) .GT. BEGIN )
     .THEN
         REC(1) = REC(1) - 1
         CALL DAFRDA ( HANDLE, OFFE+REC(1), OFFE+REC(1), DATA )
         GO TO 50002
      END IF
 
C
C     See whether there's any work to do; return immediately if not.
C
      IF (  REC(1)  .GT.  REC(2)  ) THEN
         CALL CHKOUT ( 'SPKS09' )
         RETURN
      END IF
 
C
C     Copy states REC(1) through REC(2) to the output file.
C
      DO 50003
     .    I = REC(1), REC(2)
         OFFSET = BADDR  -  1  +  (I - 1) * 6
 
         CALL DAFRDA ( HANDLE, OFFSET+1, OFFSET+6, DATA    )
         CALL DAFADA (                             DATA, 6 )
50003 CONTINUE
 
C
C     Copy epochs REC(1) through REC(2) to the output file.
C
      DO 50004
     .    I = REC(1), REC(2)
         CALL DAFRDA ( HANDLE, OFFE+I, OFFE+I, DATA    )
         CALL DAFADA (                         DATA, 1 )
50004 CONTINUE
 
C
C     Put every 100'th epoch into the directory, except the last
C     epoch, if that epoch's index would be a multiple of 100.
C
      DO 50005
     .    I = REC(1)+99, REC(2)-1, 100
         CALL DAFRDA ( HANDLE, OFFE+I, OFFE+I, DATA    )
         CALL DAFADA (                         DATA, 1 )
50005 CONTINUE
 
C
C     Store the polynomial degree and the number of records
C     to end the segment.
C
      CALL DAFADA ( DBLE ( DEGREE ),                  1 )
      CALL DAFADA ( DBLE ( REC(2) - REC(1) + 1 ),     1 )
 
      CALL CHKOUT ( 'SPKS09' )
      RETURN
      END
		
