 
C$Procedure SPKSSB ( S/P Kernel, solar system barycenter )
 
      SUBROUTINE SPKSSB ( TARG, ET, REF, STARG )
 
C$ Abstract
C
C     Return the state (position and velocity) of a target body
C     relative to the solar system barycenter.
C
C$ Required_Reading
C
C     SPK
C
C$ Keywords
C
C     EPHEMERIS
C
C$ Declarations
 
      INTEGER               TARG
      DOUBLE PRECISION      ET
      CHARACTER*(*)         REF
      DOUBLE PRECISION      STARG    ( 6 )
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     TARG       I   Target body.
C     ET         I   Target epoch.
C     REF        I   Target inertial reference frame.
C     STARG      O   State of target.
C
C$ Detailed_Input
C
C     TARG        is the standard NAIF ID code for a target body.
C
C     ET          is the epoch (ephemeris time) at which the state
C                 of the target body is to be computed.
C
C     REF         is the name of the inertial reference frame to
C                 which the vectors returned by the routine should
C                 be rotated. This may be any frame supported by
C                 the SPICELIB subroutine CHGIRF.
C
C$ Detailed_Output
C
C     STARG       contains the position and velocity of the target
C                 body, relative to the solar system barycenter,
C                 at epoch ET. These vectors are rotated into the
C                 specified inertial reference frame. Units are always
C                 km and km/sec.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If the data available from previously loaded ephemeris
C        files is not sufficient to determine the state of the
C        body relative to the solar system barycenter, the error
C        'SPICE(SPKINSUFFDATA)' is signalled.
C
C$ Files
C
C     See: $Restrictions.
C
C$ Particulars
C
C     In order to compute the state of one body relative to another,
C     the states of the two bodies must be known relative to a third
C     body. One simple solution is to use the solar system barycenter
C     as the third body.
C
C     Ephemeris data from more than one segment may be required
C     to determine the state of a body relative to the barycenter.
C     SPKSSB reads as many segments as necessary, from as many
C     files as necessary, using files that have been loaded by
C     previous calls to SPKLEF (load ephemeris file).
C
C$ Examples
C
C     In the following code fragment, SPKSSB is used to display
C     the distance from Earth (Body 399) to Mars (body 499) at
C     a series of epochs.
C
C        CALL SPKLEF ( 'DE125.SPK', HANDLE )
C         .
C         .
C
C        EARTH = 399
C        MARS  = 499
C
C        DO WHILE ( EPOCH .LE. END )
C           CALL SPKSSB ( EARTH, EPOCH, 'J2000', SEARTH )
C           CALL SPKSSB ( MARS,  EPOCH, 'J2000', SMARS  )
C
C           CALL VSUBG ( MARS, EARTH, MARS )
C           WRITE (*,*) EPOCH, VNORM ( MARS )
C
C           EPOCH = EPOCH + DELTA
C        END DO
C
C$ Restrictions
C
C     1) The ephemeris files to be used by SPKSSB must be loaded
C        by SPKLEF before SPKSSB is called.
C
C$ Literature_References
C
C     NAIF Document 168.0, "S- and P- Kernel (SPK) Specification and
C     User's Guide"
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.2, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     state relative to solar system barycenter
C
C-&
 
 
C
C     SPICELIB functions
C
      LOGICAL               FAILED
      LOGICAL               RETURN
 
C
C     Local variables
C
      INTEGER               HANDLE
      DOUBLE PRECISION      DESCR    ( 5 )
      CHARACTER*40          IDENT
      LOGICAL               FOUND
 
      INTEGER               CENTER
      DOUBLE PRECISION      TEMP     ( 6 )
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SPKSSB' )
      END IF
 
C
C     The idea is quite simple: every body moves with respect to
C     some center. The center is itself a body, which in turn moves
C     about some other center. If we follow the chain,
C
C                                   T
C                                     \
C           SSB                        \
C               \                     C[1]
C                \                     /
C                 \                   /
C                  \                 /
C                   \               /
C                  C[3]-----------C[2]
C
C     and avoid circular definitions (A moves about B, and B moves
C     about A), eventually we get the state relative to the solar
C     system barycenter (which, for our purposes, doesn't move).
C     Thus,
C
C        T    = T     + C[1]     + C[2]     + ... + C[n]
C         SSB    C[1]       C[2]       [C3]             SSB
C
C     where
C
C        X
C         Y
C
C     is the state of body X relative to body Y.
C
      CENTER = TARG
      CALL CLEARD ( 6, STARG )
 
C
C     Note the check for FAILED each time through the loop.  If SPKSFS
C     or SPKPV happens to fail during execution, and the current error
C     handling action is to NOT abort, then CENTER will never become
C     zero, and the loop will execute indefinitely.
C
50001 IF       ( ( .NOT. FAILED() )  .AND. CENTER .NE. 0 )
     .THEN
         CALL SPKSFS ( CENTER, ET, HANDLE, DESCR, IDENT, FOUND )
 
         IF ( .NOT. FOUND ) THEN
            CALL SETMSG ( 'Need state of body # at epoch #.' )
            CALL ERRINT ( '#', CENTER                        )
            CALL ERRDP  ( '#', ET                            )
            CALL SIGERR ( 'SPICE(SPKINSUFFDATA)'             )
 
            CALL CHKOUT ( 'SPKSSB' )
            RETURN
         END IF
 
         CALL SPKPV ( HANDLE, DESCR, ET, REF, TEMP, CENTER )
         CALL VADDG ( STARG, TEMP, 6, STARG )
         GO TO 50001
      END IF
 
      CALL CHKOUT ( 'SPKSSB' )
      RETURN
      END
