C$Procedure      TISBOD ( Transformation, inertial state to bodyfixed )
 
      SUBROUTINE TISBOD ( REF, BODY, ET, TSIPM )
 
C$ Abstract
C
C      Return a 6x6 matrix that transforms states in inertial
C      coordinates to states in body-equator-and-prime-meridian
C      coordinates.
C
C$ Required_Reading
C
C      KERNEL
C      NAIF_IDS
C      ROTATIONS
C      TIME
C
C$ Keywords
C
C      TRANSFORMATION
C      ROTATION
C
C$ Declarations
 
      CHARACTER*(*)         REF
      INTEGER               BODY
      DOUBLE PRECISION      ET
      DOUBLE PRECISION      TSIPM   ( 6,6 )
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      REF        I   ID of inertial reference frame to transform from
C      BODY       I   ID code of body
C      ET         I   Epoch of transformation
C      TSIPM      O   Transformation (state), inertial to prime meridian
C
C$ Detailed_Input
C
C      REF         is the NAIF name for an inertial reference frame.
C                  Acceptable names include:
C
C                    Name       Description
C                    --------   --------------------------------
C                    'J2000'    Earth mean equator, dynamical
C                               equinox of J2000
C
C                    'B1950'    Earth mean equator, dynamical
C                               equinox of B1950
C
C                    'FK4'      Fundamental Catalog (4)
C
C                    'DE-118'   JPL Developmental Ephemeris (118)
C
C                    'DE-96'    JPL Developmental Ephemeris ( 96)
C
C                    'DE-102'   JPL Developmental Ephemeris (102)
C
C                    'DE-108'   JPL Developmental Ephemeris (108)
C
C                    'DE-111'   JPL Developmental Ephemeris (111)
C
C                    'DE-114'   JPL Developmental Ephemeris (114)
C
C                    'DE-122'   JPL Developmental Ephemeris (122)
C
C                    'DE-125'   JPL Developmental Ephemeris (125)
C
C                    'DE-130'   JPL Developmental Ephemeris (130)
C
C                    'GALACTIC' Galactic System II
C
C                    'DE-200'   JPL Developmental Ephemeris (200)
C
C                    'DE-202'   JPL Developmental Ephemeris (202)
C
C                  (See the routine CHGIRF for a full list of names.)
C
C                  The output TIPM will give the transformation
C                  from this frame to the bodyfixed frame specified by
C                  BODY at the epoch specified by ET.
C
C      BODY        is the integer ID code of the body for which the
C                  state transformation matrix is requested. Bodies
C                  are numbered according to the standard NAIF
C                  numbering scheme.  The numbering scheme is
C                  explained in the NAIF_IDS required reading file.
C
C      ET          is the epoch at which the state transformation
C                  matrix is requested. (This is typically the
C                  epoch of observation minus the one-way light time
C                  from the observer to the body at the epoch of
C                  observation.)
C
C$ Detailed_Output
C
C      TSIPM       is a 6x6 transformation matrix.  It is used to
C                  transform states from inertial coordinates to body
C                  fixed (also called equator and prime meridian ---
C                  PM) coordinates.
C
C                  Given a state S in the inertial reference frame
C                  specified by REF, the corresponding bodyfixed state
C                  is given by the matrix vector product:
C
C                     TSIPM * S
C
C                  The X axis of the PM system is directed  to the
C                  intersection of the equator and prime meridian.
C                  The Z axis points along  the spin axis and points
C                  towards the same side of the invariable plane of
C                  the solar system as does earth's north pole.
C
C                  NOTE: The inverse of TSIPM is NOT its transpose.
C                        The matrix, TSIPM, has a structure as shown
C                        below:
C
C                             -            -
C                            |       :      |
C                            |   R   :  0   |
C                            | ......:......|
C                            |       :      |
C                            | dR_dt :  R   |
C                            |       :      |
C                             -            -
C
C                        where R is a time varying rotation matrix and
C                        dR_dt is its derivative.  The inverse of this
C                        matrix is:
C
C                             -              -
C                            |     T  :       |
C                            |    R   :  0    |
C                            | .......:.......|
C                            |        :       |
C                            |      T :   T   |
C                            | dR_dt  :  R    |
C                            |        :       |
C                             -              -
C
C                        The SPICE routine INVSTM is available for
C                        producing this inverse.
C
C$ Parameters
C
C      None.
C
C$ Exceptions
C
C      1) If the kernel pool does not contain all of the data required
C         for computing the transformation matrix, TSIPM, the error
C         SPICE(INSUFFICIENTANGLES) is signalled.
C
C      2) If the reference frame, REF,  is not recognized, a routine
C         called by TISBOD will diagnose the condition and invoke the
C         SPICE error handling system.
C
C      3) If the specified body code, BODY, is not recognized, the
C         error is diagnosed by a routine called by TISBOD.
C
C$ Files
C
C      None.
C
C$ Particulars
C
C      The matrix for transforming inertial states to bodyfixed
C      states is the 6x6 matrix shown below as a block structured
C      matrix.
C
C                 -            -
C                |       :      |
C                | TIPM  :  0   |
C                | ......:......|
C                |       :      |
C                | DTIPM : TIPM |
C                |       :      |
C                 -            -
C
C      The transformation from inertial to bodyfixed coordinates is
C      represented in the SPICE kernel pool as:
C
C            TIPM = [W] [HALFPI-DEC] [RA+HALFPI]
C                      3            1           3
C      Thus
C
C           DTIPM = d[W] /dt [HALFPI-DEC] [RA+HALFPI]
C                       3                1           3
C
C                 + [W] d[HALFPI-DEC] /dt [RA+HALFPI]
C                      3             1               3
C
C                 + [W] [HALFPI-DEC] d[RA+HALFPI] /dt
C                      3            1            3
C
C
C      The angles RA, DEC, and W are defined as follows:
C
C                                         2      ____
C                                    RA2*t       \
C            RA  = RA0  + RA1*t/T  + ------   +  /     a  sin theta
C                                       2        ----   i          i
C                                      T           i
C
C                                          2     ____
C                                    DEC2*t      \
C            DEC = DEC0 + DEC1*t/T + -------  +  /    d  cos theta
C                                        2       ----  i          i
C                                       T          i
C
C
C                                        2      ____
C                                    W2*t       \
C            W   = W0   + W1*t/d   + -----   +  /     w  sin theta
C                                       2       ----   i          i
C                                      d          i
C
C
C      where:
C
C            d = seconds/day
C
C            T = seconds/Julian century
C
C            a , d , and w  arrays apply to satellites only.
C             i   i       i
C
C            theta  = THETA0(i) + THETA1(i)*t/T are specific to each
C                 i
C
C            planet.
C
C
C        These angles -- typically nodal rates -- vary in number and
C        definition from one planetary system to the next.
C
C
C
C        Thus
C                                         ____
C                             2*RA2*t     \
C          dRA/dt  = RA1/T  + -------   + /   a THETA1(i)/T cos theta
C                                 2       ---- i                     i
C                                T          i
C
C                                         ____
C                             2*DEC2*t    \
C          dDEC/dt = DEC1/T + --------  - /    d  THETA1(i)/T sin theta
C                                 2       ----  i                      i
C                                T          i
C
C                                         ____
C                             2*W2*t      \
C          dW/dt   = W1/d   + ------    + /    w  THETA1(i)/T cos theta
C                                 2       ----  i                      i
C                                d          i
C
C
C
C$ Examples
C
C      Note that the items necessary to compute the Euler angles
C      must have been loaded into the kernel pool (by one or more
C      previous calls to LDPOOL).  The Euler angles are typically
C      stored in the P_constants kernel file that comes with
C      SPICELIB.
C
C      Example 1.
C
C      In the following code fragment, TISBOD is used to transform
C      a state in J2000 inertial coordinates to a state in bodyfixed
C      coordinates.
C
C      The 6-vectors STATE represents the inertial state (position and
C      velocity) of an object with respect to the center of the body
C      at time ET.
C
C      C
C      C     First load the kernel pool.
C      C
C            CALL LDPOOL ( 'PLANETARY_CONSTANTS.KER' )
C
C      C
C      C     Next get the transformation and its derivative.
C      C
C            CALL TISBOD ( 'J2000', BODY, ET, TSIPM )
C
C      C
C      C     Convert position to bodyfixed coordinates.
C      C
C            CALL MXVG    ( TSIPM, STATE, 6, 6, BDSTAT )
C
C
C      Example 2.
C
C      In the example below, TISBOD is used to compute
C      the angular velocity vector (with respect to an inertial frame)
C      of the specified body at time ET.
C
C      C
C      C     First get the state transformation matrix.
C      C
C            CALL TISBOD ( BODY,  ET,   TSIPM )
C
C      C
C      C     This matrix has the form:
C      C
C      C           -            -
C      C          |       :      |
C      C          | TIPM  :  0   |
C      C          | ......:......|
C      C          |       :      |
C      C          | DTIPM : TIPM |
C      C          |       :      |
C      C           -            -
C      C
C      C     We extract TIPM and DTIPM
C      C
C
C            DO  I = 1,3
C               DO  J = 1,3
C
C                  TIPM  ( I, J ) = TSIPM ( I,   J )
C                  DTIPM ( I, J ) = TSIPM ( I+3, J )
C
C               END DO
C            END DO
C
C      C
C      C     The transpose of TIPM and DTIPM, (TPMI and DTPMI), give
C      C     the transformation from bodyfixed coordinates to inertial
C      C     coordinates.
C      C
C      C     Here is a fact about the relationship between angular
C      C     velocity associated with a time varying rotation matrix
C      C     that gives the orientation of a body with respect to
C      C     an inertial frame.
C      C
C      C        The angular velocity vector can be read from the off
C      C        diagonal components of the matrix product:
C      C
C      C                                t
C      C        OMEGA =     DTPMI * TPMI
C      C
C      C                         t
C      C              =     DTIPM * TIPM
C      C
C      C        the components of the angular velocity V will appear
C      C        in this matrix as:
C      C
C      C             _                   _
C      C            |                     |
C      C            |   0    -V(3)  V(2)  |
C      C            |                     |
C      C            |  V(3)    0   -V(1)  |
C      C            |                     |
C      C            | -V(2)   V(1)   0    |
C      C            |_                   _|
C      C
C      C
C            CALL MTXM ( DTIPM, TIPM, OMEGA )
C
C            V(1) = OMEGA (3,2)
C            V(2) = OMEGA (1,3)
C            V(3) = OMEGA (2,1)
C
C$ Restrictions
C
C      The kernel pool must be loaded with the appropriate coefficients
C      (from the P_constants kernel) prior to calling this routine.
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C      N. J. Bachman   (JPL)
C      W. L. Taber     (JPL)
C
C$ Version
C
C-     SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 2.0.0, 04-SEP-1991 (NJB)
C
C         Updated to handle P_constants referenced to different epochs
C         and inertial reference frames.
C
C         $Required_Reading and $Literature_References sections were
C         updated.
C
C-     SPICELIB Version 1.0.0, 05-NOV-1990 (WLT)
C
C-&
 
C$ Index_Entries
C
C     transformation from inertial state to bodyfixed
C
C-&
 
 
 
C$ Revisions
C
C-     SPICELIB Version 2.0.0, 04-SEP-1991 (NJB)
C
C         Updated to handle P_constants referenced to different epochs
C         and inertial reference frames.
C
C         TISBOD now checks the kernel pool for presence of the
C         variables
C
C            BODY#_CONSTANTS_REF_FRAME
C
C         and
C
C            BODY#_CONSTANTS_JED_EPOCH
C
C         where # is the NAIF integer code of the barycenter of a
C         planetary system or of a body other than a planet or
C         satellite.  If either or both of these variables are
C         present, the P_constants for BODY are presumed to be
C         referenced to the specified inertial frame or epoch.
C         If the epoch of the constants is not J2000, the input
C         time ET is converted to seconds past the reference epoch.
C         If the frame of the constants is not the frame specified
C         by REF, the rotation from the P_constants' frame to
C         body-fixed coordinates is transformed to the rotation from
C         the requested frame to body-fixed coordinates.  The same
C         transformation is applied to the derivative of this
C         rotation.
C
C         Due to the prescience of the original author, the code
C         was already prepared to handle the possibility of
C         specification of a P_constants inertial reference frame via
C         kernel pool variables.
C
C
C         Also, the $Required_Reading and $Literature_References
C         sections were updated.  The SPK required reading has been
C         deleted from the $Literature_References section, and the
C         NAIF_IDS, KERNEL, and TIME Required Reading files have
C         been added in the $Required_Reading section.
C
C-&
 
 
 
C
C     SPICELIB functions
C
      DOUBLE PRECISION      HALFPI
      DOUBLE PRECISION      J2000
      DOUBLE PRECISION      RPD
      DOUBLE PRECISION      SPD
      DOUBLE PRECISION      VDOTG
 
      LOGICAL               BODFND
      LOGICAL               RETURN
      LOGICAL               FAILED
 
 
C
C     Local parameters
C
      INTEGER               MAXANG
      PARAMETER           ( MAXANG = 100 )
 
C
C     Local variables
C
      CHARACTER*32          ITEM
 
      INTEGER               DIM
      INTEGER               J2CODE
      INTEGER               REFID
 
      DOUBLE PRECISION      PCKEPC
      DOUBLE PRECISION      EPOCH
      DOUBLE PRECISION      PCKREF
 
      DOUBLE PRECISION      RCOEF  ( 0:2 )
      DOUBLE PRECISION      DCOEF  ( 0:2 )
      DOUBLE PRECISION      WCOEF  ( 0:2 )
 
      LOGICAL               ISSAT
      LOGICAL               FOUND
      INTEGER               BARY
 
 
 
      INTEGER               NA
      INTEGER               ND
      INTEGER               NW
      DOUBLE PRECISION      AC     ( MAXANG )
      DOUBLE PRECISION      DC     ( MAXANG )
      DOUBLE PRECISION      WC     ( MAXANG )
 
      INTEGER               NTHETA
      INTEGER               NPAIRS
 
      DOUBLE PRECISION      TCOEF  ( 2, MAXANG )
      DOUBLE PRECISION      THETA
      DOUBLE PRECISION      SINTH  (    MAXANG )
      DOUBLE PRECISION      COSTH  (    MAXANG )
 
      DOUBLE PRECISION      DTHETA
      DOUBLE PRECISION      DSINTH (    MAXANG )
      DOUBLE PRECISION      DCOSTH (    MAXANG )
 
      DOUBLE PRECISION      T
      DOUBLE PRECISION      D
 
      DOUBLE PRECISION      RA
      DOUBLE PRECISION      DEC
      DOUBLE PRECISION      W
 
 
      DOUBLE PRECISION      DRA
      DOUBLE PRECISION      DDEC
      DOUBLE PRECISION      DW
 
      DOUBLE PRECISION      WM     ( 3, 3 )
      DOUBLE PRECISION      DECM   ( 3, 3 )
      DOUBLE PRECISION      RAM    ( 3, 3 )
 
      DOUBLE PRECISION      DWM    ( 3, 3 )
      DOUBLE PRECISION      DDECM  ( 3, 3 )
      DOUBLE PRECISION      DRAM   ( 3, 3 )
 
      DOUBLE PRECISION      DWDT   ( 3, 3 )
      DOUBLE PRECISION      DDECDT ( 3, 3 )
      DOUBLE PRECISION      DRADT  ( 3, 3 )
 
      DOUBLE PRECISION      MTERM1 ( 3, 3 )
      DOUBLE PRECISION      MTERM2 ( 3, 3 )
      DOUBLE PRECISION      MTERM3 ( 3, 3 )
 
      DOUBLE PRECISION      TIPM   ( 3, 3 )
      DOUBLE PRECISION      DTIPM  ( 3, 3 )
      DOUBLE PRECISION      REQ2PC ( 3, 3 )
      INTEGER               I
      INTEGER               J
 
 
      INTEGER               PCREF
      INTEGER               REQREF
 
      LOGICAL               FIRST
 
C
C     Saved variables
C
      SAVE                  FIRST
      SAVE                  J2CODE
 
C
C     Initial values
C
      DATA  FIRST / .TRUE. /
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'TISBOD' )
      END IF
 
 
C
C     Get the code for the J2000 frame, if we don't have it yet.
C
      IF ( FIRST ) THEN
 
         CALL IRFNUM ( 'J2000', J2CODE )
         FIRST = .FALSE.
 
      END IF
 
C
C     Find the body code used to label the reference frame and epoch
C     specifiers for the orientation constants for BODY.
C
C     For planetary systems, the reference frame and epoch for the
C     orientation constants is associated with the system barycenter,
C     not with individual bodies in the system.  For any other bodies,
C     (the Sun or asteroids, for example) the body's own code is used
C     as the label.
C
      IF (  ( BODY .GT. 100 ) .AND. ( BODY .LT. 1000 )  ) THEN
         REFID = BODY / 100
      ELSE
         REFID = BODY
      END IF
 
C
C     Look up the epoch of the constants.  The epoch is specified
C     as a Julian ephemeris date.  The epoch defaults to J2000.
C
      ITEM  =  'BODY#_CONSTANTS_JED_EPOCH'
 
      CALL REPMI  ( ITEM, '#',   REFID,   ITEM   )
      CALL RTPOOL ( ITEM,  DIM,  PCKEPC,  FOUND  )
 
 
      IF ( FOUND ) THEN
C
C        The reference epoch is returned as a JED.  Convert to
C        ephemeris seconds past J2000.  Then convert the input
C        ET to seconds past the reference epoch.
C
         PCKEPC  =  SPD() * ( PCKEPC - J2000() )
         EPOCH   =  ET    -   PCKEPC
 
      ELSE
         EPOCH   =  ET
      END IF
 
C
C     Look up the reference frame of the constants.  The reference
C     frame is specified by a code recognized by CHGIRF.  The default
C     frame is J2000, symbolized by the code J2CODE.
C
      ITEM  =  'BODY#_CONSTANTS_REF_FRAME'
 
      CALL REPMI  ( ITEM,  '#',  REFID,   ITEM   )
      CALL RTPOOL ( ITEM,  DIM,  PCKREF,  FOUND  )
 
      IF ( FOUND ) THEN
         PCREF  =  NINT ( PCKREF )
      ELSE
         PCREF  =  J2CODE
      END IF
 
C
C     Next get the rotation from the user specified frame to the
C     P_constants frame.  Note:  We are doing this now primarily
C     to catch problems early in the routine.  If everything
C     works, we aren't going to need this matrix until near the
C     end of the routine.
C
      CALL IRFNUM ( REF,    REQREF         )
      CALL IRFROT ( REQREF, PCREF,  REQ2PC )
 
      IF ( FAILED() ) THEN
         CALL CHKOUT ( 'TISBOD' )
         RETURN
      END IF
 
C
C     Whatever the body, it has quadratic time polynomials for
C     the RA and Dec of the pole, and for the rotation of the
C     Prime Meridian.
C
      ITEM = 'POLE_RA'
      CALL CLEARD (              3, RCOEF )
      CALL BODVAR ( BODY, ITEM, NA, RCOEF )
 
      ITEM = 'POLE_DEC'
      CALL CLEARD (              3, DCOEF )
      CALL BODVAR ( BODY, ITEM, ND, DCOEF )
 
      ITEM = 'PM'
      CALL CLEARD (              3, WCOEF )
      CALL BODVAR ( BODY, ITEM, NW, WCOEF )
 
 
C
C     If the body is a satellite, there may be additional nutation
C     and libration (THETA) terms.
C
      NTHETA = 0
      NA     = 0
      ND     = 0
      NW     = 0
 
      ISSAT  = (        BODY              .GT. 100
     .           .AND.  BODY              .LT. 1000
     .           .AND.  MOD ( BODY, 100 ) .NE. 99   )
 
      IF ( ISSAT ) THEN
 
         BARY = BODY / 100
 
         ITEM = 'NUT_PREC_ANGLES'
 
C
C        There is something a bit obscure going on below.  We
C        are passing a two dimensional array ( TCOEF(2,MAXANG) ).
C        But BODVAR is expecting a 1-dimensional array.
C        BODVAR loads the array TCOEF in the following order
C
C         TCOEF(1,1), TCOEF(2,1), TCOEF(1,2), TCOEF(2,2), TCOEF(1,3) ...
C
C        The NTHETA that comes back is the total number of items loaded,
C        but we will need the actual limit on the second dimension.
C        That is --- NTHETA / 2.
C
         IF ( BODFND ( BARY, ITEM ) ) THEN
            CALL BODVAR ( BARY, ITEM, NTHETA, TCOEF )
            NPAIRS = NTHETA / 2
         END IF
 
C
C        Look up the right ascension nutations in the precession
C        of the pole.  NA is the number of Ascension coefficients.
C        AC are the Ascension coefficients.
C
         ITEM = 'NUT_PREC_RA'
 
         IF ( BODFND ( BODY, ITEM ) ) THEN
            CALL BODVAR ( BODY, ITEM, NA, AC )
         END IF
 
C
C        Look up the declination nutations in the precession of the
C        pole.  ND is the number of Declination coefficients. DC
C        are the Declination coefficients.
C
         ITEM = 'NUT_PREC_DEC'
 
         IF ( BODFND ( BODY, ITEM ) ) THEN
            CALL BODVAR ( BODY, ITEM, ND, DC )
         END IF
 
C
C        Finally look up the prime meridian nutations.  NW is the
C        number of coefficients.  WC is the array of coefficients.
C
         ITEM = 'NUT_PREC_PM'
 
         IF ( BODFND ( BODY, ITEM ) ) THEN
            CALL BODVAR ( BODY, ITEM, NW, WC )
         END IF
 
      END IF
 
C
C     The number of coefficients returned had better not be bigger
C     than the number of angles we are going to compute.  If it
C     is we simply signal an error and bag it, fer sure.
C
      IF ( MAX ( NA, ND, NW ) .GT. NPAIRS ) THEN
         CALL SETMSG ( 'TISBOD: Insufficient number of nutation/'  //
     .                 'precession angles for body * at time #.'     )
         CALL ERRINT ( '*', BODY )
         CALL ERRDP  ( '#', ET   )
         CALL SIGERR ( 'SPICE(INSUFFICIENTANGLES)' )
         CALL CHKOUT ( 'TISBOD' )
         RETURN
      END IF
 
 
C
C     Evaluate the time polynomials and their derivatives w.r.t. EPOCH
C     at EPOCH.
C
C
C     Evaluate the time polynomials at EPOCH.
C
      D   = SPD()
      T   = D  *  36525.D0
 
      RA   = RCOEF(0) + (EPOCH/T) * ( RCOEF(1) + (EPOCH/T) * RCOEF(2) )
      DEC  = DCOEF(0) + (EPOCH/T) * ( DCOEF(1) + (EPOCH/T) * DCOEF(2) )
      W    = WCOEF(0) + (EPOCH/D) * ( WCOEF(1) + (EPOCH/D) * WCOEF(2) )
 
      DRA  = ( RCOEF(1)  + 2.0D0*(EPOCH/T)*RCOEF(2) ) / T
      DDEC = ( DCOEF(1)  + 2.0D0*(EPOCH/T)*DCOEF(2) ) / T
      DW   = ( WCOEF(1)  + 2.0D0*(EPOCH/D)*WCOEF(2) ) / D
 
 
C
C     Compute the nutations and librations (and their derivatives)
C     as appropriate.
C
      DO 50001
     .   I = 1, NPAIRS
 
         THETA    = ( TCOEF(1,I)  + (EPOCH/T) * TCOEF(2,I)     ) * RPD()
         DTHETA   = (                           TCOEF(2,I) / T ) * RPD()
 
         SINTH(I)  =  SIN ( THETA )
         COSTH(I)  =  COS ( THETA )
         DSINTH(I) =  COS ( THETA ) * DTHETA
         DCOSTH(I) = -SIN ( THETA ) * DTHETA
 
50001 CONTINUE
 
C
C     Adjust RA, DEC, W and their derivatives by the librations and
C     nutations.
C
      RA  = RA    + VDOTG ( AC, SINTH,  NA )
      DEC = DEC   + VDOTG ( DC, COSTH,  ND )
      W   = W     + VDOTG ( WC, SINTH,  NW )
 
      DRA  = DRA  + VDOTG ( AC, DSINTH, NA )
      DDEC = DDEC + VDOTG ( DC, DCOSTH, ND )
      DW   = DW   + VDOTG ( WC, DSINTH, NW )
 
C
C     Convert from degrees to radians
C
      RA   = RA   * RPD()
      DEC  = DEC  * RPD()
      W    = W    * RPD()
 
      DRA  = DRA  * RPD()
      DDEC = DDEC * RPD()
      DW   = DW   * RPD()
 
C
C     Compute the factors of the transformation from the Euler angles.
C
      CALL ROTATE ( RA + HALFPI(),  3, RAM  )
      CALL ROTATE ( HALFPI() - DEC, 1, DECM )
      CALL ROTATE ( W,              3, WM   )
 
C
C     Compute TIPM.
C
      CALL MXM ( DECM, RAM,  TIPM )
      CALL MXM ( WM,   TIPM, TIPM )
 
C
C     Compute the derivatives of each of the above matrices w.r.t.
C     the angle of rotation.
C
      CALL DROTAT ( RA + HALFPI(),  3, DRAM )
      CALL DROTAT ( HALFPI() - DEC, 1, DDECM )
      CALL DROTAT ( W,              3, DWM   )
 
 
C
C     Compute each of the terms of the derivative.
C
C
C           DTIPM = d[W] /dt [HALFPI-DEC] [RA+HALFPI]
C                       3                1           3
C
      CALL VSCLG  ( DW,     DWM,  9, DWDT   )
      CALL MXM    ( DWDT,   DECM,    MTERM1 )
      CALL MXM    ( MTERM1, RAM,     MTERM1 )
 
C
C                 + [W] d[HALFPI-DEC] /dt [RA+HALFPI]
C                      3             1               3
C
      CALL VSCLG  ( -DDEC,  DDECM,  9, DDECDT )
      CALL MXM    ( WM,     DDECDT,    MTERM2 )
      CALL MXM    ( MTERM2, RAM,       MTERM2 )
 
C
C                 + [W] [HALFPI-DEC] d[RA+HALFPI] /dt
C                      3            1            3
C
      CALL VSCLG  ( DRA,    DRAM,  9, DRADT  )
      CALL MXM    ( WM,     DECM,     MTERM3 )
      CALL MXM    ( MTERM3, DRADT,    MTERM3 )
 
C
C     Now add all the terms together
C
      CALL VADDG ( MTERM1, MTERM2, 9, DTIPM )
      CALL VADDG ( DTIPM,  MTERM3, 9, DTIPM )
 
C
C     Rotate from the requested frame to the bodyfixed frame.
C
      CALL MXM   ( TIPM,   REQ2PC,    TIPM  )
      CALL MXM   ( DTIPM,  REQ2PC,    DTIPM )
 
C
C     Finally, pack the results into TSIPM so we get a matrix that
C     looks like:
C
C                 -            -
C                |       :      |
C                | TIPM  :  0   |
C                | ......:......|
C                |       :      |
C                | DTIPM : TIPM |
C                |       :      |
C                 -            -
C
      DO 50002
     .   I = 1, 3
         DO 50003
     .       J  = 1, 3
 
            TSIPM ( I,   J   ) = TIPM ( I, J )
            TSIPM ( I+3, J   ) = DTIPM( I, J )
            TSIPM ( I+3, J+3 ) = TIPM ( I, J )
            TSIPM ( I,   J+3 ) = 0.0D0
 
50003    CONTINUE
50002 CONTINUE
 
 
C
C     That's all folks. Check out and get out.
C
      CALL CHKOUT ( 'TISBOD' )
 
      RETURN
      END
