C$Procedure      TPARSE ( Parse a UTC time string )
 
       SUBROUTINE TPARSE ( STRING, UTCSEC, ERROR )
 
C$ Abstract
C
C      Parse a UTC time string.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      PARSING, TIME
C
C$ Declarations
 
      CHARACTER*(*)      STRING
      DOUBLE PRECISION   UTCSEC
      CHARACTER*(*)      ERROR
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      STRING     I   Input time string, UTC.
C      UTCSEC     O   Equivalent UTC seconds past J2000.
C      ERROR      O   Descriptive error message.
C
C$ Detailed_Input
C
C      STRING      is an input time string, containing a Calendar or
C                  Julian Date, UTC.  It may be in several different
C                  formats and can make use of abbreviations and
C                  default values.  Several example UTC strings and
C                  the times that they translate to are listed below.
C                  The rules by which TPARSE interprets UTC strings
C                  follow these examples.
C
C                    UTC STRING                   TRANSLATION
C                    ----------                   -----------
C
C                 '1/9/86 3 12 59.2     '    (  9 JAN 1986 03:12:59.2 )
C                 '1 9 86 3,12,59.2     '    (  9 JAN 1986 03:12:59.2 )
C                 '2 jan 91 3:00 12.2   '    (  2 JAN 1991 03:00:12.2 )
C                 '91 MAR 10 12:00:00   '    ( 10 MAR 1991 12:00:00   )
C                 '29 February 1975 3:00'    (  1 MAR 1975 03:00:00   )
C                 '10 October 29 3:58   '    ( 29 OCT 2010 03:58:00   )
C                 'dec 31 86 12         '    ( 31 DEC 1986 12:00:00   )
C                 '86-365 // 12         '    ( 31 DEC 1986 12:00:00   )
C                 'JUL 77               '    (  1 JUL 1977 00:00:00   )
C                 'JD 2451545.          '    (  1 JAN 2000 12:00:00   )
C                 'jd 2451545.          '    (  1 JAN 2000 12:00:00   )
C                 'JD2451545.           '    (  1 JAN 2000 12:00:00   )
C
C
C                  Calendar dates consist of up to six tokens:
C                  one each for Year, Month, Day, Hours, Minutes,
C                  and Seconds.
C
C                  Valid token delimiters are:
C
C                        ' '           space
C                        ','           comma
C                        '/'           slash
C                        '-'           dash
C                        ':'           colon
C
C                  With the exception of spaces, consecutive delimiters
C                  specify a blank token.  Blank tokens and tokens
C                  missing from the right of the string are assigned
C                  the following defaults:
C
C                      - Year      defaults to    1950
C                      - Month                    January
C                      - Day                      1
C                      - Hours                    0
C                      - Minutes                  0
C                      - Seconds                  0.0000000
C
C                  The month may be specified as an integer or a name.
C                  At least three characters are required in a name.
C
C                  When all six tokens are present, the last three
C                  always represent Hours, Minutes, and Seconds
C                  respectively.  The first three tokens always
C                  represent Year, Month, and Day, with the order
C                  determined according to the following rules:
C
C                     1. If a month name is present, then if only one
C                        of the other two inputs is > 31 it must be
C                        the year. Otherwise, default to M-D-Y, Y-M-D,
C                        or Y-D-M according to the placement of M.
C                        If both of the other inputs are > 31, then
C                        one and only one must be greater than 1000;
C                        it must be the year. Otherwise the input
C                        string is rejected as ambiguous.
C
C                     2. If no month name is present, first examine
C                        the third token to see if it is > 31. If so,
C                        and the first token is > 12, then parse as
C                        D-M-Y. Otherwise, the default is M-D-Y.
C
C                        If the third token is <= 31, then the order of
C                        the input is taken to be either Y-M-D or Y-D-M.
C                        If the second token is > 12 or the third token
C                        is blank, then parse as Y-D-M. Otherwise, the
C                        final default is Y-M-D.
C
C                  Years in the range 0-49 are taken to be the years
C                  2000 through 2049. Years in the range 50-200 are
C                  taken to be the years 1950 through 2100.
C                  Years beginning after 10000 return an error.
C
C
C                  Note that Day of Year may be substituted for Month
C                  and Day in either of the following ways:
C
C                     1. By setting the month to January and the day to
C                        Day of Year, e.g.,
C
C                           '1986 JAN 247 12:00:01.184'
C
C                     2. By specifying a blank token for the month.
C                        The most popular form for DOY entry is:
C
C                           '1986-247 // 12:00:01.184'
C
C                  Julian Dates consist of two tokens.
C                  The first contains the letters 'JD', in any
C                  combinations of upper- or lower-case. The
C                  second token is a Julian Date. For convenience,
C                  the two tokens may be concatenated, as shown
C                  in the examples below. Valid token delimiters
C                  are the same as for Calendar format.
C
C                  If the token 'JD' is entered by itself, the
C                  input string is rejected as ambiguous.
C
C                  The length of UTC should not exceed 80 characters.
C
C$ Detailed_Output
C
C      UTCSEC      is the equivalent of UTC, expressed in UTC
C                  seconds past J2000. If an error occurs, or if
C                  UTC is ambiguous, UTCSEC is not changed.
C
C      ERROR       is a descriptive error message, which is blank when
C                  no error occurs.
C
C$ Parameters
C
C      None.
C
C$ Exceptions
C
C      Error free.
C
C$ Files
C
C      None.
C
C$ Particulars
C
C      The input string is parsed for six tokens, delimited by any
C      of the valid delimiters (space, comma, slash, hyphen, colon).
C
C      If the first token is (or begins with) 'JD', the input is
C      taken to be a Julian Date. Extra tokens are ignored.
C
C      Otherwise, the last three tokens are assigned to hours,
C      minutes, and seconds respectively. The first three are
C      assigned to year, month, and day, according to magnitude and
C      the presence (or lack) of a month name, according to the rules
C      described under Detailed_Inputs above. The Muller-Wimberly
C      formula is used to compute the number of days past 2000 JAN 1,
C      which is then converted to UTC seconds past J2000.
C
C		 Since the routine is works by breaking the input string into
C		 a sequence of tokens whose meanings are determined by position
C      and magnitude, you can supply strings such as 1993 FEB 35 and
C      have this correctly interpreted as March 7, 1993.  However,
C      this default action can be modified so that only "proper"
C      calendar dates and times are recognized.  To do this call
C      the entry point  TPARCH as shown below:
C
C         CALL TPARCH ( 'YES' )
C
C      This will cause the routine to treat dates and times with
C      components outside the normal range as errors.
C
C      To return to the default behavior
C
C         CALL TPARCH ( 'NO' )
C
C$ Examples
C
C      The following are examples of valid inputs to TPARSE:
C
C               UTC STRING                      TRANSLATION
C               ----------                      -----------
C
C         '1/9/86 3 12 59.22451 '       (  9 JAN 1986 03:12:59.22451 )
C         '86,1,9,3,12,59.22451 '       (  9 JAN 1986 03:12:59.22451 )
C         '86-365//12           '       ( 31 DEC 1986 12:00:00       )
C         '2 jan 91 3:00 12.2   '       (  2 JAN 1991 03:00:12.2     )
C         '2 janua 91 3:00 12.2 '       (  2 JAN 1991 03:00:12.2     )
C         '29 February 1975 3:00'       (  1 MAR 1975 03:00:00       )
C         '10 October 29 3:58   '       ( 29 OCT 2010 03:58:00       )
C         'JUL 77               '       (  1 JUL 1977 00:00:00       )
C         'JUL 29               '       ( 29 JUL 1950 00:00:00       )
C         'JD 2451545.          '       (  1 JAN 2000 12:00:00       )
C         'JD-2451545.          '       (  1 JAN 2000 12:00:00       )
C         'jd 2451545.          '       (  1 JAN 2000 12:00:00       )
C         'JD2451545.           '       (  1 JAN 2000 12:00:00       )
C
C      The next set of examples are intended to help clarify how
C      default values are assigned to tokens.
C
C         '1989 Feb // 12:01:01'        (  1 FEB 1989 12:01:01       )
C         '1989 Feb /  12:01:01'        ( 12 FEB 1989 01:01:00       )
C         '1989 Feb            '        (  1 FEB 1989 00:00:00       )
C
C         ' 89-345 // 12:00:00 '        ( 11 DEC 1989 12:00:00       )
C         ' 89-345 /  12:00:00 '        ( 10 NOV 1990 00:00:00       )
C
C
C      In the first example the consecutive delimiters specify a blank
C      token and so the day is assigned the default value of one.  In
C      the second example, while a person might assume that the date
C      was left blank, TPARSE interprets the 12 as being the value for
C      the day. This is why blank tokens are needed in the middle of
C      strings. Also note that since there are only five tokens,
C      TPARSE assigns to seconds the default value of zero. In the
C      third example, since the day, hours, minutes, and seconds are
C      all missing from the right of the string, they are assigned the
C      default values of 1, 0, 0, and 0 respectively.
C
C      The fourth and fifth examples are similar to the first two
C      except that they are in the day of year format. Note that they
C      translate to two very different times.  The reason is that the
C      consecutive delimiters in the fourth string specify that the
C      third token is blank, and thus TPARSE parses the tokens in the
C      order Y-D-M with the month assigned the default value of one.
C      Since the fifth string does not contain consecutive delimiters,
C      the third token is assigned the value 12 and the order is
C      assumed to be Y-M-D.
C
C
C      The following examples would be rejected as ambiguous.
C
C          '32 jan 32'
C          '85 86 january'
C          '86 85 january'
C          'January 80 81'
C          'JD,,,2451545'
C
C
C$ Restrictions
C
C      None.
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C      W.M. Owen       (JPL)
C      I.M. Underwood  (JPL)
C      W.L. Taber      (JPL)
C
C$ Version
C
C-    SPICELIB Version 3.0.0, 30-JUL-1993 (WLT)
C
C        The entry point TPARCH was added so that users may
C        restrict the set of input calendar strings to those
C        that are in proper form.
C
C-    SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 2.0.0, 18-NOV-1991 (MJS)
C
C        TPARSE no longer accepts a blank time string.
C
C-    SPICELIB Version 1.0.1, 26-MAR-1991 (JML)
C
C        In the Detailed_Input section of the header, the
C        description of how default values are assigned to
C        tokens in STRING was clarified.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (WMO) (IMU)
C
C-&
C
C$ Index_Entries
C
C     parse a utc time string
C
C-&
 
C
C$ Revisions
C
C-    SPICELIB Version 3.0.0, 30-JUL-1993 (WLT)
C
C        The entry point TPARCH was added so that users may
C        restrict the set of input calendar strings to those
C        that are in proper form.
C
C-    SPICELIB Version 2.0.0, 18-NOV-1991 (MJS)
C
C        TPARSE no longer accepts a blank time string. Prior to
C        this fix, TPARSE interpreted a blank time string to be
C        -1577880000.000 UTC seconds (1 JAN 1950 00:00:00).
C
C-    SPICELIB Version 1.0.1, 26-MAR-1991 (JML)
C
C        In the Detailed_Input section of the header, the
C        description of how default values are assigned to
C        tokens in STRING was clarified.
C
C        NAIFers are accustomed to specifying day of year
C        formats of UTC strings in the following form:
C
C           1986-247 // 12:00:00
C
C        This revision to the header states explicitly that
C        the // is a blank token which results in the default
C        value being assigned to the month token.  The previous
C        version of the header implied that tokens could be left
C        out or "missing" from the string, and that default values
C        would automatically be assigned.  This works only for
C        tokens missing from the right end of the string.  For
C        default values to be assigned to tokens missing from the
C        middle of a UTC string, consecutive delimiters such as
C        // or :: must be included.
C
C
C-&
 
 
 
C
C     SPICELIB functions
C
      DOUBLE PRECISION      J2000
      DOUBLE PRECISION      SPD
 
      INTEGER               LASTNB
 
      LOGICAL               EQSTR
 
 
C
C     Local variables
C
      INTEGER               MAXYR
      PARAMETER           ( MAXYR = 10000 )
 
      CHARACTER*5           DELIMS
      PARAMETER           ( DELIMS = ' ,/-:' )
 
 
 
 
 
 
      CHARACTER*3           C        (  6 )
      CHARACTER*9           NAMES    ( 12 )
      CHARACTER*30          CTOKEN   (  6 )
      CHARACTER*100         INPUT
 
      DOUBLE PRECISION      JD
      DOUBLE PRECISION      SEC
 
      INTEGER               DINMON ( 12 )
      INTEGER               DINYR
      INTEGER               LEAPDY
      INTEGER               DAY
      INTEGER               DTOKEN
      INTEGER               HOUR
      INTEGER               I
      INTEGER               J
      INTEGER               L
      INTEGER               MINUTE
      INTEGER               MONTH
      INTEGER               MTOKEN
      INTEGER               N
      INTEGER               TOKEN    (  6 )
      INTEGER               YEAR
      INTEGER               YTOKEN
 
      LOGICAL               BADSEC
      LOGICAL               DOCHCK
      LOGICAL               HADMON
      LOGICAL               LEAPSD
 
      SAVE                  C
      SAVE                  DINMON
      SAVE                  DOCHCK
      SAVE                  NAMES
 
 
 
 
C
C     Initial values
C
      DATA             C        / '1: ',
     .                            '2: ',
     .                            '3: ',
     .                            '4: ',
     .                            '5: ',
     .                            '6: '  /
 
      DATA                  DINMON / 31,      28,
     .                               31,      30,
     .                               31,      30,
     .                               31,      31,
     .                               30,      31,
     .                               30,      31  /
 
 
 
      DATA             DOCHCK   / .FALSE. /
 
      DATA             NAMES    / 'JANUARY  ',   'FEBRUARY ',
     .                            'MARCH    ',   'APRIL    ',
     .                            'MAY      ',   'JUNE     ',
     .                            'JULY     ',   'AUGUST   ',
     .                            'SEPTEMBER',   'OCTOBER  ',
     .                            'NOVEMBER ',   'DECEMBER '   /
 
 
 
 
C
C    Go no further if STRING is blank.
C
      IF ( STRING .EQ. ' ' ) THEN
         ERROR  = 'The input time string is blank.'
         RETURN
      ENDIF
 
C
C     Convert INPUT to upper case, to avoid comparison problems.
C
      CALL LJUST ( STRING, INPUT )
      CALL NUCASE ( INPUT,  INPUT )
 
C
C     Parse the input character string for up to 6 tokens representing
C     year, month, day, hour, minute, second. Valid delimeters are
C     spaces, commas, slashes, hyphens, and colons.
C
      CALL CLEARC ( 6, CTOKEN )
      CALL CLEARI ( 6, TOKEN  )
 
      CALL LPARSM ( INPUT, DELIMS, 6, N, CTOKEN )
 
 
 
C
C     Julian Date is the easiest case. Let's try that first.
C     There are two possible cases. In either case, the first
C     two characters of the first token will be 'JD'.
C
      IF ( CTOKEN(1)(1:2) .EQ. 'JD' ) THEN
 
C
C        Find the date. It may be the second token, or part of
C        the first token (jammed up against the JD identifier).
C        Wherever it is, put it in the second token.
C
         IF ( CTOKEN(1)(3: ) .NE. ' ' ) THEN
            CTOKEN(2) = CTOKEN(1)(3: )
         END IF
 
C
C        A blank Julian Date is ambiguous.
C
         IF ( CTOKEN(2) .EQ. ' ' ) THEN
            ERROR  = 'The input time string is ambiguous.'
            RETURN
         END IF
 
C
C        If the date is present, and parses correctly, convert
C        directly to UTC seconds past J2000. (Do not pass GO.
C        Do not collect $200.)
C
         CALL NPARSD ( CTOKEN(2), JD, ERROR, J )
 
         IF ( ERROR .EQ. ' ' ) THEN
 
            UTCSEC = ( JD - J2000() ) * SPD()
            RETURN
 
         ELSE
            ERROR  = 'Could not parse Julian Date: ' //
     .               CTOKEN(2)
 
            RETURN
         END IF
 
      END IF
 
 
C
C     Looks like Calendar format. Set up the defaults.
C
      YEAR   = 50
      MONTH  = 1
      DAY    = 1
      HOUR   = 0
      MINUTE = 0
      SEC    = 0.D0
 
C
C     Is one of the first three tokens the name of a month?
C
      MTOKEN = 0
 
      DO 50001
     .   I = 1, MIN ( 3, N )
 
C
C        This can't be a month name unless it starts with one of
C        the right letters.
C
         IF ( INDEX ( 'JFMAMJJASOND', CTOKEN(I)(1:1) ) .GT. 0 ) THEN
 
C
C           Okay. But it must have at least three characters.
C
            L = LASTNB ( CTOKEN(I) )
 
            IF ( L .GE. 3 ) THEN
 
C
C              So far, so good. Does it match the beginning of
C              any of the month names?
C
               DO 50002
     .            J = 1, 12
                  IF ( CTOKEN(I)(1:L) .EQ. NAMES(J)(1:L) ) THEN
                     MTOKEN        = I
                     TOKEN(MTOKEN) = J
                  END IF
50002          CONTINUE
 
            END IF
 
         END IF
 
50001 CONTINUE
 
C
C     Convert the rest of the tokens to integer, except the sixth
C     (seconds), which is double precision. Skip missing tokens.
C
      DO 50003
     .   I = 1, MIN ( 5, N )
 
         IF ( I .NE. MTOKEN .AND. CTOKEN(I) .NE. ' ' ) THEN
 
            CALL NPARSI ( CTOKEN(I), TOKEN(I), ERROR, J )
 
            IF ( ERROR .NE. ' ' ) THEN
               ERROR  = 'Invalid input for token '   //
     .                  C(I)                         //
     .                  CTOKEN(I)
 
               RETURN
            END IF
 
         END IF
 
50003 CONTINUE
 
C
C     Do seconds separately.
C
      IF ( N .EQ. 6 ) THEN
 
         CALL NPARSD ( CTOKEN(6), SEC, ERROR, J )
 
            IF ( ERROR .NE. ' ' ) THEN
               ERROR  = 'Invalid input for token '   //
     .                  C(6)                         //
     .                  CTOKEN(6)
 
               RETURN
            END IF
 
      END IF
 
 
 
C
C     Determine order of year, month, day in first 3 tokens, according
C     to the following rules:
C
C     1. If a month name is present, then if only one of the other
C        two inputs is > 31 it must be the year. Otherwise, default
C        to M-D-Y, Y-M-D, or Y-D-M according to the placement of M.
C        If both of the other inputs are > 31, then one and only one
C        must be greater than 1000; it must be the year. Otherwise
C        the input string is rejected as ambiguous.  In this block of
C        code, I and J are the tokens after and before the month,
C        respectively, in circular permutation.  (So if MTOKEN is 3,
C        I would be 1 and J would be 2.)
C
      IF (  MTOKEN .GT. 0  ) THEN
 
         I = 1 + MOD ( MTOKEN, 3 )
         J = 1 + MOD ( I,      3 )
 
         IF ( TOKEN(I) .GT. 31 .AND. TOKEN(J) .GT. 31 ) THEN
 
            IF ( TOKEN(I) .GT. 1000 .AND. TOKEN(J) .LE. 1000 ) THEN
               YTOKEN = I
               DTOKEN = J
 
            ELSE IF ( TOKEN(J) .GT. 1000 .AND. TOKEN(I) .LE. 1000 ) THEN
               YTOKEN = J
               DTOKEN = I
 
            ELSE
               ERROR  = 'The input time string is ambiguous.'
 
               RETURN
            END IF
 
         ELSE IF ( TOKEN(I) .GT. 31 ) THEN
 
            YTOKEN = I
            DTOKEN = J
 
         ELSE IF ( TOKEN(J) .GT. 31 ) THEN
 
            YTOKEN = J
            DTOKEN = I
 
         ELSE IF ( MTOKEN .EQ. 1 ) THEN
 
            YTOKEN = 3
            DTOKEN = 2
 
         ELSE
 
            YTOKEN = 1
            DTOKEN = 5 - MTOKEN
 
         END IF
 
 
C
C     2. If no month name is present, first examine the third
C        token to see if it is > 31. If so, and the first token
C        is > 12, then parse as D-M-Y. Otherwise, the default
C        is M-D-Y.
C
C        If the third token is <= 31, then the order of the input
C        is taken to be either Y-M-D or Y-D-M. If the second token
C        is > 12 or the third token is blank, then parse as Y-D-M.
C        Otherwise, the final default is Y-M-D.
C
 
      ELSE IF ( TOKEN(3) .GT. 31 ) THEN
 
         YTOKEN = 3
 
         IF ( TOKEN(1) .GT. 12 ) THEN
 
            MTOKEN = 2
            DTOKEN = 1
 
         ELSE
 
            MTOKEN = 1
            DTOKEN = 2
 
         END IF
 
      ELSE
 
         YTOKEN = 1
 
         IF ( TOKEN(2) .GT. 12 .OR. CTOKEN(3) .EQ. ' ' ) THEN
 
            MTOKEN = 3
            DTOKEN = 2
 
         ELSE
 
            MTOKEN = 2
            DTOKEN = 3
 
         END IF
 
      END IF
 
 
 
C
C     Determine year, month, day, hour, and minute. (We've already
C     done seconds.) Use defaults wherever new values have not been
C     supplied.
C
      IF ( CTOKEN(YTOKEN) .NE. ' ' )    YEAR   = TOKEN(YTOKEN)
      IF ( CTOKEN(MTOKEN) .NE. ' ' )    MONTH  = TOKEN(MTOKEN)
      IF ( CTOKEN(DTOKEN) .NE. ' ' )    DAY    = TOKEN(DTOKEN)
      IF ( CTOKEN(4)      .NE. ' ' )    HOUR   = TOKEN(4)
      IF ( CTOKEN(5)      .NE. ' ' )    MINUTE = TOKEN(5)
 
C
C     Use the Muller-Wimberly formula to get days past 2000 JAN 1.
C     Then then get seconds past J2000.
C
C     Make sure that the year isn't too big.
C
      IF ( YEAR .GE. MAXYR ) THEN
         ERROR = 'The year token is too large.'
         RETURN
 
      ELSE IF ( YEAR .LT.  50 ) THEN
         YEAR = YEAR + 2000
 
      ELSE IF ( YEAR .LT. 200 ) THEN
         YEAR = YEAR + 1900
      END IF
C
C     If the user insists, then we need to make sure that
C     all calendar components are in the standard range.
C
      IF ( DOCHCK ) THEN
C
C        If there is a month token
C        then we insist that the day number be in bounds for the
C        month.
C
         HADMON = CTOKEN(MTOKEN) .NE. ' '
 
         LEAPDY =    ( ( (YEAR/4)  *4   ) / YEAR )
     .            -  ( ( (YEAR/100)*100 ) / YEAR )
     .            +  ( ( (YEAR/400)*400 ) / YEAR )
 
         DINMON(2) = 28  + LEAPDY
         DINYR     = 365 + LEAPDY
 
         IF ( MONTH .GT. 12 ) THEN
 
            ERROR = 'There are only 12 months in a year. Month '
     .      //      'number # was specified.'
				CALL REPMI ( ERROR, '#', MONTH, ERROR )
            RETURN
 
         END IF
C
C        If a month was specified, make sure that the day of month
C        is well behaved.
C
         IF ( HADMON ) THEN
 
            IF ( DAY .GT. DINMON(MONTH) ) THEN
 
               IF ( MONTH .EQ. 2 ) THEN
                  ERROR = 'February has only # in #.  The day # '
     .            //      'was specified.'
                  CALL REPMI ( ERROR, '#', DINMON(MONTH), ERROR )
                  CALL REPMI ( ERROR, '#', YEAR,          ERROR )
                  CALL REPMI ( ERROR, '#', DAY,           ERROR )
                  RETURN
               ELSE
                  ERROR = '# has only # days.  Day # was specified.'
                  CALL REPMC ( ERROR, '#', NAMES (MONTH), ERROR )
                  CALL REPMI ( ERROR, '#', DINMON(MONTH), ERROR )
                  CALL REPMI ( ERROR, '#', DAY,           ERROR )
                  RETURN
               END IF
 
            END IF
 
C
C        If no month was present, then we insist that the number
C        of days stay within the bounds for this year.
C
         ELSE
 
            IF ( DAY .GT. DINYR ) THEN
               ERROR = '# has only # days.  Day # was specified.'
               CALL REPMI ( ERROR, '#', YEAR,   ERROR )
               CALL REPMI ( ERROR, '#', DINYR,  ERROR )
               CALL REPMI ( ERROR, '#', DAY,    ERROR )
               RETURN
            END IF
 
         END IF
C
C        Hours, and minutes must always be in the range 0 to 23
C        and 0 to 59 respectively.
C
         IF ( HOUR .GT. 23 ) THEN
 
            ERROR = 'The valid values for the hour component of '
     .      //      'a calendar date are from 0 to 23. The '
     .      //      'value # was supplied. '
            CALL REPMI ( ERROR, '#', HOUR, ERROR )
            RETURN
 
         END IF
 
         IF ( MINUTE .GT. 59 ) THEN
            ERROR = 'The valid values for the minute component of '
     .      //      'a calendar date are from 0 to 59. The '
     .      //      'value # was supplied. '
            CALL REPMI ( ERROR, '#', MINUTE, ERROR )
            RETURN
         END IF
C
C        Seconds must be in the range 0 to 59 and can be in the
C        but  can go up to (but not including) 61 only if the
C        month is  6 or 12 and then only if HR = 23 and MN = 59.
C
 
         IF ( SEC .GE. 61.0D0 ) THEN
            ERROR = 'The valid values for the seconds component of '
     .      //      'a calendar date are from 0 up to but not '
     .      //      'including 61. (Values 60 and greater are only '
     .      //      'valid as the last second of December 31 or '
     .      //      'June 30.) The value # was supplied. '
            CALL REPMD ( ERROR, '#', SEC, 3, ERROR )
            RETURN
         END IF
 
 
C
C        The value 184 is the number of days in July through December.
C
         LEAPSD =       ( MONTH .EQ. 1  .AND. DAY .EQ. DINYR       )
     .            .OR.  ( MONTH .EQ. 1  .AND. DAY .EQ. DINYR - 184 )
     .            .OR.  ( MONTH .EQ. 6  .AND. DAY .EQ. 30          )
     .            .OR.  ( MONTH .EQ. 12 .AND. DAY .EQ. 31          )
 
         BADSEC =       ( SEC .GE. 60.0D0 )
     .            .AND. (       (  HOUR   .NE. 23 )
     .                     .OR. (  MINUTE .NE. 59 )
     .                     .OR. ( .NOT.    LEAPSD ) )
 
 
 
         IF (  BADSEC ) THEN
            ERROR = 'The valid values for the seconds component of '
     .      //      'a calendar date are from 0 up to but not '
     .      //      'including 60 on all but the last second of '
     .      //      'non-leapsecond days.  The epoch specified is '
     .      //      'not the last second of a leapsecond day. '
     .      //      'The value # was supplied for the seconds '
     .      //      'component.'
            CALL REPMD ( ERROR, '#', SEC, 3, ERROR )
            RETURN
         END IF
 
      END IF
 
      DAY = 367*YEAR - 7*(YEAR+(MONTH+9)/12)/4
     .               - 3*((YEAR+(MONTH-9)/7)/100+1)/4
     .               + 275*MONTH/9 + DAY - 730516
 
      UTCSEC =  ( DBLE (DAY) - 0.5D0 ) * SPD()
     .       +  DBLE (60 * ( 60*HOUR + MINUTE ) )
     .       +  SEC
 
 
 
 
      RETURN
 
 
 
C$Procedure      TPARCH ( Parse check---check format of strings )
 
      ENTRY TPARCH ( STRING )
 
C$ Abstract
C
C
C      Restrict the set of strings that are recognized by
C      TPARSE to those that have standard values for all
C      time components.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      PARSING, TIME
C
C$ Declarations
C
C     CHARACTER*(*)      STRING
C
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      STRING     I   String:  Use 'YES' to restrict TPARSE inputs.
C
C$ Detailed_Input
C
C      STRING      is a character string that is used to adjust the
C                  set of strings that will be regarded as valid
C                  time strings by TPARSE.
C
C                  The default behavior of TPARSE is to allow
C                  an extended range of values for the various
C                  components (tokens) of a time string.  For example,
C                  using its default behavior, TPARSE would regard
C                  1993 JAN 367 as a valid time string and return
C                  the UTCSEC value that corresponds to Jan 2, 1994.
C
C                  While this is a "reasonable" interpretation of
C                  such a string, there may be occasions when such
C                  a string should be regarded as an error.
C
C                  By calling TPARCH with a value of 'YES', the
C                  action of TPARSE will be modified. Strings
C                  that have components that are out of the
C                  range of values used in most English discourse
C                  will be regarded as errors.  Thus the numeric
C                  values of MONTH, DAY, HOUR, MINUTE, and SECOND
C                  must satisfy the following conditions to be
C                  regarded as legitimate calendar time strings.
C
C                  ITEM     Valid Range
C                  ----     -------------------------------------
C                  MONTH    1 to 13
C                  DAY      1 to 365 (366 for leap years) when
C                           DAY is interpreted as the day of year
C                           i.e. the month token is empty.
C                           1 to 31  if month is January
C                           1 to 28 (29 in leap years) if month is
C                                   February
C                           1 to 31  if month is March
C                           1 to 30  if month is April
C                           1 to 31  if month is May
C                           1 to 31  if month is June
C                           1 to 30  if month is July
C                           1 to 31  if month is August
C                           1 to 30  if month is September
C                           1 to 31  if month is October
C                           1 to 30  if month is November
C                           1 to 31  if month is December
C                    HOUR   0 to 23
C                    MINUTE 0 to 59
C                    SECOND 0 up to but not including 60 on days that
C                           can not have a leapsecond.
C                           0 up to but not including 61 for times
C                           that are the last second of June or
C                           December.  In other words,
C                                JUN 30, 23:59:60.xxxxxx...x
C                           and  DEC 31, 23:59:60.xxxxxx...x
C
C                    To reset the action of TPARSE to the default
C                    action, set string to a value that is not
C                    equivalent to 'YES' when case and spaces are
C                    ignored.
C
C
C$ Detailed_Output
C
C      None.
C
C$ Parameters
C
C      None.
C
C$ Exceptions
C
C      Error free.
C
C$ Files
C
C      None.
C
C$ Particulars
C
C     This routine is used to alter the collections of strings
C     that TPARSE regards as legitimate calendar strings.  The
C     default behavior of TPARSE is to accept strings such
C     as FEB 34, 1993 and to interpret these in a "natural way"
C     (FEB 34, 1993 is regarded as MARCH 6, 1993.)  This behavior
C     is sometimes useful for "private" programs that you write.
C     However, such a string may be a typo (a finger accidentally hit
C     two keys for the day instead of one).  Given that this string
C     does not appear in common usage,  you may want to consider
C     that it is more likely the result of erroneous input.  You
C     can alter the behavior of TPARSE so that it will treat such
C     a string as an error.  To do this call this entry point with
C     STRING having the value 'YES'.
C
C        CALL TPARCH ( 'YES' )
C
C     Until the behavior is reset by calling TPARCH with a value
C     other than 'YES' (such as 'NO'),  TPARSE will treat all
C     out-of-bound components of time strings as errors.
C
C     If you are happy with TPARSE's default interpretation of
C     strings, you do not need to make any calls to TPARCH.
C
C     The SPICE routine UTC2ET uses TPARSE to interpret the meaning
C     of input strings.  The checking level set using TPARCH will
C     affect which strings are regarded as errors when calling
C     UTC2ET.
C
C$ Examples
C
C     When accepting times as input interactively, you usually
C     READ a string typed at a keyboard and then pass that string
C     to UTC2ET to convert it to an ephemeris time.  If you want
C     to restrict the strings accepted by UTC2ET, place the
C     following call at a point early in your program.
C
C        CALL TPARCH ( 'YES' )
C
C$ Restrictions
C
C      None.
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C      W.L. Taber      (JPL)
C
C$ Version
C
C
C-    SPICELIB Version 3.0.0, 30-JUL-1993 (WLT)
C
C        The entry point TPARCH was added so that users may
C        restrict the set of input calendar strings to those
C        that are in proper form.
C
C-    SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 2.0.0, 18-NOV-1991 (MJS)
C
C        TPARSE no longer accepts a blank time string.
C
C-    SPICELIB Version 1.0.1, 26-MAR-1991 (JML)
C
C        In the Detailed_Input section of the header, the
C        description of how default values are assigned to
C        tokens in STRING was clarified.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (WMO) (IMU)
C
C-&
 
C$ Index_Entries
C
C     Restrict time strings to proper form
C
C-&
C
 
C
C$ Revisions
C
C
C-    SPICELIB Version 3.0.0, 30-JUL-1993 (WLT)
C
C        The entry point TPARCH was added so that users may
C        restrict the set of input calendar strings to those
C        that are in proper form.
C
C-    SPICELIB Version 2.0.0, 18-NOV-1991 (MJS)
C
C        TPARSE no longer accepts a blank time string. Prior to
C        this fix, TPARSE interpreted a blank time string to be
C        -1577880000.000 UTC seconds (1 JAN 1950 00:00:00).
C
C-    SPICELIB Version 1.0.1, 26-MAR-1991 (JML)
C
C        In the Detailed_Input section of the header, the
C        description of how default values are assigned to
C        tokens in STRING was clarified.
C
C        NAIFers are accustomed to specifying day of year
C        formats of UTC strings in the following form:
C
C           1986-247 // 12:00:00
C
C        This revision to the header states explicitly that
C        the // is a blank token which results in the default
C        value being assigned to the month token.  The previous
C        version of the header implied that tokens could be left
C        out or "missing" from the string, and that default values
C        would automatically be assigned.  This works only for
C        tokens missing from the right end of the string.  For
C        default values to be assigned to tokens missing from the
C        middle of a UTC string, consecutive delimiters such as
C        // or :: must be included.
C
C
C-&
 
      IF ( EQSTR ( STRING, 'YES' ) ) THEN
         DOCHCK = .TRUE.
      ELSE
         DOCHCK = .FALSE.
      END IF
 
      RETURN
      END
 
 
