C$Procedure      WNDIFD ( Difference two DP windows )
 
      SUBROUTINE WNDIFD ( A, B, C )
 
C$ Abstract
C
C      Place the difference of two double precision windows into
C      a third window.
C
C$ Required_Reading
C
C      WINDOWS
C
C$ Keywords
C
C      WINDOWS
C
C$ Declarations
 
      INTEGER               LBCELL
      PARAMETER           ( LBCELL = -5 )
 
      DOUBLE PRECISION      A        ( LBCELL:* )
      DOUBLE PRECISION      B        ( LBCELL:* )
      DOUBLE PRECISION      C        ( LBCELL:* )
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      A,
C      B          I   Input windows.
C      C          I   Difference of A and B.
C
C$ Detailed_Input
C
C      A,
C      B           are windows, each of which contains zero or more
C                  intervals.
C
C$ Detailed_Output
C
C      C           is the output window, containing the difference
C                  of A and B---every point contained in A, but not
C                  contained in B.
C
C                  C must be distinct from both A and B.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C      1. If the difference of the two windows results in an excess of
C         elements, the error SPICE(WINDOWEXCESS) is signalled.
C
C$ Files
C
C      None.
C
C$ Particulars
C
C      Mathematically, the difference of two windows contains every
C      point contained in the first window but not contained in the
C      second window.
C
C      Fortran offers no satisfactory floating point representation
C      of open intervals. Thus, for floating point windows we must
C      return the closure of the set theoretical difference: that is,
C      the difference plus the endpoints of the first window that are
C      contained in the second window.
C
C$ Examples
C
C      Let A contain the intervals
C
C            [ 1, 3 ]  [ 7, 11 ]  [ 23, 27 ]
C
C      and B contain the intervals
C
C            [ 2, 4 ]  [ 8, 10 ]  [ 16, 18 ]
C
C      Then the difference of A and B contains the intervals
C
C            [ 1, 2 ]  [ 7, 8 ]  [ 10, 11 ]  [ 23, 27 ]
C
C$ Restrictions
C
C      None.
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C      H.A. Neilan     (JPL)
C      W.L. Taber      (JPL)
C      I.M. Underwood  (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (WLT) (IMU)
C
C-&
 
C$ Index_Entries
C
C     difference two d.p. windows
C
C-&
 
 
 
C$ Revisions
C
C-     Beta Version 1.1.0, 27-FEB-1989  (HAN)
C
C         Due to the calling sequence and functionality changes
C         in the routine EXCESS, the method of signalling an
C         excess of elements needed to be changed.
C
C-&
 
 
 
 
C
C     SPICELIB functions
C
      INTEGER               CARDD
      INTEGER               SIZED
      LOGICAL               RETURN
 
C
C     Local variables
C
      INTEGER               ACARD
      INTEGER               BCARD
      INTEGER               BSIZE
      INTEGER               CSIZE
 
      INTEGER               AP
      INTEGER               BP
      INTEGER               CP
      CHARACTER*1           USE
      INTEGER               OVER
      INTEGER               I
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'WNDIFD' )
      END IF
 
 
 
C
C     Find the cardinality of the input windows, and the allowed size
C     of the output window. Also, save the size of the second window.
C
      ACARD = CARDD ( A )
      BCARD = CARDD ( B )
      BSIZE = SIZED ( B )
      CSIZE = SIZED ( C )
 
 
C
C     This depends on the relationship between a window and its
C     complement. The difference A-B is equal to the intersection
C     of A and the complement of B. We can "complement" B by
C     replacing element -1 by something at least as small as the
C     initial endpoints of both A and B, moving all the elements of
C     B toward the front of the window, and replacing element zero
C     by something at least as large as the final endpoints of
C     both A and B. Then we just intersect.
C
C     The cardinality of the output window is zero. And there is no
C     overflow so far.
C
C     (Note that AP and BP point to the LEFT endpoints of intervals
C     in A and B, while CP points to the RIGHT endpoint of the latest
C     interval in C.)
C
      DO 50001
     .   I = 1, BCARD
         B(I-1) = B(I)
50001 CONTINUE
 
      B(-1)    = MIN ( A(1),     B(1)     )
      B(BCARD) = MAX ( A(ACARD), B(BCARD) )
 
      AP       =  1
      BP       = -1
      CP       =  0
      OVER     =  0
 
C
C     When the end of either input window is reached, we're done.
C
50002 IF       ( AP .LT. ACARD  .AND.  BP .LT. BCARD )
     .THEN
 
C
C        Let's see what we can do with the earlier of the next
C        intervals from A and B.
C
         IF ( A(AP+1) .LT. B(BP+1) ) THEN
            USE = 'A'
 
         ELSE IF ( B(BP+1) .LE. A(AP+1) ) THEN
            USE = 'B'
         END IF
 
C
C        If there is still space in the output window, fill it
C        as necessary. Otherwise, stop filling the array, but continue
C        to count the number of elements in excess of the size of the
C        output window.
C
C        The general idea is this: if the next interval of A overlaps
C        the next interval of B, save the area of overlap. (Similarly
C        for B.)
C
         IF ( USE .EQ. 'A' ) THEN
 
            IF ( A(AP+1) .GE. B(BP) ) THEN
 
               IF ( CP .LT. CSIZE ) THEN
                  CP      = CP + 2
                  C(CP-1) = MAX ( B(BP), A(AP) )
                  C(CP  ) = A(AP+1)
               ELSE
                  OVER    = OVER + 2
               END IF
 
            END IF
 
            AP = AP + 2
 
C
C        This is the same as the last clause, with B replacing A.
C
         ELSE IF ( USE .EQ. 'B' ) THEN
 
            IF ( B(BP+1) .GE. A(AP) ) THEN
 
               IF ( CP .LT. CSIZE ) THEN
                  CP      = CP + 2
                  C(CP-1) = MAX ( A(AP), B(BP) )
                  C(CP  ) = B(BP+1)
               ELSE
                  OVER    = OVER + 2
               END IF
 
            END IF
 
            BP = BP + 2
 
         END IF
 
         GO TO 50002
      END IF
 
C
C     Undo the changes to B.
C
      DO 50003
     .   I = BCARD, 1, -1
         B(I) = B(I-1)
50003 CONTINUE
 
      CALL SSIZED ( BSIZE, B )
      CALL SCARDD ( BCARD, B )
 
C
C     Set the cardinality of the output window.
C
      CALL SCARDD ( CP, C )
 
C
C     If there was an excess of elements, signal an error amd check out
C     as usual.
C
      IF ( OVER .GT. 0 ) THEN
         CALL EXCESS ( OVER, 'window' )
         CALL SIGERR ( 'SPICE(WINDOWEXCESS)' )
      END IF
 
      CALL CHKOUT ( 'WNDIFD' )
 
      RETURN
      END
