c************************************************************************
c
c  History:
c     14oct93  jm  Original version.
c     06dec93  jm  Changed call of private key routine to public keya.
c                  Also modified syntax so that -hhmm is shorthand for
c                  pstop-hhmm (rather than lst-hhmm).
c     24jan94  jm  Added a more descriptive comment section.
c
c************************************************************************
c* keytime
c& jm
c: user-input,command-line
c+
      subroutine keytime(key, value, default, error)
      character key*(*)
      real value, default
      character error*(*)
c
c  keytime: Retrieve a character string from the command line. If
c           the keyword is not found, the default is returned.  The
c           input string is processed and converted to decimal hours.
c
c  Input:
c    key       The name of the keyword to retrieve.  The value of the
c              keyword should have the form [+|-]hhmm[+hhmm|-hhmm ...]
c              (where the terms in brackets are optional).  See the
c              description below.
c    default   The default value to return if the keyword is not
c              present on the command line.  This should be entered
c              in decimal hours.  If this value is negative, the
c              current LST will be used.
c  Output:
c    value     The returned value in decimal hours.
c    error     This will be 'OK' if the decoding succeeded or the
c              default value was used; an error message otherwise.
c
c  Description of input time string.
c
c  The value associated with the keyword should have one of the
c  following forms:
c    hhmm[+hhmm|-hhmm ...]
c    stop[+hhmm|-hhmm ...]
c    lst[+hhmm|-hhmm ...]
c    +hhmm[+hhmm|-hhmm ...]
c    -hhmm[+hhmm|-hhmm ...]
c  where the additional offset terms in brackets are optional
c  (see the examples that follow).  The first form ("hhmm") is
c  translated as an absolute time; the second ("stop") as the
c  project stop time; and the third ("lst") as the current LST.
c  The fourth form is shorthand for "lst+hhmm" and the fifth form
c  is shorthand for "stop-hhmm".  The strings "lst" and "stop"
c  are translated internally (dynamically) and do not represent
c  shell variables accessable to the user.
c
c  Note: The project stop time ("stop") and the current LST ("lst")
c        may only be specified as the first member of the time string.
c  Note: The project stop time ("stop") may never be exceeded.
c
c  Examples:
c    time=1350         : Set the time to exactly 13h 50m.
c    time=+100         : Set the time to 1h from now.
c    time=lst+100      : Set the time to 1h from now.
c    time=-0200        : Set the time to 2h before the project stop time.
c    time=stop-0200    : Set the time to 2h before the project stop time.
c    time=stop-30-100  : Set the time to 1h 30m before the project stop.
c
c--
c------------------------------------------------------------------------
      character cvalue*256
      real retval
c
c  Get the value.
c
      call keya(key, cvalue, ' ')
c
c  Decode it if found.
c
      error = 'OK'
      if ((cvalue .eq. ' ') .and. (default .ge. 0.0)) then
        retval = default
      else if (cvalue .eq. ' ') then
        call strtime('lst', retval, error)
      else
        call strtime(cvalue, retval, error)
      endif
c
      if (error(1:2) .eq. 'OK') then
        value = retval
      endif
c
      return
      end
c************************************************************************
c* strtime
c& jm
c: date, utilities
c+
      subroutine strtime(string, value, error)
      character string*(*)
      real value
      character error*(*)
c
c  strtime: Convert a character string into a time value. If the string
c           can not be converted, an error string is returned and the
c           return value is undefined.  If the string is properly
c           converted, the error string is returned with 'OK' and the
c           converted time loaded into value.  The time is always
c           returned in units of decimal hours.
c
c  Input:
c    string    The string to be converted.  The string should have
c              the form [+|-]hhmm[+hhmm|-hhmm ...] (where the terms
c              in brackets are optional).  See the description below.
c  Output:
c    value     The returned value in decimal hours (in the range 0-24).
c    error     This will be 'OK' if the decoding succeeded; an error
c              message otherwise.
c
c  Description of input time string:
c
c  The value associated with the keyword should have the form
c  [+|-]hhmm[+hhmm|-hhmm ...] (where the terms in brackets are
c  optional).  Without the leading plus or minus sign, the time
c  specified by the first "hhmm" is taken as an absolute time;
c  otherwise, the first "hhmm" is measured relative to the current
c  LST.  This can also be stated explicitly by entering the time as
c  "lst+hhmm" where the string "lst" will be translated dynamically.
c  Also, the project stop time may be accessed by using the string "stop".
c
c  Note: The project stop time and the current LST may only be specified
c        as the first member of the time string.
c  Note: The project stop time may never be exceeded.
c
c  Examples:
c    time=1350         : Set the time to exactly 13h 50m.
c    time=+100         : Set the time to 1h from now.
c    time=lst+100      : Set the time to 1h from now.
c    time=-0200        : Set the time to 2h before the project stop time.
c    time=stop-0200    : Set the time to 2h before the project stop time.
c    time=stop-30-100  : Set the time to 1h 30m before the project stop.
c
c--
c------------------------------------------------------------------------
      double precision DPI
      parameter (DPI = 3.14159265358979323846)
c
      integer i, k
      integer tsign, pstop, hhmm
      integer mon, day, year
      integer ticks
      double precision ut, lst
c
      integer len1
c
c  Ignore leading blanks.
c
      k = len1(string)
      i = 1
  100 if ((i .le. k) .and. (string(i:i) .le. ' ')) then
        i = i + 1
        goto 100
      endif
      call lcase(string(i:k))
c
c  Check for a time specified relative to the project stop time.
c
      value = 0.0
      if (((i + 3) .le. k) .and. (string(i:i+3) .eq. 'stop')) then
        call comgeti('PSTOP', pstop, 1, error)
        call write_error('STRTIME', error)
        if (error(1:2) .ne. 'OK') return
        if (pstop .ne. 0) then
          value = int(pstop / 100) + (mod(pstop, 100) / 60.0)
        endif
        i = i + 4
      else if (((i + 2) .le. k) .and. (string(i:i+2) .eq. 'lst')) then
        call get_times(ticks, 2, mon, day, year, ut, lst, error)
        call write_error('STRTIME', error)
        if (error(1:2) .ne. 'OK') return
        value = (lst * 12.0D0 / DPI)
        i = i + 3
      else if ((string(i:i) .ge. '0') .and. (string(i:i) .le. '9')) then
        hhmm = 0
  200   if ((i .le. k) .and.
     *      (string(i:i) .ge. '0') .and. (string(i:i) .le. '9')) then
          hhmm = (10 * hhmm) + (ichar(string(i:i)) - ichar('0'))
          i = i + 1
          goto 200
        endif
        if (hhmm .gt. 0) then
          value = int(hhmm / 100) + (mod(hhmm, 100) / 60.0)
        endif
      else if (string(i:i) .eq. '-') then
        call comgeti('PSTOP', pstop, 1, error)
        call write_error('STRTIME', error)
        if (error(1:2) .ne. 'OK') return
        if (pstop .ne. 0) then
          value = int(pstop / 100) + (mod(pstop, 100) / 60.0)
        endif
      else if (string(i:i) .eq. '+') then
        call get_times(ticks, 2, mon, day, year, ut, lst, error)
        call write_error('STRTIME', error)
        if (error(1:2) .ne. 'OK') return
        value = (lst * 12.0D0 / DPI)
      else
        error = 'Incorrect time format (use [+|-]hhmm).'
        return
      endif
c
c  Now check for relative times to be added or subtracted from
c  the current set time.
c
  300 if ((i .le. k) .and.
     *    ((string(i:i) .eq. '-') .or. (string(i:i) .eq. '+'))) then
        tsign = 1
        if (string(i:i) .eq. '-') tsign = -1
        i = i + 1
        hhmm = 0
  400   if ((i .le. k) .and.
     *      (string(i:i) .ne. '-') .and. (string(i:i) .ne. '+') .and.
     *      (string(i:i) .ge. '0') .and. (string(i:i) .le. '9')) then
          hhmm = (10 * hhmm) + (ichar(string(i:i)) - ichar('0'))
          i = i + 1
          goto 400
        endif
        if (hhmm .gt. 0) then
          value = value +
     *            (tsign * (int(hhmm / 100) + (mod(hhmm, 100) / 60.0)))
        endif
        goto 300
      endif
c
c  Make sure the time is in the hour range [0.0,24.0).
c
  500 if (value .ge. 24.0) then
        value = value - 24.0
        goto 500
      endif
  600 if (value .lt. 0.0) then
        value = value + 24.0
        goto 600
      endif
c
      error = 'OK'
      return
      end
