/***********************************************************************/
/*
 *  This file contains subroutines used by the scan and loop commands.
 *
 *  History:
 *    06jan94 jm  Extracted from scan.c and loop.c files.
 *    27jan94 jm  Modified spool() to only output messaages only for
 *                commands that matter to the user.  Also added
 *                Warning() and Message() commands.
 *    01feb94 jm  Modified buildCommand() to quote every value.
 *    03feb94 jm  Added testVariables() routine to point out unused
 *                setup variables.
 *    16feb94 jm  Added checkLoopSrcTime() routine and OneShot, AddTime,
 *                and SubTime global variables.  This is used to adjust
 *                the final source observation by loop.  Also modified
 *                formatTime() to call setCurrentLST() before using it.
 *                Passed history file descriptor into buildCommand()
 *                for call to Warning().  Also added call to comment
 *                inside Message() and Warning().
 *    21feb94 jm  Modified buildCommand() to test stop keyword even if
 *                it is not present in the variable list.  A default of
 *                one hour from "now" is used if keyword isn't present.
 *                Also, keyword is only tested for observing task.
 *    08mar94 jm  Changed getTask() to return pointer to a TASK item
 *                and renamed old version to getTaskName().  Also,
 *                expanded call to include a keyword to search which
 *                enables all four major tasks to be retrieved from
 *                the getTask() routine.  Also added new task setcorr3
 *                and task keywords checktask, corrtask, and tunetask.
 *    29mar94 jm  Removed ants keyword from setcorr and setcorr3 tasks.
 *    04apr94 jm  Added mint3 task.
 *    09may94 jm  Added qint task.
 *    04aug94 jm  Added hanning keyword to setcorr3 task.
 *    10aug94 jm  Modified for transition of mint3->mint corr3->corr.
 *    11aug94 jm  Modified so that an ERROR state ALWAYS returns
 *                immediately.
 *    17aug94 jm  Modified to remove restfqs keyword from setfreq and
 *                add it to setcorr.
 *    19aug94 jm  Switched the order of calling corrtask/tunetask.
 *    12sep94 jm  Changed setcorr keyword mode back to cormode.
 *
 *---------------------------------------------------------------------*/

#include "setups.h"

#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>

#ifdef __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif /*__STDC__*/

static unsigned int StartTime;  /* Integer value of the starting time. */
static unsigned int StopTime;     /* Integer value of input stop time. */
static unsigned int LstNow;           /* Integer value of current LST. */

static unsigned int OneShot;       /* Flag to adjust integration time. */
static unsigned int AddTime;     /* Amount to add to integration time. */
static unsigned int SubTime;          /* Amount to decrease stop time. */

static char *programName = (char *)NULL;

#define HASHSIZE 127
#define MAXARGS   15

#ifdef __STDC__
#define FREE(x) (void)free((void *)(x))
#else
#define FREE(x) (void)free((char *)(x))
#endif /*__STDC__*/

typedef struct taskstruct {
  char *name;
  char *keywords[MAXARGS];
  char *required[MAXARGS];
} TASK;

typedef struct variable {
  char *name;
  char *value;
  short int used;
  struct variable *next;
} VARIABLE;

static VARIABLE *hashtable[HASHSIZE];

static TASK cksrc = {
  "cksrc",
  { /* Keywords */
  "source",
  "catalog",
  "elevlim",
  "lstrange",
/*  "epoch", */
/*  "lst", */
/*  "latitude", */
  },
  { /* Required keywords */
  "source",
  },
};

static TASK setcorr = {
  "setcorr",
  { /* Keywords */
  "cormode",
  "coropt",
  "corf",
  "corbw",
  "corbits",
  "hanning",
  "restfqs",
/*  "ants", */
  },
  { /* Required keywords */
  "cor",
  },
};

static TASK setcorr3 = {
  "setcorr3",
  { /* Keywords */
  "cormode",
  "coropt",
  "corf",
  "corbw",
  "corbits",
  "hanning",
/*  "ants", */
  },
  { /* Required keywords */
  "cor",
  },
};

static TASK setfreq = {
  "setfreq",
  { /* Keywords */
  "freq",
  "iffreq",
  "dopsrc",
  "dopcat",
/*  "restfqs", */
  "obsline",
  "ants",
  "options",
  "harm",
  },
  { /* Required keywords */
  "freq",
  "iffreq",
  },
};

static TASK mint3cmd = {
  "mint3",
  { /* Keywords */
  "source",
  "catalog",
  "ants",
  "itime",
  "nchan",
  "vis",
  "trakopt",
  "start",
  "stop",
  "grid",
  "scale",
  "nreps",
/*  "device", */
  },
  { /* Required keywords */
  "source",
  "vis",
  },
};

static TASK mintcmd = {
  "mint",
  { /* Keywords */
  "source",
  "catalog",
  "ants",
  "itime",
  "nchan",
  "vis",
  "trakopt",
  "start",
  "stop",
  "grid",
  "scale",
  "nreps",
/*  "device", */
  },
  { /* Required keywords */
  "source",
  "vis",
  },
};

static TASK intcmd = {
  "int",
  { /* Keywords */
  "source",
  "catalog",
  "ants",
  "itime",
  "nchan",
  "vis",
  "trakopt",
  "start",
  "stop",
  "options",
/*  "device", */
  },
  { /* Required keywords */
  "source",
  "vis",
  },
};

static TASK int3cmd = {
  "int3",
  { /* Keywords */
  "source",
  "catalog",
  "ants",
  "itime",
  "nchan",
  "vis",
  "trakopt",
  "start",
  "stop",
  "options",
/*  "device", */
  },
  { /* Required keywords */
  "source",
  "vis",
  },
};

static TASK qintcmd = {
  "qint",
  { /* Keywords */
  "source",
  "catalog",
  "ants",
  "itime",
  "vis",
  "trakopt",
  "start",
  "stop",
  "grid",
  "scale",
  "nreps",
  "quick",
/*  "device", */
  },
  { /* Required keywords */
  "source",
  "vis",
  },
};

static char *taskList[] = {
  "checktask",
  "corrtask",
  "tunetask",
  "task",
};

/* MYlogit needs this forward declaration. */
static int spool ARGS(( const char *command, char *output, int length,
  int *status, int echo ));

/***********************************************************************/
/***********************************************************************/
/*                 M E S S A G E   C O M M A N D S                     */
/***********************************************************************/
/***********************************************************************/

/***********************************************************************/
#ifdef __STDC__
void setProgram(const char *name)
#else
void setProgram(name)
const char *name;
#endif /*__STDC__*/
{
    programName = (char *)name;
}

/***********************************************************************/
#ifdef __STDC__
static void MYfprintf(FILE *fd, const char *prefix, const char *format, va_list ap)
#else
static void MYfprintf(fd, prefix, format, ap)
FILE *fd;
const char *prefix;
const char *format;
va_list ap;
#endif /*__STDC__*/
{
    if (prefix != (const char *)NULL)
      (void)fprintf(fd, prefix);

    (void)vfprintf(fd, format, ap);

    if (format[strlen(format)-1] != '\n')
      (void)fprintf(fd, "\n");
    (void)fflush(fd);

    return;
}

/***********************************************************************/
#ifdef __STDC__
static void MYlogit(const char *prefix, const char *format, va_list ap)
#else
static void MYlogit(prefix, format, ap)
const char *prefix;
const char *format;
va_list ap;
#endif /*__STDC__*/
{
    char logText[MAXBUF];
    char parameter[MAXBUF];
    register int indx;
    int dummy;

    if (prefix != (const char *)NULL)
      (void)sprintf(logText, "comment is='%s ", prefix);
    else
      (void)sprintf(logText, "comment is='");

    indx = strlen(logText);

    (void)vsprintf(logText+indx, format, ap);

    if (format[strlen(format)-1] == '\n')
      logText[strlen(logText)-1] = '\0';

    (void)strcat(logText, "'");

    (void)spool(logText, parameter, MAXBUF, &dummy, 0);

    return;
}

/***********************************************************************/
#ifdef __STDC__
void Warning(FILE *fd, const char *format, ...)
#else
void Warning(fd, format, va_alist)
FILE *fd;
const char *format;
va_dcl
#endif /*__STDC__*/
{
    char *defprompt = "### ";
    char prompt[MAXBUF];
    va_list ap;

#ifdef __STDC__
    va_start(ap, format);
#else
    va_start(ap);
#endif /*__STDC__*/

    (void)sprintf(prompt, "%c%s", '\007',
      ((programName == (char *)NULL) ? defprompt : programName));

    if (fd != stderr)
      MYfprintf(fd, prompt+1, format, ap);
    MYfprintf(stderr, prompt, format, ap);

    MYlogit(prompt+1, format, ap);

    va_end(ap);
    return;
}

/***********************************************************************/
#ifdef __STDC__
void Message(FILE *fd, const char *format, ...)
#else
void Message(fd, format, va_alist)
FILE *fd;
const char *format;
va_dcl
#endif /*__STDC__*/
{
    char *defprompt = "### ";
    char *prompt;
    va_list ap;

#ifdef __STDC__
    va_start(ap, format);
#else
    va_start(ap);
#endif /*__STDC__*/

    prompt = (programName == (char *)NULL) ? defprompt : programName;
    if (fd != stdout)
      MYfprintf(fd, prompt, format, ap);
    MYfprintf(stdout, prompt, format, ap);

    MYlogit(prompt, format, ap);

    va_end(ap);
    return;
}

/***********************************************************************/
/***********************************************************************/
/*                   S P O O L   C O M M A N D S                       */
/***********************************************************************/
/***********************************************************************/

/***********************************************************************/
#ifdef __STDC__
static int MYpclose(FILE *fd, pid_t pid)
#else
static int MYpclose(fd, pid)
FILE *fd;
pid_t pid;
#endif /*__STDC__*/
{
    int status;
    register pid_t r;

#ifdef __STDC__
    register void (*hstat)(int), (*istat)(int), (*qstat)(int);
#else
    register void (*hstat)(), (*istat)(), (*qstat)();
#endif /*__STDC__*/

    if (fclose(fd) != 0) {
      perror("fclose(fd) error");
      return(-1);
    }

    istat = signal(SIGINT,  SIG_IGN);
    qstat = signal(SIGQUIT, SIG_IGN);
    hstat = signal(SIGHUP,  SIG_IGN);

    while (((r = wait(&status)) != pid) && r != -1) /* NULL */ ;

    if (r == -1)
      status = -1;

    (void)signal(SIGINT,  istat);
    (void)signal(SIGQUIT, qstat);
    (void)signal(SIGHUP,  hstat);

    return(status);
}

/***********************************************************************/
#ifdef __STDC__
static FILE *MYpopen(const char *cmd, pid_t *pid)
#else
static FILE *MYpopen(cmd, pid)
const char *cmd;
pid_t *pid;
#endif /*__STDC__*/
{
    int p[2];

    if (pipe(p) < 0)
      return((FILE *)NULL);

    switch (*pid = fork()) {
      case -1:
        return((FILE *)NULL);
      case  0:
        if (close(1) < 0) {                   /* Close child's stdout. */
          perror("close(1) error");
          exit(1);
        }
        if (dup(p[1]) < 0) {      /* Dup this id into the stdout slot. */
          perror("dup(p[1]) error");
          exit(1);
        }
        if ((close(p[0]) < 0) || (close(p[1]) < 0)) {   /* Close both. */
          perror("close(p[0||1]) error");
          exit(1);
        }
        (void)execlp("/bin/sh", "sh", "-c", cmd, (char *)NULL);
        exit(1);          /* Disaster has occurred if this is reached. */
      default:
        if (close(p[1]) < 0) {         /* Close the pipe's writing fd. */
          perror("close(p[1]) error");
          return((FILE *)NULL);
        }
            /* Returns a stream associated with the pipe's reading fd. */
        return(fdopen(p[0], "r"));
    }
}

/***********************************************************************/
#ifdef __STDC__
static int spool(const char *command, char *output, int length, int *status, int echo)
#else
static int spool(command, output, length, status, echo)
const char *command;
char *output;
int length;
int *status;
int echo;
#endif /*__STDC__*/
/*
  This executes the input command, writing all stdout messages directly
  to the caller.  This routine returns the final stdout string.
  The task returns 1 if (MY)popen could not be successfully called;
  otherwise, the routine returns 0.

  Input:
    command     The command to execute.
    length      The size of the output buffer.
    echo        If this is non-zero, output from the command will be
                printed to stdout.

  Output:
    output      The last output text string from the executed task.
    status      The return status of the called program (0 means success).

------------------------------------------------------------------------*/
{
    char string[MAXBUF];
    register int len;
    pid_t pid;
    FILE *fd;

    if ((fd = MYpopen(command, &pid)) == (FILE *)NULL)
      return(1);

    output[0] = Null;
    while (fgets(string, MAXBUF, fd) != NULL) {
      len = strlen(string) - 1;
      if (len >= 0) {
        (void)strncpy(output, string, length);
        output[length-1] = Null;
        if (echo != 0) (void)fputs(output, stdout);
      }
    }

    /* Strip trailing blanks out of the final string. */
    len = strlen(output) - 1;
    while ((len >= 0) && isspace(output[len]))
      output[len--] = Null;

    *status = MYpclose(fd, pid);
    return(0);
}

/***********************************************************************/
/***********************************************************************/
/*                       U T I L I T I E S                             */
/***********************************************************************/
/***********************************************************************/

/***********************************************************************/
#ifdef __STDC__
char *skipLeading(const char *token)
#else
char *skipLeading(token)
const char *token;
#endif /*__STDC__*/
/*
 *  Returns a pointer to the first non-space character in
 *  the input string.
 */
{
    char *ptr;

    ptr = (char *)token;
    while ((*ptr != Null) && isspace(*ptr))
      ptr++;

    return(ptr);
}

/***********************************************************************/
#ifdef __STDC__
char *getGroup(const char *token, char *group)
#else
char *getGroup(token, group)
const char *token;
char *group;
#endif /*__STDC__*/
/*
 *  Extract a "group".  This routine looks for a separator, which may
 *  be a delimiter or any white space.  It also worries about matching
 *  bracketing of () characters and strings enclosed within quotes.
 *  Quoted material is always retained within a particular group but
 *  the single quote at the beginning and end of the string is removed.
 *  This routine skips leading blanks and delimiters and then returns a
 *  pointer to the first trailing white space or delimiting character.
 *  It also loads the group into the string pointed to by "group" BUT
 *  DOES NOT CHECK THAT ENOUGH SPACE EXISTS for the string.
 */
{
    char *ptr, *par, *start;
    char temp[MAXBUF];
    int quoted, depth;

    ptr = (char *)token;
    while ((*ptr != Null) && (isspace(*ptr) || isdelimiter(*ptr)))
      ptr++;

    if (*ptr == Null)
      return((char *)NULL);

    quoted = depth = 0;
    start = par = temp;
    while (*ptr != Null) {
      if (*ptr == '\'') {
        quoted = !quoted;
      } else if (!quoted) {
        if (*ptr == '(') {
          depth++;
        } else if (*ptr == ')') {
          depth--;
        } else if (isspace(*ptr) || isdelimiter(*ptr)) {
          if (depth <= 0)
            break;
        }
      }
      *par++ = *ptr++;
    }
    *par = Null;

    /* Strip off the matching leading and trailing quotes. */
    while ((par > start) && ((*start == '\'') && (*(par-1) == '\''))) {
      start++;
      *(--par) = Null;
    }

    (void)strcpy(group, ((par > start) ? start : ""));

    return(ptr);
}

/***********************************************************************/
/***********************************************************************/
/*                V A R I A B L E   C O M M A N D S                    */
/***********************************************************************/
/***********************************************************************/

/***********************************************************************/
#ifdef __STDC__
static VARIABLE *findVariable(const char *name, int *hashval)
#else
static VARIABLE *findVariable(name, hashval)
const char *name;
int *hashval;
#endif /*__STDC__*/
{
    register char *t;
    register int value;
    VARIABLE *v;

    value = 0;
    for (t = (char *)name; *t != Null; t++)
      value += *t;

    value %= HASHSIZE;
    *hashval = value;

    /* Find the VARIABLE associated with this name from the hash table. */
    for (v = hashtable[value]; v != (VARIABLE *)NULL; v = v->next)
      if (strcmp(name, v->name) == 0)
        break;

    return(v);
}

/***********************************************************************/
#ifdef __STDC__
void freeVariables(void)
#else
void freeVariables()
#endif /*__STDC__*/
{
    static char firstTime = 1;
    register int j;
    VARIABLE *v, *next;

    if (firstTime) {   /* Initialize the array to be defined as empty. */
      for (j = 0; j < HASHSIZE; j++)
        hashtable[j] = (VARIABLE *)NULL;
      firstTime = 0;
      return;
    }

    for (j = 0; j < HASHSIZE; j++) {
      next = (VARIABLE *)NULL;                /* To keep lint happy... */
      for (v = hashtable[j]; v != (VARIABLE *)NULL; v = next) {
        next = v->next;
        if (v->value != (char *)NULL)
          FREE(v->value);
        FREE(v->name);
        FREE(v);
      }
      hashtable[j] = (VARIABLE *)NULL;
    }

    return;
}

/***********************************************************************/
#ifdef __STDC__
static void addVariable(const char *name, const char *value)
#else
static void addVariable(name, value)
const char *name;
const char *value;
#endif /*__STDC__*/
{
    int hashval;
    VARIABLE *v;

    v = findVariable(name, &hashval);

    /* Create the variable if necessary and fill in the value. */
    if (v == (VARIABLE *)NULL) {
      if ((v = (VARIABLE *)malloc(sizeof(VARIABLE))) == (VARIABLE *)NULL) {
        Warning(stderr,
          "Ran out of memory trying to allocate storage for a variable.\n");
        return;
      }
      if ((v->name = (char *)malloc(strlen(name) + 1)) == (char *)NULL) {
        Warning(stderr,
          "Ran out of memory trying to allocate storage for a string.\n");
        FREE(v);
        return;
      }
      (void)strcpy(v->name, name);
      v->next = hashtable[hashval];
      hashtable[hashval] = v;
      v->value = (char *)NULL;
    }

    v->used = 0;
    if (v->value != (char *)NULL)
      FREE(v->value);

    if ((v->value = (char *)malloc(strlen(value) + 1)) == (char *)NULL) {
      Warning(stderr,
        "Ran out of memory trying to allocate storage for the value.\n");
      return;
    }
    (void)strcpy(v->value, value);

    return;
}

/***********************************************************************/
#ifdef __STDC__
static void touchVariables(const char *task, const char *name)
#else
static void touchVariables(task, name)
const char *task;
const char *name;
#endif /*__STDC__*/
/*
 *  Finds the value associated with the keyword "name" and marks it as
 *  found (used).  If a local version of the keyword (task/name) is
 *  present, it takes precedence.  If there is no match to the keyword,
 *  nothing is done.
 */
{
    char fullname[MAXBUF];
    int dummy;
    VARIABLE *v, *vkey;

    v = (VARIABLE *)NULL;

    /* Search for a match to the global keyword. */
    if ((vkey = findVariable(name, &dummy)) != (VARIABLE *)NULL)
      v = vkey;

    /* Now search for a match to the local version of the keyword. */
    if (task != (const char *)NULL) {
      (void)strcpy(fullname, task);
      (void)strcat(fullname, "/");
      (void)strcat(fullname, name);

      if ((vkey = findVariable(fullname, &dummy)) != (VARIABLE *)NULL)
        v = vkey;
    }

    if (v != (VARIABLE *)NULL)
      v->used = 1;

    return;
}

/***********************************************************************/
#ifdef __STDC__
static void unusedVariables(const char *setup, FILE *fd)
#else
static void unusedVariables(setup, fd)
const char *setup;
FILE *fd;
#endif /*__STDC__*/
/*
 *  Finds setup entries that are not used by any command and prints
 *  a warning to the user.
 *
 *  This also warns about global keywords that have been ignored by
 *  local versions.  -- (perhaps this should not be?)
 */
{
    register int j;
    VARIABLE *v;

    for (j = 0; j < HASHSIZE; j++) {
      for (v = hashtable[j]; v != (VARIABLE *)NULL; v = v->next) {
        if (v->used == 0) {
          Warning(fd, "Setup [%s] variable [%s] will not be used.\n",
            setup, v->name);
        }
      }
    }
}

/***********************************************************************/
#ifdef __STDC__
int loadVariables(const char *filename, const char *project, int nest)
#else
int loadVariables(filename, project, nest)
const char *filename;
const char *project;
int nest;
#endif /*__STDC__*/
/*
 *  This routine trys to open the setup file, and read the variables
 *  (recursively, if necessarily).  It returns 1 on error; -1 if the
 *  nesting level is >10; or 0 if all went okay.
 */
{
    register char *s;
    char *name, *value;
    char line[MAXBUF];
    FILE *fd;

    if (nest > 10) {
      Warning(stderr, "Possible infinite nesting of setup [%s]?\n", filename);
      return(-1);
    }

    name = skipLeading(filename);
    if ((char *)project != (char *)NULL)
      (void)strcpy(line, project);
    else
      (void)strcpy(line, "./");

    if (line[strlen(line)-1] != '/')
      (void)strcat(line, "/");
    (void)strcat(line, name);

    if ((fd = fopen(line, "r")) == (FILE *)NULL) {
      if ((fd = fopen(name, "r")) == (FILE *)NULL) {
        /***********************************************/
        /* Also search any other default directories?? */
        /***********************************************/
        Warning(stderr, "Error opening file [%s].\n", name);
        return(1);
      }
    }

    while (fgets(line, MAXBUF, fd) != NULL) {
      s = skipLeading(line);
      if (*s == '#')                          /* Ignore comment lines. */
        continue;
      name = s;
      while ((*s != Null) && (!isspace(*s)) && (*s != '='))
        s++;
      while ((*s != Null) && ((isspace(*s)) || (*s == '=')))
        *s++ = Null;
      value = s;
      while ((*s != Null) && (!isspace(*s)) && (*s != '\n'))
        s++;
      *s = Null;
      if (*value != Null) {
        /*
         *  If name is "setup", then value is considered a
         *  setup file name to also be read.  Otherwise, it
         *  is just a keyword to be loaded.
         */
        if (strcmp(name, "setup") == 0) {
          while ((value != (char *)NULL) && (*value != Null)) {
            if ((s = strchr(value, ',')) != (char *)NULL)
              *s++ = Null;
            if (loadVariables(value, project, (nest + 1)) < 0) {
              (void)fclose(fd);
              return(-1);
            }
            value = s;
          }
        } else {
          addVariable(name, value);
        }
      }
    }

    (void)fclose(fd);
    return(0);
}

/***********************************************************************/
#ifdef __STDC__
char *getVariable(const char *task, const char *name)
#else
char *getVariable(task, name)
const char *task;
const char *name;
#endif /*__STDC__*/
/*
 *  Returns the value associated with the keyword "name."  If a local
 *  version of the keyword (task/name) is present, it takes precedence.
 *  If there is no match to the keyword, NULL is returned.
 */
{
    char *value;
    char fullname[MAXBUF];
    int dummy;
    VARIABLE *v;

    value = (char *)NULL;

    /* Search for a match to the global keyword. */
    if ((v = findVariable(name, &dummy)) != (VARIABLE *)NULL)
      value = v->value;

    /* Now search for a match to the local version of the keyword. */
    if (task != (const char *)NULL) {
      (void)strcpy(fullname, task);
      (void)strcat(fullname, "/");
      (void)strcat(fullname, name);

      if ((v = findVariable(fullname, &dummy)) != (VARIABLE *)NULL)
        value = v->value;
    }

    return(value);
}

/***********************************************************************/
/***********************************************************************/
/*                   C O M M O N   C O M M A N D S                     */
/***********************************************************************/
/***********************************************************************/

/***********************************************************************/
#ifdef __STDC__
static void incrementObs(const char *setup)
#else
static void incrementObs(setup)
const char *setup;
#endif /*__STDC__*/
/*
 *  Increments the number of times a setup has been observed by 1.
 */
{
    char *sval;
    char command[MAXBUF];
    char parameter[MAXBUF];
    int value;

    if (((sval = getVariable(setup, "maxobs")) == (char *)NULL) ||
       (sscanf(sval, "%d", &value) != 1) || (value <= 0))
      return;

    (void)sprintf(command, "svalue setup=%s value=+1", setup);

    (void)spool(command, parameter, MAXBUF, &value, 0);

    return;
}

/***********************************************************************/
#ifdef __STDC__
static int getMaxObs(const char *setup)
#else
static int getMaxObs(setup)
const char *setup;
#endif /*__STDC__*/
/*
 *  Returns 1 if the number of times observed has exceeded the
 *  value of the maxobs variable.  Returns 0 if okay to observed
 *  or if maxobs is not set (or retrievable).
 */
{
    static char *cmd = "svalue setup=%s | tail -1 | awk '{print $NF}'";
    char *sval;
    char command[MAXBUF];
    char parameter[MAXBUF];
    int value, retstate;

    if ((sval = getVariable(setup, "maxobs")) == (char *)NULL)
      return(0);

    if (sscanf(sval, "%d", &value) != 1) {
      Warning(stderr, "Trouble decoding maxobs variable [%s].\n", sval);
      Warning(stderr, "skipping test of maxobs....\n");
      return(0);
    }

    if (value <= 0)
      return(0);

    (void)sprintf(command, cmd, setup);

    if (spool(command, parameter, MAXBUF, &retstate, 0)) {
      Warning(stderr, "Could not call task [svalue].\n");
      return(0);
    } else if (retstate != 0) {
      Warning(stderr, "[svalue] returned with error status [%d].\n", retstate);
      return(0);
    } else if (sscanf(parameter, "%d", &retstate) != 1) {
      Warning(stderr, "Trouble decoding svalue value [%s].\n", parameter);
      return(0);
    }

    return(retstate >= value);
}

/***********************************************************************/
#ifdef __STDC__
static int getCommon(const char *name, int defval)
#else
static int getCommon(name, defval)
const char *name;
int defval;
#endif /*__STDC__*/
/*
 *  Returns the common block variable "name" value or "defval" on error.
 */
{
    static char *cmd = "value name=%s | tail -1 | awk '{print $NF}'";
    char command[MAXBUF];
    char parameter[MAXBUF];
    int retstate;

    (void)sprintf(command, cmd, name);

    if (spool(command, parameter, MAXBUF, &retstate, 0)) {
      Warning(stderr, "Could not get common variable [%s].\n", name);
      return(defval);
    } else if (retstate != 0) {
      Warning(stderr,
        "Command to get the common variable [%s] returned error status [%d].\n",
        name, retstate);
      return(defval);
    } else if (sscanf(parameter, "%d", &retstate) != 1) {
      Warning(stderr, "Trouble decoding common variable [%s=%s].\n",
        name, parameter);
      return(1);
    }

    return(retstate);
}

/***********************************************************************/
#ifdef __STDC__
static void setCommon(const char *name, int value)
#else
static void setCommon(name, value)
const char *name;
int value;
#endif /*__STDC__*/
{
    static char *cmd = "alter name=%s value=%d";
    char command[MAXBUF];
    char parameter[MAXBUF];
    int retstate;

    (void)sprintf(command, cmd, name, value);
    (void)spool(command, parameter, MAXBUF, &retstate, 0);

    return;
}
/***********************************************************************/
/***********************************************************************/
/*                      T I M E   C O M M A N D S                      */
/***********************************************************************/
/***********************************************************************/

/***********************************************************************/
#ifdef __STDC__
int setCurrentLST(void)
#else
int setCurrentLST()
#endif /*__STDC__*/
/*  Returns 1 if the LST could not be retrieved; 0 otherwise. */
{
    static char *cmd = "lstsymbol";
    char parameter[MAXBUF];
    int retstate;             /* Return value from the called program. */
    int hh, mm, hhmm;

    if (spool(cmd, parameter, MAXBUF, &retstate, 0)) {
      Warning(stderr, "Could not get the current LST time.\n");
      return(1);
    } else if (retstate != 0) {
      Warning(stderr,
        "Command to get the current LST returned with error status [%d].\n",
        retstate);
      return(1);
    } else if (sscanf(parameter, "%d", &hhmm) != 1) {
      Warning(stderr, "Trouble decoding current LST time [%s].\n", parameter);
      return(1);
    }

    hh = hhmm / 100;
    mm = hhmm % 100;
    LstNow = (hh * 60) + mm;
    LstNow %= FULLDAY;

    if (AppDebug == TRUE)
      (void)printf("Current LST is %d [%d minutes]\n", hhmm, LstNow);

    return(0);
}

/***********************************************************************/
#ifdef __STDC__
unsigned int getCurrentLST(void)
#else
unsigned int getCurrentLST()
#endif /*__STDC__*/
{
    return(LstNow);
}

/***********************************************************************/
#ifdef __STDC__
char *formatTime(void)
#else
char *formatTime()
#endif /*__STDC__*/
/*
 *  Returns a pointer to a static string of the (user-formatted)
 *  current Lst.
 */
{
    static char string[MAXBUF];
    register int hh, mm;
    unsigned int now;

    (void)setCurrentLST();
    now = getCurrentLST();
    hh = now / 60;
    hh %= 24;
    mm = now % 60;
    (void)sprintf(string, "[%2.2d:%2.2d]", hh, mm);
    return(string);
}

/***********************************************************************/
#ifdef __STDC__
unsigned int getTime(const char *string, unsigned int defTime)
#else
unsigned int getTime(string, defTime)
const char *string;
unsigned int defTime;
#endif /*__STDC__*/
{
    char *ptr, *s;
    char buffer[BUFSIZ];
    int itime, jtime;
    int hhmm, sign;

    if ((string == (const char *)NULL) || (*string == Null))
      return(defTime);

    s = skipLeading(string);
    if ((s == (char *)NULL) || (*s == Null))
      return(defTime);

    ptr = buffer;
    while ((hhmm = *s++) != Null)
      *ptr++ = isupper(hhmm) ? tolower(hhmm) : hhmm;
    *ptr = Null;

    ptr = buffer;
    itime = 0;
    if (strncmp(ptr, "stop", 4) == 0) {
      if ((jtime = getCommon("pstop", 0)) > 0) {
        hhmm = jtime / 100;
        itime = (hhmm * 60) + (jtime % 100);
        itime %= FULLDAY;
      }
      ptr += 4;
    } else if (strncmp(ptr, "lst", 3) == 0) {
      (void)setCurrentLST();
      itime = getCurrentLST();
      ptr += 3;
    } else if (isdigit(*ptr)) {
      while (isdigit(*ptr))
        itime = (itime * 10) + (*ptr++ - '0');
      if (itime > 0) {
        hhmm = itime / 100;
        itime = (hhmm * 60) + (itime % 100);
        itime %= FULLDAY;
      }
    } else if (*ptr == '-') {
      if ((jtime = getCommon("pstop", 0)) > 0) {
        hhmm = jtime / 100;
        itime = (hhmm * 60) + (jtime % 100);
        itime %= FULLDAY;
      }
    } else if (*ptr == '+') {
      (void)setCurrentLST();
      itime = getCurrentLST();
    } else {
      return(defTime);
    }

    while ((*ptr == '-') || (*ptr == '+')) {
      sign = (*ptr++ == '-') ? -1 : 1;
      jtime = 0;
      while (isdigit(*ptr))
        jtime = (jtime * 10) + (*ptr++ - '0');
      if (jtime > 0) {
        hhmm = jtime / 100;
        jtime = (hhmm * 60) + (jtime % 100);
        jtime %= FULLDAY;
        itime += (sign * jtime);
      }
    }

    while (itime < 0)
      itime += FULLDAY;

    itime %= FULLDAY;

    return((unsigned int)itime);
}

/***********************************************************************/
#ifdef __STDC__
static char *checkStopTime(const char *input)
#else
static char *checkStopTime(input)
const char *input;
#endif /*__STDC__*/
/*
 *  This routine tests that the amount of time needed for the
 *  observation does not exceed the task (scan/loop) stop time.
 *  If the time is acceptable as input, a pointer to NULL is
 *  returned; otherwise, a new time string is constructed that
 *  matches the task stop time and that is returned.
 */
{
    static char newtime[MAXBUF];
    register int hh, mm;
    unsigned int now, itime, deltime, stop;

    stop = LstNow + 60; /* The default stop time is one hour from now. */
    itime = getTime(input, stop);

    if (itime < StartTime)
      itime += FULLDAY;

    if ((OneShot == 0) && (itime <= StopTime))
      return((char *)NULL);                    /* No change necessary. */
    /*
     *  At this point, a new stop time keyword is needed.  Adjust
     *  the integration time and/or (local) stop time accordingly
     *  and then set the keyword stop time so as to not exceed
     *  the project StopTime.  Since most stop= keyword times will
     *  be written as stop=+mm, try to output this in the same format.
     */
    stop = StopTime;
    now = LstNow;
    if (now < StartTime)
      now += FULLDAY;
    deltime = itime - now;

    if (OneShot) {
      if (AddTime) itime += (stop - now - AddTime);
      if (SubTime) stop -= SubTime;
      OneShot = AddTime = SubTime = 0;
      itime = ((itime > stop) ? stop : itime) - now;
    } else {
      itime = stop - now;
    }

    if ((itime == deltime) && (itime <= (stop - now)))
      return((char *)NULL);             /* Still, no change necessary. */

    hh = itime / 60;
    hh %= 24;
    mm = itime % 60;
    if (hh > 0)
      (void)sprintf(newtime, "+%d%2.2d", hh, mm);
    else
      (void)sprintf(newtime, "+%d", mm);

    return(newtime);                       /* The new stop time value. */
}

/***********************************************************************/
#ifdef __STDC__
int checkTime(unsigned int offset)
#else
int checkTime(offset)
unsigned int offset;
#endif /*__STDC__*/
/*
 *  Returns 1 if (a) the LST could not be retrieved; (b) the current
 *  (LST + offset) is "past" the assigned stop time; or (c) the
 *  STOPFLAG common variable has been set.  Otherwise, the routine
 *  returns 0.
 *
 *  To insure that this test works properly, the stop time has been
 *  forced to always be greater than the project's start time.
 *  Likewise, the current time is forced (here) to always be greater
 *  than the project's start time.  This makes the termination test
 *  easier to understand (and code).
 */
{
    unsigned int now;

    if (setCurrentLST())
      return(1);

    now = getCurrentLST();
    now = (now + offset) % FULLDAY;
    if (now < StartTime)
      now += FULLDAY;

    if (now >= StopTime)
      return(1);
    else if (getCommon("stopflag", 1))
      return(1);
    else
      return(0);
}

/***********************************************************************/
#ifdef __STDC__
int checkLoopSrcTime(unsigned int src, unsigned int cal, unsigned int thresh)
#else
int checkLoopSrcTime(src, cal, thresh)
unsigned int src;
unsigned int cal;
unsigned int thresh;
#endif /*__STDC__*/
/*
 *  This is similar to the checkTime() routine above except that it
 *  is used by the loop command to test the source integration time.
 *  As loop reaches the end of its cycle, it is unlikely that there
 *  will be exactly (src + cal) minutes left.  Either the integration
 *  time of the source will need to be truncated or expanded.  This
 *  routine returns 1 if (a) the LST could not be retrieved; (b) the
 *  STOPFLAG common variable has been set; or (c) the current (LST +
 *  cal) is "past" the assigned stop time.  Otherwise, the routine
 *  returns 0.
 *
 *  The threshold specifies (in minutes) the shortest scan integration
 *  time for the source.  The src and cal arguments specify (in
 *  minutes) the duration of a typical scan on the source and
 *  calibrator, respectively.
 *
 *  To insure that this test works properly, the stop time has been
 *  forced to always be greater than the project's start time.
 *  Likewise, the current time is forced (here) to always be greater
 *  than the project's start time.  This makes the termination test
 *  easier to understand (and code).
 */
{
    unsigned int test;

    OneShot = AddTime = SubTime = 0;
    test = (2 * cal) + src + thresh;
    if (checkTime(test)) {
      if (checkTime(cal + src) == 0) {
        AddTime = cal + src;
        SubTime = cal;
        OneShot = 1;
      } else if (checkTime(cal + thresh) == 0) {
        SubTime = cal;
        OneShot = 1;
      } else {
        return(1);
      }
    }

    return(0);
}

/***********************************************************************/
#ifdef __STDC__
int setTimes(const char *starttime, const char *stoptime)
#else
int setTimes(starttime, stoptime)
const char *starttime;
const char *stoptime;
#endif /*__STDC__*/
/*
 *  Returns 1 if the current time is beyond the project/scan stop
 *  time or if there is an input error.  Returns 0 on success.
 */
{
    int hh, mm, itime;
    int delb, dele;
    unsigned int now, beg, end;

    if (setCurrentLST())        /* Failure if the LST can not be read. */
      return(1);

    if ((itime = getCommon("pstart", 0)) > 0) {
      hh = itime / 100;
      mm = itime % 100;
      itime = (hh * 60) + mm;
      itime %= FULLDAY;
    }
    StartTime = (unsigned int)itime;

    if ((itime = getCommon("pstop", 0)) > 0) {
      hh = itime / 100;
      mm = itime % 100;
      itime = (hh * 60) + mm;
      itime %= FULLDAY;
    }
    StopTime = (unsigned int)itime;

    if (AppDebug == TRUE)
      (void)printf("Project Start/Stoptime (minutes) = %d [%d:%d] %d [%d:%d]\n",
        StartTime, (StartTime / 60) % 24, StartTime % 60,
        StopTime, (StopTime / 60) % 24, StopTime % 60);

    if (StartTime == StopTime) {
      Warning(stderr, "Project Start and Stop times are the same!");
      return(1);
    }

    now = getCurrentLST();
    beg = getTime(starttime, now);
    end = getTime(stoptime, StopTime);

    if (AppDebug == TRUE)
      (void)printf("Keyword Start/Stop times (minutes) = %d [%d:%d] %d [%d:%d]\n",
        beg, (beg / 60) % 24, beg % 60, end, (end / 60) % 24, end % 60);

    /*
     *  Wrap all times so that they are (internally) always greater
     *  than the project start time.  This makes all future time math
     *  easier to deal with and gives a proper anchor for 24 hour wraps.
     *  Next, if the input start= and/or stop= arguments are outside
     *  the project time ranges, assign them to the nearest of the
     *  project start or stop time.
     */
    if (StopTime < StartTime) StopTime += FULLDAY;
    if (now < StartTime) now += FULLDAY;
    if (beg < StartTime) beg += FULLDAY;
    if (end < StartTime) end += FULLDAY;
    if (beg >= StopTime) {
      delb = (FULLDAY + StartTime - beg) % FULLDAY;
      dele = (FULLDAY + beg - StopTime) % FULLDAY;
      beg = (delb < dele) ? StartTime : StopTime;
    }
    if (end >= StopTime) {
      delb = (FULLDAY + StartTime - end) % FULLDAY;
      dele = (FULLDAY + end - StopTime) % FULLDAY;
      end = (delb < dele) ? StartTime : StopTime;
    }

    if (AppDebug == TRUE)
      (void)printf("Wrapped Start/Stop times (minutes) = %d [%d:%d] %d [%d:%d]\n",
        beg % FULLDAY, (beg / 60) % 24, beg % 60,
        end % FULLDAY, (end / 60) % 24, end % 60);

    /*
     *  If the argument stop time is earlier than the project stop time,
     *  reassign the (internal) project stop time to this earlier value.
     *  If the current (LST) time is beyond the project stop time, exit.
     *  If the argument start time has not been reached, notify the
     *  observer and wait (and the reset the internal clock).  Finally,
     *  reset the common block stop flag.
     */
    if (end < StopTime) StopTime = end;
    if (now >= StopTime)
      return(1);
    if (beg >= StopTime)
      return(1);
    if (beg > now) {
      beg -= (now + 1);
      Message(stdout, "Not at starting time yet; waiting %d minutes.\n", beg);
      (void)sleep(beg * 60);
      if (setCurrentLST())
        return(1);
    }

    setCommon("stopflag", 0);
    return(0);
}

/***********************************************************************/
/***********************************************************************/
/*                     S C A N   C O M M A N D S                       */
/***********************************************************************/
/***********************************************************************/

/***********************************************************************/
#ifdef __STDC__
static TASK *getTask(const char *varName)
#else
static TASK *getTask(varName)
const char *varName;
#endif /*__STDC__*/
{
    char *name;
    char *value;
    TASK *task;

    name = (varName == (const char *)NULL) ? "task" : (char *)varName;
    value = getVariable((const char *)NULL, name);

    task = (TASK *)NULL;

    if (strcmp("task", name) == 0) {
      if (value == (char *)NULL) {
        task = &mintcmd;
      } else if (strcmp(mintcmd.name, value) == 0) {
        task = &mintcmd;
      } else if (strcmp(mint3cmd.name, value) == 0) {
        task = &mint3cmd;
      } else if (strcmp(intcmd.name,  value) == 0) {
        task = &intcmd;
      } else if (strcmp(int3cmd.name, value) == 0) {
        task = &int3cmd;
      } else if (strcmp(qintcmd.name, value) == 0) {
        task = &qintcmd;
      }
    } else if (strcmp("corrtask", name) == 0) {
      if (value == (char *)NULL) {
        task = &setcorr;
      } else if (strcmp(setcorr.name, value) == 0) {
        task = &setcorr;
      } else if (strcmp(setcorr3.name, value) == 0) {
        task = &setcorr3;
      }
    } else if (strcmp("tunetask", name) == 0) {
      if (value == (char *)NULL) {
        task = &setfreq;
      } else if (strcmp(setfreq.name, value) == 0) {
        task = &setfreq;
      }
    } else if (strcmp("checktask", name) == 0) {
      if (value == (char *)NULL) {
        task = &cksrc;
      } else if (strcmp(cksrc.name, value) == 0) {
        task = &cksrc;
      }
    }

    return(task);
}

/***********************************************************************/
#ifdef __STDC__
char *getTaskName(const char *varName)
#else
char *getTaskName(varName)
const char *varName;
#endif /*__STDC__*/
{
    TASK *task;

    if ((task = getTask(varName)) == (TASK *)NULL)
      return(mintcmd.name);

    return(task->name);
}

/***********************************************************************/
#ifdef __STDC__
static void testVariables(const char *setup, FILE *fd)
#else
static void testVariables(setup, fd)
const char *setup;
FILE *fd;
#endif /*__STDC__*/
{
    int listSize;
    register int ntask, nargs;
    TASK *task;

    /*  Touch each of the observing task keywords. */
    listSize = sizeof(taskList) / sizeof(taskList[0]);
    for (ntask = 0; ntask < listSize; ntask++) {
      if ((task = getTask(taskList[ntask])) != (TASK *)NULL) {
        for (nargs = 0; task->keywords[nargs] != (char *)NULL; nargs++) {
          touchVariables(task->name, task->keywords[nargs]);
        }
      }
      /* Also touch the name of the observing task keywords. */
      touchVariables((const char *)NULL, taskList[ntask]);
    }

    /*  Now touch any other reserved keywords. */
    touchVariables((const char *)NULL, "maxobs");

    /*  Now notify the user about keywords that will not be used. */
    unusedVariables(setup, fd);
}

/***********************************************************************/
#ifdef __STDC__
static STATUS buildCommand(FILE *fd, char *command, int maxcom, TASK *task)
#else
static STATUS buildCommand(fd, command, maxcom, task)
FILE *fd;
char *command;
int maxcom;
TASK *task;
#endif /*__STDC__*/
{
    char *value, *newstop;
    char parameter[MAXBUF];
    register int i, j;
    int testStopTime;
    int len, reqlen;
    int nset, nargs, nreq;

    /* Count up the number of possible arguments for this command. */
    for (nargs = 0; task->keywords[nargs] != (char *)NULL; nargs++)
      /* NULL */ ;
    for (nreq = 0; task->required[nreq] != (char *)NULL; nreq++)
      /* NULL */ ;

    if (AppDebug == TRUE)
      (void)printf("%s [%d] args (%d required):\n", task->name, nargs, nreq);

    testStopTime = (strcmp(task->name, getTaskName((const char *)NULL)) == 0);
    (void)strcpy(command, task->name);
    len = strlen(command);
    nset = 0;

    for (i = 0; i < nargs; i++) {     /* Loop for each value'd keyword */
      value = getVariable(task->name, task->keywords[i]);

      if (AppDebug == TRUE)
        (void)printf("\t%s = %s\n", task->keywords[i],
          ((value == (char *)NULL) ? "NULL" : value));

      if ((testStopTime) && (strcmp(task->keywords[i], "stop") == 0)) {
        if ((newstop = checkStopTime(value)) != (char *)NULL) {
          Warning(fd, "Stop keyword to task [%s] will be adjusted to [%s].\n",
            task->name, newstop);
          value = newstop;
        }
      }

      if (value != (char *)NULL) {
        len += strlen(task->keywords[i]) + strlen(value) + 4;
        if (len > maxcom) {
          Warning(stderr, "Out of argument space to execute command [%s].\n",
            task->name);
          return(ERROR);
        }
        (void)sprintf(parameter, " %s=\"%s\"", task->keywords[i], value);
        (void)strcat(command, parameter);
        nset++;
        if (nreq > 0) {
          for (j = 0; task->required[j] != (char *)NULL; j++) {
            reqlen = strlen(task->required[j]);
            if (strncmp(task->keywords[i], task->required[j], reqlen) == 0) {
              nreq--;
              break;
            }
          }
        }
      }
    }

    if ((nset == 0) && (nargs > 0))
      return(NONE);

    if (nreq > 0)
      return(NONE);

    return(OKAY);
}

/***********************************************************************/
#ifdef __STDC__
static STATUS doSpawn(FILE *fd, TASK *task, char *retval, int maxsize, int echo)
#else
static STATUS doSpawn(fd, task, retval, maxsize, echo)
FILE *fd;
TASK *task;
char *retval;
int maxsize;
int echo;
#endif /*__STDC__*/
{
    char command[MAXBUF];
    int spoolstate, retstate;
    STATUS status;

    status = buildCommand(fd, command, MAXBUF, task);
    if (status != OKAY)                  /* ==> status = ERROR or NONE */
      return(status);

    if (AppDebug == TRUE)
      (void)printf("Command: %s\n", command);

    spoolstate = spool(command, retval, maxsize, &retstate, echo);

    if (AppDebug == TRUE) {
      (void)printf("[Spool returns = %d]\n", spoolstate);
      (void)printf("[Spool output string = %s]\n", retval);
      (void)printf("[Spool return status = %d]\n", retstate);
    }

    if (spoolstate != 0) {
      Warning(stderr, "Could not execute command [%s].\n", task->name);
      return(ERROR);
    } else if (retstate != 0) {
      Warning(stderr, "Command [%s] returned with error status [%d].\n",
        command, retstate);
      return(ERROR);
    }

    return(OKAY);
}

/***********************************************************************/
#ifdef __STDC__
static STATUS executeTask(const char *setup, FILE *fd)
#else
static STATUS executeTask(setup, fd)
const char *setup;
FILE *fd;
#endif /*__STDC__*/
{
    char retval[MAXBUF];
    int done;
    STATUS status;            /* Return status from the spawn routine. */
    TASK *task;

    if (getMaxObs(setup)) {
      Warning(fd,
        "%s Setup [%s] exceeds the maximum number of observations permitted.\n",
        formatTime(), setup);
      return(NONE);
    }

    done = 0;
    if ((task = getTask("checktask")) == (TASK *)NULL) {
      Warning(fd, "%s Setup [%s] has an unknown source checking task.\n",
        formatTime(), setup);
      return(ERROR);
    }
    status = doSpawn(fd, task, retval, MAXBUF, 0);

    if (status == ERROR) {
      Warning(fd, "%s Setup [%s] had problems with %s.\n", formatTime(),
        setup, task->name);
      return(ERROR);
    } else if (status == NONE) {
      Message(fd, "%s Setup [%s] did not call %s.\n", formatTime(),
        setup, task->name);
    } else if ((status == OKAY) && (strcmp(retval, "y") != 0)) {
      Message(fd, "%s Setup [%s] did not pass %s.\n", formatTime(),
        setup, task->name);
      return(NONE);
    }
    if (status == OKAY) done++;

    /*  Tune. */
    if ((task = getTask("tunetask")) == (TASK *)NULL) {
      Warning(fd, "%s Setup [%s] had an unknown tuning task.\n",
        formatTime(), setup);
      return(ERROR);
    }
    status = doSpawn(fd, task, retval, MAXBUF, 1);

    if ((status != NONE) && (status != OKAY)) {
      Warning(fd, "%s Setup [%s] had problems with %s.\n",
        formatTime(), setup, task->name);
      return(status);
    }
    if (status == OKAY) done++;

    /*  Set up the correlator. */
    if ((task = getTask("corrtask")) == (TASK *)NULL) {
      Warning(fd, "%s Setup [%s] had an unknown correlator task.\n",
        formatTime(), setup);
      return(ERROR);
    }
    status = doSpawn(fd, task, retval, MAXBUF, 1);

    if ((status != NONE) && (status != OKAY)) {
      Warning(fd, "%s Setup [%s] had problems with %s.\n",
        formatTime(), setup, task->name);
      return(status);
    }
    if (status == OKAY) done++;

    /*  Main observing task. */
    if ((task = getTask("task")) == (TASK *)NULL) {
      Warning(fd, "%s Setup [%s] had an unknown observing task.\n",
        formatTime(), setup);
      return(ERROR);
    }
    status = doSpawn(fd, task, retval, MAXBUF, 1);

    if ((status != NONE) && (status != OKAY)) {
      Warning(fd, "%s Setup [%s] had problems with %s.\n",
        formatTime(), setup, task->name);
      return(status);
    } else if (status == NONE) {
      if (done == 0)
        Message(fd, "%s Setup [%s] did not succeed at running anything.\n",
          formatTime(), setup);
      else
        Message(fd, "%s Setup [%s] did not call %s.\n",
          formatTime(), setup, task->name);
      return(status);
    }

    incrementObs(setup);
    return(status);
}

/***********************************************************************/
#ifdef __STDC__
static STATUS doSetup(const char *setup, FILE *fd, const char *project)
#else
static STATUS doSetup(setup, fd, project)
const char *setup;
FILE *fd;
const char *project;
#endif /*__STDC__*/
{
    STATUS status;           /* Return status from the executing task. */

    freeVariables();

    addVariable("ants", "*");          /* Set up a reasonable default. */

    if (loadVariables(setup, project, 0) != 0) {
      Warning(stderr, "Trouble accessing setup file [%s].\n", setup);
      return(ERROR);
    }

    if (AppDebug == TRUE)
      (void)printf("setup [%s] called.\n", setup);

    testVariables(setup, fd);

    status = executeTask(setup, fd);

    return(status);
}

/***********************************************************************/
/*
 *  This is the main loop routine.  The setup argument specifies
 *  a string of setups delimited by and's and or's and, perhaps,
 *  grouped by matching parentheses.  Each group is treated as one
 *  unit, the return state specifying the state of the group and not
 *  the individual members.  Every group in an 'and' step are observed
 *  but only the first valid setup in an 'or' step is observed.
 *  The file descriptor points to a history log file and each setup
 *  (attempted to be) observed will have an entry.
 *  The project argument is used to determine where the setup files
 *  will be found.
 *
 *  The return status is NONE if no sources are observed, ERROR if an
 *  error happens, or OKAY if one setup in a group is observed correctly.
 */
#ifdef __STDC__
STATUS doScan(const char *setup, FILE *fd, const char *project)
#else
STATUS doScan(setup, fd, project)
const char *setup;
FILE *fd;
const char *project;
#endif /*__STDC__*/
{
    char *ptr, *next, *token;
    char op;
    char group[MAXBUF];
    int j, doObs;
    STATUS state, retState;

    retState = NONE;

    doObs = TRUE;
    ptr = (char *)setup;
    while ((next = getGroup(ptr, group)) != (char *)NULL) {
      if (checkTime(0))
        break;

      token = group;
      if (doObs != TRUE) {
        Message(fd, "%s Setup [%s] was skipped.\n", formatTime(), token);
      } else {
        j = strlen(token) - 1;
        if ((*token == '(') && (token[j] == ')')) {
          token[j] = Null;
          token++;
          state = doScan(token, fd, project);
        } else {
          state = doSetup(token, fd, project);
        }

        /*  Write a message to the history file. */
        switch (state) {
          case OKAY:
            Message(fd, "%s Setup [%s] was observed.\n", formatTime(), token);
            break;
          case NONE:
            Message(fd, "%s Setup [%s] was not observed.\n", formatTime(),
              token);
            break;
          case ERROR:
            Message(fd, "%s Setup [%s] returned with an error.\n",
              formatTime(), token);
            return(state);
          default:                    /* This should never be reached! */
            Message(fd, "%s Setup [%s] returned in unknown state.\n",
              formatTime(), token);
            state = ERROR;
            return(state);
        }
      }

      /*
       *  After skipping leading white space, get the next delimiter
       *  character.  It is an error if the first non-white space
       *  character is not a delimiter and there is still text that
       *  follows.  The pointer "ptr" is set to the next character
       *  after the delimiter (or the incorrect character).
       */
      ptr = skipLeading(next);
      op = Null;
      if (isdelimiter(*ptr)) {
        op = *ptr++;
        ptr = skipLeading(ptr);
      }

      /*
       *  If there is more in the string to process, then a delimiter
       *  must be present.  If not, then an error has occurred.  The
       *  delimiter and the returned state will determine if anymore
       *  observing will take place within this group of setups.
       */
      if (*ptr != Null) {
        switch(op) {
          case ORSYMBOL:  /* Only the first valid observation is used. */
            if ((doObs == TRUE) && (state == OKAY))
              doObs = FALSE;
            break;
          case ANDSYMBOL:            /* Next group is always observed. */
            doObs = TRUE;
            break;
          default:
            Warning(stderr, "An operator was expected here [%s]\n", next);
            return(ERROR);
        }
      }

      if ((retState != OKAY) && (state == OKAY)) retState = state;
    }

    return(retState);
}
