#!/bin/csh -f
#-----------------------------------------------------------------------
#
#  History:
#    11sep93  jm Original script.
#    16sep93 pjt Modified for checker: HATDATA becomes /tmp.
#    07oct93  jm Expanded to full command which executes observing script.
#    29nov93  jm Removed HATCHK setting of HATDATA directory.  Added a
#                test of the stop time so alter would not treat it as
#                an octal number.  Also added a test to see if the
#                script file exists.
#    01dec93  jm Increased wait time from 30 minutes to 2 hours.
#    10dec93  jm Changed stoptime to pstart and pstop.
#    20dec93  jm Cleaned up by getting rid of debug statements.
#    28jan94  jm Added script= optional argument.
#    01feb94  jm Added debug= optional argument and added search of
#                $HATSCRIPT directory for script file.  Also, in checker
#                mode, the default is to always call resetproject.
#    08feb94  jm Added date= optional argument and added a more
#                verbose usage statement.  Also modified script and
#                project name construction to account for 14 character
#                limit.  Added alarm and message if the observing script
#                exits with a non-zero status.
#    18feb94  jm Added sleep test and optional environment variable
#                WAITTASK which specifies an alternative sleep task.
#                Also, added setalarm calls and log entries for all
#                non-zero exits.
#    24feb94  jm Added call to setfocus before obs-task execution.
#    28mar94  jm Added project to message about WAITTASK and increased
#                default wait time to 8 hours.
#    27apr94  jm Added HC archiving requests.
#    25jul94  jm Added -e flag to project script calls.  Also added
#                $DONTARCHIVE test (hopefully to be removed later).
#    19aug94  jm Added dir= optional argument to override $HATDATA as
#                the parent directory of the archive.
#
#-----------------------------------------------------------------------
#  Set up some necessary definitions and check that some inputs exist.
#
set exitstate = 1
set BELL=`echo X | tr X '\007'`
alias ErrorEcho 'set X = '"\!*"'; echo "$BELL$X"; comment is="$X"; unset X'
alias FatalEcho 'set X = '"\!*"'; echo "$BELL$X"; comment is="ALARM: $X"; unset X'
#
if ($#argv < 1) then
  set exitstate = 0
  goto usage
endif
#
#  Set up input argument defaults and unset required arguments.
#
set project = "project"
set wait = 0800
set debug = 0
#
unset date
unset script
unset email
unset name
unset stop
unset start
unset dir
#
#  Get the command line arguments.
#
while ($#argv)
  set check = `echo $argv[1] | awk -F= '{print NF}'`
  if ("$check" > 1) then
    set $argv[1]
  endif
  shift
end
#
#  Check that the required inputs have been given.
#
if ($?name == 0) then
  FatalEcho "### Fatal Error: Missing argument name="
  goto usage
endif
if ($?start == 0) then
  FatalEcho "### Fatal Error: Missing argument start="
  goto usage
endif
if ($?stop == 0) then
  FatalEcho "### Fatal Error: Missing argument stop="
  goto usage
endif
#
#  Check for the presence of the observing script.  This first looks
#  in the current directory and then, possibly, in $HATSCRIPT.  If
#  the script is not found, an error message is printed and exits.
#  If the script exists and has more than 14 characters in the
#  name, only the first 14 characters in the name are retained.
#
if ($?script == 0) then
  set script = $name
endif
#
if (! -f $script) then
  if ($?HATSCRIPT) then
    if (-d "$HATSCRIPT") then
      if (-f "${HATSCRIPT}/${script}") then
        set script = "${HATSCRIPT}/${script}"
      endif
    endif
  endif
  if (! -f $script) then
    FatalEcho "### Fatal Error: Script [$script] can not be found."
    goto usage
  endif
endif
#
set newscript = `echo ${script:t} | awk '{print substr($1,1,14)}'`
#
#  Check the input start and stop times.  If the current time is
#  out of range, then quietly exit.
#
unset sleepsec
set times = "`timesup start=$start stop=$stop wait=$wait`"
if ($status == 1) then
  FatalEcho "### Fatal Error: Could not assess input time ranges\!"
  goto cleanup
endif
if ("$times" == "quit") then
  if ($?HATCHK) then
    ErrorEcho "### Notice: Project ${name:t} skipped because time not in range."
    set exitstate = 0
  else
    FatalEcho "### Notice: Project ${name:t} skipped because time not in range."
  endif
  goto finished
endif
set $times
#
#  If in checker mode, just echo a warning and adjust the LST.
#  If not in checker mode and the time to wait is greater than
#  30 minutes (but less than the value of $wait) and the environment
#  variable WAITTASK is set, then notify the caller and execute
#  $WAITTASK with a stop time argument equal to this project's start
#  time.  Otherwise notify the caller and sleep until the start time.
#
if ($?sleepsec) then
  if ($?HATCHK) then
    echo "${BELL}Waiting [$sleepsec] seconds until the project start time."
    lstsymbol inc=`echo $sleepsec | awk '{printf("%f", $1 / 3600.0)}'`
  else if (($?WAITTASK) && ($sleepsec > 1800)) then
    set mins=`echo $sleepsec | awk '{printf("%d", ($1 / 60))}'`
    ErrorEcho "Project ${name:t} will run $WAITTASK ($mins mins) until the project start time."
#-- Until these tasks go to new time format...
#    $WAITTASK stop=${start}-1
#-- ...use the following:
    $WAITTASK stop=$start
    if ($status != 0) then
      FatalEcho "### Error: Wait task [$WAITTASK] exited with an error status."
      goto cleanup
    endif
  else
    ErrorEcho "Waiting [$sleepsec] seconds until the project start time."
    sleep $sleepsec
  endif
endif
#
#  Set up the flag to be passed to the shell.  The flag 'f' means start
#  up in fast mode and don't read the .cshrc file; 'e' means to exit the
#  script immediately if an error occurs.  If DEBUG is 0, undefined,
#  or a bad value, then debugging is turned off.  If DEBUG is 1, then
#  commands are executed in verbose mode.  If DEBUG is 2, then each
#  command is also echoed before execution in verbose mode.
#
set shellargs = "-fe"
if ($?debug != 0) then
  if ($debug == 1) then
    set shellargs = "-fev"
  else if ($debug == 2) then
    set shellargs = "-fevx"
  endif
endif
#
#  Set up a flag such that aborting this script will clean up properly.
#
onintr cleanup
#
#  Make sure the archive directory is specified.
#
if ($?HATDATA == 0) then
  setenv HATDATA $DATA/archive/newdata
endif
if (! -d "$HATDATA") then
  FatalEcho "### Fatal Error: Data directory [$HATDATA] does not exist\!"
  goto cleanup
endif
#
#  Get today's date.
#
if ($?date == 0) then
  set date=`date +%d%h%y`
endif
set thisdate=`echo $date | tr '[A-Z]' '[a-z]'`
#
#  Check the project name.  If the name has the suffix "csh", then
#  the suffix is stripped off.  Also, any characters in the resulting
#  name beyond the first 14 are stripped off.
#
set thisname = $name:t
if ("${thisname:e}" == "csh") then
  set thisname = $thisname:r
endif
set thisname = `echo $thisname | awk '{print substr($1,1,14)}'`
#
if (($?HATCHK == 0) && ($?email != 0)) then
  mail_PI "$email" "${name:t}"
endif
#
#  Create the data directory and the pseudo-Miriad data set that will
#  contain the observing information.
#
if ($?dir == 0) then
  set thisproject = $HATDATA/$thisdate/$thisname
else
  set thisproject = ${dir}/$thisdate/$thisname
endif
mkdir -p $thisproject/$project
#
#  Set the project start and stop time.  The conditional test is to
#  insure that alter does treat the time as an octal value.
#
if ($start < 1000) then
  @ start += 0
endif
alter name=pstart value=$start
set testStop = $stop
if ($testStop < 1000) then
  @ testStop += 0
endif
alter name=pstop value=$testStop
if ($status) set testStop = -1
#
#  If in CHECKER mode, reset the maxobs variables.  If at the site,
#  update the home file ".project" so finger can see what is running.
#  Then change the "project" common block value and add a comment
#  to the master log file.
#
if (($?HATCHK) && ($?RESET_VAR)) then
  resetproject name=$name date=$thisdate
endif
if ($?HATCHK == 0) then
  echo "LAST PROJECT STARTED: $thisname on `date`" >! $DATA/.project
endif
#alter name=project value=$thisname
comment is="HATPROJECT : $thisname"
#
#  Move the observing script into the project directory and move
#  there too.  Before running the observing script, call the
#  following system particular commands:
#  1. setfocus set=t  #  Only call this once per project!
#
cp -p $script $thisproject/$project/$newscript
cd $thisproject
#
set check=`svalue name=setfocus`
if ("$check" == 0) then
  setfocus set=t
  if ($status != 0) then
    FatalEcho "### Error: System call [setfocus] exited with an error status."
    goto cleanup
  endif
  svalue name=setfocus value=+1
endif
#
#  Send a request to begin archiving this project.
#
if (($?HATCHK == 0) && ($?DONTARCHIVE == 0)) then
  HCarchT -a $thisproject
  if (($status != 0) && ($status != 1)) then
    mail -s "HCarchT -a [$thisproject] failed with status [$status]." \
      bimadata@bima2.berkeley.edu < /dev/null
  endif
endif
#
#  Execute the observing script and save the exit status.
#
alter name=project value=$thisname
csh $shellargs ./$project/$newscript
set exitstate = $status
#
if ($exitstate != 0) then
  FatalEcho "### Error: Observing script [$name] exited with an error status."
  goto cleanup
endif
#
#  Send a request that this project is done writing.
#
if (($?HATCHK == 0) && ($?DONTARCHIVE == 0)) then
  HCarchT -f $thisproject
endif
#
#  Check that the project stop time was not altered... Warn Observer if it has.
#
if ($testStop > 0) then
  set tstop = `value name=pstop | awk '{print $NF}'`
  if ($tstop != $testStop) then
    if ($?HATCHK == 0) then
      mail -s "Project [$name] alters PSTOP." data@bima.berkeley.edu < /dev/null
    else
      echo "Your script alters the common variable PSTOP.  This is illegal\!"
    endif
  endif
endif
#
#  All done except for cleaning up the project stop time.
#
cleanup:
#
#  Reset the project start and stop time as well as the project name.
#
echo "Finished observing project $name."
alter name=pstart value=0
alter name=pstop value=0
alter name=project value="NONE"
goto finished
#
######################################################################
# Usage:
usage:
set task=$0; set task=$task:t
echo "### Usage: $task requiredArgs [optionalArgs]"
echo "Required Arguments:"
echo "	name=projectName	: The name of the project."
echo "	start=startTimeHHMM	: The project start time in hhmm-format."
echo "	stop=stopTimeHHMM	: The project stop time in hhmm-format."
echo "Optional Arguments:"
echo "	email=emailAddress	: No e-mail set if not present."
echo "	script=scriptName	: Defaults to name if not present."
echo "	debug={0,1,2}		: Set debug level; defaults to 0."
echo "	date=ddmmmyy		: Manually set value of day; default is today."
echo "	wait=hhmm		: If LST is outside [start-wait,stop], quit."
echo "	dir=/new/root/dir	: Override the default root archive directory."
if ($?HATDATA) then
  echo 'Default root archive directory: $HATDATA='$HATDATA
endif
if ($?HATSCRIPT) then
  echo 'Default script directory: $HATSCRIPT='$HATSCRIPT
endif
#
finished:
#
if (($exitstate != 0) && ($?HATCHK == 0)) then
  setalarm to=on
  setalarm to=test
endif
#
exit ($exitstate != 0)
